using System;
using System.Collections.Generic;
using System.Text;
using Bustools1553;
using Bustools1553.DataTypes;
using System.Runtime.InteropServices;

namespace BustoolsCsApp
{
   public static class BcTests
   {
      public static Int32 status;
      public static UInt16 messno, device;
      public static UInt32 pwFlag;
      public static UInt32 gCardNum;
      public static Form1 mainForm;

      /////////////////////////////////////////////////////////////////////////////////
      // these delegates and the next 2 routines provide thread-safe access to the text box on the main form
      public delegate void AddToOutputDelegate(String s);
      public static AddToOutputDelegate addDelegate;
      public static void AddStringToOutput(String s)
      {
         mainForm.tbOutput.AppendText(s);
      }
      public static void AddText(String s)
      {
         if (mainForm.tbOutput.InvokeRequired)
         {
            mainForm.tbOutput.Invoke(addDelegate, new object[] { s });
         }
         else
         {
            mainForm.tbOutput.AppendText(s);
         }
      }
      // end text output routines
      /////////////////////////////////////////////////////////////////////////////////

      // constructor
      static BcTests()
      {
         addDelegate = new AddToOutputDelegate(AddStringToOutput);
      }

      // main routine to setup the BC test
      public static void Init()
      {
         float wcs, lpu;
         Int32 build;

         int status;
         UInt16[] retry = new UInt16[9];
         pwFlag = 1; // h/w interrupts
         device = 1;

         // Initializing Channel
         status = API.OpenChannel(out gCardNum, pwFlag, device, Constants.CHANNEL_1);
         AddText(String.Format("API_OpenChannel status = {0}\r\n", status));

         status = API.BM_Init(gCardNum, 1, 1);
         AddText(String.Format("BM_Init status = {0}\r\n", status));

         status = API.GetFWRevision(gCardNum, out wcs, out lpu, out build);
         AddText(String.Format("WCS = {0} LPU = {1} BUILD = {2} \r\n", wcs.ToString(), lpu.ToString(), build.ToString()));

         status = API.SetInternalBus(gCardNum, Constants.EXTERNAL_BUS); // Run internally for this example
         AddText(String.Format("SetInternalBus status = {0}\r\n", status));

         status = API.SetVoltage(gCardNum, 650, Constants.DIRECT);

         // Enable Message Scheduling and Fixed Gap Timing
         status = API.BC_Init(gCardNum, Constants.FIXED_GAP | Constants.MSG_SCHD, Constants.BT1553_INT_END_OF_MESS, Constants.BC_RETRY_NRSP, 14, 12, 1000000, 1);
         AddText(String.Format("BC_Init status = {0}\r\n", status));

         status = API.BC_MessageAlloc(gCardNum, 100);
         AddText(String.Format("BC_MessageAlloc status = {0}\r\n", status));

         retry[0] = Constants.RETRY_ALTERNATE_BUS;
         retry[1] = Constants.RETRY_SAME_BUS;
         retry[2] = Constants.RETRY_ALTERNATE_BUS;
         for (int i = 3; i <= 8; i++) retry[i] = 0;

         status = API.BC_RetryInit(gCardNum, retry);
         AddText(String.Format("BC_RetryInit status = {0}\r\n", status));

         Define_Bus_Controller_Messages(gCardNum);

         SetupBcIntFifo(gCardNum);  // Setup the interrupt callback for BC messages

         status = API.BC_StartStop(gCardNum, Constants.START);
         AddText(String.Format("BC_StartStop status = {0}\r\n", status));

         //API.DumpMemory(gCardNum, Constants.DUMP_ALL, "MSG_SCHDxxx.txt", "dump mess sched");
      }

      public static void Define_Bus_Controller_Messages(UInt32 cardnum)
      {
         messno = 0;
         API_BC_MBUF bcMessage = new API_BC_MBUF();
         bcMessage.messno = messno;
         bcMessage.messno_next = (UInt16)(messno + 1);
         bcMessage.control = Constants.BC_CONTROL_MESSAGE |
                              Constants.BC_CONTROL_BUFFERA |
                              Constants.BC_CONTROL_CHANNELA |
                              Constants.BC_CONTROL_INTERRUPT |
                              Constants.BC_CONTROL_MFRAME_BEG;

         bcMessage.mess_command1.rtaddr = 2;
         bcMessage.mess_command1.subaddr = 2;
         bcMessage.mess_command1.wcount = 2;
         bcMessage.mess_command1.tran_rec = 0;
         bcMessage.start_frame = 1;
         bcMessage.rep_rate = 1;
         bcMessage.gap_time = 500;
         status = API.BC_MessageWrite(cardnum, messno, bcMessage);

         //Msg slot # 2
         messno++;
         bcMessage = new API_BC_MBUF();
         bcMessage.messno = messno;
         bcMessage.messno_next = (UInt16)(messno + 1);
         bcMessage.control = Constants.BC_CONTROL_MESSAGE |
                              Constants.BC_CONTROL_BUFFERA |
                              Constants.BC_CONTROL_CHANNELA |
                              Constants.BC_CONTROL_INTERRUPT;

         bcMessage.mess_command1.rtaddr = 3;
         bcMessage.mess_command1.subaddr = 3;
         bcMessage.mess_command1.wcount = 3;
         bcMessage.mess_command1.tran_rec = 0;
         bcMessage.start_frame = 1;
         bcMessage.rep_rate = 2;
         bcMessage.gap_time = 1000;
         status = API.BC_MessageWrite(cardnum, messno, bcMessage);

         //Msg slot # 3
         messno++;
         bcMessage = new API_BC_MBUF();
         bcMessage.messno = messno;
         bcMessage.messno_next = (UInt16)(messno + 1);
         bcMessage.control = Constants.BC_CONTROL_MESSAGE |
                              Constants.BC_CONTROL_BUFFERA |
                              Constants.BC_CONTROL_CHANNELA |
                              Constants.BC_CONTROL_INTERRUPT;

         bcMessage.mess_command1.rtaddr = 4;
         bcMessage.mess_command1.subaddr = 4;
         bcMessage.mess_command1.wcount = 4;
         bcMessage.mess_command1.tran_rec = 0;
         bcMessage.start_frame = 2;
         bcMessage.rep_rate = 2;
         bcMessage.gap_time = 2000;
         status = API.BC_MessageWrite(cardnum, messno, bcMessage);

         //Msg slot # 4
         messno++;
         bcMessage = new API_BC_MBUF();
         bcMessage.messno = messno;
         bcMessage.messno_next = (UInt16)(messno + 1);
         bcMessage.control = Constants.BC_CONTROL_MESSAGE |
                              Constants.BC_CONTROL_BUFFERA |
            //Constants.BC_CONTROL_RETRY |
                              Constants.BC_CONTROL_CHANNELA |
                              Constants.BC_CONTROL_INTERRUPT;

         bcMessage.mess_command1.rtaddr = 14;
         bcMessage.mess_command1.subaddr = 14;
         bcMessage.mess_command1.wcount = 14;
         bcMessage.mess_command1.tran_rec = 0;
         bcMessage.start_frame = 2;
         bcMessage.rep_rate = 2;
         bcMessage.gap_time = 3000;
         status = API.BC_MessageWrite(cardnum, messno, bcMessage);

         //Msg slot # 5
         messno++;
         bcMessage = new API_BC_MBUF();
         bcMessage.messno = messno;
         bcMessage.messno_next = (UInt16)(messno + 1);
         bcMessage.control = Constants.BC_CONTROL_MESSAGE |
                              Constants.BC_CONTROL_BUFFERA |
                              Constants.BC_CONTROL_CHANNELA |
                              Constants.BC_CONTROL_INTERRUPT;
         bcMessage.mess_command1.rtaddr = 6;
         bcMessage.mess_command1.subaddr = 6;
         bcMessage.mess_command1.wcount = 6;
         bcMessage.mess_command1.tran_rec = 0;
         bcMessage.start_frame = 1;
         bcMessage.rep_rate = 1;
         bcMessage.gap_time = 5000;
         status = API.BC_MessageWrite(cardnum, messno, bcMessage);

         // NOOP End of frame  last message in each frame
         messno++;
         bcMessage = new API_BC_MBUF();
         bcMessage.messno = messno;
         bcMessage.messno_next = (UInt16)(0);
         bcMessage.control = Constants.BC_CONTROL_NOP;      ///NOOP
         bcMessage.control |= Constants.BC_CONTROL_MFRAME_END;

         status = API.BC_MessageWrite(cardnum, messno, bcMessage);


         //Msg slot # 20 Aperiodic Message
         messno = 20;
         bcMessage = new API_BC_MBUF();
         bcMessage.messno = messno;
         bcMessage.messno_next = (UInt16)(0xffff);
         bcMessage.control = Constants.BC_CONTROL_MESSAGE |
                              Constants.BC_CONTROL_BUFFERA |
                              Constants.BC_CONTROL_CHANNELA |
                              Constants.BC_CONTROL_INTERRUPT;
         bcMessage.mess_command1.rtaddr = 16;
         bcMessage.mess_command1.subaddr = 16;
         bcMessage.mess_command1.wcount = 16;
         bcMessage.mess_command1.tran_rec = 0;
         bcMessage.start_frame = 1;
         bcMessage.rep_rate = 1;
         bcMessage.gap_time = 10;
         status = API.BC_MessageWrite(cardnum, messno, bcMessage);
      }

      /****************************************************************************
      *
      *  Function:  IntFifoCallbackFunc
      *
      *  Description:
      *     This routine is the user callback function invoke when the Board detects
      *     a end-of-message condition. This function read the data in the RT message
      *     and print message data to the screen.
      *
      ****************************************************************************/

      public static Int32 IntFifoCallbackFunc(UInt32 cardnum, IntPtr intFifoAddr)
      {
         API_BC_MBUF bcMessage = new API_BC_MBUF();
         int bufnum;
         int i, j;
         int status;
         int wcount;
         Int32 tail, rtrt, mc;
         Int32 messno;
         StringBuilder outbuf = new StringBuilder(80);

         // marshall the return pointer to a managed int fifo
         API_INT_FIFO intFifo = (API_INT_FIFO)Marshal.PtrToStructure(intFifoAddr, typeof(API_INT_FIFO));

         tail = intFifo.tail_index;
         while (tail != intFifo.head_index)
         {
            if ((intFifo.FilterType & Constants.EVENT_BC_MESSAGE) != 0)
            {
               messno = intFifo.fifo[tail].bufferID;
               status = API.BC_MessageRead(cardnum, messno, bcMessage);
               if (status != 0) return status;

               rtrt = 0;
               mc = 0;

               API.TimeGetString(ref bcMessage.time_tag, outbuf);
               AddText(String.Format("time tag = {0}\r\n", outbuf));

               if ((bcMessage.status & Constants.BT1553_INT_RETRY) != 0)
               {
                  AddText(String.Format("-------------------------------------------------\r\n\r\n"));
               }

               if ((bcMessage.status & Constants.BT1553_INT_RT_RT_FORMAT) != 0)
               {
                  AddText(String.Format("RT<->RT ({0})\r\n", messno.ToString()));
                  rtrt = 1;
               }
               else if ((bcMessage.status & Constants.BT1553_INT_MODE_CODE) != 0)
               {
                  AddText(String.Format("Mode Code ({0})\r\n", messno.ToString()));
                  mc = 1;
               }
               else if (bcMessage.mess_command1.tran_rec != 0)
               {
                  AddText(String.Format("BC**RT->BC ({0})\r\n", messno.ToString()));
               }
               else
               {
                  AddText(String.Format("BC->RT ({0})\r\n", messno.ToString()));
               }

               AddText(String.Format("Message Command Word 1 = {0}\r\n", bcMessage.mess_command1.cmdWord.ToString("X8")));
               AddText(String.Format("BC**RT-({0})\r\n", bcMessage.mess_command1.rtaddr.ToString()));
               AddText(String.Format("BC**SA-({0})\r\n", bcMessage.mess_command1.subaddr.ToString()));
               AddText(String.Format("BC**WC-({0})\r\n", bcMessage.mess_command1.wcount.ToString()));
               if (rtrt != 0)
               {
                  AddText(String.Format("BC**RT-({0})\r\n", bcMessage.mess_command2.rtaddr.ToString()));
                  AddText(String.Format("BC**SA-({0})\r\n", bcMessage.mess_command2.subaddr.ToString()));
                  AddText(String.Format("BC**WC-({0})\r\n", bcMessage.mess_command2.wcount.ToString()));
               }

               AddText(String.Format("BC**status-1 ({0})\r\n", bcMessage.mess_status1.ToString()));
               if (rtrt != 0)
                  AddText(String.Format("BC**status-2 ({0})\r\n", bcMessage.mess_status2.ToString()));

               AddText(String.Format("BC**int_stat- ({0})\r\n", bcMessage.status.ToString()));

               wcount = bcMessage.mess_command1.wcount;
               if (wcount == 0)
                  wcount = 32;           // Zero word count means 32 words.

               if (mc != 0 && wcount >= 16)
                  wcount = 1;

               j = wcount;               // See if this msg shorted than previous

               if ((bcMessage.control & Constants.BC_CONTROL_BUFFERA) != 0)
                  bufnum = 0;            // Display data from first BC buffer
               else
                  bufnum = 1;            // Display data from second BC buffer

               for (i = 0; i < j; i++) // Display all of the data words
               {
                  // If this is an RT->BC message, and there is a Message Error or No Response, than xxxx the data 
                  // (since the RT did not send any data words).
                  if (i >= wcount)
                  {
                     AddText(String.Format("     "));
                  }
                  else if (
                             (bcMessage.mess_command1.tran_rec != 0) &&
                             ((bcMessage.mess_status1.me != 0) || ((bcMessage.status & Constants.BT1553_INT_NO_RESP) != 0))
                           )
                  {
                     AddText(String.Format("xxxx "));
                  }
                  else
                  {
                     AddText(String.Format("BC**{0}", bcMessage.GetData(bufnum, i).ToString()));
                  }
               }
               AddText(String.Format("\r\n"));
            }

            if ((intFifo.FilterType & Constants.EVENT_RT_MESSAGE) != 0)
            {
               intFifo.nUser[0]++;
            }

            tail++;
            tail &= intFifo.mask_index;
            intFifo.tail_index = tail;

            AddText(String.Format("\r\n\r\n"));

         }

         API.UpdateTailPTR(intFifoAddr, tail);

         return 0;
      }



      // This routine setup the interrupt FIFO with the value needed for a interrupt on a BC message.  The 
      // interrupt is set for only the end-of message.  
      public static void SetupBcIntFifo(UInt32 cardnum)
      {
         // intFifoDelegate can be used as a callback pointer
         API_INT_FIFO_Callback intFifoDelegate = new API_INT_FIFO_Callback(IntFifoCallbackFunc);

         // have the API create an unmanaged int fifo, and set the callback pointer
         IntPtr pIntFifo = API.CreateIntFifo(intFifoDelegate);

         // marshall the return pointer to a manages int fifo
         API_INT_FIFO intFifo = (API_INT_FIFO)Marshal.PtrToStructure(pIntFifo, typeof(API_INT_FIFO));

         int rt, tr, sa, status;
         // Setup the int Fifo structure for this board.
         // don't need to set the callback.  we passed it to CreateIntFifo and the API set it
         intFifo.iNotification = 0;
         // FIX
         intFifo.iPriority = Constants.THREAD_PRIORITY_ABOVE_NORMAL;
         intFifo.dwMilliseconds = Constants.INFINITE;
         intFifo.bForceStartup = 0;
         intFifo.FilterType = Constants.EVENT_BC_MESSAGE;

         for (rt = 0; rt < 32; rt++)
            for (tr = 0; tr < 2; tr++)
               for (sa = 0; sa < 32; sa++)
                  intFifo.SetFilterMask(rt, tr, sa, 0xffffffff);  // Enable all Word Counts on all RT/SA/TR combos

         // tell the API to update the unmanaged int fifo with the contents from the managed one
         status = API.UpdateIntFifo(intFifo, pIntFifo);

         // Call the register function to register and start the thread.
         status = API.RegisterFunction(cardnum, pIntFifo, Constants.REGISTER_FUNCTION);
      }
   }
}
