using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Bustools1553;
using Bustools1553.DataTypes;
using System.Runtime.InteropServices;
using System.Threading;

namespace BustoolsCsApp
{
   public partial class Form1 : Form
   {
      public Form1()
      {
         InitializeComponent();
      }

      private void Form1_Load(object sender, EventArgs e)
      {
         this.CenterToScreen();

         // setup channel and device num combo boxes
         cbDeviceNum.SelectedIndex = 0;
         cbBcChannel.SelectedIndex = 0;
         cbBmChannel.SelectedIndex = 0;
         cbRt1Channel.SelectedIndex = 0;
         cbRt2Channel.SelectedIndex = 0;

         RtTests.mainForm = this;

         // first, we have to open the board, and disable the test bus option if it's not a QPCX1553 or QPCI1553
         UInt32 cardnum;
         Int32 status = API.OpenChannel(out cardnum, 1, cbDeviceNum.SelectedIndex, Constants.CHANNEL_1);
         status = API.GetBoardType(cardnum);
         if (status != Constants.QPCI1553 && status != Constants.QPCX1553)
         {
            rbTestBus.Enabled = false;
         }
         API.API_Close(cardnum);

      }

      private void btnStartStop_Click(object sender, EventArgs e)
      {
         if (btnStartStop.Text == "Start Tests")
         {
            // start up the sample test

            btnStartStop.Text = "Stop Tests";
            groupBox1.Enabled = false;
            groupBox2.Enabled = false;
            groupBox3.Enabled = false;
            label2.Enabled = false;
            cbDeviceNum.Enabled = false;

            RtTests.Init();
         }
         else
         {
            // when stopping, dump memory just for fun
            API.DumpMemory(0, Constants.DUMP_ALL, "dump.txt", "Memory Dump");

            // shut everything down
            btnStartStop.Text = "Start Tests";
            if (RtTests.m_UserTimer != null)
            {
               RtTests.m_UserTimer.Change(Timeout.Infinite, Timeout.Infinite);
            }

            UInt32 cardnum = cbBcChannel.SelectedIndex == 0 ? RtTests.gCardnumCh1 : RtTests.gCardnumCh2;
            API.BC_StartStop(cardnum, 0);

            cardnum = cbBmChannel.SelectedIndex == 0 ? RtTests.gCardnumCh1 : RtTests.gCardnumCh2;
            API.BM_StartStop(cardnum, 0);

            cardnum = cbRt1Channel.SelectedIndex == 0 ? RtTests.gCardnumCh1 : RtTests.gCardnumCh2;
            API.RT_StartStop(cardnum, 0);

            cardnum = cbRt2Channel.SelectedIndex == 0 ? RtTests.gCardnumCh1 : RtTests.gCardnumCh2;
            API.RT_StartStop(cardnum, 0);

            API.API_Close(RtTests.gCardnumCh1);
            API.API_Close(RtTests.gCardnumCh2);

            RtTests.bRtInited = false;

            groupBox1.Enabled = true;
            groupBox2.Enabled = true;
            groupBox3.Enabled = true;
            label2.Enabled = true;
            cbDeviceNum.Enabled = true;
         }
      }

      private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
      {

      }

      private void rbBus_CheckedChanged(object sender, EventArgs e)
      {
         RadioButton rb = sender as RadioButton;

         // if selected internal bus, disable the channel combo boxes and set to channel 1
         
         bool bEnabled = true;
         if (rb.Name == "rbInternal") bEnabled = false;

         cbBcChannel.Enabled = bEnabled;
         cbBmChannel.Enabled = bEnabled;
         cbRt1Channel.Enabled = bEnabled;
         cbRt2Channel.Enabled = bEnabled;

         if (bEnabled == false)
         {
            cbBcChannel.SelectedIndex = 0;
            cbBmChannel.SelectedIndex = 0;
            cbRt1Channel.SelectedIndex = 0;
            cbRt2Channel.SelectedIndex = 0;
         }
      }
   }
}


