using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using Bustools1553;
using Bustools1553.DataTypes;
using System.Runtime.InteropServices;

namespace BustoolsCsApp
{
   public static class RtTests
   {
      public static Form1 mainForm;
      public static UInt32 gCardnumCh1, gCardnumCh2;
      public static UInt32 gInternalExternalBus;
//      public static Int32 gTestBusEnableDisable;
      public static UInt32 gTestBusEnableDisable;
      public static Hashtable rtStartStop = new Hashtable();
      public static Timer m_UserTimer = null;
      public static bool bRtInited = false;

      // this mutex is used to lock access to the output window to a single callback thread
      public static Mutex mutex = new Mutex();


      /////////////////////////////////////////////////////////////////////////////////
      // these delegates and the next 2 routines provide thread-safe access to the text box on the main form
      public delegate void AddToOutputDelegate(String s);
      public static AddToOutputDelegate addDelegate;
      
      public static void AddStringToOutput(String s)
      {
         mainForm.tbOutput.AppendText(s);
      }

      public static void OutputMessage(String s)
      {
         if (mainForm.tbOutput.InvokeRequired)
         {
            mainForm.tbOutput.Invoke(addDelegate, new object[] { s });
         }
         else
         {
            mainForm.tbOutput.AppendText(s);
         }
      }
      // end text output routines
      /////////////////////////////////////////////////////////////////////////////////

      // these delegates can be used as function pointers for callbacks
      static API_INT_FIFO_Callback bcIntFifoDelegate = new API_INT_FIFO_Callback(Callback_BC);
      static API_INT_FIFO_Callback bmIntFifoDelegate = new API_INT_FIFO_Callback(Callback_BM);
      static API_INT_FIFO_Callback rt1IntFifoDelegate = new API_INT_FIFO_Callback(Callback_RT1);
      static API_INT_FIFO_Callback rt2IntFifoDelegate = new API_INT_FIFO_Callback(Callback_RT2);


      // constructor
      static RtTests()
      {
         addDelegate = new AddToOutputDelegate(AddStringToOutput);
      }

      public static void Init()
      {
         int status;
         int deviceNum = mainForm.cbDeviceNum.SelectedIndex;
         UInt32 pwFlag = 1;
         UInt32 ucode_rev = 0, api_rev = 0;
         UInt32 cardnum;

         pwFlag = Constants.API_B_MODE | Constants.API_SW_INTERRUPT;

         // open 2 channels, we don't necessarily use them both
         status = API.OpenChannel(out gCardnumCh1, pwFlag, deviceNum, Constants.CHANNEL_1);
         OutputMessage(String.Format("BusTools_API_OpenChannel for gCardnumCh1({0}) = {1}\r\n", gCardnumCh1, status));

         status = API.OpenChannel(out gCardnumCh2, pwFlag, deviceNum, Constants.CHANNEL_2);
         OutputMessage(String.Format("BusTools_API_OpenChannel for gCardnumCh2({0}) = {1}\r\n", gCardnumCh2, status));
       
         // enable external bus if external or testbus is checked
         gInternalExternalBus = (mainForm.rbExternal.Checked || mainForm.rbTestBus.Checked) ? Constants.EXTERNAL_BUS : Constants.INTERNAL_BUS;
         
         // enable testbus if testbus is checked
         gTestBusEnableDisable = mainForm.rbTestBus.Checked ? Constants.TEST_BUS_ENABLE: Constants.TEST_BUS_DISABLE;

         status = API.SetInternalBus(gCardnumCh1, gInternalExternalBus);
         OutputMessage(String.Format("SetInternalBus CH1 status = {0}\r\n", status));

         status = API.SetInternalBus(gCardnumCh2, gInternalExternalBus);
         OutputMessage(String.Format("SetInternalBus CH2 status = {0}\r\n", status));

         if (mainForm.rbTestBus.Checked)
         {
            status = API.SetTestBus(gCardnumCh1, gTestBusEnableDisable);
            OutputMessage(String.Format("SetTestBus CH1 status = {0}\r\n", status));

            status = API.SetTestBus(gCardnumCh2, gTestBusEnableDisable);
            OutputMessage(String.Format("SetTestBus CH2 status = {0}\r\n", status));
         }
         
         // Setup the BM, BC, and RT's based on the dialog channel selections.

         // For each of these we do two steps: Initialize them, then setup the communication mechanism between us and them.

         // The communication can be via IntFifo where the API calls us back periodically, or by ReadLastMessageBlock,
         // where it is up to us to poll the message buffers periodically.  

         // Set this to true or false based on the radio button.
         // "True" means use IntFifo's.
         // "False" means use a user timer callback to read the message.
         bool bUseIntFifo = mainForm.rbIntFifo.Checked;

         // if BM channel combo index == 0 (channel 1), set to channel 1, else channel 2,
         cardnum = mainForm.cbBmChannel.SelectedIndex == 0 ? gCardnumCh1 : gCardnumCh2;
         Init_BM(cardnum, bUseIntFifo);

         // if RT1 channel combo index == 0 (channel 1), set to channel 1, else channel 2,
         cardnum = mainForm.cbRt1Channel.SelectedIndex == 0 ? gCardnumCh1 : gCardnumCh2;
         Init_RT(cardnum, 1, bUseIntFifo);

         // if RT2 channel combo index == 0 (channel 1), set to channel 1, else channel 2,
         cardnum = mainForm.cbRt2Channel.SelectedIndex == 0 ? gCardnumCh1 : gCardnumCh2;
         Init_RT(cardnum, 2, bUseIntFifo);

         // if BC channel combo index == 0 (channel 1), set to channel 1, else channel 2,
         cardnum = mainForm.cbBcChannel.SelectedIndex == 0 ? gCardnumCh1 : gCardnumCh2;
         Init_BC(cardnum, bUseIntFifo);

         // if not using IntFifo setup the User Timer
         if (bUseIntFifo == false) 
            SetupUserTimer();

         // get API rev
         status = API.GetRevision(gCardnumCh1, ref ucode_rev, ref api_rev);
         OutputMessage(String.Format("LPU Revison = {0} -- API Revision = {1}\r\n", ucode_rev, api_rev));
      }

      ////////////////////////////////////////////////////////////////////////////////////////////
      // INIT ROUTINES
      #region Init BM BC RT

      // routine that sets up a BM
      public static int Init_BM(UInt32 cardnum, bool bUseIntFifo)
      {
         Int32 status;
         UInt32 actual = 0;

         status = API.BM_Init(cardnum, 1, 1);
         OutputMessage(String.Format("BM Init status = {0}\r\n", status));

         status = API.BM_MessageAlloc(cardnum, 200, ref actual, Constants.BT1553_INT_END_OF_MESS);  //BT1553_INT_END_OF_MESS
         OutputMessage(String.Format("BM_MessageAlloc status = {0}, actual = {1}\r\n", status, actual));

         // start the BM running
         status = API.BM_StartStop(cardnum, Constants.START);

         if (bUseIntFifo)
         {
            SetupBmIntFifo(cardnum);
         }
         
         return 0;
      }

      // routine that sets up a BC
      public static int Init_BC(UInt32 cardnum, bool bUseIntFifo)
      {
         Int32 status;
         UInt16 messno;
         API_BC_MBUF bcMessage;

         status = API.BC_Init(cardnum, 0, Constants.BT1553_INT_END_OF_MESS, 0, 16, 14, 1000000, 1);
         OutputMessage(String.Format("BC Init status = {0}\r\n", status));

         status = API.BC_MessageAlloc(cardnum, (Int32)50); //Allocate 50 BC message buffers
         OutputMessage(String.Format("BC_MessageAlloc status = {0}\r\n", status));

         // allocate the BC messages

         // Message 0 receive (BC->RT)    
         messno = 0;
         bcMessage = new API_BC_MBUF();
         bcMessage.messno = messno;
         bcMessage.messno_next = (UInt16)(messno + 1);
         bcMessage.control = Constants.BC_CONTROL_MESSAGE;          // show as a message
         bcMessage.control |= Constants.BC_CONTROL_BUFFERA;         // use buffer A
         bcMessage.control |= Constants.BC_CONTROL_CHANNELA;        // Transmit on Channel A
         bcMessage.control |= Constants.BC_CONTROL_INTERRUPT;
         bcMessage.control |= Constants.BC_CONTROL_MFRAME_BEG;
         bcMessage.mess_command1.rtaddr = 1;
         bcMessage.mess_command1.subaddr = 1;
         bcMessage.mess_command1.wcount = 1;
         bcMessage.mess_command1.tran_rec = Constants.RECEIVE;
         bcMessage.errorid = 0;     // Default error injection buffer (no errors)
         bcMessage.gap_time = 10;    // 8 microsecond inter-message gap.
         bcMessage.long_gap = 0; 
         status = API.BC_MessageWrite(cardnum, messno, bcMessage);
         OutputMessage(String.Format("BusTools_BC_MessageWrite {0} status = {1}\r\n", messno, status));

         // Message 1 transmit (RT->BC)    
         messno = 1;
         bcMessage = new API_BC_MBUF();
         bcMessage.messno = messno;
         bcMessage.messno_next = (UInt16)(0);
         bcMessage.control = Constants.BC_CONTROL_MESSAGE;          // show as a message
         bcMessage.control |= Constants.BC_CONTROL_BUFFERA;         // use buffer A
         bcMessage.control |= Constants.BC_CONTROL_CHANNELA;        // Transmit on Channel A
         bcMessage.control |= Constants.BC_CONTROL_INTERRUPT;
         bcMessage.control |= Constants.BC_CONTROL_MFRAME_END;
         bcMessage.mess_command1.rtaddr = 2;
         bcMessage.mess_command1.subaddr = 2;
         bcMessage.mess_command1.wcount = 32;
         bcMessage.mess_command1.tran_rec = Constants.TRANSMIT;
         bcMessage.errorid = 0;     // Default error injection buffer (no errors)
         bcMessage.gap_time = 10;    // 8 microsecond inter-message gap.
         status = API.BC_MessageWrite(cardnum, messno, bcMessage);
         OutputMessage(String.Format("BusTools_BC_MessageWrite {0} status = {1}\r\n", messno, status));

         // Start the BC Running
         status = API.BC_StartStop(cardnum, Constants.START);  
         OutputMessage(String.Format("BC StartStop status = {0}\r\n", status));

         if (bUseIntFifo)
         {
            SetupBcIntFifo(cardnum);
         }

         return 0;
      }
      

      // routine to setup an RT
      public static int Init_RT(UInt32 cardnum, UInt32 rt_addr, bool bUseIntFifo)
      {
         API_RT_CBUF cbuf = new API_RT_CBUF();       // RT Control Buffer Legal Word Count Mask (2 words)
         API_RT_MBUF_WRITE mbuf = new API_RT_MBUF_WRITE();
         API_RT_ABUF abuf = new API_RT_ABUF();
//         UInt32 RT_MBUF_COUNT = 4; // our RT has 4 buffers each for transmit and receive
         Int32 RT_MBUF_COUNT = 4; // our RT has 4 buffers each for transmit and receive
         Int32 status = 0;
         UInt32 tranRec;
         UInt32 subAddr;
         int i;

         // we can only call RT_Init once when setting up multiple RT's
         if (bRtInited == false)
         {
            status = API.RT_Init(cardnum, 0);
            bRtInited = true;
         }

         OutputMessage(String.Format("RT init status = {0}\r\n", status));

         //  Setup RT address control block
         abuf.enable_a = 1;    // Enable Bus A...Note that the API bits are
         abuf.enable_b = 1;    // Enable Bus B... reversed from the HW bits.
         abuf.status = 0x0000;

         status = API.RT_AbufWrite(cardnum, rt_addr, abuf);
         OutputMessage(String.Format("RT_AbufWrite status = {0}\r\n", status));


         // Create RT buffers.  Enable all word counts and mode codes
         cbuf.legal_wordcount = 0xffffffff;

         for (tranRec = 0; tranRec < 2; tranRec++)
         {
            for (subAddr = 0; subAddr < 32; subAddr++)
            {
               status = API.RT_CbufWrite(cardnum, rt_addr, subAddr, tranRec, RT_MBUF_COUNT, cbuf);
            }
         }

         //  Fill in message buffers for receive and transmit sub-units.
         mbuf.enable = Constants.BT1553_INT_END_OF_MESS;   // Interrupt enables on end of message
         mbuf.error_inj_id = 0;

         // For all subaddresses...
         for (subAddr = 0; subAddr < 32; subAddr++)
         {
            // For each data buffer..
//            for (int bufNum = 0; bufNum < RT_MBUF_COUNT; bufNum++)
            for (uint bufNum = 0; bufNum < RT_MBUF_COUNT; bufNum++)
               {
               for (tranRec = 0; tranRec < 2; tranRec++)
               {
                  // Create data buffer data
                  for (i = 0; i < 32; i++)
                  {
                     mbuf.mess_data[i] = (UInt16)(((UInt16)rt_addr << 12) | ((UInt16)subAddr << 8) | ((UInt16)bufNum << 5) | i);
                  }

                  // write the mbuf to the RT buffer
                  status = API.RT_MessageWrite(cardnum, rt_addr, subAddr, tranRec, bufNum, mbuf);
                  if (status != Constants.API_SUCCESS) OutputMessage(String.Format("RT_MessageWrite 0 status = {0}\r\n", status));
               }
            }
         }

         status = API.RT_StartStop(cardnum, Constants.START);

         if (bUseIntFifo)
         {
            SetupRtIntFifo(cardnum, rt_addr);
         }

         return 0;
      }

#endregion
      // INIT ROUTINES
      ////////////////////////////////////////////////////////////////////////////////////////////

      ////////////////////////////////////////////////////////////////////////////////////////////
      // CALLBACKS
      #region CALLBACKS

      // callback function for BM IntFifo
      public static Int32 Callback_BM(UInt32 cardnum, IntPtr intFifoAddr)
      {
         mutex.WaitOne();

         OutputMessage(String.Format("=========================\n"));
         OutputMessage(String.Format("Entering BM IntFifo Callback\r\n\r\n"));

         // marshall the pointer provided by the API to a managed int fifo
         API_INT_FIFO intFifo = (API_INT_FIFO)Marshal.PtrToStructure(intFifoAddr, typeof(API_INT_FIFO));
         API_BM_MBUF bmMessage = new API_BM_MBUF();

         // loop through all the IntFifo entries
         int tail = intFifo.tail_index;
         while (tail != intFifo.head_index)
         {
//            int messno = intFifo.fifo[tail].bufferID;
            uint messno = (uint)intFifo.fifo[tail].bufferID;
            // in this example, we read each message and dump the one that has RT/SA = 2/2
            int status = API.BM_MessageRead(cardnum, messno, bmMessage);

            OutputMessage(String.Format("Cardnum: {0}, BM Buffer ID: {1}, RT: {2}, SA: {3}\r\n", 
               cardnum, intFifo.fifo[tail].bufferID, intFifo.fifo[tail].rtaddress, intFifo.fifo[tail].subaddress));

            if ((bmMessage.command1.rtaddr == 2) && (bmMessage.command1.subaddr == 2))
            {
               // found it
               DumpMbuf(String.Format("Dumping Databuffer of BM message {0} \n", bmMessage.messno), bmMessage.value);
            }

            tail++;
            tail &= intFifo.mask_index;
            intFifo.tail_index = tail;
         }

         // after we have updated the IntFifo tail, we have to pass it back to the API, so it can set
         // the tail in the unmanaged IntFifo structure.  This is only required when using .NET interop.
         API.UpdateTailPTR(intFifoAddr, tail);

         OutputMessage(String.Format("\r\nExiting BM IntFifo Callback\n"));
         OutputMessage(String.Format("=========================\r\n\r\n"));

         mutex.ReleaseMutex();
         return 0;
      }

      // callback function for RT1 IntFifo
      public static Int32 Callback_RT1(UInt32 cardnum, IntPtr intFifoAddr)
      {
         mutex.WaitOne();

         OutputMessage(String.Format("=========================\n"));
         OutputMessage(String.Format("Entering RT1 IntFifo Callback\r\n\r\n"));

         // marshall the pointer provided by the API to a managed int fifo
         API_INT_FIFO intFifo = (API_INT_FIFO)Marshal.PtrToStructure(intFifoAddr, typeof(API_INT_FIFO));

         int tail = intFifo.tail_index;
         while (tail != intFifo.head_index)
         {
            OutputMessage(String.Format("Cardnum: {0}, RT Buffer ID: {1}, RT: {2}, SA: {3}\r\n",
               cardnum, intFifo.fifo[tail].bufferID, intFifo.fifo[tail].rtaddress, intFifo.fifo[tail].subaddress));
            tail++;
            tail &= intFifo.mask_index;
            intFifo.tail_index = tail;
         }

         // After we have updated the IntFifo tail, we have to pass it back to the API, so it can set
         // the tail in the unmanaged IntFifo structure.  This is only required when using .NET interop.
         API.UpdateTailPTR(intFifoAddr, tail);

         OutputMessage(String.Format("\r\nExiting RT1 IntFifo Callback\n"));
         OutputMessage(String.Format("=========================\r\n\r\n"));

         mutex.ReleaseMutex();
         return 0;
      }

      // callback function for RT2 IntFifo
      public static Int32 Callback_RT2(UInt32 cardnum, IntPtr intFifoAddr)
      {
         mutex.WaitOne();

         OutputMessage(String.Format("=========================\n"));
         OutputMessage(String.Format("Entering RT2 IntFifo Callback\r\n\r\n"));

         // marshall the return pointer to a managed int fifo
         API_INT_FIFO intFifo = (API_INT_FIFO)Marshal.PtrToStructure(intFifoAddr, typeof(API_INT_FIFO));

         int tail;
         tail = intFifo.tail_index;
         while (tail != intFifo.head_index)
         {
            OutputMessage(String.Format("Cardnum: {0}, RT Buffer ID: {1}, RT: {2}, SA: {3}\r\n",
               cardnum, intFifo.fifo[tail].bufferID, intFifo.fifo[tail].rtaddress, intFifo.fifo[tail].subaddress));
            tail++;
            tail &= intFifo.mask_index;
            intFifo.tail_index = tail;
         }

         // After we have updated the IntFifo tail, we have to pass it back to the API, so it can set
         // the tail in the unmanaged IntFifo structure.  This is only required when using .NET interop.
         API.UpdateTailPTR(intFifoAddr, tail);

         OutputMessage(String.Format("\r\nExiting RT2 IntFifo Callback\n"));
         OutputMessage(String.Format("=========================\r\n\r\n"));

         mutex.ReleaseMutex();

         return 0;
      }

      // callback function for BC IntFifo
      public static Int32 Callback_BC(UInt32 cardnum, IntPtr intFifoAddr)
      {
         mutex.WaitOne();

         OutputMessage(String.Format("=========================\n"));
         OutputMessage(String.Format("Entering BC IntFifo Callback\r\n\r\n"));

         // marshall the return pointer to a managed int fifo
         API_INT_FIFO intFifo = (API_INT_FIFO)Marshal.PtrToStructure(intFifoAddr, typeof(API_INT_FIFO));
         API_BC_MBUF bcMessage = new API_BC_MBUF();

         int tail;
         int icnt = 0;

         // loop through all the IntFifo entries
         tail = intFifo.tail_index;
         while (tail != intFifo.head_index)
         {
            OutputMessage(String.Format("Cardnum: {0}, BC Message ID: {1}, RT: {2}, SA: {3}\r\n",
               cardnum, intFifo.fifo[tail].bufferID, intFifo.fifo[tail].rtaddress, intFifo.fifo[tail].subaddress));

            // let's look for the RT->BC message we set up at RT=2, SA=2
            // in this example, we only read the message that has RT/SA = 2/2
            if (
                  ((intFifo.fifo[tail].event_type & Constants.EVENT_BC_MESSAGE) != 0) &&
                  (intFifo.fifo[tail].rtaddress == 2) && (intFifo.fifo[tail].subaddress == 2)
               )
            {
               // found it, read and dump the buffer
//               int messno = intFifo.fifo[tail].bufferID;
               uint messno = (uint)intFifo.fifo[tail].bufferID;
               int status = API.BC_MessageRead(cardnum, messno, bcMessage);

               DumpMbuf(String.Format("Dumping Databuffer of BC message {0} \n", tail), bcMessage.ReceiveData);
            }
            
            // update pointers and counters
            icnt += 1;
            tail += 1;
            tail &= intFifo.mask_index;
            intFifo.tail_index = tail;
         }

         // After we have updated the IntFifo tail, we have to pass it back to the API, so it can set
         // the tail in the unmanaged IntFifo structure.  This is only required when using .NET interop.
         API.UpdateTailPTR(intFifoAddr, tail);

         OutputMessage(String.Format("\r\nExiting BC IntFifo Callback\n"));
         OutputMessage(String.Format("=========================\r\n\r\n"));

         mutex.ReleaseMutex();
         return 0;
      }

      #endregion
      // CALLBACKS
      /////////////////////////////////////////////////////////////////////////////////////////////////

      /////////////////////////////////////////////////////////////////////////////////////////////////
      // INT FIFO SETUP
      #region INT FIFO SETUP

      // the next two functions are used when we use the ReadLastMessageBlock method instead of an IntFifo
      public static void SetupUserTimer()
      {
         // set up user timer.  on callback, read last message block
         m_UserTimer = new Timer(new TimerCallback(Callback_UserTimer), null, 500, 500);
      }

      // This is the callback routine for the user polling timer.  It polls the BC and BM in this example
      public static void Callback_UserTimer(Object stateInfo)
      {
         int xMbuf;
         int status;
//         int count = 0;
         uint count = 0;

         mutex.WaitOne();

         OutputMessage(String.Format("=========================\n"));
         OutputMessage(String.Format("Entering User Timer\r\n\r\n"));

         //////////////////////////////////////////////////////////////////////
         // BC section

         // init some BC MBUFS
         API_BC_MBUF_STRUCT[] bcMbufs = new API_BC_MBUF_STRUCT[296];

         // we have to do this so the struct constructor is called
         for (xMbuf = 0; xMbuf < 296; xMbuf++) bcMbufs[xMbuf] = new API_BC_MBUF_STRUCT(0);

         // read messages and report how many were read
         status = API.BC_ReadLastMessageBlock(0, -1, -1, -1, ref count, bcMbufs);
         OutputMessage(String.Format("Read {0} BC MBUFS\n", count));

         // let's look for the RT->BC message we set up at RT=2, SA=2
         for (xMbuf = 0; xMbuf < count; xMbuf++)
         {
            OutputMessage(String.Format("RT/SA = {0}/{1}\n", bcMbufs[xMbuf].mess_command1.rtaddr, bcMbufs[xMbuf].mess_command1.subaddr));
            if ((bcMbufs[xMbuf].mess_command1.rtaddr == 2) && (bcMbufs[xMbuf].mess_command1.subaddr == 2))
            {
               // found it
               DumpMbuf(String.Format("Dumping Databuffer of BC message {0} \n", xMbuf), bcMbufs[xMbuf].ReceiveData);
            }
         }


         //////////////////////////////////////////////////////////////////////
         // BM section

         // init some BM MBUFS
         API_BM_MBUF_STRUCT[] bmMbufs = new API_BM_MBUF_STRUCT[296];

         // we have to do this so the struct constructor is called
         for (xMbuf = 0; xMbuf < 296; xMbuf++) bmMbufs[xMbuf] = new API_BM_MBUF_STRUCT(0);
         
         // read messages and report how many were read
         status = API.BM_ReadLastMessageBlock(0, -1, -1, -1, ref count, bmMbufs);
         OutputMessage(String.Format("\r\nRead {0} BM MBUFS\n", count));

         // let's look for the RT->BC message we set up at RT=2, SA=2
         for (xMbuf = 0; xMbuf < count; xMbuf++)
         {
            OutputMessage(String.Format("RT/SA = {0}/{1}\n", bmMbufs[xMbuf].command1.rtaddr, bmMbufs[xMbuf].command1.subaddr));
            if ((bmMbufs[xMbuf].command1.rtaddr == 2) && (bmMbufs[xMbuf].command1.subaddr == 2))
            {
               // found it
               DumpMbuf(String.Format("Dumping Databuffer of BM message {0} \n", xMbuf), bmMbufs[xMbuf].value);
            }
         }

         //////////////////////////////////////////////////////////////////////
         // RT section

         // init some RT MBUFS
         API_RT_MBUF_READ_STRUCT[] rtMbufs = new API_RT_MBUF_READ_STRUCT[296];

         // we have to do this so the struct constructor is called
         for (xMbuf = 0; xMbuf < 296; xMbuf++) rtMbufs[xMbuf] = new API_RT_MBUF_READ_STRUCT(0);

         // read messages and report how many were read
         status = API.RT_ReadLastMessageBlock(0, -1, -1, Constants.TRANSMIT, ref count, rtMbufs);
         OutputMessage(String.Format("\r\nRead {0} RT MBUFS\n", count));

         // let's look for the RT->BC message we set up at RT=2, SA=2
         for (xMbuf = 0; xMbuf < count; xMbuf++)
         {
            OutputMessage(String.Format("RT/SA = {0}/{1}\n", rtMbufs[xMbuf].mess_command.rtaddr, rtMbufs[xMbuf].mess_command.subaddr));
            if ((rtMbufs[xMbuf].mess_command.rtaddr == 2) && (rtMbufs[xMbuf].mess_command.subaddr == 2))
            {
               // found it
               DumpMbuf(String.Format("Dumping Databuffer of BM message {0} \n", xMbuf), rtMbufs[xMbuf].mess_data);
            }
         }

         OutputMessage(String.Format("\r\nExiting User Timer\n"));
         OutputMessage(String.Format("=========================\r\n\r\n"));

         mutex.ReleaseMutex();
      }

      public static void DumpMbuf(string message, UInt16[] buf)
      {
         OutputMessage(String.Format("{0} \n", message));
         for (int row = 0; row < 4; row++)
         {
            for (int col = 0; col < 8; col++)
            {
               UInt16 word = buf[row * 8 + col];
               OutputMessage(String.Format("{0} ", word.ToString("X4")));
            }
            OutputMessage("\n");
         }
      }

      public static void SetupBcIntFifo(UInt32 cardnum)
      {
         // Have the API create an unmanaged IntFifo structure and give us a pointer to it.  We have to pass the API the 
         // callback pointer because the IntFifo sits in unmanaged memory, and the API must set the callback pointer.  Interop 
         // passes us copies of the IntFifo, but we can't access it directly from .NET code.

         // Note: CreateIntFifo() is only used when programming in .NET.
         IntPtr pIntFifoUnmanaged = API.CreateIntFifo(bcIntFifoDelegate);

         // marshall the returned pointer to a managed int fifo
         API_INT_FIFO intFifo = (API_INT_FIFO)Marshal.PtrToStructure(pIntFifoUnmanaged, typeof(API_INT_FIFO));

         int rt, tr, sa, status;
         // Setup the int Fifo structure for this board.
         // We don't need to set the callback here.  We passed it to CreateIntFifo and the API set it.
         intFifo.iPriority = Constants.THREAD_PRIORITY_ABOVE_NORMAL;
         intFifo.dwMilliseconds = Constants.INFINITE;
         intFifo.FilterType = Constants.EVENT_BC_MESSAGE;
         //intFifo.bForceStartup = 0;
         //intFifo.bForceShutdown = 0;

         for (rt = 0; rt < 32; rt++)
            for (tr = 0; tr < 2; tr++)
               for (sa = 0; sa < 32; sa++)
                  intFifo.SetFilterMask(rt, tr, sa, 0xffffffff);  // Enable all Word Counts on all RT/SA/TR combos

         // This call tells the API to update the unmanaged int fifo with the contents from our managed one after we set it up.
         // Note: UpdateIntFifo() is only used when programming in .NET.
         status = API.UpdateIntFifo(pIntFifoUnmanaged, intFifo);

         // Call the register function to register and start the thread.  We pass the unmanaged IntFifo pointer here,
         // because that is what the API understands.
         status = API.RegisterFunction(cardnum, pIntFifoUnmanaged, Constants.REGISTER_FUNCTION);
         OutputMessage(String.Format("BusTools_RegisterFunction BC status = {0}\r\n", status));
      }

      public static void SetupBmIntFifo(UInt32 cardnum)
      {
         // Have the API create an unmanaged IntFifo structure, and set the callback pointer.  The unmanaged
         // IntFifo sits in unmanaged memory.  Interop can make copies of it, but we don't access it from .NET code.
         IntPtr pIntFifo = API.CreateIntFifo(bmIntFifoDelegate);

         // marshall the return pointer to a managed int fifo
         API_INT_FIFO intFifo = (API_INT_FIFO)Marshal.PtrToStructure(pIntFifo, typeof(API_INT_FIFO));

         int rt, tr, sa, status;

         // Setup the FIFO structure for this board.
         intFifo.iPriority = Constants.THREAD_PRIORITY_TIME_CRITICAL;
         intFifo.dwMilliseconds = Constants.INFINITE;
         intFifo.FilterType = Constants.EVENT_BM_MESSAGE;

         for (rt = 0; rt < 32; rt++)
            for (tr = 0; tr < 2; tr++)
               for (sa = 0; sa < 32; sa++)
                  intFifo.SetFilterMask(rt, tr, sa, 0xffffffff);  // Enable all Word Counts on all RT/SA/TR combos

         // tell the API to update the unmanaged int fifo with the contents from the managed one
         status = API.UpdateIntFifo(pIntFifo, intFifo);

         // Call the register function to register and start the thread.  We pass the unmanaged pointer here,
         // because that is what the API understands.
         status = API.RegisterFunction(cardnum, pIntFifo, Constants.REGISTER_FUNCTION);
         OutputMessage(String.Format("BusTools_RegisterFunction BM status = {0}\r\n", status));
      }

      public static void SetupRtIntFifo(UInt32 cardnum, UInt32 rtNum)
      {
         // Have the API create an unmanaged IntFifo structure, and set the callback pointer.  The unmanaged
         // IntFifo sits in unmanaged memory.  Interop can make copies of it, but we don't access it from .NET code.

         IntPtr pIntFifo = IntPtr.Zero;
         if (rtNum == 1)
         {
            pIntFifo = API.CreateIntFifo(rt1IntFifoDelegate);
         }
         else
         {
            pIntFifo = API.CreateIntFifo(rt2IntFifoDelegate);
         }

         // marshall the return pointer to a managed int fifo
         API_INT_FIFO intFifo = (API_INT_FIFO)Marshal.PtrToStructure(pIntFifo, typeof(API_INT_FIFO));

         int tr, sa, status;

         // Setup the FIFO structure for this board.
         intFifo.iPriority = Constants.THREAD_PRIORITY_TIME_CRITICAL;
         intFifo.dwMilliseconds = Constants.INFINITE;
         intFifo.FilterType = Constants.EVENT_RT_MESSAGE;

         for (tr = 0; tr < 2; tr++)
            for (sa = 0; sa < 32; sa++)
               intFifo.SetFilterMask((Int32)rtNum, tr, sa, 0xffffffff);  // Enable all Word Counts on all RT/SA/TR combos

         // tell the API to update the unmanaged int fifo with the contents from the managed one
         status = API.UpdateIntFifo(pIntFifo, intFifo);

         // Call the register function to register and start the thread.  We pass the unmanaged pointer here,
         // because that is what the API understands.
         status = API.RegisterFunction(cardnum, pIntFifo, Constants.REGISTER_FUNCTION);
         OutputMessage(String.Format("BusTools_RegisterFunction RT status = {0}\r\n", status));
      }


      #endregion
      // INT FIFO SETUP
      ///////////////////////////////////////////////////////////////////////////////////////

   }
}
