/*===========================================================================*
 * FILE:              E X A M P L E _ A U T O _ S Y N C _ M O D E. C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    Demonstration of BusTools/1553-API routines. This example 
 *              shows how to setup the time tag auto sync mode.  The mode
 *              synchronizes the time tag to an external pulse.  It also sets up
 *              a deadman timer to detect if the external 1PPS pulse does not
 *              occur on time.  The is option code to have the 1553 interface
 *              board generate the 1PPS pulse instead of external 1PPS.
 *
 *===========================================================================*/

/* $Revision:  8.120 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include <time.h>
#include "busapi.h"

int BusController_Init(int);
int BusMonitor_Init(int);
int RemoteTerminal_Init(int);
void setupThe_BC_IntFIFO(int);
void setupThe_Timer_IntFIFO(int);
void setupThe_1PPS_Deadman_Timer_IntFIFO(int cardnum);

API_INT_FIFO  sIntFIFO1;     // Thread FIFO structure for BC
API_INT_FIFO  sIntFIFO2;     // Thread FIFO structure for RT
API_INT_FIFO  sIntFIFO3;     // Thread FIFO structure for BM
API_INT_FIFO  trig_fifo;     // Thread FIFO structure for deadman timer
API_INT_FIFO  timerFIFO;     // Thread FIFO structure for 1PPS timer

void main(int argc, char **argv)                   
{
   BT_INT status,cardnum;
   BT_UINT pwFlag, device;
   BT_INT isV6FW = FALSE;

   device = 0;

   pwFlag = API_B_MODE | API_SW_INTERRUPT;  //Initialixe 155B mode with Software polling interrupts

   printf("Initializing Test\n");

   status = BusTools_API_OpenChannel(&cardnum,pwFlag,device,CHANNEL_1);
   printf("BusTools_API_OpenChannel status = %d\n",status);

   status = BusTools_SetInternalBus(cardnum,EXTERNAL_BUS);  // Set up external bus
   printf("BusTools_SetInternalBus status = %d\n",status);

   if(BusTools_BoardIsV6(cardnum))
   {

      status = BusTools_SetV6TrigIn(cardnum, DISCRETE, 7);             //Enable Trigger in on Discrete 7
      printf("BusTools_DiscreteSetIO status = %d\n",status);
   
      /* Use this to have the board generate it's own 1PPS signal */
      status = BusTools_SetV6TrigOut(cardnum, DISCRETE, 8);            //Enable Trigger out on Discrete 8
      printf("BusTools_DiscreteSetIO status = %d\n",status);   

	  status = BusTools_DiscreteWrite(cardnum,7,0);                    //Clear Discrete 7
      printf("BusTools_DiscreteWrite status = %d\n",status); 
   }
   else
   {
      // Setup input trigger to be discrete 7, output trigger on discrete 8.
      status = BusTools_DiscreteSetIO(cardnum, 0x155, 0x3ff);
      printf("BusTools_DiscreteSetIO status\n", status);

      status = BusTools_DiscreteTriggerOut(cardnum, TRIGGER_OUT_DIS_8);
      printf("BusTools_DiscreteTriggerOut status\n", status);
   
      status = BusTools_DiscreteTriggerIn(cardnum, TRIGGER_IN_DIS_7);
      printf("DiscreteTriggerIn Status\n", status);
   }

   // Set up the time tag for auto sync mode with 1PPS (500000 usec) pulse, Set time tag to system time
   // This causes the time tag of the 1-second message to increment by 500000 microseconds instead of 
   // 1000000 microseconds.  This to demonstrate that the sync is working.
   status = BusTools_TimeTagMode(cardnum,API_TTD_RELM, API_TTI_DAY,API_TTM_AUTO,NULL,500000,0,0);
   printf("BusTools_TimeTagMode status = %d\n",status); 

   status = BusTools_ExtTrigIntEnable(cardnum, EXT_TRIG_ENABLE); //Enable interrupt on input trigger
   printf("BusTools_ExtTrigIntEnable status = %d\n",status); 

   status = BusMonitor_Init(cardnum);             //Setup BM
   status = BusController_Init(cardnum);          //Setup BC
   status = RemoteTerminal_Init(cardnum);         //Setup RT

   /* Use this to have the board generate it's own 1PPS signal */
   setupThe_Timer_IntFIFO(cardnum);               // Sets up a 1sec timer interrupt to generate pulse for 1PPS signal */
   setupThe_BC_IntFIFO(cardnum);                  // Sets up interrupt for BC message
   setupThe_1PPS_Deadman_Timer_IntFIFO(cardnum);  // Sets up a deadman timer to warn of loss of the trigger input

   status = BusTools_BM_StartStop(cardnum,BM_START);
   printf("BusTools_BM_StartStop status = %d\n",status);

   status = BusTools_RT_StartStop(cardnum,RT_START);
   printf("BusTools_RT_StartStop status = %d\n",status);

   status = BusTools_BC_StartStop(cardnum,BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);

   while(getc(stdin) != '\n');

   printf("main: closing API\n");

   status = BusTools_BC_StartStop(cardnum,BC_STOP);
   printf("BusTools_BC_StartStop status = %d\n",status);

   status = BusTools_RT_StartStop(cardnum,RT_STOP);
   printf("BusTools_RT_StartStop status = %d\n",status);

   status = BusTools_BM_StartStop(cardnum,BM_STOP);
   printf("BusTools_RT_StartStop status = %d\n",status);

   status = BusTools_RegisterFunction(cardnum,&sIntFIFO1,UNREGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);

   status = BusTools_RegisterFunction(cardnum,&trig_fifo,UNREGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);

   status = BusTools_RegisterFunction(cardnum,&timerFIFO,UNREGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);

   status = BusTools_API_Close(cardnum);
   printf("BusTools_API_Close status = %d\n",status);
}

int BusController_Init(int cardnum)
{
   int status, messno;
   API_BC_MBUF bcmessage;

   status = BusTools_BC_Init(cardnum,0,BT1553_INT_END_OF_MESS,0,20,20,1000000,1);
   printf("BusTools_BC_Init status = %d\n",status);

   status = BusTools_BC_MessageAlloc(cardnum,50);
   printf("BusTools_BC_MessageAlloc status = %d\n",status); 

   messno = 0;
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on BUS A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;
   bcmessage.mess_command1.rtaddr   = 2;
   bcmessage.mess_command1.subaddr  = 2;
   bcmessage.mess_command1.wcount   = 2;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                     // 100 microsecond inter-message gap.
   bcmessage.data[0][0] = 1;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);
    
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on BUS A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 3;
   bcmessage.mess_command1.subaddr  = 3;
   bcmessage.mess_command1.wcount   = 3;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 200;                      // 200 microsecond inter-message gap.
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on BUS A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 4;
   bcmessage.mess_command1.wcount   = 4;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 300;                      // 300 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on BUS A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_MFRAME_END;
   bcmessage.mess_command1.rtaddr   = 5;
   bcmessage.mess_command1.subaddr  = 5;
   bcmessage.mess_command1.wcount   = 5;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 12;                      // 12 microsecond inter-message gap.
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;
   bcmessage.messno = messno;
   bcmessage.messno_next = 0x0;
   bcmessage.messno_branch = 0;
   bcmessage.control = BC_CONTROL_NOP;        // Do it again and again.
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   return status;
}


/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   BT_INT       status;
   BT_INT tail;
   BT_UINT messno,wcount,j,i,bufnum;
   char outbuf[128];

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      messno = sIntFIFO->fifo[tail].bufferID;
      status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);
      if (status)
         return status;
   
      if(BusTools_BoardIsV6(cardnum))
         strcpy(outbuf,"NANO");
      BusTools_TimeGetString(&bcmessage.time_tag,outbuf);
      printf("\n_____________________\n");
      printf("Time Tag = %s --\n",outbuf);

      if(sIntFIFO->fifo->transrec == 0)
         printf("BC->RT (%d)  \n", messno);
      else
         printf("RT->BC (%d)  \n", messno);

      printf("BC**RT-%d  \n",bcmessage.mess_command1.rtaddr);
      printf("BC**SA-%d  \n",bcmessage.mess_command1.subaddr);
      printf("BC**WC-%d  \n",bcmessage.mess_command1.wcount);

      // If there is no response, don't display the RT status word,
      //  since the RT did not send it!
      if ( bcmessage.status & BT1553_INT_NO_RESP )
          printf("status 0xXXXX\n");  // No status word was sent
      else
          printf("BC**status 0x%04x\n",*((WORD*)(&bcmessage.mess_status1)));

      printf("BC**int_stat-0x%08lx\n",*((DWORD*)(&bcmessage.status)));

      wcount = bcmessage.mess_command1.wcount;
      if (wcount == 0)
         wcount = 32;           // Zero word count means 32 words.

      j = wcount;               // See if this msg shorted than previous
    
      if (bcmessage.control & BC_CONTROL_BUFFERA)
         bufnum = 0;            // Display data from first BC buffer
      else
         bufnum = 1;            // Display data from second BC buffer

      for ( i = 0; i < j; i++ ) // Display all of the data words
      {
         // If this is an RT->BC message, and there is a Message Error
         //  or No Response, than xxxx the data (since the RT did not
         //    send any data words).
         if ( i >= wcount )
         {
            printf("    ");     // Data does not apply to this msg
         }
         else if ( bcmessage.mess_command1.tran_rec &&
              (bcmessage.mess_status1.me || (bcmessage.status&BT1553_INT_NO_RESP) ) )
         {
            printf("xxxx ");      // No data was transfered because of error
         }
         else
         {
            printf("BC**%04x ",bcmessage.data[bufnum][i]);
         }
      }
      printf("\n");

      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
  }
  return 0;
}

BT_INT _stdcall intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{
   static BT_U32BIT last_micro=0;
   BT_INT tail;

   if(sIntFIFO->timeout == BTD_EVENT_WAIT_TIMEOUT)
      printf("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!1PPS Pulse Error\n");

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {	
      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }
   return 0;
}

BT_INT _stdcall timer_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{
   int status;

   status = BusTools_DiscreteWrite(cardnum,8,DISCRETE_ON);
   MSDELAY(50);
   status = BusTools_DiscreteWrite(cardnum,8,DISCRETE_OFF);
     
   return 0;
}

int RemoteTerminal_Init(int cardnum)
{
   int status;
   int RT_MBUF_COUNT;
   API_RT_CBUF cbuf;       // RT Control Buffer Legal Word Count Mask (2 words)
   API_RT_MBUF_WRITE mbuf;
   API_RT_ABUF abuf;
   int         tr;
   int         subaddress;
   int i,j,ii;
   int rt_addr[] = {2,3,4,5};

   RT_MBUF_COUNT = 10;

   status = BusTools_RT_Init(cardnum,0);
   printf("BusTools_RT_Init status = %d\n",status);

   /**********************************************************************
   *  Output RT control buffers
   *   Enable or disable all word counts and mode codes as specified by
   *    the caller's arguements.
   **********************************************************************/

   cbuf.legal_wordcount = 0xffffffff;  // legailize all word counts
   for ( i = 0; i < 4; i++)
   {
      for ( tr = 0; tr < 2; tr++ )
      {
         for ( subaddress = 0; subaddress < 32; subaddress++ )
         {
            status = BusTools_RT_CbufWrite(cardnum,rt_addr[i],subaddress,tr,
			                            RT_MBUF_COUNT,&cbuf);
         }
      }
   }

   /*******************************************************************
   *  Fill in message buffers for receive and transmit sub-units.
   *******************************************************************/

   mbuf.enable = BT1553_INT_END_OF_MESS;   // Interrupt enables on Broadcast Message
   mbuf.error_inj_id = 0;

   // For all subaddresses...
   for (ii = 0; ii<4; ii++)
   {
      for (i=0; i<32; i++)
      {
         // Create data buffer data
         for (j=0; j<32; j++)
            mbuf.mess_data[j] = (BT_U16BIT)(0xF000 + i);
         // For each data buffer...
         for (j=0; j<RT_MBUF_COUNT; j++)
         {
            status = BusTools_RT_MessageWrite(cardnum,rt_addr[ii],i,0,j,&mbuf);
            status = BusTools_RT_MessageWrite(cardnum,rt_addr[ii],i,1,j,&mbuf);
         }
      }
   }

   /************************************************
   *  Setup RT address control block
   ************************************************/

   abuf.enable_a          = 1;    // Enable Bus A...Note that the API bits are
   abuf.enable_b          = 1;    // Enable Bus B... reversed from the HW bits.
   abuf.inhibit_term_flag = 0;
   abuf.status            = 0x0000;
   abuf.command           = 0;
   abuf.bit_word          = 0;


   for(i=0;i<4;i++)
   {
      status = BusTools_RT_AbufWrite(cardnum,rt_addr[i],&abuf);
      printf("BusTools_RT_AbufWrite status = %d\n",status);
   }

   return status;
}

int BusMonitor_Init(int cardnum)
{
   int status;
   BT_UINT actual;

   status = BusTools_BM_Init(cardnum,1,1);
   printf("BusTools_BM_Init status = %d\n",status);

   status = BusTools_BM_MessageAlloc(cardnum, 300, &actual, BT1553_INT_END_OF_MESS ); //interrupt on END_OF_MESS
   printf("BusTools_BM_MessageAlloc status = %d - actual =%d\n",status,actual);

   return status;
}

/****************************************************************************
*
*  Function:  setupThe_Timer_IntFIFO1
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a RT message.  The interrupt is set for only the end-of
*     message.  
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupThe_Timer_IntFIFO(int cardnum)
{
   int status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO2, 0, sizeof(API_INT_FIFO));
   timerFIFO.function     = timer_intFunction;
   timerFIFO.iPriority    = THREAD_PRIORITY_NORMAL;
   timerFIFO.dwMilliseconds = 1000;
   timerFIFO.iNotification  = 0;
   timerFIFO.FilterType     = 0x0;
		 
   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &timerFIFO, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}


/****************************************************************************
*
*  Function:  setupThe_BC_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a BC message.  The interrupt is set for only the end-of
*     message.  
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupThe_BC_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.function     = bc_intFunction;
   sIntFIFO1.iPriority    = THREAD_PRIORITY_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.iNotification  = 0;
   sIntFIFO1.FilterType     = EVENT_BC_MESSAGE;

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO1.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}

/****************************************************************************
*
*  Function:  setupThe_1PPS_Deadman_timer
*
*  Description:
*     This routine setup the interrupt FIFO to detect when the 1PPS pulse is
*     not delivered on time
****************************************************************************/
void setupThe_1PPS_Deadman_Timer_IntFIFO(int cardnum)
{
   BT_INT status;

   // Setup the FIFO structure for trigger interrupt event.
   memset(&trig_fifo, 0, sizeof(trig_fifo));
   trig_fifo.function       = intFunction; // This is the interrupt function the API calls
   trig_fifo.iPriority      = THREAD_PRIORITY_ABOVE_NORMAL;
   trig_fifo.dwMilliseconds = 750;         // Timeout every 500 milliseconds to demonstrate timing out
   trig_fifo.iNotification  = 0;
   trig_fifo.FilterType     = EVENT_EXT_TRIG; 
   status = BusTools_RegisterFunction(cardnum, &trig_fifo, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);  
}