/*===========================================================================*
 * FILE:                    E X A M P L E _ B C _ A P E R I O D I C. C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    Demonstration of BusTools/1553-API routines.  This example shows
 *              how to setup and run aperiodic messages.  This example sets up a
 *              periodic bus list running at 1hz and two aperiodic bust lists. 
 *              The user can command either of the  aperiodic list to run low 
 *              or priorty or high priority.           
 *
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include <time.h>
#include "busapi.h"

int BusController_Init(int);
void setupThe_BC_IntFIFO(int);
API_INT_FIFO  sIntFIFO1;     // Thread FIFO structure for BC
BT_INT change_gap = 0;

BT_U32BIT mess_addr[6];

void main(int argc, char **argv)                     
{
   int status,cardnum;
   unsigned mode, device = 0;
   char ichar=' ';
  
   if(argc == 2)
      device = atoi(argv[1]);

   mode = API_SW_INTERRUPT | API_B_MODE;
   
   printf("Initializing Test\n");

   //Initial the channel running the BC
   status = BusTools_API_OpenChannel(&cardnum,mode,device,CHANNEL_1);
   printf("BusTools_API_OpenChannel BC status = %d\n",status);

   //Set the Bus to External for BC
   status = BusTools_SetInternalBus(cardnum,EXTERNAL_BUS);  // Set up External bus 0 on Internal 1
   printf("BusTools_SetInternalBus BC status = %d\n",status);

   //Set up the BC for 1Shot mode
   status = BusController_Init(cardnum);

   //Setting up a callback function to show the BC data
   setupThe_BC_IntFIFO(cardnum); //Set up BC interrupts using BusTools_RegisterFunction

   //Start the BC. 
   status = BusTools_BC_StartStop(cardnum,BC_START);
  
   while(1)
   {
      char inp[10];
	  printf("Enter Aperiodic Option:\n1 - Run Aperiodic Block 1 Low Priority\n2 - Run Aperiodic Block 2 Low Priority\n3 - Run Aperiodic Block 1 High Priority\n4 - Run aperiodic Block 2 High Priority\n");
      printf("Enter Q/q to quit\n");
      scanf("%s", inp);
      if(inp[0] == 'q' || inp[0] == 'Q')
         break;
      if(inp[0] == '1')
      {
         BusTools_BC_AperiodicRun(cardnum,10,0,0,0);  // Run aperiodic block 1 low
      }
      if(inp[0] == '2')
      {
         BusTools_BC_AperiodicRun(cardnum,20,0,0,0);  // Run aperiodic block 2 low
      }
      if(inp[0] == '3')
      {
         BusTools_BC_AperiodicRun(cardnum,10,1,0,0);  // Run aperiodic block 1 high
      }
      if(inp[0] == '4')
      {
         BusTools_BC_AperiodicRun(cardnum,20,1,0,0);  // Run aperiodic block 2 high
      }
   }

   status = BusTools_BC_StartStop(cardnum,BC_STOP); 
   printf("main: closing API\n");

   status = BusTools_RegisterFunction(cardnum,&sIntFIFO1,UNREGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);

   status = BusTools_API_Close(cardnum);
   printf("BusTools_API_Close BC status = %d\n",status);

}

int BusController_Init(int cardnum)
{
   int status, messno,i;
   API_BC_MBUF bcmessage;

   // Initialize the BC.  Select the interrupt, timeouts and frame time. 
   status = BusTools_BC_Init(cardnum,0,BT1553_INT_END_OF_MESS,0,16,14,1000000,1);
   printf("BusTools_BC_Init status = %d\n",status);

   // Allocate enough BC Buffer to hold the maximum number of meesage you are using
   status = BusTools_BC_MessageAlloc(cardnum,50);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);

   // #1
   messno = 0;
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;
   bcmessage.mess_command1.rtaddr   = 2;
   bcmessage.mess_command1.subaddr  = 2;
   bcmessage.mess_command1.wcount   = 8;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 10;                      // 10 microsecond inter-message gap.
   for(i=0;i<8;i++)
      bcmessage.data[0][i] = 0xA200+i;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   // #2
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 2;
   bcmessage.mess_command1.subaddr  = 0;          // Mode Code
   bcmessage.mess_command1.wcount   = 1;          // Sync W/O Data 
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 20;                       // 20 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   // #3
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 2;
   bcmessage.mess_command1.subaddr  = 4;
   bcmessage.mess_command1.wcount   = 6;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                       // 15 microsecond inter-message gap.
   for(i=0;i<6;i++)
      bcmessage.data[0][i] = 0xA400+i;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   // #4
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 2;
   bcmessage.mess_command1.subaddr  = 5;
   bcmessage.mess_command1.wcount   = 5;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 10;                      // 10 microsecond inter-message gap.
   for(i=0;i<5;i++)
      bcmessage.data[0][i] = 0xA500+i;

   // This the end-of-frame on a Noop Message
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(0);       // Back to frame start
   bcmessage.control = BC_CONTROL_NOP;           // Noop
   bcmessage.control |= BC_CONTROL_MFRAME_END;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

// This is the aperiodic message

   //aperodic message block 1  Start at buffer 10
   messno=10;
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(0xffff);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 8;
   bcmessage.mess_command1.subaddr  = 8;
   bcmessage.mess_command1.wcount   = 8;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 20;                       // 20 microsecond inter-message gap.
   for(i=0;i<8;i++)
      bcmessage.data[0][i] = 0x8800+i;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //aperiodic message block 2 start at buffer 20
   ///aperiodic message 1
   messno=20;
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 6;
   bcmessage.mess_command1.wcount   = 12;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 10;                       // 10 microsecond inter-message gap.
   for(i=0;i<12;i++)
      bcmessage.data[0][i] = 0xa800+i;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //aperiodic message 2
   messno=21;
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(0xffff);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 0;
   bcmessage.mess_command1.wcount   = 16;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 20;                      // 20 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   return 0;
}  


/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   static int sa_index;
   int       status,i;
   BT_INT tail, j,wcount;
   BT_UINT messno;
   int mcode;

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      messno = sIntFIFO->fifo[tail].bufferID;
      status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);
      if (status)
         return status; 

      if(bcmessage.mess_command1.subaddr == 0)
      {
         mcode = 1;
         printf("Mode Code ....\n");
      }
      else
         mcode = 0;
      
      if (bcmessage.mess_command1.tran_rec)
         printf("BC**RT->BC (%d)  \n", messno);
      else
         printf("BC->RT (%d)  \n", messno);
 
      printf("BC**RT-%d  \n",bcmessage.mess_command1.rtaddr);
      printf("BC**SA-%d  \n",bcmessage.mess_command1.subaddr);
      printf("BC**WC-%d  \n",bcmessage.mess_command1.wcount);
      printf("BC**status-1 0x%04x\n",*((WORD*)(&bcmessage.mess_status1)));
      printf("BC**int_stat-0x%08lx\n",*((DWORD*)(&bcmessage.status)));

      wcount = bcmessage.mess_command1.wcount;
 
      if(mcode)
	  {
         if(wcount >=16)
            wcount = 1;
		 else
            wcount = 0;
	  }
	  else
	  {
         if (wcount == 0)
            wcount = 32;           // Zero word count means 32 words.
	  }
      j = wcount;
      
      for ( i = 0; i < j; i++ ) // Display all of the data words
      {
         printf("BC**%04x ",bcmessage.data[0][i]);
      }
      printf("\n");

      tail++;
      tail &= sIntFIFO->mask_index;
    sIntFIFO->tail_index = tail;
   }

   return 0;
}


/****************************************************************************
*
*  Function:  setupThe_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a BC message.  The interrupt is set for only the end-of
*     message.  
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupThe_BC_IntFIFO(int cardnum)
{
   int status,tr,sa;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.function     = bc_intFunction;
   sIntFIFO1.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.iNotification  = 0;
   sIntFIFO1.FilterType     = EVENT_BC_MESSAGE;

   for (tr = 0; tr < 2; tr++ )
   {
      for (sa = 0; sa < 32; sa++ )
      {
         sIntFIFO1.FilterMask[4][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos
         sIntFIFO1.FilterMask[8][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos          
      }
  }

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}
