/*===========================================================================*
 * FILE:              E X A M P L E _ B C _ A U T O _ I N C R 
 *===========================================================================*
 *
  *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    Demonstration of BusTools/1553-API routines. This exampe sets 
 *              up the Bus Controller, Remote Terminal and Bus Monitor and then
 *              enable auto-increment on the Bus Controller message buffer 3. 
 *              that buffer contains a 32-word receive command to RT4 SA4.  The
 *              last word in the command, data word word 32 increment each time
 *              the message transacts.
 *
 *===========================================================================*/

/* $Revision:  8.12lease $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/
#include <stdio.h>
#include "busapi.h"

void BusController_Init(int,int);
void BusMonitor_Init(int);
void RemoteTerminal_Init(int);
void setupThe_BC_IntFIFO(int cardnum);

API_INT_FIFO  sIntFIFO1;     // Thread FIFO structure for BC

#if defined (__WIN32__)
void main(int argc, char **argv)                     
#else
int tst_bc_incr()
#endif
{
   int status,cardnum;
   char ichar;
   unsigned channel,device =0,mode;

   mode = API_B_MODE | API_SW_INTERRUPT;  // Use 1553B and Polling interrupts
   channel = CHANNEL_1;                   // Default channel 1
#if defined (__WIN32__)
   if(argc == 2)                          // Get device ID from command line.
      device = atoi(argv[1]);
   else
      device = 0;  // Default device 0
   
#endif

   // Initialize 1553 channel
   status = BusTools_API_OpenChannel(&cardnum,mode,device,channel);
   printf("BusTools_API_OpenChannel %d status = %d\n",cardnum,status);

   // Use the external bus
   status = BusTools_SetInternalBus(cardnum,EXTERNAL_BUS);  // Set External Bus
   printf("BusTools_SetInternalBus status = %d\n",status);

   // When running a Bus Monitor Initialize it first then the RT or BC
   BusMonitor_Init(cardnum);
   RemoteTerminal_Init(cardnum);  
   BusController_Init(cardnum,50);

   // Set up the auto increment on message 3 RT4 SA4 RX on the last word (32)
   // Increment between 0 and 512 by steps of 1
   BusTools_BC_AutoIncrMessageData(cardnum,3,31,0,1,1,512,1);

   setupThe_BC_IntFIFO(cardnum);
   
   status = BusTools_BM_StartStop(cardnum,BM_START);
   printf("BusTools_BM_StartStop status = %d\n",status);

   status = BusTools_RT_StartStop(cardnum,RT_START);
   printf("BusTools_RT_StartStop status = %d\n",status);

   status = BusTools_BC_StartStop(cardnum,BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);

   while((ichar=getc(stdin))!='q')
   {
      if(ichar == 'd') // memory dump
      {
         static int count=0;

         char dump_file_name[20];
         sprintf(dump_file_name,"mem_dump%d.txt",count++);
         printf("Dumping memory to %s\n",dump_file_name);
         BusTools_DumpMemory(cardnum,DUMP_ALL,dump_file_name,"TST_RT Memory Dump");
      }
   }

   // Stop the Bus Controller
   status = BusTools_BC_StartStop(cardnum,BC_STOP); 
   printf("BusTools_BC_StartStop status = %d\n",status);

   // Stop the Remote Terminal
   status = BusTools_RT_StartStop(cardnum,RT_STOP);
   printf("BusTools_RT_StartStop status = %d\n",status);

   // Stop the Bus Monitor
   status = BusTools_BM_StartStop(cardnum,BM_STOP);
   printf("BusTools_BM_StartStop status = %d\n",status);

   BusTools_BC_AutoIncrMessageData(cardnum,3,0,0,1,1,512,0);

   status = BusTools_API_Close(cardnum);//
   printf("BusTools_API_Close status = %d\n",status);

}

void BusController_Init(int cardnum,int num_buffers)
{
   int status, messno,i;
   API_BC_MBUF bcmessage;

   status = BusTools_BC_Init(cardnum,0,BT1553_INT_END_OF_MESS,0,16,14,1000000,1);
   printf("BusTools_BC_Init status = %d\n",status);

   status = BusTools_BC_MessageAlloc(cardnum,50);//num_buffers);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);

   messno = 0;
   memset((char*)&bcmessage,0,sizeof(bcmessage));       
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_MFRAME_BEG; 
   bcmessage.mess_command1.rtaddr   = 2;
   bcmessage.mess_command1.subaddr  = 2;
   bcmessage.mess_command1.wcount   = 8;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 10;                      // Microsecond inter-message gap.

   for(i=0;i<32;i++)
       bcmessage.data[0][i] = 0xA000+i;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;

   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 4;
   bcmessage.mess_command1.wcount   = 4;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 10;                      // Microsecond inter-message gap
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);


   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 3;
   bcmessage.mess_command1.subaddr  = 3;
   bcmessage.mess_command1.wcount   = 3;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 20;                       // Microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 4;
   bcmessage.mess_command1.wcount   = 32;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 50;                      // Microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;

   bcmessage.mess_command1.rtaddr   = 5;
   bcmessage.mess_command1.subaddr  = 5;
   bcmessage.mess_command1.wcount   = 8;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // Microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   // RT - RT Message
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_RTRTFORMAT;
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 10;
   bcmessage.mess_command1.wcount   = 6;
   bcmessage.mess_command1.tran_rec = 0;

   bcmessage.mess_command2.rtaddr   = 6;
   bcmessage.mess_command2.subaddr  = 6;
   bcmessage.mess_command2.wcount   = 6;
   bcmessage.mess_command2.tran_rec = 1;
   bcmessage.errorid = 0;                       // Default error injection buffer (no errors)
   bcmessage.gap_time = 25;                     // Microsecond inter-message gap.
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 6;
   bcmessage.mess_command1.subaddr  = 0;
   bcmessage.mess_command1.wcount   = 17;  
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                     // Microsecond inter-message gap.
   bcmessage.data[0][0] = 0xABCD;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)0;         // Start back at the first command
   bcmessage.control = BC_CONTROL_NOP;           // No-op command
   bcmessage.control |= BC_CONTROL_MFRAME_END;   // End of minor frame

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);
 
} 

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a BC end-of-message condition. This function read the data in the BC 
*     and prints data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   int       bufnum;
   static int sa_index;
   int       i,j;
   int       status;
   int       wcount;
   BT_INT tail, rtrt,mc;
   BT_UINT messno;
   static int nFlag = 1;

   char outbuf[80];

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      messno = sIntFIFO->fifo[tail].bufferID;
      status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);
      if (status)
         return status;
          
      rtrt=0;
      mc=0;
      if(bcmessage.status & BT1553_INT_RETRY)
         printf("Retry - %d - %x\n",messno,bcmessage.status);
      else
         printf("%d - %x\n",messno,bcmessage.status);

      BusTools_TimeGetString(&bcmessage.time_tag, outbuf);
      printf("tag time = %s\n",outbuf);
       
      if(bcmessage.status & BT1553_INT_RT_RT_FORMAT)
      {
         printf("RT<->RT (%d) ",messno);
         rtrt=1;
      }
      else if(bcmessage.status & BT1553_INT_MODE_CODE)
      {
         printf("Mode Code --  (%d) ", messno);
         mc=1;
      }
      else if (bcmessage.mess_command1.tran_rec)
          printf("BC**RT->BC (%d)  ", messno);
      else
         printf("BC->RT (%d)  ", messno);
 
      printf("BC**RT-%d  \n",bcmessage.mess_command1.rtaddr);
      printf("BC**SA-%d  \n",bcmessage.mess_command1.subaddr);
      printf("BC**WC-%d  \n",bcmessage.mess_command1.wcount);
      if(rtrt)
      {   
         printf("BC**RT-%d  \n",bcmessage.mess_command2.rtaddr);
         printf("BC**SA-%d  \n",bcmessage.mess_command2.subaddr);
         printf("BC**WC-%d  \n",bcmessage.mess_command2.wcount);
      }
        
      printf("BC**status-1 0x%04x\n",*((WORD*)(&bcmessage.mess_status1)));
      if(rtrt)
         printf("BC**status-2 0x%04x\n",*((WORD*)(&bcmessage.mess_status2)));          
      printf("BC**int_stat-0x%08lx\n",*((DWORD*)(&bcmessage.status)));

      wcount = bcmessage.mess_command1.wcount;
      if (wcount == 0)
         wcount = 32;           // Zero word count means 32 words.

      if(mc)
         if(wcount >=16)
            wcount = 1;

      j = wcount;               // See if this msg shorted than previous
    
      if (bcmessage.control & BC_CONTROL_BUFFERA)
         bufnum = 0;            // Display data from first BC buffer
      else
         bufnum = 1;            // Display data from second BC buffer

      for ( i = 0; i < j; i++ ) // Display all of the data words
      {
         if(i % 8 == 0)
            printf("\n");
         // If this is an RT->BC message, and there is a Message Error
         //  or No Response, than xxxx the data (since the RT did not
         //    send any data words).
         if ( i >= wcount )
         {
            printf("    ");     // Data does not apply to this msg
         }
         else if(bcmessage.mess_command1.tran_rec && (bcmessage.mess_status1.me || (bcmessage.status&BT1553_INT_NO_RESP)))
         {
            printf("xxxx ");      // No data was transfered because of error
         }
         else
         {
            printf("BC**%04x ",bcmessage.data[bufnum][i]);
         }
      }
      printf("\n");

      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }

   return API_SUCCESS;
}

void RemoteTerminal_Init(int cardnum)
{
   int status;
   int RT_MBUF_COUNT;
   API_RT_CBUF cbuf;       // RT Control Buffer Legal Word Count Mask (2 words)
   API_RT_MBUF_WRITE mbuf;
   API_RT_ABUF abuf;
   int         tr;
   int         subaddress;
   int i,j,ii,jj;
   int rt_addr[] = {2,3,4,5,6,7,8,30};
   int rt_cnt = 8;

   RT_MBUF_COUNT = 1;

   status = BusTools_RT_Init(cardnum,0);
   printf("BusTools_RT_Init status = %d\n",status);

   /**********************************************************************
   *  Output RT control buffers
   *   Enable or disable all word counts and mode codes as specified by
   *    the caller's arguements.
   **********************************************************************/

   cbuf.legal_wordcount = 0xffffffff;  // legailize all word counts
   for ( i = 0; i < rt_cnt; i++)
   { 
      for ( tr = 0; tr < 2; tr++ )
      {
         for ( subaddress = 0; subaddress < 32; subaddress++ )
         {
            status = BusTools_RT_CbufWrite(cardnum,rt_addr[i],subaddress,tr,
			                            RT_MBUF_COUNT,&cbuf);
         }
      }
   }

   /*******************************************************************
   *  Fill in message buffers for receive and transmit sub-units.
   *******************************************************************/
   mbuf.error_inj_id = 0;
   mbuf.enable = BT1553_INT_END_OF_MESS;   // Interrupt enables
   // For all subaddresses...
   for (ii = 0; ii<rt_cnt; ii++)
   {  
      for (i=0; i<8; i++)
      {
         // For each data buffer...
         for (j=0; j<RT_MBUF_COUNT; j++)
         {
            for(jj=0;jj<8;jj++)
               mbuf.mess_data[jj] = (BT_U16BIT)(0xcdef + j);
            status = BusTools_RT_MessageWrite(cardnum,rt_addr[ii],i,0,j,&mbuf);
            status = BusTools_RT_MessageWrite(cardnum,rt_addr[ii],i,1,j,&mbuf);
         }
      }
   }

   /************************************************
   *  Setup RT address control block
   ************************************************/

   abuf.enable_a          = 1;    // Enable Bus A...Note that the API bits are
   abuf.enable_b          = 1;    // Enable Bus B... reversed from the HW bits.
   abuf.inhibit_term_flag = 0;
   abuf.status            = 0x0000;
   abuf.command           = 0;
   abuf.bit_word          = 0;

   for(i=0;i<rt_cnt;i++)
   {
      status = BusTools_RT_AbufWrite(cardnum,i,&abuf);
      printf("BusTools_RT_AbufWrite status = %d\n",status);
   }
}

void BusMonitor_Init(int cardnum)
{
 int status;
   BT_UINT actual;

   status = BusTools_BM_Init(cardnum,1,1);
   printf("BusTools_BM_Init status = %d\n",status);

   status = BusTools_BM_MessageAlloc(cardnum, 100, &actual ,BT1553_INT_END_OF_MESS); // NO interrupt on BT1553_INT_END_OF_MESS
   printf("BusTools_BM_MessageAlloc status = %d - actual =%d\n",status,actual);
}

/****************************************************************************
*
*  Function:  setupThe_BC_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a BC message.  The interrupt is set for only the end-of
*     message.  
*
****************************************************************************/
void setupThe_BC_IntFIFO(int cardnum)
{
   int status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.iNotification = 0;
   sIntFIFO1.function     = bc_intFunction;
   sIntFIFO1.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.bForceStartup  = 0;
   sIntFIFO1.FilterType     =  EVENT_BC_MESSAGE;
   sIntFIFO1.FilterMask[4][0][4] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, 1);
   printf("BusTools_RegisterFunction status = %d\n",status);
}
