/*===========================================================================*
 * FILE:             E X M A P L E _ B C _ B R A N C H_ O N _ D A T A . C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    Demonstration of BusTools/1553-API routines. This 
 *              application shows how to setup Message Scheduling, Frame-Start-
 *              Timing and Conditional Branch 2. 
 *
 *              Conditional Branch 2 branches based on the value in an 
 *              on-board memory location allocated using BusTools_MemoryAlloc.
 *              The application writes walues to this location to control
 *              which conditional messages transactions. There are 10 conditional
 *              messages. Each is run based on a different bit-wise (1,2,4,8...)
 *              vaule written to the allocated memory address.  By using the
 *              data_mask and data_value the application can send any combination 
 *              of conditional messages by entering a value between 1 and 0x3ff.
 *              That value is written to the allocated memory location.  The
 *              Conditional Branch message evaluate the data at that test address
 *              to determine whether to branch.
 *  
 *              This is an alternate approach to apperiodic messaging,  The
 *              application can select any combination of messages to run after
 *              the periodic messages transact. Unlike aperiodic messages you
 *              need to ensure there is enough time in the frame after the 
 *              periodic messages for the selected conditional messages to run.
 *              All messages run in the current frame. After each conditional 
 *              message runs the corresponding bit at the test address is cleared
 *              so the message only runs once per data update.
 *              
 *              Note: This example run on both the new V6 firmware and older 
 *              V4/V5 firmware.  In the new V6 firmware allocate 32 bit of 
 *              memory rather than 16 bits used in the older firmware.
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/

#include "busapi.h"					 
#include <stdio.h>

void setupThe_BC_IntFIFO(int cardnum);
void setupThe_BCCTL_intFIFO(cardnum);
int	             status,j;
int              messno,cardnum;
static BT_U32BIT messno_cond;
BT_U32BIT        mem_addr;
API_INT_FIFO     bcIntFIFO;
API_INT_FIFO     bcCtlIntFIFO;
API_BC_MBUF		 bcmessage;	         // bloc message .
char			 c;
BT_UINT			 priority;
BT_UINT branc_count = 0;

int add_msg_in_minorframe(int start_frame,int rep_rate,int delay,int rt,
						  int sa,int direction,int wordcount)
{
   //*******************
   // Create message 
   //*******************	
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = messno+1;    
   bcmessage.control = BC_CONTROL_MESSAGE;
   bcmessage.control |= BC_CONTROL_BUFFERA;     
   bcmessage.control |= BC_CONTROL_CHANNELA;

   if (messno == 0)
   {
      //bcmessage.control |= BC_CONTROL_INTERRUPT;
      bcmessage.control |= BC_CONTROL_MFRAME_BEG;  // Set Begin Frame on first message
   }

   bcmessage.mess_command1.rtaddr   = rt;
   bcmessage.mess_command1.subaddr  = sa;
   bcmessage.mess_command1.wcount   = wordcount;
   bcmessage.mess_command1.tran_rec = direction;
   bcmessage.start_frame            = start_frame;
   bcmessage.rep_rate               = rep_rate;
   bcmessage.long_gap               = delay;
   for (j=0; j<wordcount; j++)
      bcmessage.data[0][j] = sa;
			
   status = BusTools_BC_MessageWrite(cardnum, messno, &bcmessage);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status);   
   
   messno++;
   return(status);
}

void defineConditionalMessages(BT_INT cardnum)
{
   API_BC_MBUF bc_msg;

   memset(&bc_msg,0,sizeof (bc_msg));            // Clear all fields
   bc_msg.messno        = messno;                // Message number      
   bc_msg.control       = BC_CONTROL_CONDITION2; // Branch on data in a memory location
   bc_msg.test_address  = mem_addr;              // Address of memory location
   bc_msg.data_mask     = 0xffff;                   // Branch data mask
   bc_msg.data_value    = 0x5555;                   // Branch value
   bc_msg.messno_next   = messno+2;              // next message on false
   bc_msg.messno_branch = messno+1;              // Branch here on true
   status = BusTools_BC_MessageWrite (cardnum,messno,&bc_msg);        //Write branch Message to buffer.    
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); 

   messno++;
   memset(&bc_msg, 0, sizeof(bc_msg));           // Clear all fields.
   bc_msg.messno  = messno;		                 // Message number
   bc_msg.control = BC_CONTROL_MESSAGE;		     // BC message.
   bc_msg.control |= BC_CONTROL_BUSA;	         // Send message on bus A (primary).
   bc_msg.control |= BC_CONTROL_BUFFERA;	     // Only using one buffer, buffer A.
   bc_msg.control |= BC_CONTROL_INTERRUPT;
   bc_msg.messno_next			 = messno+1;     // Next message number
   bc_msg.mess_command1.rtaddr	 = 20;		     // Command word 1, RT address
   bc_msg.mess_command1.subaddr  = 1;		     // Command word 1, Subaddress
   bc_msg.mess_command1.wcount   = 6;
   bc_msg.mess_command1.tran_rec = 0;		     // Command word 1, transmit (1) or receive (0)
   bc_msg.gap_time				 = 1000;         // Gap time in microseconds from start of Frame
   bc_msg.start_frame            = 1;            // Message Scheduling Start-Frame
   bc_msg.rep_rate               = 1;            // Message Scheduling Repeat-Rate
   status = BusTools_BC_MessageWrite(cardnum,messno, &bc_msg);        //Write the BC Message to buffer
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); 

   messno++;
   memset(&bc_msg,0,sizeof (bc_msg));
   bc_msg.messno        = messno;
   bc_msg.control       = BC_CONTROL_CONDITION2;
   bc_msg.control |= BC_CONTROL_INTERRUPT;
   bc_msg.test_address  = mem_addr; 
   bc_msg.data_mask     = 0x2; 
   bc_msg.data_value    = 0x2;
   bc_msg.messno_next   = messno+2;
   bc_msg.messno_branch = messno+1;
   status = BusTools_BC_MessageWrite (cardnum,messno,&bc_msg);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); 

   messno++;
   memset(&bc_msg, 0, sizeof(bc_msg));           // See Comments above
   bc_msg.messno  = messno;		              
   bc_msg.control = BC_CONTROL_MESSAGE;		  
   bc_msg.control |= BC_CONTROL_CHANNELA;	  
   bc_msg.control |= BC_CONTROL_BUFFERA;	 
   bc_msg.control |= BC_CONTROL_INTERRUPT;
   bc_msg.messno_next			  = messno+1; 
   bc_msg.mess_command1.rtaddr	  = 21;	
   bc_msg.mess_command1.subaddr   = 3;
   bc_msg.mess_command1.wcount    = 16;
   bc_msg.mess_command1.tran_rec  = 1;	
   bc_msg.start_frame             = 1;
   bc_msg.rep_rate                = 1;
   bc_msg.gap_time				  = 1000;	 
   status = BusTools_BC_MessageWrite(cardnum,messno, &bc_msg);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); 

   messno++;
   memset(&bc_msg,0,sizeof (bc_msg));           // See Comments above               
   bc_msg.messno        = messno;
   bc_msg.control       = BC_CONTROL_CONDITION2;
   bc_msg.control |= BC_CONTROL_INTERRUPT;
   bc_msg.test_address  = mem_addr; 
   bc_msg.data_mask     = 0x4; 
   bc_msg.data_value    = 0x4;
   bc_msg.messno_next   = messno+2;
   bc_msg.messno_branch = messno+1;
   status = BusTools_BC_MessageWrite (cardnum,messno,&bc_msg);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); 
   
   messno++;
   memset(&bc_msg, 0, sizeof(bc_msg));           // See Comments above
   bc_msg.messno  = messno;		                
   bc_msg.control = BC_CONTROL_MESSAGE;   
   bc_msg.control |= BC_CONTROL_CHANNELA;	   
   bc_msg.control |= BC_CONTROL_BUFFERA;	    
   bc_msg.control |= BC_CONTROL_INTERRUPT;
   bc_msg.messno_next			  = messno+1;
   bc_msg.mess_command1.rtaddr	  = 22;	
   bc_msg.mess_command1.subaddr   = 1;	
   bc_msg.mess_command1.tran_rec  = 0;		
   bc_msg.mess_command1.wcount    = 18;
   bc_msg.start_frame             = 1;
   bc_msg.rep_rate                = 1;
   bc_msg.gap_time				  = 1000;	
   status = BusTools_BC_MessageWrite(cardnum,messno, &bc_msg);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); ;
     
   messno++;
   memset(&bc_msg,0,sizeof (bc_msg));           // See Comments above
   bc_msg.messno         = messno;
   bc_msg.control        = BC_CONTROL_CONDITION2;
   bc_msg.test_address   = mem_addr; 
   bc_msg.data_mask      = 0x8; 
   bc_msg.data_value     = 0x8;
   bc_msg.messno_next    = messno+2;
   bc_msg.messno_branch  = messno+1;
   status = BusTools_BC_MessageWrite (cardnum,messno,&bc_msg);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); 

   messno++;
   memset(&bc_msg, 0, sizeof(bc_msg));           // See Comments above.
   bc_msg.messno  = messno;		// Message number
   bc_msg.control = BC_CONTROL_MESSAGE;		
   bc_msg.control |= BC_CONTROL_CHANNELA;
   bc_msg.control |= BC_CONTROL_BUFFERA;	
   bc_msg.control |= BC_CONTROL_INTERRUPT;
   bc_msg.messno_next			  = messno+1;	
   bc_msg.mess_command1.rtaddr	  = 23;		
   bc_msg.mess_command1.subaddr   = 30;	
   bc_msg.mess_command1.tran_rec  = 0;	
   bc_msg.mess_command1.wcount    = 4;
   bc_msg.start_frame             = 1;
   bc_msg.rep_rate                = 1;
   bc_msg.gap_time				  = 1000;			
   status = BusTools_BC_MessageWrite(cardnum,messno, &bc_msg);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); 

   messno++;
   memset(&bc_msg,0,sizeof (bc_msg));           // See Comments above
   bc_msg.messno        = messno;
   bc_msg.control       = BC_CONTROL_CONDITION2;
   bc_msg.test_address  = mem_addr; 
   bc_msg.data_mask     = 0x10; 
   bc_msg.data_value    = 0x10;
   bc_msg.messno_next   = messno+2;
   bc_msg.messno_branch = messno+1;
   status = BusTools_BC_MessageWrite (cardnum,messno,&bc_msg);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); 

   messno++;
   memset(&bc_msg, 0, sizeof(bc_msg));           // See Comments above
   bc_msg.messno  = messno;		// Message number
   bc_msg.control = BC_CONTROL_MESSAGE;		 
   bc_msg.control |= BC_CONTROL_CHANNELA;	 
   bc_msg.control |= BC_CONTROL_BUFFERA; 
   bc_msg.control |= BC_CONTROL_INTERRUPT;
   bc_msg.messno_next			  = messno+1;
   bc_msg.mess_command1.rtaddr	  = 24;	
   bc_msg.mess_command1.subaddr   = 3;	
   bc_msg.mess_command1.wcount    = 0;
   bc_msg.mess_command1.tran_rec  = 0;		
   bc_msg.start_frame             = 1;
   bc_msg.rep_rate                = 1;
   bc_msg.gap_time				  = 1000;	
   status = BusTools_BC_MessageWrite(cardnum,messno, &bc_msg);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); 

   messno++;
   memset(&bc_msg,0,sizeof (bc_msg));           // See Comments above
   bc_msg.messno        = messno;
   bc_msg.control       = BC_CONTROL_CONDITION2;
   bc_msg.test_address  = mem_addr; 
   bc_msg.data_mask     = 0x20; 
   bc_msg.data_value    = 0x20;
   bc_msg.messno_next   = messno+2;
   bc_msg.messno_branch = messno+1;
   status = BusTools_BC_MessageWrite (cardnum,messno,&bc_msg);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); 

   messno++;
   memset(&bc_msg, 0, sizeof(bc_msg));           // See Comments above.
   bc_msg.messno  = messno;		// Message number
   bc_msg.control = BC_CONTROL_MESSAGE;	
   bc_msg.control |= BC_CONTROL_CHANNELA;	
   bc_msg.control |= BC_CONTROL_BUFFERA;
   bc_msg.control |= BC_CONTROL_INTERRUPT;
   bc_msg.messno_next			 = messno+1;
   bc_msg.mess_command1.rtaddr	 = 25;		
   bc_msg.mess_command1.subaddr  = 5;	
   bc_msg.mess_command1.wcount   = 3;
   bc_msg.mess_command1.tran_rec = 0;	
   bc_msg.start_frame            = 1;
   bc_msg.rep_rate               = 1;
   bc_msg.gap_time				 = 1000;		
   status = BusTools_BC_MessageWrite(cardnum,messno, &bc_msg);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); ;
 
   messno++;
   memset(&bc_msg,0,sizeof (bc_msg));           // See Comments above
   bc_msg.messno        = messno;
   bc_msg.control       = BC_CONTROL_CONDITION2;
   bc_msg.test_address  = mem_addr; 
   bc_msg.data_mask     = 0x40; 
   bc_msg.data_value    = 0x40;
   bc_msg.messno_next   = messno+2;
   bc_msg.messno_branch = messno+1;
   status = BusTools_BC_MessageWrite (cardnum,messno,&bc_msg);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); 

   messno++;
   memset(&bc_msg, 0, sizeof(bc_msg));           // See Comments above.
   bc_msg.messno  = messno;		// Message number
   bc_msg.control = BC_CONTROL_MESSAGE;	 
   bc_msg.control |= BC_CONTROL_CHANNELA;
   bc_msg.control |= BC_CONTROL_BUFFERA;	
   bc_msg.control |= BC_CONTROL_INTERRUPT;
   bc_msg.messno_next			 = messno+1; 
   bc_msg.mess_command1.rtaddr	 = 26;	 
   bc_msg.mess_command1.subaddr  = 1;		 
   bc_msg.mess_command1.tran_rec = 0;	 
   bc_msg.start_frame            = 1;
   bc_msg.rep_rate               = 1;
   bc_msg.mess_command1.wcount   = 18;
   bc_msg.gap_time				 = 1000;	 
   status = BusTools_BC_MessageWrite(cardnum,messno, &bc_msg);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); ;

   memset(&bc_msg,0,sizeof (bc_msg));           // See Comments above
   messno++;
   bc_msg.messno        = messno;
   bc_msg.control       = BC_CONTROL_CONDITION2;
   bc_msg.test_address  = mem_addr; 
   bc_msg.data_mask     = 0x80; 
   bc_msg.data_value    = 0x80;
   bc_msg.messno_next   = messno+2;
   bc_msg.messno_branch = messno+1;
   status = BusTools_BC_MessageWrite (cardnum,messno,&bc_msg);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); 

   messno++;
   memset(&bc_msg, 0, sizeof(bc_msg));           // See Comments above
   bc_msg.messno  = messno;	 
   bc_msg.control = BC_CONTROL_MESSAGE;	 
   bc_msg.control |= BC_CONTROL_CHANNELA; 
   bc_msg.control |= BC_CONTROL_BUFFERA; 
   bc_msg.control |= BC_CONTROL_INTERRUPT;
   bc_msg.messno_next			 = messno+1; 
   bc_msg.mess_command1.rtaddr	 = 27;	 
   bc_msg.mess_command1.subaddr  = 30;	 
   bc_msg.mess_command1.tran_rec = 0;	 
   bc_msg.mess_command1.wcount   = 4;
   bc_msg.start_frame            = 1;
   bc_msg.rep_rate               = 1;
   bc_msg.gap_time				 = 1000; 
   status = BusTools_BC_MessageWrite(cardnum,messno, &bc_msg);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); 

   messno++;
   memset(&bc_msg,0,sizeof (bc_msg));           // See Comments above
   bc_msg.messno = messno;
   bc_msg.control = BC_CONTROL_CONDITION2;
   bc_msg.test_address = mem_addr; 
   bc_msg.data_mask  = 0x100; 
   bc_msg.data_value = 0x100;
   bc_msg.messno_next = messno+2;
   bc_msg.messno_branch = messno+1;
   status = BusTools_BC_MessageWrite (cardnum,messno,&bc_msg);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); 

   messno++;
   memset(&bc_msg, 0, sizeof(bc_msg));           // See Comments above.
   bc_msg.messno  = messno;		// Message number
   bc_msg.control = BC_CONTROL_MESSAGE;	 
   bc_msg.control |= BC_CONTROL_CHANNELA; 
   bc_msg.control |= BC_CONTROL_BUFFERA;	 
   bc_msg.control |= BC_CONTROL_INTERRUPT;
   bc_msg.messno_next			 = messno+1; 
   bc_msg.mess_command1.rtaddr	 = 28;	 
   bc_msg.mess_command1.subaddr  = 3;	 
   bc_msg.mess_command1.tran_rec = 0;	 
   bc_msg.mess_command1.wcount   = 32;
   bc_msg.start_frame            = 1;
   bc_msg.rep_rate               = 1;
   bc_msg.long_gap			     = 1000;	   
   status = BusTools_BC_MessageWrite(cardnum,messno, &bc_msg);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); 

   messno++;
   memset(&bc_msg,0,sizeof (bc_msg));           // See Comments above
   bc_msg.messno        = messno;
   bc_msg.control       = BC_CONTROL_CONDITION2;
   bc_msg.test_address  = mem_addr; 
   bc_msg.data_mask     = 0x200; 
   bc_msg.data_value    = 0x200;
   bc_msg.messno_next   = messno+2;
   bc_msg.messno_branch = messno+1;
   status = BusTools_BC_MessageWrite (cardnum,messno,&bc_msg);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); 

   messno++;
   memset(&bc_msg, 0, sizeof(bc_msg));            // See Comments above
   bc_msg.messno  = messno;		// Message number
   bc_msg.control = BC_CONTROL_MESSAGE;		
   bc_msg.control |= BC_CONTROL_CHANNELA;	
   bc_msg.control |= BC_CONTROL_BUFFERA;
   bc_msg.control |= BC_CONTROL_INTERRUPT;
   bc_msg.messno_next			 = messno+1;
   bc_msg.mess_command1.rtaddr	 = 29;		
   bc_msg.mess_command1.subaddr  = 5;	
   bc_msg.mess_command1.tran_rec = 0;	
   bc_msg.mess_command1.wcount   = 3;
   bc_msg.start_frame            = 1;
   bc_msg.rep_rate               = 1;
   bc_msg.long_gap				 = 1000;		
   status = BusTools_BC_MessageWrite(cardnum,messno, &bc_msg);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); 

   // NOOP End of frame  last message in each frame
   messno++;
   memset((char*)&bc_msg,0,sizeof(bc_msg));
   bc_msg.messno = messno;                  // Message Number
   bc_msg.messno_next = (BT_U16BIT)(0);     // Start Next frame at message buffer 0
   bc_msg.control  = BC_CONTROL_NOP;        // NOOP Message
   bc_msg.control |= BC_CONTROL_MFRAME_END; // End-of-Frame
   bc_msg.long_gap = 1000; 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bc_msg);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); 
}

void main(int argc, char** argv) 
{
   BT_UINT mode;
   BT_UINT device;

   printf("Initializing API with BusTools_API_OpenChannel . . . ");

    // Open the device and get the card number.
   mode = API_B_MODE | API_SW_INTERRUPT;  // 1553B protocol with SW interrupts.

   if(argc == 2)                          
      device = atoi(argv[1]);            // Device number from command-line
   else
      device = 0;                        // Default to device 0

   status = BusTools_API_OpenChannel( &cardnum, mode, device, CHANNEL_1);  // Initialize select device/channel
   printf("BusTools_API_OpenChannel status = %d\n",status);

   // Select External Bus.
   printf("Bus select . . . ");
   status = BusTools_SetInternalBus(cardnum, EXTERNAL_BUS);
   printf("BusTools_SetInternalBus status = %d\n",status);

   printf("BC Init . . . ");  //Message Scheduling and Frame-Start-Timing, Interrupt on end-of-message
   status = BusTools_BC_Init(cardnum,MSG_SCHD | FRAME_START_TIMING, BT1553_INT_END_OF_MESS, 0,14,12,1000000,1);
   printf("BusTools_BC_Init status = %d\n",status);

   printf("\nMSG Alloc . . . \n"); 
   status = BusTools_BC_MessageAlloc(cardnum, 50);  // Allocate 50 BC Message Buffers
   printf("BusTools_BC_MessageAlloc status = %d\n",status);
		
   // Set up the periodic messages (gap is the time from start of frame)
   //                    rr sf   gap  rt  sa  r/t wc
   add_msg_in_minorframe(1, 1,     0,  1,  2,  1, 15);
   add_msg_in_minorframe(1, 1,  1000,  2,  2,  1, 15);
   add_msg_in_minorframe(1, 1,  4000,  3,  5,  1,  2);
   add_msg_in_minorframe(1, 1,  5000,  4,  5,  1,  2);
   add_msg_in_minorframe(1, 2,  6000, 10, 15,  1,  7);
   add_msg_in_minorframe(1, 2,  7000, 11, 15,  1,  7);
   add_msg_in_minorframe(1, 1, 11000,  5,  1,  1, 28);
   add_msg_in_minorframe(1, 2, 12000, 12,  2,  1, 20);
   add_msg_in_minorframe(1, 5, 13000, 17, 30,  0,  2);
   add_msg_in_minorframe(1, 5, 14000, 18, 30,  1,  2);

   //Set up the Condition Branch memory address
   messno_cond=0;
   if(BusTools_BoardIsV6(cardnum))
   {
      status = BusTools_MemoryAlloc32(cardnum,0,4,&mem_addr);                   // Use memory address for bit encoded branchning 
      printf("BusTools_MemoryAlloc status = %d\n",status);            
      status = BusTools_MemoryWrite2(cardnum,RAM32,mem_addr,4,&messno_cond);  // Clear this address
      printf("BusTools_MemoryWrite2 status = %d\n",status);
   }
   else
   {
      status = BusTools_MemoryAlloc(cardnum,0,2,&mem_addr);                   // Use memory address for bit encoded branchning 
      printf("BusTools_MemoryAlloc status = %d\n",status);            
      status = BusTools_MemoryWrite(cardnum,mem_addr,2,(BT_U16BIT *)(&messno_cond));  // Clear this address
      printf("BusTools_MemoryWrite2 status = %d\n",status);
   }

   // Define conditional messages     
   defineConditionalMessages(cardnum);    
   setupThe_BC_IntFIFO(cardnum);  

   status = BusTools_BC_StartStop(cardnum, BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);

   // BC is running, wait for user to tell us to stop or read data.
   printf("\nEnter bit encoded HEX value 0 - 0x3ff.\nEnter Q/q to quit\n");

   // Loop until a 0 or 'q' is input			
   while (1)      
   {
      scanf("%x",&messno_cond);
      if(messno_cond==0)
         break;
      printf("the message value is %d\n",messno_cond);

      if(messno_cond > 0xffffffff)
      {
         BusTools_DumpMemory(cardnum,DUMP_ALL,"Branch_on_data_dump.txt","Branch On Data");
         messno_cond = 0;
      }

      if(BusTools_BoardIsV6(cardnum))
         BusTools_MemoryWrite2(cardnum,RAM32,mem_addr,4,&messno_cond);  // Write the input data to the branch address
      else
         BusTools_MemoryWrite(cardnum,mem_addr,2,(BT_U16BIT *)&messno_cond);  // Write the input data to the branch address
      fflush(stdin);
   }

   // Stop the BC.
   printf("\nBC stopping . . . ");
   status = BusTools_BC_StartStop(cardnum, BC_STOP);
   printf("BusTools_BC_StartStop status = %d\n",status);

   status = BusTools_RegisterFunction(cardnum,&bcIntFIFO,UNREGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);

   // We're done.  Close API and board
   status = BusTools_API_Close(cardnum);
   printf("BusTools_API_Close status = %d\n",status);
} // End of main

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoked when the a Bus
*     Controller message issues an interrupt. It prints the RT address and clears
*     The conditional bit value associated with that RT address in the allocated
*     memory location used by the Conditional Branch messages.
*
****************************************************************************/
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   BT_INT tail;
   BT_UINT messno;
   BT_UINT mdata[]={1,0,2,0,4,0,8,0,16,0,32,0,64,0,128,0,256,0,512};

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      messno = sIntFIFO->fifo[tail].bufferID;
      status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);
      printf("BC interrupts: RT - %d\n",bcmessage.mess_command1.rtaddr);

      messno_cond &= ~mdata[(messno - 0xb)];
      if(BusTools_BoardIsV6(cardnum))
         BusTools_MemoryWrite2(cardnum,RAM32,mem_addr,4,&messno_cond);  //Clear Branch address
      else
         BusTools_MemoryWrite(cardnum,mem_addr,2,&messno_cond);         //Clear Branch address        
      
      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }

   return API_SUCCESS;
}

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoked when the a Bus
*     Controller message issues an interrupt. It prints the RT address and clears
*     The conditional bit value associated with that RT address in the allocated
*     memory location used by the Conditional Branch messages.
*
****************************************************************************/
BT_INT  _stdcall bc_intCtlFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   BT_INT tail,branch_count;
   
   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      branch_count++;
      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }

   return API_SUCCESS;
}

/****************************************************************************
*
*  Function:  setupThe_BC_IntFIFO
*
*  Description:
*     This routine sets up the interrupt FIFO for a interrupt on a BC message.  
* 
****************************************************************************/
void setupThe_BC_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;
   // Setup the FIFO structure for this board.
   memset(&bcIntFIFO, 0, sizeof(bcIntFIFO));;
   bcIntFIFO.function     = bc_intFunction;
   bcIntFIFO.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   bcIntFIFO.dwMilliseconds = INFINITE;
   bcIntFIFO.FilterType     =  EVENT_BC_MESSAGE;

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            bcIntFIFO.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &bcIntFIFO, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}


