/*============================================================================*
 * FILE:       E X A M P L E _ B C _B R A N C H _ O N _ S T A S U S
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:    EXAMPLE PROGRAM
 *				This example program demonstrates the use of Conditional
 *				Branches. There are several option for branching. In this 
 *              example there are two branches used, CONDITIONAL BRANCH and
 *              CONDITIONAL BRANCH 3. CONDITIONAL BRANCH branches on data in the
 *              imeediately preceeding message in the bus list, either command
 *              word, status word or data. CONDITIONAL BRANCH 3 branches the 
 *              same values, but the message buffer is specified in the setup.
 *              
 *              In this example, 8 message buffers are created.  Message 
 *              1 is a CONDITIONAL BRANCH that branches when the Status Request
 *              bit (SRQ) is set in the status response on message 0, the 
 *              immediately preceeding message.  If the SRQ is set then Msg 2
 *              transact otherwise it is skipped.  
 *
 *              Message buffer 6 is a CONDITIONAL BRANCH 3 that branches if the
 *              data in the last data word of Msg 4 is 0xabcd. If the data is
 *              set to 0xabcd then Msg 7 transact, otherwise it is skipped.
 *
 *===========================================================================*/

/* $Revision: 8.12. Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
	6//	Initial version.  RSW
	8/3/	Minor cleanup, removed unnecessary BM init code.  RSW.
	3/5/	Removed conio.h and getc calls.  RSW.
    9//4    Added option to initialize with BusTools_API_LoadChannel.  RSW
*/

#include "busapi.h"
#include <stdio.h>

void setupTheBusController(BT_INT cardnum);
void setupThe_BC_IntFIFO(int cardnum);
void setupThe_BC_CtlIntFIFO(int cardnum);

API_INT_FIFO sIntFIFO1;
API_INT_FIFO sIntFIFO2;

// Main program
int main(int argc, char ** argv)
{
   BT_INT status, cardnum;
   BT_UINT device, mode, build;
   float wrev, lrev;

   if(argc == 2)
      device = atoi(argv[1]);
   else
      device = 0;

   // Open the device and get the channel id.
   mode = API_B_MODE | API_SW_INTERRUPT;  // 1553B protocol, use SW interrupts.
   status = BusTools_API_OpenChannel( &cardnum, mode, device, CHANNEL_1);
   printf("BusTools_API_OpenChannel status = %d\n",status);

   // Get firmware version information
   status = BusTools_GetFWRevision(cardnum, &wrev, &lrev, &build);
   if(status)
      printf("BusTools_GetFWRevision status = %d\n",status);

   printf("LPU Revision       = %f\n",lrev);
   printf("Microcode Revision = %f\n",wrev);
   if(BusTools_BoardIsV6(cardnum))
      printf("FPGA Build         = %x\n", build);  // Use hex to display the FPGA/LPU build number
   else
      printf("LPU Build          = %x\n");

   // Select External Bus.
   status = BusTools_SetInternalBus(cardnum, EXTERNAL_BUS);
   printf("BusTools_SetInternalBus status = %d\n",status);

   // Intialize the Bus Controller and setup the messages and branches
   setupTheBusController(cardnum);

   // Setup an interrupt callback to print BC data to the console.
   setupThe_BC_IntFIFO(cardnum);
   //setupThe_BC_CtlIntFIFO(cardnum);

   printf("\nHit Enter to start BC\n  Hit Enter again to stop the BC\n");
   getchar();
   fflush(stdin);

   // Start the Bus Controller
   status = BusTools_BC_StartStop(cardnum, BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);

   getchar();

   // Stop the Bus Controller
   status = BusTools_BC_StartStop(cardnum, BC_STOP);
   printf("BusTools_BC_StartStop status = %d\n",status);

   // Close API and board
   status = BusTools_API_Close(cardnum);
   printf("BusTools_API_Close status = %d\n",status);
		
} // End of main


/****************************************************************************
*
*  Function:  setupTheBusController
*
*  Description:
*     This function initialize the Bus Controller and creates the messages in
*     the frame.  The last message in the frame is a conditional branch on count.
*     When the count has not reached zero (0), the next message is the start of
*     the frame.  If the count reaches zero (0), the branch command is a
*     BC_CONTROL_HALT.
*
****************************************************************************/
void setupTheBusController(BT_INT cardnum)
{
   BT_INT status, messno;
   API_BC_MBUF bcmessage;

   // Initialize the BC.  
   status = BusTools_BC_Init( cardnum, 0, BT1553_INT_END_OF_MESS, 0, 14, 12, 1000000, 1 );
   printf("BusTools_BC_Init status = %d\n",status);

   // Allocate enough BC Buffer
   status = BusTools_BC_MessageAlloc(cardnum,50);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);

   // Create messages for The Minor Frame
   messno = 0; // Msg 0
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;
   bcmessage.mess_command1.rtaddr   = 1; 
   bcmessage.mess_command1.subaddr  = 1;
   bcmessage.mess_command1.wcount   = 1;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // inter-message gap in microseconds.
   bcmessage.data[0][0] = 1;
   bcmessage.data[0][1] = 1;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   // Setup deadman message branch on status of message 0.
   messno++;  // 1
   memset( (char*)&bcmessage, 0, sizeof(bcmessage) );  //Clear the bcmessage structure
   bcmessage.messno         = messno;
   bcmessage.messno_next    = (BT_U16BIT)(messno + 2); // Next message end-of-frame
   bcmessage.messno_branch  = (BT_U16BIT)(messno + 1); // Last message ID BC Halt
   bcmessage.control        = BC_CONTROL_CONDITION;    // Branch command on previous message data
   bcmessage.control       |= BC_CONTROL_INTERRUPT;    // Enable interrupt of this branch
   bcmessage.address        = API_BRANCH_MSTAT1;       // Use the status 
   bcmessage.data_mask      = 0x0100;                  // Apply mask to status word	
   bcmessage.data_value     = 0x0000;                  // Look for SRQ bit set (bit 8, 0x100)
      
   status = BusTools_BC_MessageWrite(cardnum, bcmessage.messno, &bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   // This message only transact if the SRQ bit is set by the RT in the response to message 0 
   messno++;  // Msg 2
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 2;
   bcmessage.mess_command1.subaddr  = 2;
   bcmessage.mess_command1.wcount   = 2;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;  // Msg 3
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 10;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.mess_command1.subaddr  = 10;
   bcmessage.mess_command1.wcount   = 10;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   // The data from this transmit command is the branch condition for Msg 6
   // If the last data word (zero based 10) is 0xabcd then the last message
   // In the frame transact.  Otherwise it is skipped.
   messno++;  // Msg 4
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 11;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.mess_command1.subaddr  = 11;
   bcmessage.mess_command1.wcount   = 11;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;  // Msg 5
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 20;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.mess_command1.subaddr  = 20;
   bcmessage.mess_command1.wcount   = 20;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                     // inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   // Setup a branch on data word 11 of message 4.
   messno++;  // Msg 6
   memset( (char*)&bcmessage, 0, sizeof(bcmessage) );  //Clear the bcmessage structure
   bcmessage.messno         = messno;
   bcmessage.messno_next    = (BT_U16BIT)(messno + 2); // Next message end-of-frame Msg 8
   bcmessage.messno_branch  = (BT_U16BIT)(messno + 1); // Branch message Msg 7
   bcmessage.control        = BC_CONTROL_CONDITION3;   // Branch command on defined message data
   bcmessage.messno_compare = 4;                       // Specify message buffer 4
   bcmessage.address        = API_BC_BRANCH_DATA + 10; // Branch on the last data work of message 4 (11 - 1) 
   bcmessage.data_mask      = 0xffff;                  // Apply mask to status word	
   bcmessage.data_value     = 0xabcd;                  // Look for SRQ bit set (bit 8, 0x100)
      
   status = BusTools_BC_MessageWrite(cardnum, bcmessage.messno, &bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   // The command only transact if the data value in the 11 data word is 0xabcd
   messno++;  // Msg 7
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 21;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.mess_command1.subaddr  = 21;
   bcmessage.mess_command1.wcount   = 21;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum, bcmessage.messno, &bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //==========================================================================
   // This is the bus list END-FRAME
   messno++; // Msg 8
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(0);      // Go back to start of the frame
   bcmessage.control =  BC_CONTROL_MFRAME_END;  // End-frame marker
   bcmessage.control |= BC_CONTROL_NOP;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);
} 

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   static int sa_index;
   int       status;
   BT_INT tail;
   BT_UINT messno;
   static int nFlag = 1;

   char outbuf[80];

   tail = sIntFIFO->tail_index;         // Get the last recorded tail pointer
   while(tail != sIntFIFO->head_index)
   {
      printf("_________________________________________________________\n");

      messno = sIntFIFO->fifo[tail].bufferID;  //  Get the messno                  
      status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);  // Read the BC message buffer
      if (status)
         return status;

      printf("%d - %x\n",messno,bcmessage.status);

      // Print time base on time tag format (microseconds or nanoseconds)
      if(BusTools_BoardIsV6(cardnum))
         BusTools_TimeGetFmtString(API_TTD_IRIG_NS, &bcmessage.time_tag, outbuf);
      else
         BusTools_TimeGetFmtString(API_TTD_IRIG, &bcmessage.time_tag, outbuf);
      printf("tag time = %s\n",outbuf);     

      // Print message details
      if (bcmessage.mess_command1.tran_rec)
         printf("BC**RT->BC (%d)  ", messno);
      else
         printf("BC->RT (%d)  ", messno);
      printf("BC**RT-%d  \n",bcmessage.mess_command1.rtaddr);
      printf("BC**SA-%d  \n",bcmessage.mess_command1.subaddr);
      printf("BC**WC-%d  \n",bcmessage.mess_command1.wcount);
      printf("BC**status-1 0x%04x\n",*((WORD*)(&bcmessage.mess_status1)));
      printf("BC**int_stat-0x%08lx\n",*((DWORD*)(&bcmessage.status)));
            
      tail++;                           // Index tail pointer
      tail &= sIntFIFO->mask_index;     // Wrap pointer if needed
      sIntFIFO->tail_index = tail;      // Save tail pointer 
   }

   return API_SUCCESS;
}

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_CtlIntFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   int status;
   BT_INT tail;
   BT_UINT messno;

   tail = sIntFIFO->tail_index;         // Get the last recorded tail pointer
   while(tail != sIntFIFO->head_index)
   {
      messno = sIntFIFO->fifo[tail].bufferID;  //  Get the messno   
               
      status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);  // Read the BC message buffer
      if (status)
         return status;

      printf("Branched to addr %x\n",bcmessage.next_branch_addr);
          
      tail++;                           // Index tail pointer
      tail &= sIntFIFO->mask_index;     // Wrap pointer if needed
      sIntFIFO->tail_index = tail;      // Save tail pointer 
   }

   return API_SUCCESS;
}


/****************************************************************************
*
*  Function:  setupThe_BC_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO for a interrupt on a BC message.
*     The interrupt is set for the end-of message.  
*
****************************************************************************/
void setupThe_BC_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.iNotification = 0;
   sIntFIFO1.function     = bc_intFunction;
   sIntFIFO1.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.bForceStartup  = 0;
   sIntFIFO1.FilterType     =  EVENT_BC_MESSAGE;

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO1.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}

/****************************************************************************
*
*  Function:  setupThe_BC_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO for a interrupt on a BC message.
*     The interrupt is set for the end-of message.  
*
****************************************************************************/
void setupThe_BC_CtlIntFIFO(int cardnum)
{
   int status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO2, 0, sizeof(API_INT_FIFO));
   sIntFIFO2.iNotification  = 0;
   sIntFIFO2.function       = bc_CtlIntFunction;
   sIntFIFO2.iPriority      = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO2.dwMilliseconds = INFINITE;
   sIntFIFO2.FilterType     =  EVENT_BC_CONTROL;

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO2, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}