
/*===========================================================================*
 * FILE:             E X M A P L E _ B C _ D A T A _ L O A D. C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    Demonstration of BusTools/1553-API routines. This 
 *              application shows to setup a data load scenario running off a 
 *              conditional branch. 
 *
 *              Conditional Branch 2 branches based on the value in an 
 *              on-board memory location allocated using BusTools_MemoryAlloc.
 *              The application writes walues to this location to control
 *              When the data transfer starts.  The transfer sends 10000 32-word
 *              messages to an external RT.  It also queries the RT to verify the
 *              correct data is sent.  after the 1000 message are sent the conditional
 *              branch memory location is clear and just the periodic message 
 *              transacts.
 *              
 *              Note: This example run on both the new V6 firmware and older 
 *              V4/V5 firmware.  In the new V6 firmware allocate 32 bit of 
 *              memory rather than 16 bits used in the older firmware.
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/

#include "busapi.h"					 
#include <stdio.h>

void setupThe_BC_IntFIFO(int cardnum);
int	             status,j;
int              messno,cardnum;
static BT_U32BIT messno_cond;
BT_U32BIT        mem_addr;
API_INT_FIFO     bcIntFIFO;
API_INT_FIFO     bcCtlIntFIFO;
API_BC_MBUF		 bcmessage;	         // bloc message .
BT_UINT load_count = 0;
BT_UINT error_count = 0;


void defineMessages(BT_INT cardnum)
{
   API_BC_MBUF bc_msg;
   BT_INT i;

   // Periodic message sent evry 50 milliseconds
   messno=0;
   memset(&bc_msg, 0, sizeof(bc_msg));           // Clear all fields.
   bc_msg.messno  = messno;		                 // Message number
   bc_msg.control = BC_CONTROL_MESSAGE;		     // BC message.
   bc_msg.control |= BC_CONTROL_BUSA;	         // Send message on bus A (primary).
   bc_msg.control |= BC_CONTROL_BUFFERA;	     // Only using one buffer, buffer A.
   bc_msg.control |= BC_CONTROL_MFRAME_BEG;      // start-of-Frame
   bc_msg.messno_next			 = messno+1;     // Next message number
   bc_msg.mess_command1.rtaddr	 = 4;		     // Command word 1, RT address
   bc_msg.mess_command1.subaddr  = 4;		     // Command word 1, Subaddress
   bc_msg.mess_command1.wcount   = 4;
   bc_msg.mess_command1.tran_rec = 0;		     // Command word 1, transmit (1) or receive (0)
   bc_msg.gap_time				 = 1000;         // Gap time in microseconds from start of Frame
   status = BusTools_BC_MessageWrite(cardnum,messno, &bc_msg);        //Write the BC Message to buffer
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); 

   // Conditional branch when true transfers data when false skip the next two messages
   messno++;
   memset(&bc_msg,0,sizeof (bc_msg));
   bc_msg.messno        = messno;
   bc_msg.control       = BC_CONTROL_CONDITION2;
   bc_msg.test_address  = mem_addr; 
   bc_msg.data_mask     = 0x2; 
   bc_msg.data_value    = 0x2;
   bc_msg.messno_next   = messno+3;
   bc_msg.messno_branch = messno+1;
   status = BusTools_BC_MessageWrite (cardnum,messno,&bc_msg);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); 

   // BC recevie command sends data to RT
   messno++;
   memset(&bc_msg, 0, sizeof(bc_msg));           // See Comments above
   bc_msg.messno  = messno;		              
   bc_msg.control = BC_CONTROL_MESSAGE;		  
   bc_msg.control |= BC_CONTROL_CHANNELA;	  
   bc_msg.control |= BC_CONTROL_BUFFERA;	 
   bc_msg.messno_next			  = messno+1; 
   bc_msg.mess_command1.rtaddr	  = 22;	
   bc_msg.mess_command1.subaddr   = 3;
   bc_msg.mess_command1.wcount    = 32;
   bc_msg.mess_command1.tran_rec  = RECEIVE;	
   bc_msg.gap_time				  = 1000;	 
   
   // Fill data buffer with initial data 
   for (i=0; i<32; i++)
      bc_msg.data[0][i] = 0x0000;

   status = BusTools_BC_MessageWrite(cardnum,messno, &bc_msg);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); 
   
   // BC Transmit command to verify the data is correct
   messno++;
   memset(&bc_msg, 0, sizeof(bc_msg));           // See Comments above
   bc_msg.messno  = messno;		                
   bc_msg.control = BC_CONTROL_MESSAGE;   
   bc_msg.control |= BC_CONTROL_CHANNELA;	   
   bc_msg.control |= BC_CONTROL_BUFFERA;	    
   bc_msg.control |= BC_CONTROL_INTERRUPT;
   bc_msg.messno_next			  = messno+1;
   bc_msg.mess_command1.rtaddr	  = 22;	
   bc_msg.mess_command1.subaddr   = 3;	
   bc_msg.mess_command1.wcount    = 32;
   bc_msg.mess_command1.tran_rec  = TRANSMIT;		
   bc_msg.gap_time				  = 1000;	
   status = BusTools_BC_MessageWrite(cardnum,messno, &bc_msg);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); ;
     
   // NOOP End of frame  last message in each frame
   messno++;
   memset((char*)&bc_msg,0,sizeof(bc_msg));
   bc_msg.messno = messno;                  // Message Number
   bc_msg.messno_next = (BT_U16BIT)(0);     // Start Next frame at message buffer 0
   bc_msg.control  = BC_CONTROL_NOP;        // NOOP Message
   bc_msg.control |= BC_CONTROL_MFRAME_END; // End-of-Frame
   bc_msg.long_gap = 1000; 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bc_msg);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status); 
}

void main(int argc, char** argv) 
{
   BT_UINT mode, mess_cond=0;
   BT_UINT device;
   BT_UINT cardnum;

   printf("Initializing API with BusTools_API_OpenChannel . . . ");

    // Open the device and get the card number.
   mode = API_B_MODE | API_SW_INTERRUPT;  // 1553B protocol with SW interrupts.

   if(argc == 2)                          
      device = atoi(argv[1]);            // Device number from command-line
   else
      device = 0;                        // Default to device 0

   status = BusTools_API_OpenChannel( &cardnum, mode, device, CHANNEL_1);  // Initialize select device/channel
   printf("BusTools_API_OpenChannel status = %d\n",status);

   // Select External Bus.
   status = BusTools_SetInternalBus(cardnum, EXTERNAL_BUS);
   printf("BusTools_SetInternalBus status = %d\n",status);

   // Initialize the BC with a 50 millisecond frame time and interupt on end-of-message
   status = BusTools_BC_Init(cardnum, REL_GAP, BT1553_INT_END_OF_MESS, 0,14,12,50000,1);
   printf("BusTools_BC_Init status = %d\n",status);

   status = BusTools_BC_MessageAlloc(cardnum, 10);  // Allocate 10 BC Message Buffers
   printf("BusTools_BC_MessageAlloc status = %d\n",status);
		
   //Set up the Condition Branch memory address
   messno_cond=0;
   if(BusTools_BoardIsV6(cardnum))
   {
      status = BusTools_MemoryAlloc32(cardnum,0,4,&mem_addr);                   // Use memory address for bit encoded branchning 
      printf("BusTools_MemoryAlloc status = %d\n",status);            
      status = BusTools_MemoryWrite2(cardnum,RAM32,mem_addr,4,&messno_cond);  // Clear this address
      printf("BusTools_MemoryWrite2 status = %d\n",status);
   }
   else
   {
      status = BusTools_MemoryAlloc(cardnum,0,2,&mem_addr);                   // Use memory address for bit encoded branchning 
      printf("BusTools_MemoryAlloc status = %d\n",status);            
      status = BusTools_MemoryWrite(cardnum,mem_addr,2,(BT_U16BIT *)(&messno_cond));  // Clear this address
      printf("BusTools_MemoryWrite2 status = %d\n",status);
   }

   // Define messages  
   defineMessages(cardnum);

   // setup the callback
   setupThe_BC_IntFIFO(cardnum);  

   status = BusTools_BC_StartStop(cardnum, BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);

   // BC is running, wait for user to tell us to stop or read data.
   printf("\n Enter:\n 2 to start transfer \n 1 to dump memory \n 0 to quit\n : ");
			
   do      
   {
      scanf("%x",&messno_cond);
      printf("the message value is %d\n",messno_cond);

      switch(messno_cond)
      {
	  case 1://dum memory
         BusTools_DumpMemory(cardnum,DUMP_ALL,"Branch_on_data_dump.txt","Branch On Data");
         break;
	  case 2://Start branch
         if(BusTools_BoardIsV6(cardnum))
            BusTools_MemoryWrite2(cardnum,RAM32,mem_addr,4,&messno_cond);  // Write the input data to the branch address
         else
            BusTools_MemoryWrite(cardnum,mem_addr,2,(BT_U16BIT *)&messno_cond);  // Write the input data to the branch address
         fflush(stdin);
         break;
	  default:
		  continue;
	  }
   }while(messno_cond>0);

   // Stop the BC.
   status = BusTools_BC_StartStop(cardnum, BC_STOP);
   printf("BusTools_BC_StartStop status = %d\n",status);

   status = BusTools_RegisterFunction(cardnum,&bcIntFIFO,UNREGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);

   // We're done.  Close API and board
   status = BusTools_API_Close(cardnum);
   printf("BusTools_API_Close status = %d\n",status);
} // End of main

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoked when the a Bus
*     Controller message issues an interrupt. It prints the RT address and clears
*     The conditional bit value associated with that RT address in the allocated
*     memory location used by the Conditional Branch messages.
*
****************************************************************************/
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   BT_INT tail,bindex;
   BT_UINT messno;
   static BT_U16BIT new_buffer[32] = {0x0,};

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      messno = sIntFIFO->fifo[tail].bufferID;
      status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);
      if(bcmessage.status & BT1553_INT_ERROR_BITS) // Check for transaction errors
	  {
	     error_count++;
	  }  
	  else
	  {
	     for(bindex=0;bindex<32;bindex++) // Test that the data is correct
	     {
            if(bcmessage.data[0][bindex] != new_buffer[bindex])
            {
               error_count++;
			   break;
			}
			else
			{
	           if(++load_count == 10000)  // Transfer 10000 messages
	           {
                  messno_cond = 0;
                  if(BusTools_BoardIsV6(cardnum))
                     BusTools_MemoryWrite2(cardnum,RAM32,mem_addr,4,&messno_cond);  //Clear Branch address
                  else
                     BusTools_MemoryWrite(cardnum,mem_addr,2,&messno_cond);         //Clear Branch address 
	           }
	           else  // update the BC message buffer with the new data
	           {
                  for(bindex=0;bindex<32;bindex++)
 	                 new_buffer[bindex]++;
                  BusTools_BC_MessageUpdateBuffer(cardnum,2,0,new_buffer);  // write the data to the BC receive command
	           }
			}
		 }
	  }
      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }

   return API_SUCCESS;
}

/****************************************************************************
*
*  Function:  setupThe_BC_IntFIFO
*
*  Description:
*     This routine sets up the interrupt FIFO for a interrupt on a BC message.  
* 
****************************************************************************/
void setupThe_BC_IntFIFO(int cardnum)
{
   int status;
   // Setup the FIFO structure for this board.
   memset(&bcIntFIFO, 0, sizeof(bcIntFIFO));;
   bcIntFIFO.function     = bc_intFunction;
   bcIntFIFO.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   bcIntFIFO.dwMilliseconds = INFINITE;
   bcIntFIFO.FilterType     =  EVENT_BC_MESSAGE;

   // interrpt only on RT22 SA3 TX commnad
   bcIntFIFO.FilterMask[22][1][3] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &bcIntFIFO, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}

