/*===========================================================================*
 * FILE:         E X A M P L E _ B C _ D A T A _ W R A P. C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    Demonstration of BusTools/1553-API routines. This example shows
 *              how to wrap transmit and receive buffer so that data from a BC
 *              transits command can be used for the data on BC receive command. 
 *              In this example the data buffer from the first message (0) is 
 *              linked to the second message (1) by reading the data buffer address
 *              in buffer 0, the transmit command, and over-writing the data buffer
 *              address in the second buffer (1) with the transmit buffer address.
 *              That way the data from the transmit command is used by the 
 *              receive command.
 *
 *              Once you wrap the data buffer the Bus Controller read functions are
 *              no able to completely read the receive command.  This is because
 *              the linked data buffer points back to the receive message.  If you
 *              want to verify the transmit transaction use a bus monitor to
 *              process the messages on the bus.  In this example a BM is setup to
 *              monitor for the RT2 SA2 RX command that is linked with the TX data.
 *===========================================================================*/

/* $Revision:  8.13 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include "busapi.h"

//forward declarations
BT_INT BusController_Init(int,int);
void setupThe_BM_IntFIFO(int);

API_INT_FIFO  sIntFIFO1;          // Thread FIFO structure for BC
API_INT_FIFO  sIntFIFO2;          // Thread FIFO for branch
BT_INT KILL_THE_BC = 0;           // activate deadman timer
BT_U32BIT m_msgAddr1, m_msgAddr2, bc_dataAddr;

//******************************************************************************
//*                                                                            *
//******************************************************************************
int main(int argc, char **argv)                     
{
   BT_UINT cardnum, actual;
   BT_INT status,sa,tr,rt;
   BT_UINT mode=3;
   BT_INT device = 0; 
   API_BM_CBUF control_buffer;                  
   char ichar;

   if(argc == 2)
      device = atoi(argv[1]);             // Read command line device input

   mode = API_B_MODE | API_SW_INTERRUPT;  // 1553B protocol, use SW interrupts.
   status = BusTools_API_OpenChannel( &cardnum, mode, device, CHANNEL_1);
   printf("BusTools_API_OpenChannel status=%d\n", status, BusTools_StatusGetString(status));

   // Set the Bus to External for BC
   status = BusTools_SetInternalBus(cardnum,EXTERNAL_BUS);  // Set up External bus
   printf("BusTools_SetInternalBus BC status = %d (%s)\n",status, BusTools_StatusGetString(status));

   // Set the time mode for both channels
   status = BusTools_TimeTagMode(cardnum, API_TTD_IRIG_NS, API_TTI_IRIG, API_TTM_FREE, NULL, 0, 0, 0 );
   printf("BusTools_TimeTagMode BC status = %d (%s)\n",status, BusTools_StatusGetString(status));

   // Initial the BM 
   BusTools_BM_Init(cardnum,BM_ENABLE_BUS ,BM_ENABLE_BUS);

   // Set up the BC for Trigger-User mode
   status = BusController_Init(cardnum,0);

   // Allocate BM message buffer region with interrupt on end-of-message
   BusTools_BM_MessageAlloc(cardnum, 50, &actual, BT1553_INT_END_OF_MESS);

   // Disable monitoring of ALL RT/SA/TR combinations
   for (rt=0; rt<32; rt++)
   {
      for (sa=0; sa<32; sa++)
      {
         for (tr=0; tr<2; tr++)
         {
            control_buffer.t.wcount   = 0x00000000;  // NO word counts accepted.
            control_buffer.pass_count = 1;

            status = BusTools_BM_FilterWrite(cardnum, rt, sa, tr, &control_buffer);
            if(status)
               printf("BusTools_BM_FilterWrite status %d\n",status);
         }
      }
   }

   // Enable monitoring for our RT/SA of interest, RT2 SA2 RECEIVE.
   control_buffer.t.wcount   = 0xffffffff;  // Word counts 2.
   control_buffer.pass_count = 1; 
   rt=2; sa=2; tr=0;
   status = BusTools_BM_FilterWrite(cardnum, rt, sa, tr, &control_buffer);
   printf("BusTools_BM_FilterWrite = %d\n",status);

   // Setting up a callback function to show the BM data
   setupThe_BM_IntFIFO(cardnum);   // Sets up BM message interrupt

   printf("Hit Enter to start BC.\n");
   getchar();
   fflush(stdin);

   // Start the BM.  
   status = BusTools_BM_StartStop(cardnum,BM_START);
   printf("BusTools_BM_StartStop status = %d\n",status);

   // Start the BC. 
   status = BusTools_BC_StartStop(cardnum,BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);

   do
   {
      ichar = getchar();
      switch(ichar)
      {
         case 'd':
         case 'D':
            BusTools_DumpMemory(cardnum,DUMP_ALL,"bc_trigger_dump.txt","exmaple_bc_trigger-user Dumping Memory");
            break;
      }
   }while (ichar != 'q' && ichar != 'Q');

   printf("main: closing API...\n");

   // Stop the bus controller
   status = BusTools_BC_StartStop(cardnum,BC_STOP); 
   printf("BusTools_BC_StartStop status = %d\n",status);

   status = BusTools_BM_StartStop(cardnum,BM_STOP); 
   printf("BusTools_BM_StartStop status = %d\n",status);

   // Unregister BC Message callback function
   status = BusTools_RegisterFunction(cardnum,&sIntFIFO1,UNREGISTER_FUNCTION);
   printf("unregistering via BusTools_RegisterFunction status = %d\n",status);

   // Close the channel
   status = BusTools_API_Close(cardnum);
   printf("BusTools_API_Close BC status = %d\n",status);
}


/****************************************************************************
*
*  Function:  BusController_Init
*
*  Description:
*     This function initialize the Bus Controller and creates the messages in
*     the frame.  The last message in the frame is a conditional branch on count.
*     When the count has not reached zero (0), the next message is the start of
*     the frame.  If the count reaches zero (0), the branch command is a
*     BC_CONTROL_HALT.
*
****************************************************************************/
int BusController_Init(int cardnum,int sa_index)
{
   BT_INT status, messno;
   API_BC_MBUF bcmessage;

   // Initialize the BC.  
   status = BusTools_BC_Init( cardnum, 0, BT1553_INT_END_OF_MESS, 0, 14, 12, 1000000, 1 );
   printf("BusTools_BC_Init status = %d\n",status);

   // Allocate enough BC Buffer
   status = BusTools_BC_MessageAlloc(cardnum,50);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);

   // Create messages for The Minor Frame
   messno = 0; // Msg 0                          // This command requests 32 words of data from the RT
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
//   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;
   bcmessage.mess_command1.rtaddr   = 1; 
   bcmessage.mess_command1.subaddr  = 1;
   bcmessage.mess_command1.wcount   = 32;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 10;                       // inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);
 
   messno++;  // Msg 1                            // This command will send to 32 data words received from RT1 SA1 
                                                  // to RT2 SA2 
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 2;
   bcmessage.mess_command1.subaddr  = 2;
   bcmessage.mess_command1.wcount   = 32;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 10;                      // inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;  // Msg 2
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
//   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 10;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.mess_command1.subaddr  = 10;
   bcmessage.mess_command1.wcount   = 10;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;  // Msg 3
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
//   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 11;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.mess_command1.subaddr  = 11;
   bcmessage.mess_command1.wcount   = 11;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;  // 4
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
//   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 20;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.mess_command1.subaddr  = 20;
   bcmessage.mess_command1.wcount   = 20;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                     // inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;  // 5
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
//   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 21;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.mess_command1.subaddr  = 21;
   bcmessage.mess_command1.wcount   = 21;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum, bcmessage.messno, &bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   // This is the bus list END-FRAME
   messno++; //7
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(0);      // Go back to start of the frame
   bcmessage.control =  BC_CONTROL_MFRAME_END;  // End-frame marker
   bcmessage.control |= BC_CONTROL_NOP;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   // Get the address of the BC BC Message buffer 0 and 1
   status = BusTools_BC_MessageGetaddr(cardnum, 0, &m_msgAddr1);  // Message Buffer 0 address
   status = BusTools_BC_MessageGetaddr(cardnum, 1, &m_msgAddr2);  // Message Buffer 1 address
  
   if(BusTools_BoardIsV6(cardnum))  // If using V6 firmware get address of the data buffer
   {
      status = BusTools_MemoryRead2(cardnum,RAM32,m_msgAddr1 + BC_MSG_DATA_OFFSET,4,&bc_dataAddr);
      printf("Data Buffer Address 0x%08x\n", bc_dataAddr);  // Print the address of the data buffer
      status = BusTools_MemoryWrite2(cardnum,RAM32,m_msgAddr2 + BC_MSG_DATA_OFFSET,4,&bc_dataAddr);
   }
   else
   {
      status = BusTools_MemoryRead(cardnum, (m_msgAddr1 + (4*2)), 2, &bc_dataAddr);
      printf("Data Buffer Address 0x%08x\n", bc_dataAddr);  // Print the address of the data buffer 
      status = BusTools_MemoryWrite(cardnum, (m_msgAddr2 + (4*2)), 2, &bc_dataAddr); 
   }

   return status;
} 


/****************************************************************************
*
*  Function:  bm_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message on a Bus Monitor  message. This function read the data 
*     in the BM message and prints message data to the screen.
****************************************************************************/
BT_INT _stdcall bm_intFunction(BT_UINT ch_id, struct api_int_fifo *sIntFIFO)
{
   BT_INT         tail, wcnt, status, i, messno;
   API_BM_MBUF    mbuf;
   char           outbuf[80];

   // Loop through all entries in the FIFO
   // Fetch entries from the FIFO: Get the tail pointer and extract the entry
   // it points to.   When (head == tail) then FIFO is empty.
   tail = sIntFIFO->tail_index;
   while ( tail != sIntFIFO->head_index )
   {
      //  Process a BM interrupt:
      //sIntFIFO->fifo[tail].buffer_off       // Byte address of buffer
      //sIntFIFO->fifo[tail].rtaddress        // RT address
      //sIntFIFO->fifo[tail].transrec         // Transmit/Receive
      //sIntFIFO->fifo[tail].subaddress       // Subaddress number
      //sIntFIFO->fifo[tail].wordcount        // Word count
      //sIntFIFO->fifo[tail].bufferID         // BM message number

      messno = sIntFIFO->fifo[tail].bufferID;

      // Read the message from the board.
      status = BusTools_BM_MessageRead(ch_id, messno, &mbuf);

      //print out the BM data.  NOTE: FOR EXAMPLE ONLY 
      // printing not recommended in user-callback functions.
		

      BusTools_TimeGetString(&mbuf.time, outbuf);

      printf("BM>>>>> -- time tag = %s\n",outbuf);

      if(mbuf.int_status & BT1553_INT_RETRY)
         printf("Retry - %d\n",messno);

      if (mbuf.int_status & BT1553_INT_BROADCAST)
         printf("Broadcast - %d\n",messno);

      if(mbuf.int_status & BT1553_INT_RT_RT_FORMAT)
         printf("BM>>>>> RT<->RT (%d) \n",mbuf.messno);
      else if(mbuf.int_status & BT1553_INT_MODE_CODE)  
         printf("BM>>>>> Mode Code (%d) \n",mbuf.messno);
      else if (mbuf.command1.tran_rec)
         printf("BM>>>>> RT->BC (%d)  \n", mbuf.messno);
      else
         printf("BM>>>>> BC->RT (%d)  \n", mbuf.messno);
          
      printf("BM>>>>> RT-%d  \n",mbuf.command1.rtaddr);
      printf("BM>>>>> SA-%d  \n",mbuf.command1.subaddr);
      printf("BM>>>>> WC%d  \n",mbuf.command1.wcount);
      printf("BM>>>>> ST1 %4X \n",*(BT_U16BIT *)&mbuf.status1);
 
      if(mbuf.command1.wcount == 0)
         wcnt = 32;
      else
         wcnt = mbuf.command1.wcount;

      printf("BM>>>>> 0x%08lx\n",mbuf.int_status);
      for(i=0;i<wcnt; i++)
      {
         if(i%8==0 && i>0)
            printf("\n");
         printf("%04X ",mbuf.value[i]);
      }
      printf("\n\n");

      // Now update and store the tail pointer.
      tail++;                         // Next entry
      tail &= sIntFIFO->mask_index;   // Wrap the index
      sIntFIFO->tail_index = tail;    // Save the index
   }
   return( API_SUCCESS );
}

/****************************************************************************
*
*  Function:  setupThe_BM_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the values needed for a 
*     interrupt on a BC messages.  The interrupt is set for the end-of message.  
*
****************************************************************************/
void setupThe_BM_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.iNotification = 0;
   sIntFIFO1.function     = bm_intFunction;
   sIntFIFO1.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.bForceStartup  = 0;
   sIntFIFO1.FilterType     =  EVENT_BM_MESSAGE;

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO1.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}


