/*===========================================================================*
 * FILE:           E X A M P L E _ B C _ J O I N . C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    Demonstration of BusTools/1553-API routines. Channel Sharing
 *              allows mulitple applications to run on a single 1553 channel. 
 *              Channel sharing requires that only one application intialize a
 *              channel.  That application must share the channel by calling 
 *              BusTools_API_ShareChannel.  Other application can join the 
 *              shared channel by calling BusTools_API_JoinChannel. There can be
 *              only one Bus Controller application, one Bus Monitor application
 *              and one Remote Terminal application per channel.
 *
 *              This example Shows a BC application joining an already 
 *              initialized channel.
 *
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <conio.h>
#include "busapi.h"

int BusController_Init(int,int);
void setupThe_BC_IntFIFO(int);

API_INT_FIFO  sIntFIFO1;     // Thread FIFO structure for BC

#if defined (__WIN32__)
void main(int argc, char **argv)                     
#else
int tstall()
#endif
{
   int status,cardnum;
   int help=0;
   unsigned channel,device,mode;
   char ichar;

   channel = 0;  //default channel 1
   device = 0;   //default device 0


   status = BusTools_API_JoinChannel(&cardnum,device,CHANNEL_1);
   printf("BusTools_API_JoinChannel %d status = %d\n",cardnum,status);

   status = BusTools_SetInternalBus(cardnum,EXTERNAL_BUS);  // Set up bus 0 External or  1 Internal
   printf("BusTools_SetInternalBus status = %d\n",status);
 
   status = BusTools_TimeTagMode(cardnum, API_TTD_RELM, API_TTI_ZERO, API_TTM_FREE, 0, 0, 0, 0);
   printf("BusTools_TimeTagMode status = %d\n",status);
  
   status = BusController_Init(cardnum,50);

   setupThe_BC_IntFIFO(cardnum);

   status = BusTools_BC_StartStop(cardnum,BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);

   while((ichar=getc(stdin))!='q'){};


   printf("main: closing API\n");

   status = BusTools_BC_StartStop(cardnum,BC_STOP); 
   printf("BusTools_BC_StartStop status = %d\n",status);

   status = BusTools_RegisterFunction(cardnum,&sIntFIFO1,0);

   status = BusTools_API_QuitChannel(cardnum, BC_QUIT);//
   printf("BusTools_API_Close status = %d\n",status);

}

int BusController_Init(int cardnum,int num_buffers)
{
   int status, messno,i;
   API_BC_MBUF bcmessage;

   status = BusTools_BC_Init(cardnum,MSG_SCHD,BT1553_INT_END_OF_MESS,0,16,14,500000,1);
   printf("BusTools_BC_Init status = %d\n",status);

   status = BusTools_BC_MessageAlloc(cardnum,50);//num_buffers);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);

   messno = 0;
   memset((char*)&bcmessage,0,sizeof(bcmessage));       
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;   // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_MFRAME_BEG; 
   bcmessage.mess_command1.rtaddr   = 2;
   bcmessage.mess_command1.subaddr  = 2;
   bcmessage.mess_command1.wcount   = 2;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.start_frame = 1;
   bcmessage.rep_rate = 1;
   bcmessage.errorid = 0;     // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;    // 15 microsecond inter-message gap.

   for(i=0;i<32;i++)
       bcmessage.data[0][i] = 0xA000+i;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;   // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_INTERRUPT;

   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 4;
   bcmessage.mess_command1.wcount   = 4;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.start_frame = 1;
   bcmessage.rep_rate = 1;
   bcmessage.errorid = 0;     // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;    // 15 microsecond inter-message gap.q
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;   // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 3;
   bcmessage.mess_command1.subaddr  = 3;
   bcmessage.mess_command1.wcount   = 3;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.start_frame = 1;
   bcmessage.rep_rate = 2;
   bcmessage.errorid = 0;     // Default error injection buffer (no errors)
   bcmessage.gap_time = 8;    // 8 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;   // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 4;
   bcmessage.mess_command1.wcount   = 4;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.start_frame = 1;
   bcmessage.rep_rate = 3;
   bcmessage.errorid = 0;     // Default error injection buffer (no errors)
   bcmessage.gap_time = 8;    // 8 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;   // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_INTERRUPT;

   bcmessage.mess_command1.rtaddr   = 5;
   bcmessage.mess_command1.subaddr  = 5;
   bcmessage.mess_command1.wcount   = 5;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;     // Default error injection buffer (no errors)
   bcmessage.gap_time = 8;    // 8 microsecond inter-message gap.
   bcmessage.start_frame = 4;
   bcmessage.rep_rate = 4;
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;   // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_MFRAME_END;
   bcmessage.mess_command1.rtaddr   = 6;
   bcmessage.mess_command1.subaddr  = 0;
   bcmessage.mess_command1.wcount   = 17;  
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;     // Default error injection buffer (no errors)
   bcmessage.gap_time = 12;   // 12 microsecond inter-message gap.
   bcmessage.data[0][0] = 0xABCD;
   bcmessage.start_frame = 1;
   bcmessage.rep_rate = 4;
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)0;
   bcmessage.control = BC_CONTROL_NOP;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   return status; 
} 

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   int       bufnum;
   static int sa_index;
   int       i,j;
   int       status;
   int       wcount;
   BT_INT tail, rtrt,mc;
   BT_UINT messno;
   static int nFlag = 1;

   char outbuf[80];

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
	   if(sIntFIFO->fifo[tail].event_type & EVENT_BC_MESSAGE)
	   {
		  messno = sIntFIFO->fifo[tail].bufferID;
          status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);
     
          if (status)
             return status;
          rtrt=0;
          mc=0;
          if(bcmessage.status & BT1553_INT_RETRY)
            printf("Retry - %d - %x\n",messno,bcmessage.status);
          else
            printf("%d - %x\n",messno,bcmessage.status);


         BusTools_TimeGetString(&bcmessage.time_tag, outbuf);
         printf("tag time = %s\n",outbuf);
       

          if(bcmessage.status & BT1553_INT_RT_RT_FORMAT)
          {
            printf("RT<->RT (%d) ",messno);
             rtrt=1;
          }
          else if(bcmessage.status & BT1553_INT_MODE_CODE)
          {
             printf("Mode Code --  (%d) ", messno);
             mc=1;
          }
          else if (bcmessage.mess_command1.tran_rec)
             printf("BC**RT->BC (%d)  ", messno);
          else
             printf("BC->RT (%d)  ", messno);
 
          printf("BC**RT-%d  \n",bcmessage.mess_command1.rtaddr);
          printf("BC**SA-%d  \n",bcmessage.mess_command1.subaddr);
          printf("BC**WC-%d  \n",bcmessage.mess_command1.wcount);
          if(rtrt)
          {   
             printf("BC**RT-%d  \n",bcmessage.mess_command2.rtaddr);
             printf("BC**SA-%d  \n",bcmessage.mess_command2.subaddr);
             printf("BC**WC-%d  \n",bcmessage.mess_command2.wcount);
          }
        
          printf("BC**status-1 0x%04x\n",*((WORD*)(&bcmessage.mess_status1)));
          if(rtrt)
             printf("BC**status-2 0x%04x\n",*((WORD*)(&bcmessage.mess_status2)));
          
          printf("BC**int_stat-0x%08lx\n",*((DWORD*)(&bcmessage.status)));
          if(bcmessage.status & BT1553_INT_RETRY)
             printf("XXXXXXXXXXXXXXXXXX RETRY %x!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n",bcmessage.mess_status1);

          wcount = bcmessage.mess_command1.wcount;
          if (wcount == 0)
             wcount = 32;           // Zero word count means 32 words.

          if(mc)
            if(wcount >=16)
               wcount = 1;

          j = wcount;               // See if this msg shorted than previous
    
          if (bcmessage.control & BC_CONTROL_BUFFERA)
             bufnum = 0;            // Display data from first BC buffer
          else
             bufnum = 1;            // Display data from second BC buffer

          for ( i = 0; i < j; i++ ) // Display all of the data words
          {
             // If this is an RT->BC message, and there is a Message Error
             //  or No Response, than xxxx the data (since the RT did not
             //    send any data words).
             if ( i >= wcount )
             {
                printf("    ");     // Data does not apply to this msg
             }
             else if ( bcmessage.mess_command1.tran_rec &&
               (bcmessage.mess_status1.me || (bcmessage.status&BT1553_INT_NO_RESP) ) )
             {
                printf("xxxx ");      // No data was transfered because of error
             }
             else
             {
                printf("BC**%04x ",bcmessage.data[bufnum][i]);
             }
          }
          printf("\n");

      }
   
            
       tail++;
       tail &= sIntFIFO->mask_index;
       sIntFIFO->tail_index = tail;
   }

   return 0;
}

/****************************************************************************
*
*  Function:  setupThe_BC_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a BC message.  The interrupt is set for only the end-of
*     message.  
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupThe_BC_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.iNotification = 0;
   sIntFIFO1.function     = bc_intFunction;
   sIntFIFO1.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.bForceStartup  = 0;
   sIntFIFO1.FilterType     =  EVENT_BC_MESSAGE;

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO1.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, 1);
   printf("BusTools_RegisterFunction status = %d\n",status);
}


