/*============================================================================*
 * FILE:                 E X A M P L E _ B C _ M S G _ T Y P E S . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION: This example program shows how to program the Bus Controller to 
 *  run a multiple minor frames and to demonstrate programming all of the message 
 *  types available to the Bus Controller. The example initializes the channel and 
 *  sets it for external bus.  Following those initialization steps it initializes 
 *  the Bus Controller and builds two minor frames.  There are examples of each of 
 *  the BC message in the frames.
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
	05/29/01	Initial version.  
	08/13/01	Minor cleanup, removed unnecessary BM init code.  
	03/15/02	Removed conio.h and getc calls.
    09/20/04    Added option to initialize with BusTools_API_LoadChannel.
    05/18/10    re-designed to show traditional frame implementation.
*/

#include "busapi.h"
#include <stdio.h>

void bc_init(int cardnum);
void setupThe_BC_IntFIFO(int cardnum);
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO);

API_INT_FIFO sIntFIFO1;

// Main program
int main(int argc, char **argv) 
{
   BT_INT status,cardnum,noop=0;
   char c;
   BT_INT dev_num, mode;

   printf("Initializing API with BusTools_API_OpenChannel . . . ");
 
   //Set the device ID 
   if(argc == 2)
      dev_num = atoi(argv[1]);
   else
      dev_num = 0;

   // Open the device and get the channel handle.
   mode = API_B_MODE | API_SW_INTERRUPT;                      // 1553B protocol using SW interrupts.
   status = BusTools_API_OpenChannel( &cardnum, mode, dev_num, CHANNEL_1);
   printf("BusTools_API_OpenChannel status = %d\n",status);

   BusTools_TimeTagMode(cardnum,API_TTD_DATE,API_TTI_IRIG,API_TTM_FREE,0,0,0,0);
   printf("BusTools_TimeTagMode status = %d\n",status);

   // Select External Bus.
   status = BusTools_SetInternalBus(cardnum, EXTERNAL_BUS);
   printf("BusTools_SetInternalBus status = %d\n",status);

   // Enable broadcast message 
   status = BusTools_SetBroadcast(cardnum,BROADCAST);     
   printf("BusTools_SetBroadcast status = %d\n",status);
        
   bc_init(cardnum);
   setupThe_BC_IntFIFO(cardnum);

   printf("\nHit Enter to start the BC\n");            
   printf("type Q/q to Quit\n");
   c = getc(stdin);
   fflush(stdin);

   //Start the bus controller.
   status = BusTools_BC_StartStop(cardnum, BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);

   // BC is up and running, wait for input to NOOP, UN-NOOP, or QUIT.
   do
   {
      c = getchar();
      fflush(stdin);
   } while (c != 'Q' && c != 'q'); // End of do-while

   // Stop the BC.
   status = BusTools_BC_StartStop(cardnum, BC_STOP);
   printf("BusTools_BC_StartStop status = %d\n",status);

   // Close API and board
   status = BusTools_API_Close(cardnum);
   printf("BusTools_API_Close status = %d\n",status);

   printf("FINISHED.\n");
} // End of main

void bc_init(int cardnum)
{
   int status, messno,i;
   API_BC_MBUF bcmessage;

   // Initialize BC for no interrupts, no retries, no resp at 16us, late resp at 14us,
   // minor frame time of 500000us, 1 data buffer per message.
   status = BusTools_BC_Init(cardnum, 0, BT1553_INT_END_OF_MESS, 0, 16, 14, 1000000, 1);
   printf("BusTools_BC_Init status = %d\n",status);
   
   // Allocate memory on the board for 30 message buffers.
   // Only three are used in this example.
   status = BusTools_BC_MessageAlloc(cardnum, 30);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);
		
   // Write the Bus Controllere messages to the allocated message buffers.
   // Start of the first frame
   // RECEIVE Command RT2 SA2 WC2 RX  
   messno = 0;                                     // Set the message buffer index to the first buffer
   memset((char*)&bcmessage,0,sizeof(bcmessage));  // Clear out the message buffer data       
   bcmessage.messno = messno;                      // Current buffer index
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);// Link to next message buffer
   bcmessage.control = BC_CONTROL_MESSAGE;         // Set as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;        // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;           // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;       // Enable interrupt on this message
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;     // Start of the frame
   bcmessage.mess_command1.rtaddr   = 2;           // RT Address 2
   bcmessage.mess_command1.subaddr  = 2;           // Subassress 2
   bcmessage.mess_command1.wcount   = 2;           // Word count 2
   bcmessage.mess_command1.tran_rec = 0;           // receive command.
   bcmessage.errorid = 0;                          // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                        // 15 microsecond inter-message gap_time.
   // Fill in the data for the receive command
   for(i=0;i<2;i++)
       bcmessage.data[0][i] = 0xA000+i;            // Add data for the specified number of data elements
   // Write the message buffer data to the buffer
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //TRANMIT Command RT3 SA3 WC3 TX  
   messno++;                                       // Index to the next message buffer                                             
   memset((char*)&bcmessage,0,sizeof(bcmessage));  // Clear out the message buffer data
   bcmessage.messno = messno;                      // Current buffer index
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);// Link to next message buffer
   bcmessage.control = BC_CONTROL_MESSAGE;         // Set as a NOP message
   bcmessage.control |= BC_CONTROL_BUFFERA;        // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSB;           // Transmit on Bus B
   bcmessage.control |= BC_CONTROL_INTERRUPT;       // Enable interrupt on this messageT
   bcmessage.mess_command1.rtaddr   = 3;           // RT Address 3
   bcmessage.mess_command1.subaddr  = 3;           // Subaddress 3
   bcmessage.mess_command1.wcount   = 3;           // Word count 3
   bcmessage.mess_command1.tran_rec = 1;           // Transmit command 
   bcmessage.errorid = 0;                          // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                        // 15 microsecond inter-message gap_time
   // Write the message buffer data to the buffer 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Mode Code - No Data RT4 SA0 MC2 TX  -- Transmit Status Word 
   messno++;                                       // Index to the next message buffer                                             
   memset((char*)&bcmessage,0,sizeof(bcmessage));  // Clear out the message buffer data
   bcmessage.messno = messno;                      // Current buffer index
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);// Link to next message buffer
   bcmessage.control = BC_CONTROL_MESSAGE;         // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;        // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;           // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;       // Enable interrupt on this message
   bcmessage.mess_command1.rtaddr   = 4;           // RT address 4
   bcmessage.mess_command1.subaddr  = 0;           // Subaddress 0 - Mode Code
   bcmessage.mess_command1.wcount   = 4;           // Word count 4 - Transmit Status Word
   bcmessage.mess_command1.tran_rec = 1;           // Transmit command
   bcmessage.errorid = 0;                          // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                        // 15 microsecond inter-message gap_time.
   // Write the message buffer data to the buffer
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Mode Code - With Data RT4 SA0 MC16 TX  -- Transmit Vector Word 
   messno++;                                       // Index to the next message buffer                                             
   memset((char*)&bcmessage,0,sizeof(bcmessage));  // Clear out the message buffer data
   bcmessage.messno = messno;                      // Current buffer index
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);// Link to next message buffer
   bcmessage.control = BC_CONTROL_MESSAGE;         // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;        // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;           // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;       // Enable interrupt on this message
   bcmessage.mess_command1.rtaddr   = 4;           // RT address 4
   bcmessage.mess_command1.subaddr  = 0;           // Subaddress 0 - Mode Code
   bcmessage.mess_command1.wcount   = 16;          // Word count 4 - Transmit Vector Word
   bcmessage.mess_command1.tran_rec = 1;           // Transmit command
   bcmessage.errorid = 0;                          // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                        // 15 microsecond inter-message gap_time.
   // Write the message buffer data to the buffer
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Mode Code - With Data RT4 SA0 MC17 RX  -- Synchronize with data
   messno++;                                       // Index to the next message buffer                                             
   memset((char*)&bcmessage,0,sizeof(bcmessage));  // Clear out the message buffer data
   bcmessage.messno = messno;                      // Current buffer index
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);// Link to next message buffer
   bcmessage.control = BC_CONTROL_MESSAGE;         // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;        // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;           // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;       // Enable interrupt on this message
   bcmessage.mess_command1.rtaddr   = 4;           // RT address 4  
   bcmessage.mess_command1.subaddr  = 0;           // Subaddress 0  - Mode Code
   bcmessage.mess_command1.wcount   = 17;          // Word count 4  - Transmit Status Word
   bcmessage.mess_command1.tran_rec = 0;           // Transmit command
   bcmessage.errorid = 0;                          // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                        // 15 microsecond inter-message gap_time.

   bcmessage.data[0][1] = 0xABCD;                  // Fill in the data for the synchronize mode code

   // Write the message buffer data to the buffer
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Broadcast Mode Code - No Data RT31 SA0 MC1 TX  -- Synchronize  
   messno++;                                       // Index to the next message buffer                                             
   memset((char*)&bcmessage,0,sizeof(bcmessage));  // Clear out the message buffer data
   bcmessage.messno = messno;                      // Current buffer index
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);// Link to next message buffer
   bcmessage.control = BC_CONTROL_MESSAGE;         // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;        // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;           // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;       // Enable interrupt on this message
   bcmessage.mess_command1.rtaddr   = 31;          // RT address 31 - broadcast
   bcmessage.mess_command1.subaddr  = 0;           // Subaddress 0  - Mode Code
   bcmessage.mess_command1.wcount   = 1;           // Word count 4  - Synchronize
   bcmessage.mess_command1.tran_rec = 1;           // Transmit command
   bcmessage.errorid = 0;                          // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                        // 15 microsecond inter-message gap_time.
   // Write the message buffer data to the buffer
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Broadcast Mode Code - With Data RT31 SA0 MC20 TX  -- Selected Transmitter shutdown  
   messno++;                                       // Index to the next message buffer                                             
   memset((char*)&bcmessage,0,sizeof(bcmessage));  // Clear out the message buffer data
   bcmessage.messno = messno;                      // Current buffer index
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);// Link to next message buffer
   bcmessage.control = BC_CONTROL_MESSAGE;         // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;        // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;           // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;       // Enable interrupt on this message
   bcmessage.control |= BC_CONTROL_MFRAME_END;     // End of frame  
   bcmessage.mess_command1.rtaddr   = 31;          // RT address 31 - broadcast
   bcmessage.mess_command1.subaddr  = 0;           // Subaddress 0  - Mode Code
   bcmessage.mess_command1.wcount   = 20;          // Word count 4  - Synchronize
   bcmessage.mess_command1.tran_rec = 0;           // Transmit command
   bcmessage.errorid = 0;                          // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                        // 15 microsecond inter-message gap_time.
   bcmessage.data[0][1] = 0x0;                     // Fill in the data for the transmitter shutdown

   // Write the message buffer data to the buffer
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   // Start of the second frame

   // Broadcast   
   messno++;                                       // Set the message buffer index to the first buffer
   memset((char*)&bcmessage,0,sizeof(bcmessage));  // Clear out the message buffer data       
   bcmessage.messno = messno;                      // Current buffer index
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);// Link to next message buffer
   bcmessage.control = BC_CONTROL_MESSAGE;         // Set as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;        // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;           // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;       // Enable interrupt on this message
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;     // Start of the frame
   bcmessage.mess_command1.rtaddr   = 31;          // RT Address 31 - Broadcast
   bcmessage.mess_command1.subaddr  = 2;           // Subassress 2
   bcmessage.mess_command1.wcount   = 0;           // Word count 32
   bcmessage.mess_command1.tran_rec = 0;           // receive command.
   bcmessage.errorid = 0;                          // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                        // 15 microsecond inter-message gap_time.
   // Fill in the data for the receive command
   for(i=0;i<32;i++)
       bcmessage.data[0][i] = 0xA000+i;            // Add data for the specified number of data elements
   // Write the message buffer data to the buffer
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   // RT - RT Message
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);// Link to next message buffer
   bcmessage.control = BC_CONTROL_MESSAGE;         // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;        // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;       // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_RTRTFORMAT;     // Define RT-RT Message
   bcmessage.control |= BC_CONTROL_INTERRUPT;       // Enable interrupt on this message
   bcmessage.mess_command1.rtaddr   = 4;           // RT address 4
   bcmessage.mess_command1.subaddr  = 10;          // Subaddress 10
   bcmessage.mess_command1.wcount   = 6;           // Word count 6
   bcmessage.mess_command1.tran_rec = 0;           // Receive 
   // Use Command 2 for RT-RT
   bcmessage.mess_command2.rtaddr   = 3;           // RT address 3
   bcmessage.mess_command2.subaddr  = 3;           // Subaddress 3
   bcmessage.mess_command2.wcount   = 6;           // Word count 6
   bcmessage.mess_command2.tran_rec = 1;           // Transmit
   bcmessage.errorid = 0;                          // Default error injection buffer (no errors)
   bcmessage.long_gap = 20;                        // 20 microsecond inter-message gap.
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Broadcast RT-RT Message
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;         // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;        // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;       // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_RTRTFORMAT;     // Define RT-RT message
   bcmessage.control |= BC_CONTROL_INTERRUPT;       // Enable interrupt on this message
   bcmessage.mess_command1.rtaddr   = 31;          // Broadcast RT address
   bcmessage.mess_command1.subaddr  = 10;          // Subaddress 10
   bcmessage.mess_command1.wcount   = 6;           // Word count 6
   bcmessage.mess_command1.tran_rec = 0;           // Receive
   // Use Command 2 for RT-RT
   bcmessage.mess_command2.rtaddr   = 3;           // Transmit RT address 3
   bcmessage.mess_command2.subaddr  = 3;           // Subadress 3
   bcmessage.mess_command2.wcount   = 6;           // Word count 6
   bcmessage.mess_command2.tran_rec = 1;           // Transmit address
   bcmessage.errorid = 0;                          // Default error injection buffer (no errors)
   bcmessage.long_gap = 20;                        // 20 microsecond inter-message gap.
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //TRANMIT Command RT24 SA16 WC32 TX  
   messno++;                                       // Index to the next message buffer                                             
   memset((char*)&bcmessage,0,sizeof(bcmessage));  // Clear out the message buffer data
   bcmessage.messno = messno;                      // Current buffer index
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);// Link to next message buffer
   bcmessage.control = BC_CONTROL_MESSAGE;         // Set as a NOP message
   bcmessage.control |= BC_CONTROL_BUFFERA;        // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSB;           // Transmit on Bus B
   bcmessage.control |= BC_CONTROL_INTERRUPT;       // Enable interrupt on this message
   bcmessage.mess_command1.rtaddr   = 24;          // RT Address 24
   bcmessage.mess_command1.subaddr  = 15;          // Subaddress 16
   bcmessage.mess_command1.wcount   = 0;           // Word count 32
   bcmessage.mess_command1.tran_rec = 1;           // Transmit command 
   bcmessage.errorid = 0;                          // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                        // 15 microsecond inter-message gap_time
   // Write the message buffer data to the buffer 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                       // Index to the next message buffer                                             
   memset((char*)&bcmessage,0,sizeof(bcmessage));  // Clear out the message buffer data
   bcmessage.messno = messno;                      // Current buffer index
   bcmessage.messno_next = (BT_U16BIT)0;           // link back to the start of first frame
   bcmessage.control = BC_CONTROL_NOP;             // Noop buffer
   bcmessage.control |= BC_CONTROL_MFRAME_END;     // End of second frame
   // Write the message buffer data to the buffer
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

} 

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   int       bufnum;
   static int sa_index;
   int       i,j;
   int       status;
   int       wcount;
   BT_INT tail, rtrt,mc;
   BT_UINT messno;

   char outbuf[80];

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      messno = sIntFIFO->fifo[tail].bufferID;
      status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);         
      if (status)
         return status;

       if(BusTools_BoardIsV6(cardnum))  //check for the F/w version to display time correctly
          BusTools_TimeGetFmtString(API_TTD_IRIG_NS, &bcmessage.time_tag, outbuf);  //MSB=nanoseconds
       else
          BusTools_TimeGetFmtString(API_TTD_IRIG, &bcmessage.time_tag, outbuf);     //MSB=microesconds 
       printf("BC**Message Time - %s\n",outbuf);    
 
       rtrt=0;
       mc=0;

       if(bcmessage.status & BT1553_INT_RETRY)
         printf("Retry - %d - %x\n",messno,bcmessage.status);
       else if (bcmessage.status & BT1553_INT_BROADCAST)
         printf("Broadcast - %d - %x\n",messno,bcmessage.status);
       else
         printf("%d - %x\n",messno,bcmessage.status);

       if(bcmessage.status & BT1553_INT_RT_RT_FORMAT)
       {
         printf("RT<->RT (%d) ",messno);
          rtrt=1;
       }
       else if(bcmessage.status & BT1553_INT_MODE_CODE)
       {
          printf("Mode Code --  (%d) ", messno);
          mc=1;
       }
       else if (bcmessage.mess_command1.tran_rec)
          printf("BC**RT->BC (%d)  ", messno);
       else
          printf("BC->RT (%d)  ", messno);
 
       printf("BC**RT-%d  \n",bcmessage.mess_command1.rtaddr);
       printf("BC**SA-%d  \n",bcmessage.mess_command1.subaddr);
       printf("BC**WC-%d  \n",bcmessage.mess_command1.wcount);
       if(rtrt)
       {   
          printf("BC**RT-%d  \n",bcmessage.mess_command2.rtaddr);
          printf("BC**SA-%d  \n",bcmessage.mess_command2.subaddr);
          printf("BC**WC-%d  \n",bcmessage.mess_command2.wcount);
       }
        
       printf("BC**status-1 0x%04x\n",*((WORD*)(&bcmessage.mess_status1)));
       if(rtrt)
          printf("BC**status-2 0x%04x\n",*((WORD*)(&bcmessage.mess_status2)));
          
       printf("BC**int_stat-0x%08lx\n",*((DWORD*)(&bcmessage.status)));
       if(bcmessage.status & BT1553_INT_NO_RESP)
          printf("XXXXXXXXXXXXXXXXXX NO RESPONSE %x!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n",bcmessage.mess_status1);

       wcount = bcmessage.mess_command1.wcount;
       if (wcount == 0)
          wcount = 32;           // Zero word count means 32 words.

       if(mc)
         if(wcount >=16)
            wcount = 1;

       j = wcount;               // See if this msg shorted than previous
    
       if (bcmessage.control & BC_CONTROL_BUFFERA)
          bufnum = 0;            // Display data from first BC buffer
       else
          bufnum = 1;            // Display data from second BC buffer

       for ( i = 0; i < j; i++ ) // Display all of the data words
       {
          // If this is an RT->BC message, and there is a Message Error
          //  or No Response, than xxxx the data (since the RT did not
          //    send any data words).
          if ( i >= wcount )
          {
             printf("    ");     // Data does not apply to this msg
          }
          else if ( bcmessage.mess_command1.tran_rec &&
            (bcmessage.mess_status1.me || (bcmessage.status&BT1553_INT_NO_RESP) ) )
          {
             printf("xxxx ");      // No data was transfered because of error
          }
          else
          {
             printf("BC**%04x ",bcmessage.data[bufnum][i]);
          }
       }
       printf("\n_____________________________________________________________\n");
   
       tail++;
       tail &= sIntFIFO->mask_index;
       sIntFIFO->tail_index = tail;
   }

   return 0;
}

/****************************************************************************
*
*  Function:  setupThe_BC_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a BC message.  The interrupt is set for only the end-of
*     message.  
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupThe_BC_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.iNotification = 0;
   sIntFIFO1.function     = bc_intFunction;
   sIntFIFO1.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.bForceStartup  = 0;
   sIntFIFO1.FilterType     =  EVENT_BC_MESSAGE;

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO1.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, 1);
   printf("BusTools_RegisterFunction status = %d\n",status);
}

