/*============================================================================*
 * FILE:                 E X A M P L E _ B C _ M S G _ R U N . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:   This example program shows how to program the Bus Controller
 *             to run a simple three message minor-frame. The example
 *             initializes the channel the Bus Controller is running on. 
 *             Then sets the channel for external bus.  Following 
 *             those initialization step it initializes the Bus Controller and
 *             builds a simple 3-message frame.  The message in the frame are
 *             linked together and the frame will run continually until stopped
 *             by the user.
 *             
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
	05/29/01	Initial version.  
	08/13/01	Minor cleanup, removed unnecessary BM init code.  
	03/15/02	Removed conio.h and getc calls.
    09/20/04    Added option to initialize with BusTools_API_LoadChannel.
    05/18/10    re-designed to show traditional frame implementation.
*/

#include "busapi.h"
#include <stdio.h>

void bc_init(int cardnum);

// Main program
int main(int argc, char **argv) 
{
   BT_INT status,cardnum,noop=0;
   char c;
   BT_INT dev_num, mode;

   printf("Initializing API with BusTools_API_OpenChannel . . . ");
 
   //Set the device ID 
   if(argc == 2)
      dev_num = atoi(argv[1]);
   else
      dev_num = 0;

   // Open the device and get the channel handle.
   mode = API_B_MODE | API_SW_INTERRUPT;                      // 1553B protocol using SW interrupts.
   status = BusTools_API_OpenChannel( &cardnum, mode, dev_num, CHANNEL_1);
   printf("BusTools_API_OpenChannel status = %d\n",status);

   // Select External Bus.
   status = BusTools_SetInternalBus(cardnum, EXTERNAL_BUS);
   printf("BusTools_SetInternalBus status = %d\n",status);
        
   bc_init(cardnum);

   printf("\nHit Enter to start the BC\n");            
   printf("type Q/q to Quit\n");
   c = getc(stdin);
   fflush(stdin);

   //Start the bus controller.
   status = BusTools_BC_StartStop(cardnum, BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);

   // BC is up and running, wait for input to NOOP, UN-NOOP, or QUIT.
   do
   {
      c = getchar();
      fflush(stdin);
   } while (c != 'Q' && c != 'q'); // End of do-while

   // Stop the BC.
   status = BusTools_BC_StartStop(cardnum, BC_STOP);
   printf("BusTools_BC_StartStop status = %d\n",status);

   // Close API and board
   status = BusTools_API_Close(cardnum);
   printf("BusTools_API_Close status = %d\n",status);

   printf("FINISHED.\n");
} // End of main

void bc_init(int cardnum)
{
   int status, messno,i;
   API_BC_MBUF bcmessage;

   // Initialize BC for no interrupts, no retries, no resp at 16us, late resp at 14us,
   // minor frame time of 500000us, 1 data buffer per message.
   status = BusTools_BC_Init(cardnum, 0, 0, 0, 16, 14, 500000, 1);
   printf("BusTools_BC_Init status = %d\n",status);
   
   // Allocate memory on the board for 10 message buffers.
   // Only three are used in this example.
   status = BusTools_BC_MessageAlloc(cardnum, 10);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);
		
   // Write the Bus Controllere messages to the allocated message buffers.

   //RT2 SA2 WC2 RX  
   messno = 0;                                     // Set the message buffer index to the first buffer
   memset((char*)&bcmessage,0,sizeof(bcmessage));  // Clear out the message buffer data       
   bcmessage.messno = messno;                      // Current buffer index
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);// Link to next message buffer
   bcmessage.control = BC_CONTROL_MESSAGE;         // Set as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;        // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;           // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;     // Start of the frame
   bcmessage.mess_command1.rtaddr   = 2;           // RT Address 2
   bcmessage.mess_command1.subaddr  = 2;           // Subassress 2
   bcmessage.mess_command1.wcount   = 2;           // Word count 2
   bcmessage.mess_command1.tran_rec = 0;           // receive command.
   bcmessage.errorid = 0;                          // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                        // 15 microsecond inter-message gap_time.
   // Fill in the data for the receive command
   for(i=0;i<2;i++)
       bcmessage.data[0][i] = 0xA000+i;            // Add data for the specified number of data elements
   // Write the message buffer data to the buffer
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //RT3 SA3 WC3 TX  
   messno++;                                       // Index to the next message buffer                                             
   memset((char*)&bcmessage,0,sizeof(bcmessage));  // Clear out the message buffer data
   bcmessage.messno = messno;                      // Current buffer index
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);// Link to next message buffer
   bcmessage.control = BC_CONTROL_MESSAGE;         // Set as a NOP message
   bcmessage.control |= BC_CONTROL_BUFFERA;        // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSB;           // Transmit on Bus B
   bcmessage.mess_command1.rtaddr   = 3;           // RT Address 3
   bcmessage.mess_command1.subaddr  = 3;           // Subaddress 3
   bcmessage.mess_command1.wcount   = 3;           // Word count 3
   bcmessage.mess_command1.tran_rec = 1;           // Transmit command 
   bcmessage.errorid = 0;                          // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                        // 15 microsecond inter-message gap_time
   // Write the message buffer data to the buffer 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);


   //RT4 SA4 WC4 TX 
   messno++;                                       // Index to the next message buffer                                             
   memset((char*)&bcmessage,0,sizeof(bcmessage));  // Clear out the message buffer data
   bcmessage.messno = messno;                      // Current buffer index
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);// Link to next message buffer
   bcmessage.control = BC_CONTROL_MESSAGE;         // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;        // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;           // Transmit on Bus A
   bcmessage.mess_command1.rtaddr   = 4;           // RT address 4
   bcmessage.mess_command1.subaddr  = 4;           // Subaddress 4
   bcmessage.mess_command1.wcount   = 4;           // Word count 4
   bcmessage.mess_command1.tran_rec = 1;           // Transmit command
   bcmessage.errorid = 0;                          // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                        // 15 microsecond inter-message gap_time.
   // Write the message buffer data to the buffer
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                       // Index to the next message buffer                                             
   memset((char*)&bcmessage,0,sizeof(bcmessage));  // Clear out the message buffer data
   bcmessage.messno = messno;                      // Current buffer index
   bcmessage.messno_next = (BT_U16BIT)0;           // link back to the first message buffer
   bcmessage.control = BC_CONTROL_NOP;             // Noop buffer
   bcmessage.control |= BC_CONTROL_MFRAME_END;     // End of frame
   // Write the message buffer data to the buffer
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

} 

