/*===========================================================================*
 * FILE:             E X M A P L E _ B C _ M S G _ S C H E D . C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:   This application shows how to setup the bus Controller for Message 
 * Scheduling.  That is a technique that allows the user to schedule message at
 * differing rates in the major frame.  In this example, messages going out on 
 * every frame, every other frame, and every fifth frame.   The frame rate is set 
 * at 1 Hz, so the message traffic can be visually seen on an analyzer.  When 
 * using Message Scheduling the user programs the start-frame and the repeat rate 
 * for each message.  A message with a start frame of one and repeat rate of one 
 * goes in every frame.  A start frame of 1 and repeat rate of 2 causes the message 
 * to transact in every other frame.  By setting the base frame rate and the start 
 * frame and repeat rate for each message the user can control the rate at which the 
 * messages in the major frame transact. You can vary frame rate and the start-frame 
 * and repeat rates of the messages in this example and see how the changes affect 
 * message traffic.
 *
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include "busapi.h"

void Define_Bus_Controller_Messages(int);
void setupThe_BC_IntFIFO(int cardnum);
int RemoteTerminal_Init(int cardnum);

int status;
API_BC_MBUF bcmessage;
unsigned messno,device;
unsigned icount=0,  last_count=0;
static unsigned total_bc_messages_processed=0;
unsigned errcount = 0;
unsigned ccount=0;
int pwFlag;
API_INT_FIFO sIntFIFO1, sIntFIFO2;

void main(int argc, char **argv)
{
   int status;
   int cardnum;

   pwFlag = API_SW_INTERRUPT; // S/w interrupts
   device = 0;
   
   /* Initializing Channel */
   status = BusTools_API_OpenChannel(&cardnum,pwFlag,device,CHANNEL_1);
   printf("BusTools_API_OpenChannel status = %d\n",status);

   {
      float wcs,lpu;
      BT_INT build;

      status = BusTools_GetFWRevision(cardnum,&wcs,&lpu,&build);
      printf("WCS = %f LPU = %f BUILD = %i \n",wcs,lpu,build);
   }

   status = BusTools_SetInternalBus(cardnum, EXTERNAL_BUS);      /*Run internally for this example */
   printf("BusTools_SetInternalBus status = %d\n",status);

   RemoteTerminal_Init(cardnum);
   Define_Bus_Controller_Messages(cardnum);
   setupThe_BC_IntFIFO(cardnum);  /*Setup the interrupt callback for BC messages */

   printf("Hit enter to start the test\n");
   getc(stdin);

   status = BusTools_RT_StartStop(cardnum,RT_START);
   printf("BusTools_RT_StartStop status = %d\n",status);

   status = BusTools_BC_StartStop(cardnum,BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);
 
   getchar();
   status = BusTools_DumpMemory(cardnum,DUMP_ALL,"test.txt","test");

   status = BusTools_BC_StartStop(cardnum,BC_STOP);
   printf("BusTools_BC_StartStop status = %d\n",status);

   status = BusTools_RT_StartStop(cardnum,RT_STOP);
   printf("BusTools_RT_StartStop status = %d\n",status);

   BusTools_API_Close(cardnum);

   printf("BusTools_API_Close status = %d\n",status);
  
}

void Define_Bus_Controller_Messages(int cardnum)
{
   /*Enable Message Scheduling and interrupt on end-of-message */
   status = BusTools_BC_Init(cardnum,MSG_SCHD, BT1553_INT_END_OF_MESS, 0,14,12,1000000,1);
   printf("BusTools_BC_Init status = %d\n",status);

   /* Allocate 20 BC Message Buffers      */
   status = BusTools_BC_MessageAlloc(cardnum,20);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);
   
   // Start Frame 1 - Repeat Rate 1  (always) RT->BC First Message in each frame
   messno = 0;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno+1);    
   bcmessage.control = BC_CONTROL_MESSAGE;       
   bcmessage.control |= BC_CONTROL_BUFFERA;     
   bcmessage.control |= BC_CONTROL_CHANNELA;
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_MFRAME_BEG; 
   bcmessage.mess_command1.rtaddr   = 1;
   bcmessage.mess_command1.subaddr  = 1;
   bcmessage.mess_command1.wcount   = 1;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.start_frame = 1;
   bcmessage.rep_rate = 1;
   bcmessage.gap_time = 100;
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);

   // Start Frame 1 - Repeat Rate 1  every frame
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno+1);                     
   bcmessage.control = BC_CONTROL_MESSAGE;      
   bcmessage.control |= BC_CONTROL_BUFFERA;   
   bcmessage.control |= BC_CONTROL_CHANNELA;
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 2;
   bcmessage.mess_command1.subaddr  = 9;
   bcmessage.mess_command1.wcount   = 9;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.start_frame = 1;
   bcmessage.rep_rate = 1;
   bcmessage.gap_time = 100;
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);

   // Start Frame 1 - Repeat Rate 0  (once-only in frame 1) RT->BC
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno+1);    
   bcmessage.control = BC_CONTROL_MESSAGE;  
   bcmessage.control |= BC_CONTROL_BUFFERA;  
   bcmessage.control |= BC_CONTROL_CHANNELA;
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 3;
   bcmessage.mess_command1.subaddr  = 9;
   bcmessage.mess_command1.wcount   = 9;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.start_frame = 1;
   bcmessage.rep_rate = 0;
   bcmessage.gap_time = 100;
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);

   // Start Frame 1 - Repeat Rate 2  (Every other frame starting on frame 1)  BC->RT
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno+1);                
   bcmessage.control = BC_CONTROL_MESSAGE;     
   bcmessage.control |= BC_CONTROL_BUFFERA;   
   bcmessage.control |= BC_CONTROL_CHANNELA;
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 11;
   bcmessage.mess_command1.subaddr  = 11;
   bcmessage.mess_command1.wcount   = 11;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.start_frame = 1;
   bcmessage.rep_rate = 2;
   bcmessage.gap_time = 100;
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);

   // Start Frame 1 - Repeat Rate 2  (Every other frame starting on frame 1) RT->BC
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno+1);                
   bcmessage.control = BC_CONTROL_MESSAGE;     
   bcmessage.control |= BC_CONTROL_BUFFERA; 
   bcmessage.control |= BC_CONTROL_CHANNELA;
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 12;
   bcmessage.mess_command1.subaddr  = 12;
   bcmessage.mess_command1.wcount   = 12;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.start_frame = 1;    
   bcmessage.rep_rate = 2;
   bcmessage.gap_time = 100;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);

   // Start Frame 1 - Repeat Rate 5  (Every fifth frame starting on frame 1)
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno+1);            
   bcmessage.control = BC_CONTROL_MESSAGE;  
   bcmessage.control |= BC_CONTROL_BUFFERA; 
   bcmessage.control |= BC_CONTROL_CHANNELA;
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 23;
   bcmessage.mess_command1.subaddr  = 23;
   bcmessage.mess_command1.wcount   = 23;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.start_frame = 1;
   bcmessage.rep_rate = 5;
   bcmessage.gap_time = 100;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);

   // Start Frame 1 - Repeat Rate 4 (Every fifth frame starting on frame 1) RT-RT
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno+1);
   bcmessage.control = BC_CONTROL_MESSAGE;  
   bcmessage.control |= BC_CONTROL_BUFFERA;  
   bcmessage.control |= BC_CONTROL_CHANNELA; 
   bcmessage.control |= BC_CONTROL_RTRTFORMAT;
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 24;
   bcmessage.mess_command1.subaddr  = 10;
   bcmessage.mess_command1.wcount   = 6;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.mess_command2.rtaddr   = 26;
   bcmessage.mess_command2.subaddr  = 6;
   bcmessage.mess_command2.wcount   = 6;
   bcmessage.mess_command2.tran_rec = 1;
   bcmessage.start_frame = 1;    
   bcmessage.rep_rate = 5;
   bcmessage.errorid = 0;  
   bcmessage.gap_time = 100; 
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   // Start Frame 6 - Repeat Rate 5  (Every fifth frame starting on frame 5) Mode Code
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno+1);
   bcmessage.control = BC_CONTROL_MESSAGE;      
   bcmessage.control |= BC_CONTROL_BUFFERA;
   bcmessage.control |= BC_CONTROL_CHANNELA;
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 25;
   bcmessage.mess_command1.subaddr  = 0;
   bcmessage.mess_command1.wcount   = 17;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.start_frame = 6;
   bcmessage.rep_rate = 5;
   bcmessage.gap_time = 100;
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);


   // NOOP End of frame  last message in each frame
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(0);      // Restart the frame
   bcmessage.control  = BC_CONTROL_NOP;         // NOOP
   bcmessage.control |= BC_CONTROL_MFRAME_END;  // End-of-frame

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
}

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   int       bufnum;
   static int sa_index;
   int       i,j;
   int       status;
   int       wcount;
   BT_INT tail, rtrt,mc;
   BT_UINT messno;
   char outbuf[80];

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
       if(sIntFIFO->FilterType & EVENT_BC_MESSAGE)
       {
          messno = sIntFIFO->fifo[tail].bufferID;
          status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);     
          if (status)
             return status;
          rtrt=0;
          mc=0;

          BusTools_TimeGetString(&bcmessage.time_tag, outbuf);
          printf("time tag = %s\n",outbuf);
       

          if(bcmessage.status & BT1553_INT_RT_RT_FORMAT)
          {
            printf("RT<->RT (%d) ",messno);
             rtrt=1;
          }
          else if(bcmessage.status & BT1553_INT_MODE_CODE)
          {
             printf("Mode Code --  (%d) ", messno);
             mc=1;
          }
          else if (bcmessage.mess_command1.tran_rec)
             printf("BC**RT->BC (%d)  ", messno);
          else
             printf("BC->RT (%d)  ", messno);
 
          printf("BC**RT-%d  \n",bcmessage.mess_command1.rtaddr);
          printf("BC**SA-%d  \n",bcmessage.mess_command1.subaddr);
          printf("BC**WC-%d  \n",bcmessage.mess_command1.wcount);
          if(rtrt)
          {   
             printf("BC**RT-%d  \n",bcmessage.mess_command2.rtaddr);
             printf("BC**SA-%d  \n",bcmessage.mess_command2.subaddr);
             printf("BC**WC-%d  \n",bcmessage.mess_command2.wcount);
          }
        
          printf("BC**status-1 0x%04x\n",*((WORD*)(&bcmessage.mess_status1)));
          if(rtrt)
             printf("BC**status-2 0x%04x\n",*((WORD*)(&bcmessage.mess_status2)));
          
          printf("BC**int_stat-0x%08lx\n",*((DWORD*)(&bcmessage.status)));

          wcount = bcmessage.mess_command1.wcount;
          if (wcount == 0)
             wcount = 32;           // Zero word count means 32 words.

          if(mc)
            if(wcount >=16)
               wcount = 1;

          j = wcount;               // See if this msg shorted than previous
    
          if (bcmessage.control & BC_CONTROL_BUFFERA)
             bufnum = 0;            // Display data from first BC buffer
          else
             bufnum = 1;            // Display data from second BC buffer

          for ( i = 0; i < j; i++ ) // Display all of the data words
          {
             // If this is an RT->BC message, and there is a Message Error
             //  or No Response, than xxxx the data (since the RT did not
             //    send any data words).
             if ( i >= wcount )
             {
                printf("    ");     // Data does not apply to this msg
             }
             else if ( bcmessage.mess_command1.tran_rec &&
               (bcmessage.mess_status1.me || (bcmessage.status&BT1553_INT_NO_RESP) ) )
             {
                printf("xxxx ");      // No data was transfered because of error
             }
             else
             {
                printf("BC**%04x ",bcmessage.data[bufnum][i]);
             }
          }
          printf("\n");
      }
      if(sIntFIFO->FilterType & EVENT_RT_MESSAGE)
      {
          sIntFIFO->nUser[0]++;
      }
            
       tail++;
       tail &= sIntFIFO->mask_index;
       sIntFIFO->tail_index = tail;
   }

   return 0;
}

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_noopIntFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;

   BT_INT tail;
   BT_UINT messno;


   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
       printf("Head = %d Tail = %d\n",sIntFIFO->head_index,tail);
       if(sIntFIFO->FilterType & EVENT_BC_CONTROL)
       {
          messno = sIntFIFO->fifo[tail].bufferID;
          BusTools_BC_MessageRead(cardnum,messno,&bcmessage);
          printf("\n!!!!!!!!!!!!!!!!!!!!!This is the Timed No-op\n");
       }     
       tail++;
       tail &= sIntFIFO->mask_index;
       sIntFIFO->tail_index = tail;
   }

   return 0;
}
 
/****************************************************************************
*
*  Function:  setupthe_Timed_NoopFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a BC message.  The interrupt is set for only the end-of
*     message.  
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupThe_Timed_NoopFIFO(int cardnum)
{
   int rt,tr,sa,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO2, 0, sizeof(sIntFIFO2));
   sIntFIFO2.iNotification = 0;
   sIntFIFO2.function     = bc_noopIntFunction;
   sIntFIFO2.iPriority    = THREAD_PRIORITY_BELOW_NORMAL;
   sIntFIFO2.dwMilliseconds = INFINITE;
   sIntFIFO2.bForceStartup  = 0;
   sIntFIFO2.FilterType     =  EVENT_BC_CONTROL;

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO2.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO2, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}

/****************************************************************************
*
*  Function:  setupThe_BC_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a BC message.  The interrupt is set for only the end-of
*     message.  
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupThe_BC_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.iNotification = 0;
   sIntFIFO1.function     = bc_intFunction;
   sIntFIFO1.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.bForceStartup  = 0;
   sIntFIFO1.FilterType     =  EVENT_BC_MESSAGE;

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO1.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}

/* Set up RT to response to the BC Messages */

int RemoteTerminal_Init(int cardnum)
{
   int status;
   int RT_MBUF_COUNT;
   API_RT_CBUF cbuf;       // RT Control Buffer Legal Word Count Mask (2 words)
   API_RT_MBUF_WRITE mbuf;
   API_RT_ABUF abuf;
   int         tr;
   int         subaddress;
   int i,j,ii,jj;
   int rt_addr[] = {1,2,3,11,12,23,24,25,26};
   int rt_cnt = 9;

   RT_MBUF_COUNT = 2;

   status = BusTools_RT_Init(cardnum,0);
   printf("BusTools_RT_Init status = %d\n",status);

   /**********************************************************************
   *  Output RT control buffers
   *   Enable or disable all word counts and mode codes as specified by
   *    the caller's arguements.
   **********************************************************************/
   cbuf.legal_wordcount = 0xffffffff;  // Legailize all word counts
   for ( i = 0; i < rt_cnt; i++)
   {
      for ( tr = 0; tr < 1; tr++ )
      {
         for ( subaddress = 0; subaddress < 8; subaddress++ )
         {
            status = BusTools_RT_CbufWrite(cardnum,rt_addr[i],subaddress,tr,
                                        RT_MBUF_COUNT,&cbuf);
            if(status)
               return status;
         }
      }
   }

   /*******************************************************************
   *  Fill in message buffers for receive and transmit sub-units.
   *******************************************************************/
   mbuf.error_inj_id = 0;
   mbuf.enable = BT1553_INT_END_OF_MESS;   // Interrupt enables
   // For all subaddresses...
   for (ii = 0; ii<rt_cnt; ii++)
   {  
      for (i=0; i<8; i++)
      {
         // For each data buffer...
         for (j=0; j<RT_MBUF_COUNT; j++)
         {
            for(jj=0;jj<9;jj++)
               mbuf.mess_data[jj] = (BT_U16BIT)(0xA000 + j);
            status = BusTools_RT_MessageWrite(cardnum,rt_addr[ii],i,0,j,&mbuf);
            status = BusTools_RT_MessageWrite(cardnum,rt_addr[ii],i,1,j,&mbuf);
         }
      }
   }

   /************************************************
   *  Setup RT address control block
   ************************************************/

   abuf.enable_a          = 1;    // Enable Bus A...Note that the API bits are
   abuf.enable_b          = 1;    // Enable Bus B... reversed from the HW bits.
   abuf.inhibit_term_flag = 0;
   abuf.status            = 0x0000;
   abuf.command           = 0;
   abuf.bit_word          = 0;

   for(i=0;i<rt_cnt;i++)
   {
      status = BusTools_RT_AbufWrite(cardnum,rt_addr[i],&abuf);
   }

   return status;
}