/*===========================================================================*
 * FILE:             E X A M P L E _ B C _ M U L T I _ B U F F E R . C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    Demonstration of BusTools/1553-API routines. This example shows
 *              how to initialize the Bus Controller with multiple data buffers.
 *              This examples requires Firmware version 6.0 or greater and
 *              BusTools/1553-API version 8.0 or greater. Previous F/W and API
 *              versions only support 1 or two data buffers.  When using the 
 *              Multiple BC buffer option you can create multiple data buffers 
 *              for each BC message.
 *
 *              This function initializes the board and steps through the process
 *              of enabling, creating, filling, and processing the multiple
 *              buffers. Three messages are created with 10, 15, and 5 data 
 *              buffers. A interrupt callback function is registerd using
 *              BusTools_RegisterFunction.  In that callback the data from the 
 *              buffer is optionally printed and the receive command (RT4) data
 *              is incremented to show how to update the data.
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#if defined (__WIN32__)
#include <conio.h>
#endif
#include "busapi.h"
#include "lowlevel.h"

int BusController_Init(int);
void setupThe_BC_IntFIFO(int cardnum);
void setupThe_BC_readIntFIFO(int cardnum);

API_INT_FIFO sIntFIFO1;      // Thread GIFO structure for BC.
API_INT_FIFO sIntFIFO2;      // Thread GIFO structure for BC Read.
static unsigned bc_tcnt=0;
//CEI_MUTEX csec;

#if defined (__WIN32__)
void main(int argc, char **argv)                     
#else
int tst_rt_mrt_bm()
#endif
{
   int status,cardnum;
   unsigned channel,device = 0,mode;
   int i=0;

   mode = API_B_MODE | API_HW_INTERRUPT;  // MIL-STD-1553B with S/W interrupts enabled
   channel = CHANNEL_1;                   // Default channel 1
#if defined (__WIN32__)
   if(argc == 2)
      device = atoi(argv[1]);
   else
      device = 0;  // Default device 0
   
#endif

   // Initialize the channel
   status = BusTools_API_OpenChannel(&cardnum,mode,device,channel);
   printf("BusTools_API_OpenChannel status = %d\n",status);

   status = BusTools_SetInternalBus(cardnum,EXTERNAL_BUS);  // Setup bus External
   printf("BusTools_SetInternalBus status = %d\n",status);

   // Set the time tag mode for display and initial setting
   BusTools_TimeTagMode(cardnum,API_TTD_IRIG_NS,API_TTI_IRIG,API_TTM_FREE,0,0,0,0);
 
   status = BusController_Init(cardnum);                    // Setup Bus Controller
   setupThe_BC_IntFIFO(cardnum);                            // Seup interrupt callback
   setupThe_BC_readIntFIFO(cardnum);

   //CEI_MUTEX_CREATE(&csec);


   status = BusTools_BC_StartStop(cardnum,BC_START);        // Start BC running
   printf("BusTools_BC_StartStop status = %d\n",status);

   printf("Hit Enter to stop and exit\n");
   getchar();                                              

   printf("main: closing API\n");

   status = BusTools_BC_StartStop(cardnum,BC_STOP);        // Stop BC running       
   printf("BusTools_BC_StartStop status = %d\n",status);

   // Unregister the callback
   status = BusTools_RegisterFunction(cardnum,&sIntFIFO1,UNREGISTER_FUNCTION); 
   status = BusTools_RegisterFunction(cardnum,&sIntFIFO2,UNREGISTER_FUNCTION);

   status = BusTools_API_Close(cardnum);                  // Close the channel
   printf("BusTools_API_Close status = %d bc_tcnt = %d\n",status,bc_tcnt);
}

int BusController_Init(int cardnum)
{
   int status, messno,index, i,j;
   API_BC_MBUF bcmessage;
   BT_U16BIT bc_data[34];             
   BT_INT num_bc_messages = 4;          // Number of BC message used in this example
   BT_INT  num_bc_data[4]={10,15,5,1};  // Array with the buffer count for each message

   // Initialize the BC with interrupt on end-of-message 16 us no response 14 us late response, 1sec frame (1000000 usec). 
   // Use MULTIPLE_BC_BUFFERS to specify multiple BC Buffers  
   status = BusTools_BC_Init(cardnum,MULTIPLE_BC_BUFFERS,BT1553_INT_END_OF_MESS,0,16,14,1000000,1);
   printf("BusTools_BC_Init status = %d\n",status);

   // New Mechanism to allocate buffers. Each buffer is allocated separately in loop
   for(index=0;index<num_bc_messages-1;index++) // Loop through all messages specify buffer count
   {
      status = BusTools_BC_MessageBlockAlloc(cardnum,BC_BLOCK_NEXT,num_bc_data[index]);  // Allocate each buffer 
      if(status)
         return status;
   }
   status = BusTools_BC_MessageBlockAlloc(cardnum,BC_BLOCK_LAST,num_bc_data[index]);     // Allocate last buffer
   if(status)
      return status; 

   //Write out command data and first data buffer.
   messno = 0;
   memset((char*)&bcmessage,0,sizeof(bcmessage));       
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control  = BC_CONTROL_MESSAGE;       // Show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A when programming multiple buffers
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_MFRAME_BEG; 
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 4;
   bcmessage.mess_command1.wcount   = 4;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.long_gap = 10;                      // Microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   // New method to write data buffers.  
   for(j=0;j<num_bc_data[messno];j++)
   {
      for(i=0;i<4;i++)
         bc_data[i] = 0x2000+j;    // Add the buffer ID (0 - 9) to the starting data value

      // Write out the data to each buffer   
      BusTools_BC_DataBufferWrite(cardnum,messno,j,bc_data);
   }

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control  = BC_CONTROL_MESSAGE;       // Show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on BUS A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 6;
   bcmessage.mess_command1.subaddr  = 6;
   bcmessage.mess_command1.wcount   = 6;  
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.long_gap = 10;                      // Microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control  = BC_CONTROL_MESSAGE;       // Show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on BUS A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 8;
   bcmessage.mess_command1.subaddr  = 8;
   bcmessage.mess_command1.wcount   = 8;  
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.long_gap = 10;                      // Microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = 0;
   bcmessage.control = BC_CONTROL_NOP;   
   bcmessage.control |= BC_CONTROL_MFRAME_END; 

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   return status; 
} 

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   static int sa_index;
   int       status,i;
   BT_INT tail,mc=0, j,wcount;
   BT_UINT bufaddr;
   static int nFlag = 1;
   BT_U16BIT temp[34];
   char outbuf[80];

   //CEI_MUTEX_LOCK(&csec);
   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      bufaddr = sIntFIFO->fifo[tail].buffer_off;

      bc_tcnt++;
      status = BusTools_BC_MessageBufferRead(cardnum,bufaddr,&bcmessage);//New function to read BC data
      if (status)
         return status;
      
      wcount = bcmessage.mess_command1.wcount;

       printf("%d - %x\n",bcmessage.messno,bcmessage.status);
      
       strncpy(outbuf,"NANO",4);  
       BusTools_TimeGetString(&bcmessage.time_tag, outbuf);
       printf("tag time = %s\n",outbuf);     

       if (bcmessage.mess_command1.tran_rec)
          printf("BC**RT->BC (%d)  ", bcmessage.messno);
       else
          printf("BC->RT (%d)  ", bcmessage.messno);

       printf("BC**RT-%d  \n",bcmessage.mess_command1.rtaddr);
       printf("BC**SA-%d  \n",bcmessage.mess_command1.subaddr);
       printf("BC**WC-%d  \n",bcmessage.mess_command1.wcount);
       printf("BC**status-1 0x%04x\n",*((WORD*)(&bcmessage.mess_status1)));
       printf("BC**int_stat-0x%08lx\n",*((DWORD*)(&bcmessage.status)));


       if (wcount == 0)
          wcount = 32;           // Zero word count means 32 words.

       if(bcmessage.status & BT1553_INT_MODE_CODE)
          mc=1;

       if(mc)
         if(wcount >=16)
           wcount = 1;

       j = wcount;               // See if this msg shorted than previous
       for ( i = 0; i < j; i++ ) // Display all of the data words
       {
          if ( i >= wcount )
          {
             printf("    ");     // Data does not apply to this msg
          }
          else if ( bcmessage.mess_command1.tran_rec &&
            (bcmessage.mess_status1.me || (bcmessage.status & BT1553_INT_NO_RESP) ) )
          {
             printf("xxxx ");      // No data was transfered because of error
          }
          else
          {
             printf("BC**%04x ",bcmessage.data[0][i]);
          }
          if(bcmessage.mess_command1.tran_rec==0)   
             temp[i] = bcmessage.data[0][i] + 10;          // Increment the data on the RCV command
       }                                                  // There 10 pre-load buffers, so add 10 to increment by 1
       printf("\n________________________________________________________________\n");  
       if(bcmessage.mess_command1.tran_rec==0)
          BusTools_BC_DataBufferUpdate(cardnum,bufaddr,wcount,temp);  // Write out new data to current buffer
       
       tail++;
       tail &= sIntFIFO->mask_index;
       sIntFIFO->tail_index = tail;
   }
   //CEI_MUTEX_UNLOCK(&csec);
   return 0;
}

/****************************************************************************
*
*  Function:  bc_readIntFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_readIntFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   BT_U16BIT bcdata[32];
   int status,i, tail;
   BT_UINT bufaddr;

   //CEI_MUTEX_LOCK(&csec);
   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      bufaddr = sIntFIFO->fifo[tail].buffer_off;

      if(sIntFIFO->fifo[tail].int_status & BT1553_INT_ERROR_BITS)
      {
         //CEI_MUTEX_UNLOCK(&csec);
         return -1;
      }

      status = BusTools_BC_ReadDataBuffer(cardnum,bufaddr,bcdata); //New function to read BC data
      if (status)
         return status;


       for ( i = 0; i < 8; i++ )   // Display all of the data words
       {
          printf("RT8**%04x ",bcdata[i]); 
       }
       printf("\n\n");

       tail++;
       tail &= sIntFIFO->mask_index;
       sIntFIFO->tail_index = tail;
   }
   //CEI_MUTEX_UNLOCK(&csec);
   return 0;
}

/****************************************************************************
*
*  Function:  setupThe_BC_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO for a interrupt on a BC message.
*     The interrupt is set for the end-of message.  
*
****************************************************************************/
void setupThe_BC_IntFIFO(int cardnum)
{
   int status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(API_INT_FIFO));
   sIntFIFO1.iNotification  = 0;
   sIntFIFO1.function       = bc_intFunction;
   sIntFIFO1.iPriority      = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.bForceStartup  = 0;
   sIntFIFO1.FilterType     = EVENT_BC_MESSAGE;

   sIntFIFO1.FilterMask[4][0][4] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos
   sIntFIFO1.FilterMask[6][1][6] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}

/****************************************************************************
*
*  Function:  setupThe_BC_readIntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO for a interrupt on a BC message.
*     The interrupt is set for the end-of message.  
*
****************************************************************************/
void setupThe_BC_readIntFIFO(int cardnum)
{
   int status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO2, 0, sizeof(API_INT_FIFO));
   sIntFIFO2.iNotification  = 0;
   sIntFIFO2.function       = bc_readIntFunction;
   sIntFIFO2.iPriority      = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO2.dwMilliseconds = INFINITE;
   sIntFIFO2.bForceStartup  = 0;
   sIntFIFO2.FilterType     = EVENT_BC_MESSAGE;

   sIntFIFO2.FilterMask[8][1][8] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO2, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}

