/*============================================================================*
 * FILE:                 E X A M P L E _ B C _ M S G _ N O O P . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:    EXAMPLE PROGRAM
 * This example shows how to NOOP and un-NOOP a message.  When a message is 
 *  NOOPed, It does not transact.  The firmware skips over the message as if it 
 *  were not in the bus list.  A single minor frame is created with three messages 
 *  in the frame.  The Second message in the bus list (message 1) is created as a 
 *  Noop message (BC_CONTROL_MSG_NOP).   That means it created as a BC message 
 *  buffer but set in the NOOP state.   When the frame runs only message 0 and 2 
 *  transact.  The user can toggle message 1 ON or OFF by entering 'U' or 'N" to 
 *  Un-NOOP or NOOP the message.  The function BusTools_BC_MessageNoop is called 
 *  change the NOOP setting for message 1.  You can use BusTools_BC_MessageNoop on 
 *  any BC message created with BC_CONTORL_MESSAGE or BC_CONTROL_MSG_NOP.
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
	05/29/01	Initial version.  
	08/13/01	Minor cleanup, removed unnecessary BM init code.  
	03/15/02	Removed conio.h and getc calls.
    09/20/04    Added option to initialize with BusTools_API_LoadChannel.
    05/18/10    re-designed to show traditional frame implementation.
*/

#include "busapi.h"
#include <stdio.h>

API_INT_FIFO sIntFIFO1;
BT_UINT noop_message;

void setupThe_BC_IntFIFO(int cardnum);
//---------------------------------------------------------------------------------

void bc_init(int cardnum);

// Main program
int main(int argc, char **argv) 
{
   BT_INT status,cardnum,noop=0;
   char c;
   BT_INT dev_num, mode;

   printf("Initializing API with BusTools_API_OpenChannel . . . ");
 
   //Set the device ID 
   if(argc == 2)
      dev_num = atoi(argv[1]);
   else
      dev_num = 0;

   // Open the device and get the channel handle.
   mode = API_B_MODE | API_SW_INTERRUPT;                      // 1553B protocol, use SW interrupts.
   status = BusTools_API_OpenChannel( &cardnum, mode, dev_num, CHANNEL_1);
   printf("BusTools_API_OpenChannel status = %d\n",status);

   // Select External Bus.
   status = BusTools_SetInternalBus(cardnum, EXTERNAL_BUS);
   printf("BusTools_SetInternalBus status = %d\n",status);

   BusTools_TimeTagMode(cardnum,API_TTD_DATE,API_TTI_IRIG,API_TTM_FREE,0,0,0,0);
   printf("BusTools_TimeTagMode status = %d\n",status);
        
   bc_init(cardnum);
   setupThe_BC_IntFIFO(cardnum);

   printf("\nHit Enter to start the BC\n");
   printf("type U/u to \"un-noop\" message 1 (RT3 SA3)\n");
   printf("type N/n to \"noop\" message 1 (RT3 SA3)\n");                  
   printf("type Q/q to Quit\n");
   c = getc(stdin);
   fflush(stdin);

   //Start the bus controller.
   status = BusTools_BC_StartStop(cardnum, BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);

   // BC is up and running, wait for input to NOOP, UN-NOOP, or QUIT.
   do
   {
      c = getchar();
      fflush(stdin);
      if(c == 'u' || c == 'U')
      {
         BusTools_BC_MessageNoop(cardnum,noop_message,MSG_OP,NOWAIT);
         noop = 1;
      }
      if(c == 'n' || c == 'N')
      {
         BusTools_BC_MessageNoop(cardnum,noop_message,NOOP,NOWAIT);
         noop = 0;
      }
             
   } while (c != 'Q' && c != 'q'); // End of do-while

   // Stop the BC.
   status = BusTools_BC_StartStop(cardnum, BC_STOP);
   printf("BusTools_BC_StartStop status = %d\n",status);

   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, UNREGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);

   // Close API and board
   status = BusTools_API_Close(cardnum);
   printf("BusTools_API_Close status = %d\n",status);

   printf("FINISHED.\n");
} // End of main

void bc_init(int cardnum)
{
   int status, messno,i;
   API_BC_MBUF bcmessage;

   // Initialize BC for no interrupts, no retries, no resp at 16us, late resp at 14us,
   // minor frame time of 10000us, 1 data buffer per message.
   status = BusTools_BC_Init(cardnum, 0, BT1553_INT_END_OF_MESS, 0, 16, 14, 333333, 1); // interrupt at the end-of-message
   printf("BusTools_BC_Init status = %d\n",status);
   
   // Allocate memory on the board for 20 message blocks.
   status = BusTools_BC_MessageAlloc(cardnum, 20);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);
		
   // Define our messages and write them to the allocated board memory. 
   messno = 0;
   memset((char*)&bcmessage,0,sizeof(bcmessage));       
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Set as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // Add this message to the interrupt queue
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;    // Start of the frame
   bcmessage.mess_command1.rtaddr   = 2;
   bcmessage.mess_command1.subaddr  = 2;
   bcmessage.mess_command1.wcount   = 2;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                      // 15 microsecond inter-message gap_time.

   for(i=0;i<2;i++)
       bcmessage.data[0][i] = 0xA000+i;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //RT3 SA# WC3 TX  This message is noop at the start
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MSG_NOP;        // Set as a NOP message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // Add this message to the interrupt queue
   bcmessage.mess_command1.rtaddr   = 3;
   bcmessage.mess_command1.subaddr  = 3;
   bcmessage.mess_command1.wcount   = 3;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                      // 15 microsecond inter-message gap_time
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);
   noop_message=messno;

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // Add this message to the interrupt queue
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 4;
   bcmessage.mess_command1.wcount   = 4;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                      // 15 microsecond inter-message gap_time.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)0;
   bcmessage.control = BC_CONTROL_NOP;
   bcmessage.control |= BC_CONTROL_MFRAME_END; //End of frame

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

} 

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   static int sa_index;
   int       i,j;
   int       status;
   int       wcount;
   BT_INT tail, rtrt,mc;
   BT_UINT messno;

   char outbuf[80];

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      messno = sIntFIFO->fifo[tail].bufferID;
      status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);         
      if (status)
         return status;

      if(BusTools_BoardIsV6(cardnum))  //check for the F/w version to display time correctly
         BusTools_TimeGetFmtString(API_TTD_IRIG_NS, &bcmessage.time_tag, outbuf);  //MSB=nanoseconds
      else
         BusTools_TimeGetFmtString(API_TTD_IRIG, &bcmessage.time_tag, outbuf);     //MSB=microesconds 
      printf("BC**Message Time - %s\n",outbuf);              
 
      rtrt=0;
      mc=0;

      if(bcmessage.status & BT1553_INT_RETRY)
         printf("Retry - %d - %x\n",messno,bcmessage.status);
      else if (bcmessage.status & BT1553_INT_BROADCAST)
       printf("Broadcast - %d - %x\n",messno,bcmessage.status);
      else
       printf("%d - %x\n",messno,bcmessage.status);

      if(bcmessage.status & BT1553_INT_RT_RT_FORMAT)
      {
         printf("RT<->RT (%d) ",messno);
         rtrt=1;
      }
      else if(bcmessage.status & BT1553_INT_MODE_CODE)
      {
         printf("Mode Code --  (%d) ", messno);
         mc=1;
      }
      else if (bcmessage.mess_command1.tran_rec)
         printf("BC**RT->BC (%d)  ", messno);
      else
         printf("BC->RT (%d)  ", messno);
 
      printf("BC**RT-%d  \n",bcmessage.mess_command1.rtaddr);
      printf("BC**SA-%d  \n",bcmessage.mess_command1.subaddr);
      printf("BC**WC-%d  \n",bcmessage.mess_command1.wcount);
      if(rtrt)
      {   
         printf("BC**RT-%d  \n",bcmessage.mess_command2.rtaddr);
         printf("BC**SA-%d  \n",bcmessage.mess_command2.subaddr);
         printf("BC**WC-%d  \n",bcmessage.mess_command2.wcount);
      }
        
      printf("BC**status-1 0x%04x\n",*((WORD*)(&bcmessage.mess_status1)));
      if(rtrt)
         printf("BC**status-2 0x%04x\n",*((WORD*)(&bcmessage.mess_status2)));
         
      printf("BC**int_stat-0x%08lx\n",*((DWORD*)(&bcmessage.status)));

      wcount = bcmessage.mess_command1.wcount;
      if (wcount == 0)
         wcount = 32;           // Zero word count means 32 words.

      if(mc)
        if(wcount >=16)
           wcount = 1;

      j = wcount;               // See if this msg shorted than previous
    
      for ( i = 0; i < j; i++ ) // Display all of the data words
      {
         // If this is an RT->BC message, and there is a Message Error
         //  or No Response, than xxxx the data (since the RT did not
         //    send any data words).
         if ( i >= wcount )
         {
            printf("    ");     // Data does not apply to this msg
         }
         else if ( bcmessage.mess_command1.tran_rec &&
           (bcmessage.mess_status1.me || (bcmessage.status&BT1553_INT_NO_RESP) ) )
         {
            printf("xxxx ");      // No data was transfered because of error
         }
         else
         {
            printf("BC**%04x ",bcmessage.data[0][i]);
         }
      }
      if(bcmessage.mess_command1.rtaddr == 4)
         printf("\n_____________________________________________________________\n");
      else
         printf("\n\n");
   
      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }  

   return 0;
}

/****************************************************************************
*
*  Function:  setupThe_BC_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a BC message.  The interrupt is set for only the end-of
*     message.  
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupThe_BC_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.iNotification  = 0;
   sIntFIFO1.function       = bc_intFunction;
   sIntFIFO1.iPriority      = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.bForceStartup  = 0;
   sIntFIFO1.FilterType     =  EVENT_BC_MESSAGE;

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO1.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}