/*===========================================================================*
 * FILE:                   E X A M P L E _ B C _ R T _ B M . C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    Demonstration of BusTools/1553-API routines. This console 
 *              application shows how to configure the Remote Terminal,
 *              Bus Controller and Bus Monitor  and set up interrupt on BC, RT,
 *              BM messages.  The RT and BC callback function change the data
 *              for Transmit and Receive commands.  The Bus Monitor callback
 *              displays the data. 
 *
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "busapi.h"

void BusController_Init(int,int);
void BusMonitor_Init(int);
void RemoteTerminal_Init(int);
void setupThe_RT_IntFIFO(int cardnum);
void setupThe_BC_IntFIFO(int cardnum);
void setupThe_BM_IntFIFO(int cardnum);

BT_UINT rt_int_count[32];
API_INT_FIFO  rtIntFIFO;     // Thread FIFO structure for RTs
API_INT_FIFO sIntFIFO1;      // Thread FIFO structure for BC.
API_INT_FIFO sIntFIFO2;      // Thread FIFO structure for BM.
static unsigned bm_tcnt=0,bc_tcnt=0,rt_tcnt=0;

#if defined (__WIN32__)
int main(int argc, char **argv)                     
#else
int tst_mrt()
#endif
{
   BT_INT status, cardnum;
   BT_UINT channel,device,mode;
   char ichar;
   BT_INT i=0;

   mode = API_B_MODE | API_SW_INTERRUPT; //API_B_MODE | API_SW_INTERRUPT;  //MIL-STD-1553B with Software (Polling) interrupts enabled
   channel = CHANNEL_1;  //default channel 1
   device = 0;

   status = BusTools_API_OpenChannel(&cardnum,mode,device,channel);
   printf("BusTools_API_OpenChannel status = %d\n",status);

   status = BusTools_SetInternalBus(cardnum,EXTERNAL_BUS);  // Set up bus External
   printf("BusTools_SetInternalBus status = %d\n",status);

   BusMonitor_Init(cardnum);   
   BusController_Init(cardnum,50);
   RemoteTerminal_Init(cardnum);  

   //setupThe_BC_IntFIFO(cardnum);
   setupThe_RT_IntFIFO(cardnum);
   //setupThe_BM_IntFIFO(cardnum);

   status = BusTools_BM_StartStop(cardnum,BM_START);
   printf("BusTools_BM_StartStop status = %d\n",status);

   status = BusTools_RT_StartStop(cardnum,RT_START);
   printf("BusTools_RT_StartStop status = %d\n",status);

   status = BusTools_BC_StartStop(cardnum,BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);

   do
   {
      ichar= getchar();
   }while(ichar != 'q' && ichar != 'Q'); 

   printf("main: closing API\n");

   status = BusTools_BC_StartStop(cardnum,BC_STOP); 
   printf("BusTools_BC_StartStop status = %d\n",status);

   MSDELAY(1000);

   status = BusTools_RT_StartStop(cardnum,RT_STOP);
   printf("BusTools_RT_StartStop status = %d\n",status);

   status = BusTools_BM_StartStop(cardnum,BM_STOP);
   printf("BusTools_BM_StartStop status = %d\n",status);

   status = BusTools_RegisterFunction(cardnum, &sIntFIFO2, UNREGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);

   status = BusTools_API_Close(cardnum);//
   printf("BusTools_API_Close status = %d -- %d\n",status, rt_tcnt);

}

void BusController_Init(int cardnum,int num_buffers)
{
   int status, messno,i;
   API_BC_MBUF bcmessage;

   //initialize the BC with interrupt on end-of-message 16 us no response 14 us late response, 1sec frame (1000000 usec).     
   status = BusTools_BC_Init(cardnum,0,BT1553_INT_END_OF_MESS,0,16,14,1000000,1);
   printf("BusTools_BC_Init status = %d\n",status);

   status = BusTools_BC_MessageAlloc(cardnum,50);//num_buffers);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);

   //Receive (BC->RT) Command)
   messno = 0;
   memset((char*)&bcmessage,0,sizeof(bcmessage));       
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on bus A
   bcmessage.control |= BC_CONTROL_RETRY;         // enable interrupt on message
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;    // Start of a minor frame
   bcmessage.mess_command1.rtaddr   = 7;          
   bcmessage.mess_command1.subaddr  = 7;
   bcmessage.mess_command1.wcount   = 0;
   bcmessage.mess_command1.tran_rec = RECEIVE;    // BC->RT
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 10;                       // 10 microsecond inter-message gap.

   for(i=0;i<32;i++)
       bcmessage.data[0][i] = 0xA000+i;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Transmit (RT->BC) Command
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message
   bcmessage.mess_command1.rtaddr   = 1;
   bcmessage.mess_command1.subaddr  = 23;
   bcmessage.mess_command1.wcount   = 32;
   bcmessage.mess_command1.tran_rec = TRANSMIT;   // RT->BC
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                       // 15 microsecond inter-message gap
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);


   //Receive (BC->RT)
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message
   bcmessage.mess_command1.rtaddr   = 1;
   bcmessage.mess_command1.subaddr  = 23;
   bcmessage.mess_command1.wcount   = 32;
   bcmessage.mess_command1.tran_rec = RECEIVE;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 20;                      // 20 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Receive (BC->RT) Command
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 4;
   bcmessage.mess_command1.wcount   = 4;
   bcmessage.mess_command1.tran_rec = RECEIVE;    // BC->RT
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 50;                       // 50 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Receive (BC->RT) Command
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;      // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message
   bcmessage.mess_command1.rtaddr   = 5;
   bcmessage.mess_command1.subaddr  = 5;
   bcmessage.mess_command1.wcount   = 8;
   bcmessage.mess_command1.tran_rec = 0;          // BC->RT
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // 100 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   // RT - RT Command
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_RTRTFORMAT;    // define RT-RT message
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 10;
   bcmessage.mess_command1.wcount   = 6;
   bcmessage.mess_command1.tran_rec = 0;          //RT-RT receive command

   bcmessage.mess_command2.rtaddr   = 6;
   bcmessage.mess_command2.subaddr  = 6;
   bcmessage.mess_command2.wcount   = 6;
   bcmessage.mess_command2.tran_rec = 1;          // RT-RT transmit command
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 25;                       // 25 microsecond inter-message gap.
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Mode Code
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message
   bcmessage.mess_command1.rtaddr   = 6;
   bcmessage.mess_command1.subaddr  = 0;          // Subaddress 0 is a mode code
   bcmessage.mess_command1.wcount   = 17;         // Mode Code 17 sync w/data  
   bcmessage.mess_command1.tran_rec = RECEIVE;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // 100 microsecond inter-message gap.
   bcmessage.data[0][0] = 0xABCD;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status); 

  
   //Noop message at end-of-frame
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)0;          // repeat starting at message 0
   bcmessage.control = BC_CONTROL_NOP;            // 
   bcmessage.control |= BC_CONTROL_MFRAME_END;    // End-of-Frame

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);
} 

void RemoteTerminal_Init(int cardnum)
{
   int status;
   int RT_MBUF_COUNT;
   API_RT_CBUF cbuf;       // RT Control Buffer Legal Word Count Mask (2 words)
   API_RT_MBUF_WRITE mbuf;
   API_RT_ABUF abuf;
   int         tr;
   int         subaddress;
   int i,j,ii,jj;
   int rt_cnt = 5;
   int rt_addr[] = {1,4,5,6,7};

   RT_MBUF_COUNT = 5;

   status = BusTools_RT_Init(cardnum,0);
   printf("BusTools_RT_Init status = %d\n",status);

   /**********************************************************************
   *  Output RT control buffers
   *   Enable or disable all word counts and mode codes as specified by
   *    the caller's arguements.
   **********************************************************************/

   cbuf.legal_wordcount = 0xffffffff;  // legailize all word counts
   for ( i = 0; i < rt_cnt; i++)
   { 
      for ( tr = 0; tr < 2; tr++ )
      {
         for ( subaddress = 0; subaddress < 32; subaddress++ )
         {
            status = BusTools_RT_CbufWrite(cardnum,rt_addr[i],subaddress,tr,RT_MBUF_COUNT,&cbuf);
         }
      }
   }

   /*******************************************************************
   *  Fill in message buffers for receive and transmit sub-units.
   *******************************************************************/
   mbuf.error_inj_id = 0;
   mbuf.enable = BT1553_INT_END_OF_MESS;   // Interrupt enables
   // For all subaddresses...
   for (ii = 0; ii<rt_cnt; ii++)
   {  
      for (i=0; i<32; i++)
      {
         // For each data buffer...
         for (j=0; j<RT_MBUF_COUNT; j++)
         {
            for(jj=0;jj<32;jj++)
               mbuf.mess_data[jj] = (BT_U16BIT)(0xAA00 + jj);
            status = BusTools_RT_MessageWrite(cardnum,rt_addr[ii],i,0,j,&mbuf);
            status = BusTools_RT_MessageWrite(cardnum,rt_addr[ii],i,1,j,&mbuf);
         }
      }
   }

   /************************************************
   *  Setup RT address control block
   ************************************************/

   abuf.enable_a          = 1;    // Disable Bus A...Note that the API bits are
   abuf.enable_b          = 1;    // Disable Bus B... reversed from the HW bits.
   abuf.inhibit_term_flag = 0;
   abuf.status            = 0x0000;
   abuf.command           = 0;
   abuf.bit_word          = 0;

   for(i=0;i<rt_cnt;i++)
   {
      status = BusTools_RT_AbufWrite(cardnum,rt_addr[i],&abuf);
      printf("BusTools_RT_AbufWrite status = %d\n",status);
   }
}
void BusMonitor_Init(int cardnum)
{
 int status;

   BT_UINT actual;

   status = BusTools_BM_Init(cardnum,1,1);
   printf("BusTools_BM_Init status = %d\n",status);
   status = BusTools_BM_MessageAlloc(cardnum, 40, &actual ,BT1553_INT_END_OF_MESS); // Interrupt enabled
   printf("BusTools_BM_MessageAlloc status = %d - actual =%d\n",status,actual);
}

/****************************************************************************
*
*  Function:  rt_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen. 
****************************************************************************/
BT_INT _stdcall rt_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{
   int status;
   API_RT_MBUF_READ rt_mbuf;
   BT_INT tail;

   BT_UINT messno,rt_addr,sub_addr,trans,nextmessno;
   BT1553_TIME ttime;
   static int sflg = 0;
   char      outbuf[100];
   int wcount,i,mc;
   static int cnt = 1;
   mc = 0;
   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      printf("RT msg\n");
      if(sIntFIFO->fifo[tail].event_type & EVENT_RT_MESSAGE)
	  {
	       messno   = sIntFIFO->fifo[tail].bufferID;
          rt_addr  = sIntFIFO->fifo[tail].rtaddress;        // RT address
          trans    = sIntFIFO->fifo[tail].transrec;         // Transmit/Receive
          sub_addr = sIntFIFO->fifo[tail].subaddress;       // Subaddress number
	       status = BusTools_RT_MessageRead( cardnum, rt_addr, sub_addr, trans, messno, &rt_mbuf);
          if (status)
             return status;
          status = BusTools_RT_MessageBufferNext(cardnum, rt_addr, sub_addr, trans, &nextmessno);
		  
	       printf("\nRT = %d, current messno =  %d,next messno =  %d\n", rt_addr, messno,nextmessno);
		  
         if (rt_mbuf.mess_command.tran_rec)
             printf("RT->BC (%d) \n", messno);
          else
             printf("BC->RT (%d)  \n", messno);
          printf("RT**RT-%d  \n",rt_mbuf.mess_command.rtaddr);
          printf("RT**SA-%d  \n",rt_mbuf.mess_command.subaddr);
          printf("RT**WC-%d  \n",rt_mbuf.mess_command.wcount);
          printf("RT**mstat-0x%04x\n",*((WORD*)(&rt_mbuf.mess_status)));
          printf("RT**int_stat-0x%08lx\n",rt_mbuf.status);
          printf("\n");
       }   
       tail++;
       tail &= sIntFIFO->mask_index;
       sIntFIFO->tail_index = tail;
   }
   return 0;
}

/****************************************************************************
*
*  Function:  bm_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bm_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BM_MBUF mbuf;
   int       status;
   BT_INT tail,rtrt;
   BT_UINT maddr,junk, wcnt,i;
   char      outbuf[100];

   rtrt = 0;
   tail = sIntFIFO->tail_index;
   junk=0;

   while(tail != sIntFIFO->head_index)
   {
      maddr = sIntFIFO->fifo[tail].bufferID;
      status = BusTools_BM_MessageRead(cardnum,maddr,&mbuf);// read the BM message data
      bm_tcnt++;

      if(mbuf.int_status & BT1553_INT_RT_RT_FORMAT)
      {
         printf("RT<->RT (%d) \n",mbuf.messno);
      }

      BusTools_TimeGetString(&mbuf.time, outbuf);
      printf("BM>>>>>@%08X -- time tag = %s\n",maddr,outbuf);

      if (mbuf.command1.tran_rec)
         printf("BM>>>>> RT->BC (%d)  \n", mbuf.messno);
      else
         printf("BM>>>>> BC->RT (%d)  \n", mbuf.messno);
      printf("BM>>>>> RT-%d  \n",mbuf.command1.rtaddr);
      printf("BM>>>>> SA-%d  \n",mbuf.command1.subaddr);
      printf("BM>>>>> WC%d  \n",mbuf.command1.wcount);
      printf("BM>>>>> ST1 %4X \n",*(BT_U16BIT *)&mbuf.status1);
 
      if(mbuf.command1.wcount == 0)
         wcnt = 32;
      else
         wcnt = mbuf.command1.wcount;

      if(mbuf.command1.subaddr == 0)
         wcnt = 1;

      printf("BM>>>>> 0x%08lx\n",mbuf.int_status);
      for(i=0;i<wcnt; i++)
      {
         printf("%04X ",mbuf.value[i]);
      }
      printf("\n\n");  
      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }

   return 0;
}

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   int       bufnum;
   static int sa_index;
   int       i,j;
   int       status;
   int       wcount;
   BT_INT tail, rtrt,mc;
   BT_UINT messno;
   char outbuf[80];

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
       if(sIntFIFO->FilterType & EVENT_BC_MESSAGE)
       {
          messno = sIntFIFO->fifo[tail].bufferID;
          status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);     
          if (status)
             return status;
		  if(bcmessage.status & BT1553_INT_ERROR_BITS ) //Check for transaction errors
		  {      
			// tansaction error, do something
		  }
          rtrt=0;
          mc=0;

          BusTools_TimeGetString(&bcmessage.time_tag, outbuf);
          printf("time tag = %s\n",outbuf);
       

          if(bcmessage.status & BT1553_INT_RT_RT_FORMAT)
          {
            printf("RT<->RT (%d) ",messno);
             rtrt=1;
          }
          else if(bcmessage.status & BT1553_INT_MODE_CODE)
          {
             printf("Mode Code --  (%d) ", messno);
             mc=1;
          }
          else if (bcmessage.mess_command1.tran_rec)
             printf("BC**RT->BC (%d)  ", messno);
          else
             printf("BC->RT (%d)  ", messno);
 
          printf("BC**RT-%d  \n",bcmessage.mess_command1.rtaddr);
          printf("BC**SA-%d  \n",bcmessage.mess_command1.subaddr);
          printf("BC**WC-%d  \n",bcmessage.mess_command1.wcount);
          if(rtrt)
          {   
             printf("BC**RT-%d  \n",bcmessage.mess_command2.rtaddr);
             printf("BC**SA-%d  \n",bcmessage.mess_command2.subaddr);
             printf("BC**WC-%d  \n",bcmessage.mess_command2.wcount);
          }
        
          printf("BC**status-1 0x%04x\n",*((WORD*)(&bcmessage.mess_status1)));
          if(rtrt)
             printf("BC**status-2 0x%04x\n",*((WORD*)(&bcmessage.mess_status2)));
          
          printf("BC**int_stat-0x%08lx\n",*((DWORD*)(&bcmessage.status)));

          wcount = bcmessage.mess_command1.wcount;
          if (wcount == 0)
             wcount = 32;           // Zero word count means 32 words.

          if(mc)
            if(wcount >=16)
               wcount = 1;

          j = wcount;               // See if this msg shorted than previous
    
          if (bcmessage.control & BC_CONTROL_BUFFERA)
             bufnum = 0;            // Display data from first BC buffer
          else
             bufnum = 1;            // Display data from second BC buffer

          for ( i = 0; i < j; i++ ) // Display all of the data words
          {
             // If this is an RT->BC message, and there is a Message Error
             //  or No Response, than xxxx the data (since the RT did not
             //    send any data words).
             if ( i >= wcount )
             {
                printf("    ");     // Data does not apply to this msg
             }
             else if ( bcmessage.mess_command1.tran_rec &&
               (bcmessage.mess_status1.me || (bcmessage.status&BT1553_INT_NO_RESP) ) )
             {
                printf("xxxx ");      // No data was transfered because of error
             }
             else
             {
                printf("BC**%04x ",bcmessage.data[bufnum][i]);
             }
          }
          printf("\n");
      }
      if(sIntFIFO->FilterType & EVENT_RT_MESSAGE)
      {
          sIntFIFO->nUser[0]++;
      }
            
       tail++;
       tail &= sIntFIFO->mask_index;
       sIntFIFO->tail_index = tail;
   }

   return 0;
}


/****************************************************************************
*
*  Function:  setupThe_RT_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a RT message.  The interrupt is set for only the end-of
*     message.  
*
****************************************************************************/
void setupThe_RT_IntFIFO(int cardnum)
{
   int  status;
   // Setup the FIFO structure for this board.
   memset(&rtIntFIFO, 0, sizeof(API_INT_FIFO));
   rtIntFIFO.function       = rt_intFunction;
   rtIntFIFO.iPriority      = THREAD_PRIORITY_ABOVE_NORMAL;
   rtIntFIFO.dwMilliseconds = INFINITE;
   rtIntFIFO.FilterType     = EVENT_RT_MESSAGE;

   rtIntFIFO.FilterMask[1][0][23] =  0xffffffff;  // Enable all Word Counts on RT1 SA23 RECEIVE
  
   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &rtIntFIFO, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}

/****************************************************************************
*
*  Function:  setupThe_BC_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO for a interrupt on a BC message.
*     The interrupt is set for the end-of message.  
*
****************************************************************************/
void setupThe_BC_IntFIFO(int cardnum)
{
   int rt,sa,tr,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.function     = bc_intFunction;
   sIntFIFO1.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.FilterType     =  EVENT_BC_MESSAGE;

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
         sIntFIFO1.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA for receive commands

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}

/****************************************************************************
*
*  Function:  setupThe_BM_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO for a interrupt on a BM message.
*     The interrupt is set for the end-of message.  
*
****************************************************************************/
void setupThe_BM_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO2, 0, sizeof(API_INT_FIFO));
   sIntFIFO2.function     = bm_intFunction;
   sIntFIFO2.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO2.dwMilliseconds = INFINITE;
   sIntFIFO2.FilterType     =  EVENT_BM_MESSAGE;

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO2.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO2, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}



