/*===========================================================================*
 * FILE:            E X A M P L E _ B C _ R T _B C S T . C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    Demonstration of BusTools/1553-API routines. This example
 *              shows how to setup broadcast for BC and RT.  It also
 *              how the Remote Terminal processes Broadcast messages.
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include "busapi.h"

void setupTheIntFIFO_RT(int,int,API_INT_FIFO *);
int BusController_Init(int);
int RemoteTerminal_Init(int);

API_INT_FIFO  sIntFIFO_RT;               // RT Thread FIFO structure
BT_U16BIT  RT_ADDR[] = {2,4,6,8,10}; // This is the RT used 
#define RT_MBUF_COUNT  1                 // Number of data buffers per RT 
BT_UINT rt_bcst_int_count[] = {0,0,0,0,0,0,0,0,0,
                               0,0,0,0,0,0,0,0,0,
                               0,0,0,0,0,0,0,0,0,
                               0,0,0,0,0,0,0,0,0};

void main(int argc, char **argv)
{
   BT_INT status,i,cardnum=0;

   BT_UINT mode = API_B_MODE | API_SW_INTERRUPT;
   BT_UINT device = 0;

   status = BusTools_API_OpenChannel(&cardnum,mode,device,CHANNEL_1);
   printf("BusTools_API_Init status = %d\n",status);

   status = BusTools_SetInternalBus(cardnum,EXTERNAL_BUS);
   printf("BusTools_SetInternalBus status = %d\n",status);

   status = BusTools_SetBroadcast(cardnum, BROADCAST);      // Enable boradcast (this is the default after initialization)
   printf("BusTools_SetBroadcast status = %d\n",status);

   status = BusController_Init(cardnum);
   status = RemoteTerminal_Init(cardnum);

   setupTheIntFIFO_RT(cardnum,2,&sIntFIFO_RT);

   status = BusTools_RT_StartStop(cardnum,RT_START);
   printf("BusTools_RT_StartStop status = %d\n",status);

   status = BusTools_BC_StartStop(cardnum,BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);

   while(getc(stdin) != '\n');

   status = BusTools_BC_StartStop(cardnum,BC_STOP);
   printf("BusTools_BC_StartStop status = %d\n",status);

   status = BusTools_RT_StartStop(cardnum,RT_STOP);
   printf("BusTools_RT_StartStop status = %d\n",status);

   status = BusTools_RegisterFunction(cardnum,&sIntFIFO_RT, UNREGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);

   for(i=0;i<5;i++)
      printf("Broadcast interrupt count for RT %d = %d\n",RT_ADDR[i],rt_bcst_int_count[RT_ADDR[i]]); 

   BusTools_API_Close(cardnum);
   printf("BusTools_API_Close status = %d\n",status);
}

/****************************************************************************
*
*  Function:  intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT _stdcall intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{
   int status,wc;
   API_RT_MBUF_READ rt_mbuf;
   BT_INT tail,i;
   BT_UINT messno;
   BT_UINT rt_addr,sub_addr,trans;

   tail = sIntFIFO->tail_index;
 
   while(tail != sIntFIFO->head_index)
   {
      printf("\n____________________________________________________________\n");
      messno   = sIntFIFO->fifo[tail].bufferID;
      rt_addr  = sIntFIFO->fifo[tail].rtaddress;        // RT address
      trans    = sIntFIFO->fifo[tail].transrec;         // Transmit/Receive
      sub_addr = sIntFIFO->fifo[tail].subaddress;       // Subaddress number
	  wc       = sIntFIFO->fifo[tail].wordcount;

      status = BusTools_RT_MessageRead( cardnum, rt_addr, sub_addr, trans, messno, &rt_mbuf);
      if (status)
         return status;

      printf( "Messno = %d - RT = %d - SA = %d WC = %d - int status = %x\n",messno, rt_addr,sub_addr,wc,rt_mbuf.status);
      printf("Broadcast ");
   
      if(rt_mbuf.status & BT1553_INT_MODE_CODE)
      {
         printf("Mode Code");
         if(wc >= 16)
            wc = 1;
         else 
            wc = 0;  
      }
      else
         if(wc==0)
            wc=32;
      if(rt_mbuf.status & BT1553_INT_RT_RT_FORMAT)
         printf("RT - RT");

      printf("\n");

      for ( i = 0; i < wc; i++ ) // Display all of the data words
      {
         // If this is an RT->BC message, and there is a Message Error
         //  or No Response, than xxxx the data (since the RT did not
         //    send any data words).
         if(i%8==0 && i != 0)
            printf("\n");
         if ( i >= wc )
            printf("    ");     // Data does not apply to this msg
         else
            printf("RT**%04x ",rt_mbuf.mess_data[i]);
      }
      for(i=0;i<5;i++)
         rt_bcst_int_count[RT_ADDR[i]]++;

      printf("\n");

      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }

   return API_SUCCESS;
}

/****************************************************************************
*
*  Function:  setupTheIntFIFO1
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a BC message.  The interrupt is set for only the end-of
*     message.  
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupTheIntFIFO_RT(int cardnum,int rt, API_INT_FIFO * sIntFIFO_RT)
{
   int sa,status;
   // Setup the FIFO structure for this board.
   memset(sIntFIFO_RT, 0, sizeof(API_INT_FIFO));
   sIntFIFO_RT->function       = intFunction;
   sIntFIFO_RT->iPriority      = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO_RT->dwMilliseconds = INFINITE;
   sIntFIFO_RT->iNotification  = 0;
   sIntFIFO_RT->FilterType     = EVENT_RT_MESSAGE;
   for (sa = 0; sa < 32; sa++ )
   {
      sIntFIFO_RT->FilterMask[31][0][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos
      sIntFIFO_RT->FilterMask[31][1][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos
   }
 
   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, sIntFIFO_RT, REGISTER_FUNCTION);
}

int RemoteTerminal_Init(int cardnum)
{
   API_RT_CBUF cbuf;       // RT Control Buffer Legal Word Count Mask (2 words)
   API_RT_CBUFBROAD cbufb;
   API_RT_MBUF_WRITE mbuf;
   API_RT_ABUF abuf;
   BT_INT         tr;
   BT_INT         subaddress;
   BT_INT status,i,j;

   status = BusTools_RT_Init(cardnum,0);
   printf("BusTools_RT_Init status = %d\n",status);

   /**********************************************************************
   *  Output RT control buffers
   *   Enable or disable all word counts and mode codes as specified by
   *    the caller's arguements.
   **********************************************************************/

   cbuf.legal_wordcount = 0xffffffff;  // Legailize all word counts
   for(i = 0;i<5;i++)
   {
      for ( tr = 0; tr < 2; tr++ )
      {
         for ( subaddress = 0; subaddress < 32; subaddress++ )
         {
            status = BusTools_RT_CbufWrite(cardnum,RT_ADDR[i],subaddress,tr,
			                            RT_MBUF_COUNT,&cbuf);
         }
      }
   }
   /**********************************************************************
   * Setup the Broadcast buffer to legalize all word counts and to use 5
   * message buffers
   ***********************************************************************/
   cbufb.mbuf_count = 1;
 
   for ( subaddress = 0; subaddress < 32; subaddress++ )
   {
      cbufb.legal_wordcount[subaddress] = 0xffffffff;                   // Legailize all word counts
      status = BusTools_RT_CbufbroadWrite(cardnum,subaddress,0,&cbufb); // Write only the receive buffer
      status = BusTools_RT_CbufbroadWrite(cardnum,subaddress,1,&cbufb); // Write only the transmit buffer   
   }

   /*******************************************************************
   *  Fill in message buffers for receive and transmit sub-units.
   *******************************************************************/
   
   mbuf.error_inj_id = 0;
   // For all subaddresses...
   for (i=0; i<32; i++)
   {
      mbuf.enable = BT1553_INT_BROADCAST;   // Interrupt enables on Broadcast Message 
      // Create data buffer data
      // For each data buffer...
      for(j=0;j<5;j++)
      {
         status = BusTools_RT_MessageWrite(cardnum,RT_ADDR[i],i,0,0,&mbuf);
      }
   }
   for(i=0;i<6;i++)
     mbuf.mess_data[i] = 0xcaf0 + i; 
   status = BusTools_RT_MessageWrite(cardnum,10,3,1,0,&mbuf);

   /************************************************
   *  Setup RT address control block
   ************************************************/
   abuf.enable_a          = 1;    // Enable Bus A
   abuf.enable_b          = 1;    // Enable Bus B.
   abuf.inhibit_term_flag = 0;
   abuf.status            = 0x0000;
   abuf.command           = 0;
   abuf.bit_word          = 0;
   
   for(i=0;i<5;i++)
      status = BusTools_RT_AbufWrite(cardnum,RT_ADDR[i],&abuf);
   
   return 0;
}

int BusController_Init(int cardnum)
{
   int status, messno,i;
   API_BC_MBUF bcmessage;

   status = BusTools_BC_Init(cardnum,FRAME_START_TIMING,0,0,14,12,1000000,1);
   printf("BusTools_BC_Init status = %d\n",status);

   status = BusTools_BC_MessageAlloc(cardnum,10);
   printf("BusTools_BC_MessageAlloc status = %d\n",status); 

   messno = 0;
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control  = BC_CONTROL_MESSAGE;       // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;    // Begin minor frame
   bcmessage.mess_command1.rtaddr   = 31;         // Broadcast
   bcmessage.mess_command1.subaddr  =  0;         // SA=0 Mode Code
   bcmessage.mess_command1.wcount   = 17;         // Sync w/Data (MC17)
   bcmessage.mess_command1.tran_rec =  0;         // Receive
   bcmessage.errorid  = 0;                        // Default error injection buffer (no errors)
   bcmessage.long_gap = 0;                        // Microsecond gap from start of frame.
   bcmessage.data[0][0] = 0xABCD;                 // Data transmited with the Mode Code

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control  = BC_CONTROL_MESSAGE;       // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.mess_command1.rtaddr   = 31;         // Broadcast
   bcmessage.mess_command1.subaddr  =  0;         // SA=0 Mode Code
   bcmessage.mess_command1.wcount   =  1;         // Sync w/o Data (MC17)
   bcmessage.mess_command1.tran_rec =  1;         // Receive
   bcmessage.errorid  = 0;                        // Default error injection buffer (no errors)
   bcmessage.long_gap = 1000;                     // Microsecond gap from start of frame.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control  = BC_CONTROL_MESSAGE;       // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.mess_command1.rtaddr   = 31;         // Broadcast
   bcmessage.mess_command1.subaddr  = 16;         // SA=16
   bcmessage.mess_command1.wcount   = 32;         // Send 32 word of data
   bcmessage.mess_command1.tran_rec =  0;         // Receive
   bcmessage.errorid  = 0;                        // Default error injection buffer (no errors)
   bcmessage.long_gap = 2000;                     // Microsecond gap from start of frame.
   for(i=0;i<32;i++)
      bcmessage.data[0][i] = 0xf000 + i;                 // Data transmited with the Mode Code

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Broadcast RT-RT Message
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control  = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;        // Use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;       // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_RTRTFORMAT;     // Define RT-RT message
   bcmessage.mess_command1.rtaddr   = 31;          // Broadcast RT address
   bcmessage.mess_command1.subaddr  = 10;          // Subaddress 10
   bcmessage.mess_command1.wcount   = 6;           // Word count 6
   bcmessage.mess_command1.tran_rec = 0;           // Receive
   // Use Command 2 for RT-RT
   bcmessage.mess_command2.rtaddr   = 10;          // Transmit RT address 10
   bcmessage.mess_command2.subaddr  = 3;           // Subadress 3
   bcmessage.mess_command2.wcount   = 6;           // Word count 6
   bcmessage.mess_command2.tran_rec = 1;           // Transmit address
   bcmessage.errorid  = 0;                         // Default error injection buffer (no errors)
   bcmessage.long_gap = 3000;                      // Microsecond gap from start of frame.
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 5;
   bcmessage.mess_command1.wcount   = 5;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid  = 0;                        // Default error injection buffer (no errors)
   bcmessage.long_gap = 4000;                     // Microsecond gap from start of frame.
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status); 

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next   = 0x0;
   bcmessage.control  = BC_CONTROL_NOP;           // Do it again and again.
   bcmessage.control |= BC_CONTROL_MFRAME_END;    // End Minor-Frame
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   return status;
}

