/*===========================================================================*
 * FILE:          E X A M P L E _ B C _ S T A R T _ F R A M E . C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    Demonstration of BusTools/1553-API routines. This example 
 *              application shows how to configure the Bus Controller for an 
 *              initial frame that runs once then runs a periodic frame.  The 
 *              application uses BusTools_BC_Start to start the initial frame.at
 *              message 40.  After that the frame starting at message 
 *              zero (0) runs.
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "busapi.h"

void BusController_Init(int,int);
void setupThe_BC_IntFIFO(int cardnum);

BT_UINT rt_int_count[32];

API_INT_FIFO sIntFIFO1;      // Thread FIFO structure for BC.
static unsigned bm_tcnt=0,bc_tcnt=0,rt_tcnt=0;

int main(int argc, char **argv)                     
{
   int status,cardnum;
   unsigned channel,device,mode;
   int i=0;

   mode = API_B_MODE | API_SW_INTERRUPT;  //MIL-STD-1553B with Software (Polling) interrupts enabled
   channel = CHANNEL_1;  //default channel 1
   device = 0;           //default device 0

   status = BusTools_API_OpenChannel(&cardnum,mode,device,channel);
   printf("BusTools_API_OpenChannel status = %d\n",status);

   status = BusTools_SetInternalBus(cardnum,EXTERNAL_BUS);  // Set up External bus
   printf("BusTools_SetInternalBus status = %d\n",status);
   
   BusController_Init(cardnum,50);
   setupThe_BC_IntFIFO(cardnum);

   status = BusTools_BC_Start(cardnum,40);
   printf("BusTools_BC_StartStop status = %d\n",status);

   getchar();

   printf("main: closing API\n");

   status = BusTools_BC_StartStop(cardnum,BC_STOP); 
   printf("BusTools_BC_StartStop status = %d\n",status);

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, UNREGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);

   status = BusTools_API_Close(cardnum);//
   printf("BusTools_API_Close status = %d\n",status);
}

void BusController_Init(int cardnum,int num_buffers)
{
   int status, messno,i;
   API_BC_MBUF bcmessage;

   //initialize the BC with interrupt on end-of-message 16 us no response 14 us late response, 1sec frame (1000000 usec).     
   status = BusTools_BC_Init(cardnum,0,BT1553_INT_END_OF_MESS,0,16,14,1000,1);
   printf("BusTools_BC_Init status = %d\n",status);

   status = BusTools_BC_MessageAlloc(cardnum,50);//num_buffers);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);

   //Receive (BC->RT) Command)
   messno = 0;
   memset((char*)&bcmessage,0,sizeof(bcmessage));       
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;    // Start of a minor frame
   bcmessage.mess_command1.rtaddr   = 7;          
   bcmessage.mess_command1.subaddr  = 7;
   bcmessage.mess_command1.wcount   = 9;
   bcmessage.mess_command1.tran_rec = 0;          // BC->RT
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 5;                       // 10 microsecond inter-message gap.

   for(i=0;i<7;i++)
       bcmessage.data[0][i] = 0xA000+i;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Transmit (RT->BC) Command
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message

   bcmessage.mess_command1.rtaddr   = 2;
   bcmessage.mess_command1.subaddr  = 2;
   bcmessage.mess_command1.wcount   = 4;
   bcmessage.mess_command1.tran_rec = 1;          // RT->BC
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 5;                       // 15 microsecond inter-message gap
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Transmit (RT->BC) Command
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message
   bcmessage.mess_command1.rtaddr   = 3;
   bcmessage.mess_command1.subaddr  = 3;
   bcmessage.mess_command1.wcount   = 4;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 5;                      // 20 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Receive (BC->RT) Command
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 4;
   bcmessage.mess_command1.wcount   = 4;
   bcmessage.mess_command1.tran_rec = 0;          // BC->RT
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 5;                       // 50 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Receive (BC->RT) Command
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message

   bcmessage.mess_command1.rtaddr   = 5;
   bcmessage.mess_command1.subaddr  = 5;
   bcmessage.mess_command1.wcount   = 8;
   bcmessage.mess_command1.tran_rec = 0;          // BC->RT
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 5;                      // 100 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   // RT - RT Command
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_RTRTFORMAT;    // define RT-RT message
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 10;
   bcmessage.mess_command1.wcount   = 6;
   bcmessage.mess_command1.tran_rec = 0;          //RT-RT receive command

   bcmessage.mess_command2.rtaddr   = 6;
   bcmessage.mess_command2.subaddr  = 6;
   bcmessage.mess_command2.wcount   = 6;
   bcmessage.mess_command2.tran_rec = 1;          // RT-RT transmit command
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 5;                       // 25 microsecond inter-message gap.
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Mode Code
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message
   bcmessage.mess_command1.rtaddr   = 6;
   bcmessage.mess_command1.subaddr  = 0;          // Subaddress 0 is a mode code
   bcmessage.mess_command1.wcount   = 17;         // Mode Code 17 sync w/data  
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 5;                      // 100 microsecond inter-message gap.
   bcmessage.data[0][0] = 0xABCD;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Noop message at end-of-frame
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)0;          // repeat starting at message 0
   bcmessage.control = BC_CONTROL_NOP;            // 
   bcmessage.control |= BC_CONTROL_MFRAME_END;    // End-of-Frame

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   /////////////////////////////////////////////////////////////
   //Receive (BC->RT) Command)
   messno = 40;
   memset((char*)&bcmessage,0,sizeof(bcmessage));       
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;    // Start of a minor frame
   bcmessage.mess_command1.rtaddr   = 17;          
   bcmessage.mess_command1.subaddr  = 17;
   bcmessage.mess_command1.wcount   = 17;
   bcmessage.mess_command1.tran_rec = 0;          // BC->RT
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 10;                       // 10 microsecond inter-message gap.

   for(i=0;i<7;i++)
       bcmessage.data[0][i] = 0xA000+i;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Transmit (RT->BC) Command
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message

   bcmessage.mess_command1.rtaddr   = 12;
   bcmessage.mess_command1.subaddr  = 12;
   bcmessage.mess_command1.wcount   = 12;
   bcmessage.mess_command1.tran_rec = 1;          // RT->BC
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                       // 15 microsecond inter-message gap
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);


   //Transmit (RT->BC) Command
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message
   bcmessage.mess_command1.rtaddr   = 13;
   bcmessage.mess_command1.subaddr  = 13;
   bcmessage.mess_command1.wcount   = 13;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 20;                      // 20 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Receive (BC->RT) Command
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message
   bcmessage.mess_command1.rtaddr   = 14;
   bcmessage.mess_command1.subaddr  = 14;
   bcmessage.mess_command1.wcount   = 14;
   bcmessage.mess_command1.tran_rec = 0;          // BC->RT
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 50;                       // 50 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Receive (BC->RT) Command
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message

   bcmessage.mess_command1.rtaddr   = 15;
   bcmessage.mess_command1.subaddr  = 15;
   bcmessage.mess_command1.wcount   = 18;
   bcmessage.mess_command1.tran_rec = 0;          // BC->RT
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // 100 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   // RT - RT Command
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_RTRTFORMAT;    // define RT-RT message
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message
   bcmessage.mess_command1.rtaddr   = 14;
   bcmessage.mess_command1.subaddr  = 10;
   bcmessage.mess_command1.wcount   = 6;
   bcmessage.mess_command1.tran_rec = 0;          //RT-RT receive command

   bcmessage.mess_command2.rtaddr   = 16;
   bcmessage.mess_command2.subaddr  = 6;
   bcmessage.mess_command2.wcount   = 6;
   bcmessage.mess_command2.tran_rec = 1;          // RT-RT transmit command
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 25;                       // 25 microsecond inter-message gap.
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Mode Code
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message
   bcmessage.mess_command1.rtaddr   = 16;
   bcmessage.mess_command1.subaddr  = 0;          // Subaddress 0 is a mode code
   bcmessage.mess_command1.wcount   = 17;         // Mode Code 17 sync w/data  
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // 100 microsecond inter-message gap.
   bcmessage.data[0][0] = 0xABCD;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Noop message at end-of-frame
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)0;          // repeat starting at message 0
   bcmessage.control = BC_CONTROL_NOP;            // 
   bcmessage.control |= BC_CONTROL_MFRAME_END;    // End-of-Frame

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

} 

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   static int sa_index;
   int       status,i;
   BT_INT tail,mc=0, j,wcount;
   BT_UINT bufaddr;
   static int nFlag = 1;
   BT_U16BIT temp[34];
   char outbuf[80];

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      bufaddr = sIntFIFO->fifo[tail].buffer_off;

      bc_tcnt++;
      status = BusTools_BC_MessageBufferRead(cardnum,bufaddr,&bcmessage);//New function to read BC data
      if (status)
         return status;
      
      wcount = bcmessage.mess_command1.wcount;

       printf("%d - %x\n",bcmessage.messno,bcmessage.status);
      
	   if(BusTools_BoardIsV6(cardnum))
          strncpy(outbuf,"NANO",4);  
       BusTools_TimeGetString(&bcmessage.time_tag, outbuf);
       printf("tag time = %s\n",outbuf);     

       if (bcmessage.mess_command1.tran_rec)
          printf("BC**RT->BC (%d)  ", bcmessage.messno);
       else
          printf("BC->RT (%d)  ", bcmessage.messno);

       printf("BC**RT-%d  \n",bcmessage.mess_command1.rtaddr);
       printf("BC**SA-%d  \n",bcmessage.mess_command1.subaddr);
       printf("BC**WC-%d  \n",bcmessage.mess_command1.wcount);
       printf("BC**status-1 0x%04x\n",*((WORD*)(&bcmessage.mess_status1)));
       printf("BC**int_stat-0x%08lx\n",*((DWORD*)(&bcmessage.status)));


       if (wcount == 0)
          wcount = 32;           // Zero word count means 32 words.

       if(bcmessage.status & BT1553_INT_MODE_CODE)
          mc=1;

       if(mc)
         if(wcount >=16)
           wcount = 1;

       j = wcount;               // See if this msg shorted than previous
       for ( i = 0; i < j; i++ ) // Display all of the data words
       {
          if ( i >= wcount )
          {
             printf("    ");     // Data does not apply to this msg
          }
          else if ( bcmessage.mess_command1.tran_rec &&
            (bcmessage.mess_status1.me || (bcmessage.status & BT1553_INT_NO_RESP) ) )
          {
             printf("xxxx ");      // No data was transfered because of error
          }
          else
          {
             printf("BC**%04x ",bcmessage.data[0][i]);
          }
          if(bcmessage.mess_command1.tran_rec==0)   
             temp[i] = bcmessage.data[0][i] + 10;          // Increment the data on the RCV command
       }                                                  // There 10 pre-load buffers, so add 10 to increment by 1
       printf("\n________________________________________________________________\n");  
       if(bcmessage.mess_command1.tran_rec==0)
          BusTools_BC_DataBufferUpdate(cardnum,bufaddr,wcount,temp);  // Write out new data to current buffer
       
       tail++;
       tail &= sIntFIFO->mask_index;
       sIntFIFO->tail_index = tail;
   }

   return 0;
}



/****************************************************************************
*
*  Function:  setupThe_BC_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO for a interrupt on a BC message.
*     The interrupt is set for the end-of message.  
*
****************************************************************************/
void setupThe_BC_IntFIFO(int cardnum)
{
   int rt,sa,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.iNotification = 0;
   sIntFIFO1.function     = bc_intFunction;
   sIntFIFO1.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.bForceStartup  = 0;
   sIntFIFO1.FilterType     =  EVENT_BC_MESSAGE;

   for ( rt=0; rt < 32; rt++ )
      for (sa = 0; sa < 32; sa++ )
         sIntFIFO1.FilterMask[rt][0][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA for receive commands

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}
