/*============================================================================*
 * FILE:       E X A M P L E _ B C _ T R I G G E R _ O N E S H O T . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:    EXAMPLE PROGRAM
 * This example program demonstrates how to start the Bus Controller using a 
 * trigger input.  The example sets up a simple BC message list with 1-R-1-32 
 * and 2-T-2-32, in a 500ms minor frame.  This BC list runs until stopped by 
 * user input.  Data is automatically displayed for 1-R-1-32 and 2-T-2-32.  
 * The user hits Enter to quit, shutdown the application and exit. 
 * BusTools_BC_Trigger is used to setup a 1shot trigger that starts the Bus 
 * 
 * Controller running.  After calling BusTools_BC_StartStop the BC waits for 
 * an external trigger input before running.  The trigger can be from an external 
 * trigger source or generated by the board.  If the trigger source in the board 
 * you must wrap discrete 7 and 8 together. 
 *
 * Note:  not all boards have discrete channels or are configure with the 
 * discrete 7 and 8.  You will need to check the configuration of the board 
 * installed to make sure that discrete channels are available.
 *
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
	08/15/01	Initial version.
	03/15/02    Removed conio.h and getc calls.
    09/20/04    Added option to initialize with BusTools_API_LoadChannel.
    05/26/10    Update to use internally generated trigger option.
*/

#include "busapi.h"
#include <stdio.h>

//---------------------------------------------------------------------------------
// Function Prototypes
void setupTheBusController(BT_INT ch_id);
void setupThe_BC_IntFIFO(int cardnum);
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO);

API_INT_FIFO sIntFIFO1;

// Main program
int main(int argc, char **argv)
{
   BT_INT status, ch_id;
   char c;

   // Initialize API and board.

   int dev_num, mode;

   if(argc == 2)
      dev_num = atoi(argv[1]);
   else
      dev_num = 0;  

   // Open the device and get the channel id.
   mode = API_B_MODE | API_SW_INTERRUPT;  // 1553B protocol, use SW interrupts.
   status = BusTools_API_OpenChannel( &ch_id, mode, dev_num, CHANNEL_1);
   printf("BusTools_API_OpenChannel status = %d\n",status);

   // Select External Bus.
   printf("Bus select . . . ");
   status = BusTools_SetInternalBus(ch_id, EXTERNAL_BUS);
   printf("BusTools_SetInternalBus status = 5d\n",status);

   setupTheBusController(ch_id);  // Setup The Bus Controller and enable external trigger to start	 
   setupThe_BC_IntFIFO(ch_id);    // Register a user-callback function to process BC messages
       
   status = BusTools_BC_StartStop(ch_id, BC_START);  // Start the BC. No messages will transact until 
                                                     // Trigger by an external pulse
   printf("BusTools_BC_StartStop status = %d\n",status);

   printf("Would you like to use 1553 board to generate the trigger Y/N: ");
   scanf("%c",&c);
   printf("\n");
            
   if(c=='y'||c=='Y')
   { 
      printf("(You will need to jumper pins 9 and 10 on the transition module D50 connector)\n"); 
      fflush(stdin);
      printf("Hit Enter when ready to start the BC witht the trigger\n");
      getchar();

      if(BusTools_BoardIsV6(ch_id))
      {
         status = BusTools_SetV6TrigOut(ch_id, DISCRETE, 7);            //Enable Trigger out on Discrete 7
         printf("BusTools_DiscreteSetIO status = %d\n",status);

         status = BusTools_SetV6TrigIn(ch_id, DISCRETE, 8);             //Enable Trigger in on Discrete 8
         printf("BusTools_DiscreteSetIO status = %d\n",status);

         status = BusTools_DiscreteWrite(ch_id,7,0);                    //Clear Discrete 7
         printf("BusTools_DiscreteSetIO status = %d\n",status);
      }
      else
      {
         status = BusTools_DiscreteSetIO(ch_id,0x080,0x3ff);              //Set the DISC8 for out and DISC7 for in
         printf("BusTools_DiscreteSetIO status = %d\n",status);

         status = BusTools_DiscreteTriggerOut(ch_id,TRIGGER_OUT_DIS_7);
         printf("BusTools_DiscreteTriggerOut status = %d\n",status);

         status = BusTools_DiscreteTriggerIn(ch_id,TRIGGER_IN_DIS_8);
         printf("BusTools_DiscreteTriggerOut status = %d\n",status);
      }               
      BusTools_ExtTriggerOut(ch_id,100);  // Generate 100 uS pulse

   }
   else
   {
      if(BusTools_BoardIsV6(ch_id))
      {
         status = BusTools_SetV6TrigIn(ch_id, DISCRETE, 8);             //Enable Trigger in on Discrete 8
         printf("BusTools_DiscreteSetIO status = %d\n",status);
      }
      else
      {
         status = BusTools_DiscreteSetIO(ch_id,0x080,0x3ff);              //Set the DISC8 for out and DISC7 for in
         printf("BusTools_DiscreteSetIO status = %d\n",status);

         status = BusTools_DiscreteTriggerOut(ch_id,TRIGGER_OUT_DIS_7);
         printf("BusTools_DiscreteTriggerOut status = %d\n",status);
      }
      // Now BC is up and running, wait for user to tell us to stop or read data.
      printf("\nBC READY FOR EXTERNAL TRIGGER.  ASSERT TRIGGER SIGNAL TO START BC.\n"); 
      fflush(stdin); 
   }

   printf("Hit Enter to quit\n");
   getchar();

   // Stop the BC.
   status = BusTools_BC_StartStop(ch_id, BC_STOP);
   printf("BusTools_BC_StartStop status = %d\n",status);   

   // We're done.  Close API and board
   status = BusTools_API_Close(ch_id);
   printf("BusTools_API_Close status = %d\n",status);

} // End of main

void setupTheBusController(BT_INT ch_id)
{
   BT_INT status,messno,i;
   API_BC_MBUF bc_msg;	// Structures to define BC messages

   // Initialize BC for no interrupts, no retries, no resp at 16us, late resp at 14us,
   // minor frame time of 20000us, 1 data buffer per message.

   status = BusTools_BC_Init(ch_id, 0, BT1553_INT_END_OF_MESS, 0, 16, 14, 500000, 1);
   printf("BusTools_BC_Init status = %d\n",status);

   // Allocate memory on the board for two message blocks.
   status = BusTools_BC_MessageAlloc(ch_id, 2);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);
		
   // Define messages and write them to the allocated board memory.
   // First message (1-R-1-32).
   messno = 0;
   memset((char*)&bc_msg,0,sizeof(bc_msg)); 
   bc_msg.messno   = messno;                  // Message number
   bc_msg.control  = BC_CONTROL_MESSAGE;      // This is a standard BC message.
   bc_msg.control |= BC_CONTROL_BUSA;         // Send message on bus A (primary).
   bc_msg.control |= BC_CONTROL_BUFFERA;      // Only using one buffer, buffer A
   bc_msg.control |= BC_CONTROL_INTERRUPT;    // Log to interrupt queue 
   bc_msg.control |= BC_CONTROL_MFRAME_BEG;   // Beginning of minor frame.
   bc_msg.messno_next			 = messno + 1;// Next message number, go to msg #1
   bc_msg.mess_command1.rtaddr	 = 1;         // Command word 1, RT address
   bc_msg.mess_command1.subaddr  = 1;         // Command word 1, Subaddress
   bc_msg.mess_command1.tran_rec = 0;         // Command word 1, transmit (1) or receive (0)
   bc_msg.mess_command1.wcount	 = 0;         // Command word 1, word count, 0-31, 0=32 words
   bc_msg.errorid				 = 0;         // Error injection buffer ID, no err inj
   bc_msg.gap_time				 = 10;        // Intermessage gap time in microseconds
   // Fill data buffer
   for (i=0; i<32; i++)
      bc_msg.data[0][i] = 0xAB00 + i;

   // Write the message to board memory
   status = BusTools_BC_MessageWrite(ch_id, messno, &bc_msg);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;
   memset((char*)&bc_msg,0,sizeof(bc_msg));
   bc_msg.messno   = messno;                  // Message number
   bc_msg.control  = BC_CONTROL_MESSAGE;      // This is a standard BC message.
   bc_msg.control |= BC_CONTROL_BUSA;         // Send message on bus A (primary).
   bc_msg.control |= BC_CONTROL_BUFFERA;      // Only using one buffer, buffer A.
   bc_msg.control |= BC_CONTROL_INTERRUPT;    // Log to interrupt queue 
   bc_msg.control |= BC_CONTROL_MFRAME_END;   // End of minor frame.
   bc_msg.messno_next			  = 0;        // Next message number, go to msg #0.
   bc_msg.mess_command1.rtaddr	  = 2;        // Command word 1, RT address
   bc_msg.mess_command1.subaddr   = 2;        // Command word 1, Subaddress
   bc_msg.mess_command1.tran_rec  = 1;        // Command word 1, transmit (1) or receive (0)
   bc_msg.mess_command1.wcount	  = 0;        // Command word 1, word count, 0-31, 0=32 words
   bc_msg.errorid				  = 0;        // Error injection buffer ID, no err inj
   bc_msg.gap_time				  = 10;       // Intermessage gap time in microseconds

   // Write the message to board memory
   status = BusTools_BC_MessageWrite(ch_id, messno, &bc_msg);
   printf("BusTools_BC_MessageWrite status = %d\n",status);
	
   // Enable the Bus Controller to start on a trigger input	
   status = BusTools_BC_Trigger(ch_id, BC_TRIGGER_ONESHOT);
   printf("BusTools_BC_MessageWrite status = %d\n",status);
}

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   static int sa_index;
   int       status;
   BT_INT tail;
   BT_UINT messno;
   static int nFlag = 1;

   char outbuf[80];

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      printf("_________________________________________________________\n");
      messno = sIntFIFO->fifo[tail].bufferID;
      status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);
      if (status)
         return status;
      printf("%d - %x\n",messno,bcmessage.status);

      if(BusTools_BoardIsV6(cardnum))
         BusTools_TimeGetFmtString(API_TTD_IRIG_NS, &bcmessage.time_tag, outbuf);
      else
         BusTools_TimeGetFmtString(API_TTD_IRIG, &bcmessage.time_tag, outbuf);

      printf("tag time = %s\n",outbuf);     

      if (bcmessage.mess_command1.tran_rec)
         printf("BC**RT->BC (%d)  ", messno);
      else
         printf("BC->RT (%d)  ", messno);
 
      printf("BC**RT-%d  \n",bcmessage.mess_command1.rtaddr);
      printf("BC**SA-%d  \n",bcmessage.mess_command1.subaddr);
      printf("BC**WC-%d  \n",bcmessage.mess_command1.wcount);
      printf("BC**status-1 0x%04x\n",*((WORD*)(&bcmessage.mess_status1)));
      printf("BC**int_stat-0x%08lx\n",*((DWORD*)(&bcmessage.status)));
            
      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }

   return API_SUCCESS;
}

/****************************************************************************
*
*  Function:  setupThe_BC_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO for a interrupt on a BC message.
*     The interrupt is set for the end-of message.  
*
****************************************************************************/
void setupThe_BC_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.iNotification = 0;
   sIntFIFO1.function     = bc_intFunction;
   sIntFIFO1.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.bForceStartup  = 0;
   sIntFIFO1.FilterType     =  EVENT_BC_MESSAGE;

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO1.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}
