/*============================================================================*
 * FILE:                E X A M P L E _ B C B M _ 2 C H . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:    EXAMPLE PROGRAM
 *
 *              This is a 2 CHANNEL example program that sets up Channel 2
 *              as A BM and another 1 as a BC on a multi-channel
 *              board.
 *
 *              The first channel is setup as BM to monitor the BC messages.
 *              This example tests that 2 message are received by the BM one
 *              Bus A and the other on Bus B.  It also validates the data.
 *
 *              The first channel is setup as BC with a list of two messages, 
 *				1-R-1-32 and 1-T-2-32.  This BC list runs once.
 *              
 *
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
	04/20/04	Initial version.  RSW
    09/20/04    Added option to initialize with BusTools_API_LoadChannel.  RSW
*/


#include <stdio.h>
#include "busapi.h"

static BT_U16BIT pMessage0[] = { 0x0, 0x55, 0xaa, 0xff, 0xf0, 0x0f, 0xa5, 0x5a, 9,10,11,12,13,14,15,16,
                                17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32};
static BT_U16BIT pMessage1[] = {32,31,30,29,28,27,26,25,24,23,22,21,20,19,18,17,
                                16,15,14,13,12,11,10, 9, 0x5a, 0xa5, 0x0f, 0xf0, 0xff, 0xaa, 0x55, 0x0};
//---------------------------------------------------------------------------------
// function protocols and globals
//---------------------------------------------------------------------------------
API_INT_FIFO  sIntFIFO;     // Thread FIFO structure for RT

void bc_init(BT_INT ch_id);
void bm_init(BT_INT ch_id);

//---------------------------------------------------------------------------------


// Main program
int main(int argc, char **argv)
{
   BT_INT  status, ch1_id, ch2_id;
   BT_UINT mcnt, actual;
   API_BM_MBUF bmmessage[3];      // BM message being written to board

   /*************************************************************************************/
   // Initialize first channel as remote terminal.
   int dev_num, mode;

   if(argc == 2)
      dev_num = atoi(argv[1]);
   else
      dev_num = 0;  // default to device 0

   mode = API_B_MODE | API_SW_INTERRUPT;  // 1553B protocol, use SW interrupts.

   // Open the device and get the channel id.
   status = BusTools_API_OpenChannel( &ch1_id, mode, dev_num, CHANNEL_1);
   printf("BusTools_API_OpenChannel ch1 status = %d\n",status);

   // check for a multi-channel board
   if(BusTools_GetChannelCount(ch1_id) == 1)
   {
      printf("This is a single-channel board.  This test requires a mutli-channel board.\n");
      printf("Hit enter to exit\n");
      getchar();
      exit(0);
   }

   // Open the device and get the channel id.
   status = BusTools_API_OpenChannel( &ch2_id, mode, dev_num, CHANNEL_2);
   printf("BusTools_API_OpenChannel ch2 status = %d\n",status);

   // Select External Bus on the two channels.
   status = BusTools_SetInternalBus(ch1_id, EXTERNAL_BUS);
   printf("BusTools_SetInternalBus status = %d\n",status);

   status = BusTools_SetInternalBus(ch2_id, EXTERNAL_BUS);
   printf("BusTools_SetInternalBus status = %d\n",status);

    // Initialize the Bus Monitor.
   printf("BM Init . . . ");
   status = BusTools_BM_Init(ch2_id, BM_ENABLE_BUS, BM_ENABLE_BUS);
   printf("BusTools_BM_Init = %d\n",status);

   status = BusTools_BM_MessageAlloc(ch2_id, 20, &actual, BT1553_INT_END_OF_MESS);
   printf("BusTools_SetInternalBus = %d\n",status);

   bc_init(ch1_id);             // Initialize the BC

   // Start the Remote Terminal on channel 1
   status = BusTools_BM_StartStop(ch2_id, BM_START);
   printf("BusTools_BM_StartStop status = %d\n",status);

   // Start the bus controller.on channel 2
   status = BusTools_BC_StartStop(ch1_id, BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);
   
   while (BusTools_BC_IsRunning2(ch1_id) == API_BC_IS_RUNNING){}; // while BC is running
  
   status = BusTools_BM_ReadLastMessageBlock(ch2_id, -1,-1,-1,&mcnt,bmmessage);
   if(mcnt!=2)
   {
      printf("Test fails wrong number of BM messagages detected %d\n",mcnt);
   }

   if(mcnt == 2)
   {
      int i;
	  printf("\nMessage 1 Expected Size:%d Value: [",sizeof(pMessage0));
      for(i=0; i < 32; i++)
         printf("%x ",pMessage0[i]);
      printf("]\n");

	  printf("\nMessage 2 Expected Size:%d Value: [",sizeof(pMessage0));
      for(i=0; i < 32; i++)
         printf("%x ",pMessage1[i]);
      printf("]\n");

      status = memcmp(pMessage0, bmmessage[0].value, sizeof(pMessage0));
      if ( status )
      {
         printf("BM Message 1 (bus A) data fail\n");
      }
      status = memcmp(pMessage1, bmmessage[1].value, sizeof(pMessage1));
      if ( status )
      {
         printf("BM Message 2 (bus B)  data fail\n");
      }

	  printf("\nMessage 1 Received  Size:%d Value: [",sizeof(bmmessage[0].value));
      for(i=0; i < 32; i++)
         printf("%x ",bmmessage[0].value[i]);
      printf("]\n");

	  printf("\nMessage 2 Received  Size:%d Value: [",sizeof(bmmessage[1].value));
      for(i=0; i < 32; i++)
         printf("%x ",bmmessage[1].value[i]);
      printf("]\n");

   }
   // Stop the RT
   status = BusTools_BM_StartStop(ch2_id, BM_STOP);
   printf("BusTools_BM_StartStop status = %d\n",status);

   // Close each channel 
   status = BusTools_API_Close(ch1_id);
   printf("BusTools_API_Close status = %d\n",status);

   status = BusTools_API_Close(ch2_id);
   printf("BusTools_API_Close status = %d\n",status); 
  

} // End of main



void bc_init(BT_INT ch_id)
{
   API_BC_MBUF		    bc_msg;  	  // Structures to define BC messages
   BT_INT status,i;
   BT_UINT messno;

   // Initialize for BC relative gap, no interrupts, no retries, no resp at 16us, late resp at 14us,
   // Minor frame time of 500000us, 1 data buffer per message.
   printf("BC Init . . . ");
   status = BusTools_BC_Init(ch_id, REL_GAP, NO_INTERUPT, NO_BC_RETRY, 16, 14, 500000, 1);
   if (status != API_SUCCESS) printf("ERROR = %d\n",status);

   // Allocate memory on the board for two message blocks.
   printf("\nMSG Alloc . . . ");
   status = BusTools_BC_MessageAlloc(ch_id, 4);  //Allocate memory for two message buffers
   if (status != API_SUCCESS)
      printf("ERROR = %d\n",status);
		
   // Define messages and write them to the allocated board memory.
   // First message (1-R-1-32).
   messno = 0;
   memset((char*)&bc_msg,0,sizeof(bc_msg)); // Clear the BC Message Buffer
   bc_msg.messno				 = messno;  // Message number
   bc_msg.control = BC_CONTROL_MESSAGE;		// This is a standard BC message.
   bc_msg.control |= BC_CONTROL_BUSA;	    // Send message on bus A (primary).
   bc_msg.control |= BC_CONTROL_BUFFERA;	// Only using one buffer, buffer A
   bc_msg.control |= BC_CONTROL_MFRAME_BEG;	// Beginning of minor frame.
   bc_msg.messno_next	= messno + 1;		// Next message number, go to msg #1
   bc_msg.mess_command1.rtaddr	 = 1;		// Command word 1, RT address
   bc_msg.mess_command1.subaddr  = 1;		// Command word 1, Subaddress
   bc_msg.mess_command1.tran_rec = 0;		// Command word 1, transmit (1) or receive (0)
   bc_msg.mess_command1.wcount	 = 0;		// Command word 1, word count, 0-31, 0=32 words
   bc_msg.errorid				 = 0;		// Error injection buffer ID, no err inj
   bc_msg.gap_time				 = 10;		// Intermessage gap time in microseconds

   // Fill data buffer
   for (i=0; i<32; i++)
      bc_msg.data[0][i] = pMessage0[i];

   // Write the message to board memory
   status = BusTools_BC_MessageWrite(ch_id, messno, &bc_msg);
   if (status != API_SUCCESS)
      printf("ERROR = %d\n",status);

   // Second message (2-T-2-32).
   messno++;
   memset((char*)&bc_msg,0,sizeof(bc_msg)); // Clear the BC Message Buffer
   bc_msg.messno				= messno;   // Message number
   bc_msg.control = BC_CONTROL_MESSAGE;		// This is a standard BC message.
   bc_msg.control |= BC_CONTROL_BUSA;	    // Send message on bus B (secondary).
   bc_msg.control |= BC_CONTROL_BUFFERA;	// Only using one buffer, buffer A.
   bc_msg.messno_next			 = messno+1;		// Next message number, go to msg #0.
   bc_msg.mess_command1.rtaddr	 = 1;		// Command word 1, RT address
   bc_msg.mess_command1.subaddr  = 2;		// Command word 1, Subaddress
   bc_msg.mess_command1.tran_rec = 0;		// Command word 1, transmit (1) or receive (0)
   bc_msg.mess_command1.wcount	 = 0;		// Command word 1, word count, 0-31, 0=32 words
   bc_msg.errorid				 = 0;		// Error injection buffer ID, no err inj
   bc_msg.gap_time				 = 10;		// Intermessage gap time in microseconds\

   // Fill data buffer
   for (i=0; i<32; i++)
      bc_msg.data[0][i] = pMessage1[i];

   // Write the message to board memory
   status = BusTools_BC_MessageWrite(ch_id, messno, &bc_msg);
   if (status != API_SUCCESS)
      printf("ERROR = %d\n",status);

   //Noop message at end-of-frame
   messno++;
   memset((char*)&bc_msg,0,sizeof(bc_msg));
   bc_msg.messno = messno;
   bc_msg.messno_next = (BT_U16BIT)0;          // repeat starting at message 0
   bc_msg.control = BC_CONTROL_LAST;           // 

   status = BusTools_BC_MessageWrite(ch_id,messno,&bc_msg); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);
}
