/*============================================================================*
 * FILE:                E X A M P L E _ B C R T _ System_Test . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:    EXAMPLE PROGRAM
 *
 *              This is a 2 CHANNEL system test example program that sets up one
 *              channel as RT and another channel as BC on a multi-channel
 *              board.
 *
 *              The first channel is setup as RT1 with two subaddresses,
 *              SA1 RECEIVE and SA2 TRANSMIT.  BusTools_RegisterFunction is
 *              called to implement a user callback function to process the
 *              RT tranmit message data.
 *
 *              The second channel is setup as BC with a list of two messages,
 *				1-R-1-32 and 1-T-2-32 in a 500ms minor frame,   This BC list runs
 *				until stopped by user input.
 *				
 *				Message data[0] is initialized to 0xDEAD and data[1]is 0xBEEF.  The data are
 *				verified upon receiving and results with a pass or fail is reported.
 *
 *              A interrupt callback function is setup by calling
 *              BusTools_RegisterFunction. The rt_intFunction displays the RT
 *              data.  The user can also switch between Transformer or direct
 *              couple or dump the channels memory contents through command-line
 *              input.
 *
 *              This test requires a multi-channel board and either a physical
 *              connection between any two channels.
 *              
 *				The command input arguments 
 *				argv[1]= dev_num  Device number
 *				argv[2]= Frist channel
 *				argv[3] = Second channel
 *				argv[4] = Bus A or Bus B
 *              argv[5] = Test durationSec
 *   
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
	04/20/04	Initial version.  RSW
    09/20/04    Added option to initialize with BusTools_API_LoadChannel.  RSW
*/


#include <stdio.h>
#include "busapi.h"
#include <time.h>

//---------------------------------------------------------------------------------
// function protocols and globals
//---------------------------------------------------------------------------------
API_INT_FIFO  sIntFIFO;     // Thread FIFO structure for RT

void bc_init(BT_INT ch_id, BT_INT bus);
void rt_init(BT_INT ch_id);
void setupThe_RT_IntFIFO(int cardnum);
BT_INT _stdcall rt_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO);
int retVal = 1;
//---------------------------------------------------------------------------------


// Main program
int main(int argc, char **argv)
{
   BT_INT  status, ch1_id, ch2_id, board_type, durationSec;
   BT_INT  exit_loop=0;
   char  c;
   BT_INT bus;

   /*************************************************************************************/
   // Initialize first channel as remote terminal.
   int dev_num, mode, chan;
   if (argc >= 6)
   {
        sscanf (argv[1], "%d", &dev_num);
        sscanf (argv[2], "%d", &ch1_id);
        sscanf (argv[3], "%d", &ch2_id);
        sscanf (argv[4], "%d", &bus);
        sscanf (argv[5], "%d", &durationSec);

   }
   //if(argc == 2)
   //   dev_num = atoi(argv[1]);
   else
   {
       printf("Enter args\n");
       return 2;
   }
   mode = API_B_MODE | API_SW_INTERRUPT;  // 1553B protocol, use SW interrupts.

   // Open the device and get the channel id.
   switch (ch1_id)
   {
      case(0):
         status = BusTools_API_OpenChannel( &ch1_id, mode, dev_num, CHANNEL_1);
         break;
      case(1):
         status = BusTools_API_OpenChannel( &ch1_id, mode, dev_num, CHANNEL_2);
         break;
      case(2):
         status = BusTools_API_OpenChannel( &ch1_id, mode, dev_num, CHANNEL_3);
         break;
      case(3):
         status = BusTools_API_OpenChannel( &ch1_id, mode, dev_num, CHANNEL_4);
         break;
   } 
   // check for a multi-channel board
   if(BusTools_GetChannelCount(ch1_id) == 1)
   {
      printf("This is a single-channel board.  This test requires a mutli-channel board.\n");
      printf("Hit enter to exit\n");
      getchar();
      exit(0);
   }
   // Open the device and get the channel id.
   switch (ch2_id)
   {
      case(0):
         status = BusTools_API_OpenChannel( &ch2_id, mode, dev_num, CHANNEL_1);
         break;
      case(1):
         status = BusTools_API_OpenChannel( &ch2_id, mode, dev_num, CHANNEL_2);
         break;
      case(2):
         status = BusTools_API_OpenChannel( &ch2_id, mode, dev_num, CHANNEL_3);
         break;
      case(3):
         status = BusTools_API_OpenChannel( &ch2_id, mode, dev_num, CHANNEL_4);
         break;
   }

   // Select External Bus on the two channels.
   status = BusTools_SetInternalBus(ch1_id, EXTERNAL_BUS);
   
   status = BusTools_SetInternalBus(ch2_id, EXTERNAL_BUS);

   // Set the time tag mode for the RT to start at 0 and free run.
   // Make sure you select the correct time base for the channel's firmware
   if(BusTools_BoardIsV6(ch1_id))  // Use nanosecond time base for V6 firmare
      status = BusTools_TimeTagMode(ch1_id, API_TTD_RELM_NS,API_TTI_ZERO,API_TTM_FREE,NULL,0,0,0);
   else                            // Else use microsecond time base for all other firmware
      status = BusTools_TimeTagMode(ch1_id, API_TTD_RELM,API_TTI_ZERO,API_TTM_FREE,NULL,0,0,0);

   board_type = BusTools_GetBoardType(ch1_id);

   if(board_type == QPCI1553 || board_type == QPCX1553) // If test bus available use that
   {
      status = BusTools_SetTestBus(ch1_id,TEST_BUS_ENABLE);
      BusTools_SetTestBus(ch2_id,TEST_BUS_ENABLE);
   }
   else // Otherwise connect the two channels together using proper termination
   { //assume Transformer coupled settings. Otherwise these need to change!!
      BusTools_SetVoltage(ch1_id,  19.5, 1);
      BusTools_SetVoltage(ch2_id,  19.5, 1);
   }

   rt_init(ch1_id);             // Initialize the RT
   bc_init(ch2_id, bus);             // Initialize the BC
   setupThe_RT_IntFIFO(ch1_id); // Setup the interrupt callback

   // Start the Remote Terminal on channel 1
   status = BusTools_RT_StartStop(ch1_id, RT_START);
   if(status != API_SUCCESS )
   {
      printf("BusTools_RT_StartStop status = %d\n",status);
      return 3;
   }
   // Start the bus controller.on channel 2
   status = BusTools_BC_StartStop(ch2_id, BC_START);
   if(status != 0)
   {
      printf("BusTools_RT_StartStop failed. Status = %d\n",status);
   }
   
   //Uncomment for continuous test
   //do
   //{
   //   c=getchar();
   //} while (c != 'Q' && c != 'q'); // End of do-while
   if(status != API_SUCCESS )
   {
      printf("BusTools_BC_StartStop failed. Status = %d\n",status);
      return 4;
   }
   //run for duration in seconds
   while(durationSec != 0)
   {
      usleep (1000000);
      durationSec = durationSec-1;
   }
   // Stop the BC.
   status = BusTools_BC_StartStop(ch2_id, BC_STOP);

   // Stop the RT
   status = BusTools_RT_StartStop(ch1_id, RT_STOP);

   status = BusTools_RegisterFunction(ch1_id, &sIntFIFO, UNREGISTER_FUNCTION);

   // Close each channel
   status = BusTools_API_Close(ch1_id);

   status = BusTools_API_Close(ch2_id);
   if(retVal != 0)
   {
      printf("\nFAIL\n");
   }
   return retVal;

} // End of main

/****************************************************************************
*
*  Function:  rt_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a RT end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT _stdcall rt_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{
   int status;
   API_RT_MBUF_READ rt_mbuf;
   BT_INT tail;

   BT_UINT messno,rt_addr,sub_addr,trans;
   char outbuf[100];
   int wcount,i;

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
	  messno   = sIntFIFO->fifo[tail].bufferID;
      rt_addr  = sIntFIFO->fifo[tail].rtaddress;        // RT address
      trans    = sIntFIFO->fifo[tail].transrec;         // Transmit/Receive
      sub_addr = sIntFIFO->fifo[tail].subaddress;       // Subaddress number

	  status = BusTools_RT_MessageRead( cardnum, rt_addr, sub_addr, trans, messno, &rt_mbuf);
      if (status)
         return status;

      BusTools_TimeGetString(&rt_mbuf.time, outbuf);

      //if (rt_mbuf.mess_command.tran_rec)
      //   printf("RT->BC (%d) \n", messno);
      //else
      //   printf("BC->RT (%d)  \n", messno);

      if (rt_mbuf.mess_command.wcount == 0)
         wcount = 32;           // Zero word count means 32 words.
      else
         wcount = rt_mbuf.mess_command.wcount ;               // See if this msg shorted than previous

      //printf("RT**%04x ",rt_mbuf.mess_data[0]);
      //printf("RT**%04x ",rt_mbuf.mess_data[1]);
      if(rt_mbuf.mess_data[0] != 0xDEAD && rt_mbuf.mess_data[1] != 0xBEEF)
      {
         printf("\nFAIL\n");
      }
      else
      {
         printf("\nPASS\n");
         retVal = 0;
      }
      //printf("\n\n");

      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }
   return 0;
}


/****************************************************************************
*
*  Function:  setupThe_RT_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a
*     interrupt on a RT message.  The interrupt is set for only the end-of
*     message.
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupThe_RT_IntFIFO(int cardnum)
{
   int status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO, 0, sizeof(sIntFIFO));
   sIntFIFO.function     = rt_intFunction;
   sIntFIFO.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO.dwMilliseconds = INFINITE;
   sIntFIFO.iNotification  = 0;
   sIntFIFO.FilterType     = EVENT_RT_MESSAGE;

   //                  rt tr   sa
   sIntFIFO.FilterMask[1] [0] [1] =  0xffffffff;  // Enable RT1 RX SA1
   sIntFIFO.FilterMask[1] [1] [2] =  0xffffffff;  // Enable RT1 TX SA2

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO, REGISTER_FUNCTION);
}

void rt_init(BT_INT ch_id)
{
   API_RT_ABUF			abuf;	      // RT address buffer structure.
   API_RT_CBUF			cbuf;         // RT control buffer structures.
   API_RT_MBUF_WRITE	msg_buffer_write;
   BT_UINT RX=0, TX=1,rtaddr=1,sub1=1, sub2=2, rt_buffer_count=1, bufid=0;
   BT_INT status, i;

   // Set up an RT.
   status = BusTools_RT_Init(ch_id, 0);
   if (status)
   {
      printf("BusTools_RT_Init -- %s\n",BusTools_StatusGetString(status));
      exit(0);
   }

   // Setup RT address buffer for our RT (RT1)
   abuf.enable_a = 1;			// Respond on bus A
   abuf.enable_b = 1;			// Respond on bus B
   abuf.inhibit_term_flag = 0;	// Inhibit terminal flag in status word
   abuf.status   = 0x0000;		// Set status word no status bits set
   abuf.bit_word = 0x0000;		// Set BIT word (for mode code 19)
   status = BusTools_RT_AbufWrite(ch_id, rtaddr, &abuf);
   if (status != API_SUCCESS)
      printf("Error %d on BusTools_RT_AbufWrite.\n",status);

   cbuf.legal_wordcount = 0xFFFFFFFF;  // any word count is legal.

   status = BusTools_RT_CbufWrite(ch_id, rtaddr, sub1, RX, rt_buffer_count, &cbuf);
   if (status != API_SUCCESS)
      printf("Error %d on BusTools_RT_CbufWrite.\n",status);

   status = BusTools_RT_CbufWrite(ch_id, rtaddr, sub2, TX, rt_buffer_count, &cbuf);
   if (status != API_SUCCESS)
      printf("Error %d on BusTools_RT_CbufWrite.\n",status);

   // Enable interrupts
   msg_buffer_write.enable = BT1553_INT_END_OF_MESS;		// Interrupt on the end of this message
   msg_buffer_write.error_inj_id = 0;	                    // No error injection

   status = BusTools_RT_MessageWrite(ch_id, rtaddr, sub1, RX, bufid, &msg_buffer_write);
   if (status != API_SUCCESS)
      printf("Error %d on BusTools_RT_MessageWrite.\n",status);

   //Add data to the transmit buffer
   //for (i=0; i<32; i++)
      //msg_buffer_write.mess_data[i] = 0xAB00 + i;           // Data
   msg_buffer_write.mess_data[0] = 0xDEAD;           // Data
   msg_buffer_write.mess_data[1] = 0xBEEF;           // Data
   status = BusTools_RT_MessageWrite(ch_id, rtaddr, sub2, TX, bufid, &msg_buffer_write);
   if (status != API_SUCCESS)
      printf("Error %d on BusTools_RT_MessageWrite.\n",status);

}

void bc_init(BT_INT ch_id, BT_INT bus)
{
   API_BC_MBUF		    bc_msg;  	  // Structures to define BC messages
   BT_INT status,i;
   BT_UINT messno;

   // Initialize for BC relative gap, no interrupts, no retries, no resp at 16us, late resp at 14us,
   // Minor frame time of 500000us, 1 data buffer per message.
   status = BusTools_BC_Init(ch_id, REL_GAP, NO_INTERUPT, NO_BC_RETRY, 16, 14, 500000, 1);
   if (status != API_SUCCESS) printf("ERROR = %d\n",status);

   // Allocate memory on the board for two message blocks.
   status = BusTools_BC_MessageAlloc(ch_id, 2);  //Allocate memory for two message buffers
   if (status != API_SUCCESS)
      printf("ERROR = %d\n",status);

   // Define messages and write them to the allocated board memory.
   // First message (1-R-1-32).
   messno = 0;
   memset((char*)&bc_msg,0,sizeof(bc_msg)); // Clear the BC Message Buffer
   bc_msg.messno				 = messno;  // Message number
   bc_msg.control = BC_CONTROL_MESSAGE;		// This is a standard BC message.
   if(bus == 0)
   {
      bc_msg.control |= BC_CONTROL_CHANNELA;	// Send message on bus A (primary).
      //bc_msg.control |= BC_CONTROL_BUSA;
      //printf("Testing on BUS A\n");
   }
   else
   {
      bc_msg.control |= BC_CONTROL_CHANNELB;	// Send message on bus A (primary).
      //bc_msg.control |= BC_CONTROL_BUSB;
      //printf("Testing on BUS B\n");
   }
   //bc_msg.control |= BC_CONTROL_BUSB;	// Send message on bus A (primary).
   bc_msg.control |= BC_CONTROL_BUFFERA;	// Only using one buffer, buffer A
   bc_msg.control |= BC_CONTROL_MFRAME_BEG;	// Beginning of minor frame.
   bc_msg.messno_next	= messno + 1;		// Next message number, go to msg #1
   bc_msg.mess_command1.rtaddr	 = 1;		// Command word 1, RT address
   bc_msg.mess_command1.subaddr  = 1;		// Command word 1, Subaddress
   bc_msg.mess_command1.tran_rec = 0;		// Command word 1, transmit (1) or receive (0)
   bc_msg.mess_command1.wcount	 = 0;		// Command word 1, word count, 0-31, 0=32 words
   bc_msg.errorid				 = 0;		// Error injection buffer ID, no err inj
   bc_msg.gap_time				 = 10;		// Intermessage gap time in microseconds

   // Fill data buffer
   //for (i=0; i<32; i++)
   //   bc_msg.data[0][i] = 0xCD00 + i;
   bc_msg.data[0][0] = 0xDEAD;
   bc_msg.data[0][1] = 0xBEEF;
   // Write the message to board memory
   status = BusTools_BC_MessageWrite(ch_id, messno, &bc_msg);
   if (status != API_SUCCESS)
      printf("ERROR = %d\n",status);

   // Second message (2-T-2-32).
   messno++;
   memset((char*)&bc_msg,0,sizeof(bc_msg)); // Clear the BC Message Buffer
   bc_msg.messno = messno;   // Message number
   bc_msg.control = BC_CONTROL_MESSAGE;		// This is a standard BC message.
   if(bus == 0)
   {
      bc_msg.control |= BC_CONTROL_CHANNELA;	// Send message on bus A (primary).
      //bc_msg.control |= BC_CONTROL_BUSA;
      //printf("\n------------Testing on BUS A------------\n");
   }
   else
   {
      bc_msg.control |= BC_CONTROL_CHANNELB;	// Send message on bus A (primary).
      //bc_msg.control |= BC_CONTROL_BUSB;
      //printf("\n------------Testing on BUS B------------\n");
   }
   bc_msg.control |= BC_CONTROL_BUFFERA;	// Only using one buffer, buffer A.
   bc_msg.control |= BC_CONTROL_MFRAME_END;	// End of minor frame.
   bc_msg.messno_next			 = 0;		// Next message number, go to msg #0.
   bc_msg.mess_command1.rtaddr	 = 1;		// Command word 1, RT address
   bc_msg.mess_command1.subaddr  = 2;		// Command word 1, Subaddress
   bc_msg.mess_command1.tran_rec = 1;		// Command word 1, transmit (1) or receive (0)
   bc_msg.mess_command1.wcount	 = 0;		// Command word 1, word count, 0-31, 0=32 words
   bc_msg.errorid				 = 0;		// Error injection buffer ID, no err inj
   bc_msg.gap_time				 = 10;		// Intermessage gap time in microseconds

   // Write the message to board memory
   status = BusTools_BC_MessageWrite(ch_id, 1, &bc_msg);
   if (status != API_SUCCESS)
      printf("ERROR = %d\n",status);
}
