/*============================================================================*
 * FILE:                 E X A M P L E _ B I T . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:    EXAMPLE PROGRAM
 *              This example program that initializes a board, runs Internal Built-In-
 *              Test and the Cable Wrap Test.
 *
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
	05/10/01	Initial version.  RSW
	05/25/01	Minor cleanup, added MY_CARD_NUM to call to BusTools_API_Close.
	03/15/02	Removed conio.h and getc calls.  RSW.
    09/20/04    Added option to initialize with BusTools_API_LoadChannel.  RSW
*/

#include "busapi.h"
#include <stdio.h>


// Main program
int main(int argc, char ** argv) 
{
   int status, cardnum;
   char c;

   BT_INT device, mode;

   if(argc == 2)
      device = atoi(argv[1]);
   else
      device = 0;

   printf("Initializing API with BusTools_API_OpenChannel . . . ");

   // Open the device and get the channel id.
   mode = API_B_MODE | API_SW_INTERRUPT;  // 1553B protocol, use SW interrupts.
   status = BusTools_API_OpenChannel( &cardnum, mode, device, CHANNEL_1);
   printf("BusTools_API_OpenChannel status = %d\n",status);


   //Check structure sizes.  Can be done before initialization
   if(BusTools_BoardIsV6(cardnum))
      status = BusTools_BIT_StructureAlignmentCheckV6(1);
   else
      status = BusTools_BIT_StructureAlignmentCheck(1);

   // Run Built-In-Tests
   printf("\nRunning internal BIT, 10 iterations . . . ");
   status = BusTools_BIT_InternalBit(cardnum, 10);
   printf("BusTools_BIT_InternalBit status = %d\n",status);

   printf("\nEnsure that bus cabling is configured for cable wrap test.\nInput Enter when ready.\n Or Q/q to Quit");
   scanf("%c",&c);
   if(c =='q' || c =='Q')
   {
      printf("Exit without Cable Warp test\n");
   }
   else
   {
      printf("Running cable wrap BIT, 10 iterations . . . ");
      status = BusTools_BIT_CableWrap(cardnum, 10);
      printf("BusTools_BIT_CableWrap status = %d\n",status);
   }

   // Close API and board
   printf("\nClosing API . . . ");
   status = BusTools_API_Close(cardnum);
   printf("BusTools_API_Close status = %d\n",status);

   return API_SUCCESS;
}