/*============================================================================*
 * FILE:       E X A M P L E _ B M _ F I L T E R _ M E S S A G E S . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:  
 * This example program sets up a Bus Monitor to record the message traffic to a 
 * file. The user is prompted for the number of messages to capture.  The program 
 * then captures those messages and writes them to a file. This is a version of 
 * example_bm_recorder that demonstrates message filtering. This program only 
 * captures messages for RT1 SA2 TX.
 * 
 * This is a monitor only example.  It needs something to generating bus traffic
 * that includes a RT1 SA2 TX message to capture data.
 *				
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
    08/21/13   Compatible with all O/S and F/W version
*/

#include "busapi.h"
#include <stdio.h>

void setupThe_BM_IntFIFO(cardnum);
BT_INT  _stdcall bm_REC_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO);
void setupTheBusMonitor(BT_INT ch_id);

API_INT_FIFO sIntFIFO;
BT_INT        msg_count = 0;
FILE          *BMDfile;
BT_INT   num_msgs_to_capture;

// Main program
int main(int argc, char **argv) 
{
   BT_INT   status, ch_id;
   BT_UINT  device, mode;
   BT_INT   max_buffers_read = 0;

   char    c;
   
   printf("Initializing API with BusTools_API_OpenChannel . . . ");

   if(argc == 2)
      device = atoi(argv[1]);
   else
      device = 0;

   // Open the device and get the channel id.
   mode = API_B_MODE | API_SW_INTERRUPT;  // 1553B protocol, use SW interrupts.
   status = BusTools_API_OpenChannel( &ch_id, mode, device, CHANNEL_1);
   printf("BusTools_API_OpenChannel = %d\n",status);

   // Select External Bus.
   status = BusTools_SetInternalBus(ch_id, EXTERNAL_BUS);
   printf("BusTools_SetInternalBus = %d\n",status);

   setupTheBusMonitor(ch_id);
   setupThe_BM_IntFIFO(ch_id);   // This function uses BusTools_RegisterFunction to set up a user-callback 
                                   // that records BM data

   printf("Hit Enter to start Monitoring.\n");
   scanf("%c",&c);


   BusTools_DumpMemory(ch_id,DUMP_ALL, "BM_FILTER_DUMP.txt","BM FILTERING");

   // Start monitoring.
   printf("BM Start . . . ");
   status = BusTools_BM_StartStop(ch_id, BM_START);
   printf("BusTools_BM_StartStop = %d\n",status);

   getchar();
   
   status = BusTools_BM_StartStop(ch_id,BM_STOP);
   printf("BusTools_BM_StartStop = %d\n",status);  

   status = BusTools_RegisterFunction(ch_id, &sIntFIFO, UNREGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);		
			
   // We're done.  Close API and board
   printf("\nClosing API . . . ");
   status = BusTools_API_Close(ch_id);

   printf("FINISHED.\n");
} // End of main

void setupTheBusMonitor(BT_INT ch_id)
{
   BT_INT status,sa,tr,rt;
   API_BM_CBUF control_buffer;
   BT_UINT NUM_BM_BUFFERS, num_bm_buffers_actual;

   // Initialize the Bus Monitor.
   status = BusTools_BM_Init(ch_id, 1, 1);
   printf("BusTools_BM_Init = %d\n",status);

   BusTools_DumpMemory(ch_id,DUMP_ALL, "BM_INIT.txt","BM FILTERING");

   if(BusTools_GetBoardType(ch_id)==R15USB)  // Set the number of BM Message stored in the BM Buffer
      NUM_BM_BUFFERS = 1000;                 // For R15-USB make sure to allocate a large number of messages
   else
      NUM_BM_BUFFERS = 100;                  // Otherwise a smaller number                

   // By default the API capture of messages for all RT/SA/TR/WC combinations.
   // You can set filters to limit monitored messages to specific RT/SA/TR/WC combinations
   // By using the BusTools_BM_FilterWrite function.
   // Code below sets a filter to ONLY capture messages for:
   // RT1 SA2 TRANSMIT WORD COUNT less than 16 MODE CODE greater than 16

   // Disable monitoring of ALL RT/SA/TR combinations
   for (rt=0; rt<32; rt++)
   {
      for (sa=0; sa<32; sa++)
      {
         for (tr=0; tr<2; tr++)
         {
            control_buffer.t.wcount   = 0x00000000;  // NO word counts accepted.
            control_buffer.pass_count = 1;

            status = BusTools_BM_FilterWrite(ch_id, rt, sa, tr, &control_buffer);
            if(status)
               printf("BusTools_BM_FilterWrite status %d\n",status);
         }
      }
   }

   // Enable monitoring for our RT/SA of interest, RT1 SA2 RECEIVE.
   control_buffer.t.wcount   = 0x01000000;  // Word counts 2.
   control_buffer.pass_count = 1; 
  
   // Set the filter to only allow RT1 SA2 RX messages
   rt=1; sa=2; tr=0;
   status = BusTools_BM_FilterWrite(ch_id, rt, sa, tr, &control_buffer);
   printf("BusTools_BM_FilterWrite = %d\n",status);

   // Enable monitoring for our RT/SA of interest, RT1 SA2 Mode Code (Transmit).
   control_buffer.t.modecode = 0xFFFF0000;  // Mode codes greater than 16.
   control_buffer.pass_count = 1;

   rt=1; sa=0; tr=1;
   status = BusTools_BM_FilterWrite(ch_id, rt, sa, tr, &control_buffer);
   printf("BusTools_BM_FilterWrite = %d\n",status);

   status = BusTools_BM_MessageAlloc(ch_id, NUM_BM_BUFFERS, &num_bm_buffers_actual, BT1553_INT_END_OF_MESS);
   printf("BusTools_SetInternalBus = %d\n",status);

}

/****************************************************************************
*
*  Function:  bm_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message on a Bus Monitor  message. This function read the data 
*     in the BM message and prints message data to the screen.
****************************************************************************/
BT_INT _stdcall bm_intFunction(BT_UINT ch_id, struct api_int_fifo *sIntFIFO)
{
   BT_INT         tail, wcnt, status, i, messno;
   API_BM_MBUF    mbuf;
   char           outbuf[80];

   // Loop through all entries in the FIFO
   // Fetch entries from the FIFO: Get the tail pointer and extract the entry
   // it points to.   When (head == tail) then FIFO is empty.
   tail = sIntFIFO->tail_index;
   while ( tail != sIntFIFO->head_index )
   {
      //  Process a BM interrupt:
      //sIntFIFO->fifo[tail].buffer_off       // Byte address of buffer
      //sIntFIFO->fifo[tail].rtaddress        // RT address
      //sIntFIFO->fifo[tail].transrec         // Transmit/Receive
      //sIntFIFO->fifo[tail].subaddress       // Subaddress number
      //sIntFIFO->fifo[tail].wordcount        // Word count
      //sIntFIFO->fifo[tail].bufferID         // BM message number

      messno = sIntFIFO->fifo[tail].bufferID;

      // Read the message from the board.
      status = BusTools_BM_MessageRead(ch_id, messno, &mbuf);

      //print out the BM data.  NOTE: FOR EXAMPLE ONLY 
      // printing not recommended in user-callback functions.
		
      if(BusTools_BoardIsV6(ch_id))
         BusTools_TimeGetFmtString(API_TTD_IRIG_NS, &mbuf.time, outbuf);
      else
         BusTools_TimeGetFmtString(API_TTD_IRIG, &mbuf.time, outbuf);

      printf("BM>>>>>@%08X -- time tag = %s\n",outbuf);


      if(mbuf.int_status & BT1553_INT_RETRY)
         printf("Retry - %d\n",messno);

      if (mbuf.int_status & BT1553_INT_BROADCAST)
         printf("Broadcast - %d\n",messno);

      if(mbuf.int_status & BT1553_INT_RT_RT_FORMAT)
         printf("BM>>>>> RT<->RT (%d) \n",mbuf.messno);
      else if(mbuf.int_status & BT1553_INT_MODE_CODE)  
         printf("BM>>>>> Mode Code (%d) \n",mbuf.messno);
      else if (mbuf.command1.tran_rec)
         printf("BM>>>>> RT->BC (%d)  \n", mbuf.messno);
      else
         printf("BM>>>>> BC->RT (%d)  \n", mbuf.messno);
          
      printf("BM>>>>> RT-%d  \n",mbuf.command1.rtaddr);
      printf("BM>>>>> SA-%d  \n",mbuf.command1.subaddr);
      printf("BM>>>>> WC%d  \n",mbuf.command1.wcount);
      printf("BM>>>>> ST1 %4X \n",*(BT_U16BIT *)&mbuf.status1);
 
      if(mbuf.command1.wcount == 0)
         wcnt = 32;
      else
         wcnt = mbuf.command1.wcount;

      printf("BM>>>>> 0x%08lx\n",mbuf.int_status);
      for(i=0;i<wcnt; i++)
      {
         if(i%8==0 && i>0)
            printf("\n");
         printf("%04X ",mbuf.value[i]);
      }
      printf("\n");

      // Now update and store the tail pointer.
      tail++;                         // Next entry
      tail &= sIntFIFO->mask_index;   // Wrap the index
      sIntFIFO->tail_index = tail;    // Save the index
   }
   return( API_SUCCESS );
}

/****************************************************************************
*
*  Function:   
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a BM message.  The interrupt is set for the end-of
*     message.  
****************************************************************************/
void setupThe_BM_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;

   // Setup the FIFO structure for this board.
   memset(&sIntFIFO, 0, sizeof(API_INT_FIFO));   // clear out the API_INT_FIFO structure
   sIntFIFO.function       = bm_intFunction;     // Pointer to the user-callback function 
   sIntFIFO.iPriority      = THREAD_PRIORITY_TIME_CRITICAL; // System priority
   sIntFIFO.dwMilliseconds = INFINITE;           // Wait forever 
   sIntFIFO.FilterType     = EVENT_BM_MESSAGE;   // This will record BM messages

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register the user function.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);

}