/*============================================================================*
 * FILE:        E X A M P L E _ B M _ P R O C E S S _ M E S S A G E S . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:   
 * This example program sets up a Bus Monitor to process each message
 * transacted on the 1553 bus.  The example shows how to use 
 * BusTools_RegisterFunction to setup  a user-callback function to processes 
 * individual Bus Monitor Messages.  It also shows how to write a user-callback 
 * that processes each Bus Monitor message.  In the callback function the Bus 
 * Monitor message data is displayed on the console.  This is a monitor only 
 * example, you will need to connect to a 1553 bus with traffic to capture 
 * messages.
 *				
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
    08/21/13   Compatible with all O/S and F/W version
*/

#include "busapi.h"
#include <stdio.h>

void setupThe_BM_IntFIFO(int ch_id);
BT_INT  _stdcall bm_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO);
void bc_init(int cardnum);

API_INT_FIFO sIntFIFO;
API_INT_FIFO sIntFIFO1;
BT_INT        msg_count = 0;

// Main program
int main(int argc, char **argv) 
{
   BT_INT   status, ch_id;
   BT_UINT  num_bm_buffers_actual, device, mode;
   BT_INT   exit_loop = 0;
   BT_UINT NUM_BM_BUFFERS;
   char    c;
   
   printf("Initializing API with BusTools_API_OpenChannel . . . ");

   if(argc == 2)
      device = atoi(argv[1]);
   else
      device = 0;

   // Open the device and get the channel id.
   mode = API_B_MODE | API_SW_INTERRUPT;  // 1553B protocol, use HW Only interrupts.
   status = BusTools_API_OpenChannel( &ch_id, mode, device, CHANNEL_1);
   printf("BusTools_API_OpenChannel = %d\n",status);

   // Set the time mode for both channels
   status = BusTools_TimeTagMode(ch_id, API_TTD_IRIG_NS, API_TTI_IRIG, API_TTM_FREE, NULL, 0, 0, 0 );
   printf("BusTools_TimeTagMode BC status = %d (%s)\n",status, BusTools_StatusGetString(status));

   // Initialize the Bus Monitor.
   printf("BM Init . . . ");
   status = BusTools_BM_Init(ch_id,BM_ENABLE_BUS, BM_ENABLE_BUS);  // Enable bus needed only for v4.99 and below
   printf("BusTools_BM_Init = %d\n",status);


   // Select External Bus.
   printf("Bus select . . . ");
   status = BusTools_SetInternalBus(ch_id, EXTERNAL_BUS);
   printf("BusTools_SetInternalBus = %d\n",status);

   if(BusTools_GetBoardType(ch_id)==R15USB)  // Set the number of BM Message stored in the BM Buffer
      NUM_BM_BUFFERS = 1000;                 // For R15-USB make sure to allocate a large number of messages
   else
      NUM_BM_BUFFERS = 100;                  // Otherwise a smaller number                

   printf("BM MSG Alloc . . . ");
   status = BusTools_BM_MessageAlloc(ch_id, NUM_BM_BUFFERS, &num_bm_buffers_actual, BT1553_INT_END_OF_MESS);
   printf("BusTools_SetInternalBus = %d\n",status);

   // The API enables capture of messages for all RT/SA/TR/WC combinations.

   bc_init(ch_id);                 // Set up a BC so we don't need an external one.
   setupThe_BM_IntFIFO(ch_id);     // This function uses BusTools_RegisterFunction to set up a user-callback 
                                   // That records BM data

   printf("Hit Enter to start Monitoring.\n");
   printf("  Enter 'Q/q' to stop Monitoring.\n");
   printf("  Enter 'D/d' to dump memory to file.\n");
   getchar();

   // Start monitoring.
   printf("BM Start . . . ");
   status = BusTools_BM_StartStop(ch_id, BM_START);
   printf("BusTools_BM_StartStop = %d\n",status);

   status = BusTools_BC_StartStop(ch_id, BC_START);
   printf("BusTools_BC_StartStop = %d\n",status);

   do
   {
      c = getchar();
      switch (c)
      {
         case 'd':
         case 'D':
            BusTools_DumpMemory(ch_id, DUMP_ALL, "example_bm2_dump.txt","Example_bm2");
            break;
         case 'q':
         case 'Q':
            exit_loop = 1;
            break;
         default:
            continue;
      }            
   }while(!exit_loop);
   
   status = BusTools_BM_StartStop(ch_id,BM_STOP);
   printf("BusTools_BM_StartStop = %d\n",status);  

   status = BusTools_BC_StartStop(ch_id,BC_STOP);
   printf("BusTools_BC_StartStop = %d\n",status);

   status = BusTools_RegisterFunction(ch_id, &sIntFIFO, UNREGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);		
			
   // We're done.  Close API and board
   printf("\nClosing API . . . ");
   status = BusTools_API_Close(ch_id);

   printf("FINISHED.\n");
} // End of main


void bc_init(int cardnum)
{
   int status, messno,i;
   API_BC_MBUF bcmessage;

   // Initialize BC for End-of-Message interrupts, no retries, no resp at 16us, late resp at 14us,
   // minor frame time of 500000us, 1 data buffer per message.
   status = BusTools_BC_Init(cardnum, 0, BT1553_INT_END_OF_MESS, 0, 16, 14, 1000000, 1);
   printf("BusTools_BC_Init status = %d\n",status);
   
   // Allocate memory on the board for 10 message buffers.
   // Only three are used in this example.
   status = BusTools_BC_MessageAlloc(cardnum, 10);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);
		
   // Write the Bus Controllere messages to the allocated message buffers.

   //RT2 SA2 WC2 RX  
   messno = 0;                                     // Set the message buffer index to the first buffer
   memset((char*)&bcmessage,0,sizeof(bcmessage));  // Clear out the message buffer data       
   bcmessage.messno = messno;                      // Current buffer index
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);// Link to next message buffer
   bcmessage.control = BC_CONTROL_MESSAGE;         // Set as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;        // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;           // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;     // Start of the frame
   bcmessage.mess_command1.rtaddr   = 2;           // RT Address 2
   bcmessage.mess_command1.subaddr  = 2;           // Subassress 2
   bcmessage.mess_command1.wcount   = 2;           // Word count 2
   bcmessage.mess_command1.tran_rec = 0;           // receive command.
   bcmessage.errorid = 0;                          // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                        // 15 microsecond inter-message gap_time.
   // Fill in the data for the receive command
   for(i=0;i<2;i++)
       bcmessage.data[0][i] = 0xA000+i;            // Add data for the specified number of data elements
   // Write the message buffer data to the buffer
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //RT3 SA3 WC3 TX  
   messno++;                                       // Index to the next message buffer                                             
   memset((char*)&bcmessage,0,sizeof(bcmessage));  // Clear out the message buffer data
   bcmessage.messno = messno;                      // Current buffer index
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);// Link to next message buffer
   bcmessage.control = BC_CONTROL_MESSAGE;         // Set as a NOP message
   bcmessage.control |= BC_CONTROL_BUFFERA;        // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSB;           // Transmit on Bus B
   bcmessage.mess_command1.rtaddr   = 3;           // RT Address 3
   bcmessage.mess_command1.subaddr  = 3;           // Subaddress 3
   bcmessage.mess_command1.wcount   = 3;           // Word count 3
   bcmessage.mess_command1.tran_rec = 1;           // Transmit command 
   bcmessage.errorid = 0;                          // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                        // 15 microsecond inter-message gap_time
   // Write the message buffer data to the buffer 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //RT4 SA4 WC4 TX 
   messno++;                                       // Index to the next message buffer                                             
   memset((char*)&bcmessage,0,sizeof(bcmessage));  // Clear out the message buffer data
   bcmessage.messno = messno;                      // Current buffer index
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);// Link to next message buffer
   bcmessage.control = BC_CONTROL_MESSAGE;         // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;        // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;           // Transmit on Bus A
   bcmessage.mess_command1.rtaddr   = 4;           // RT address 4
   bcmessage.mess_command1.subaddr  = 4;           // Subaddress 4
   bcmessage.mess_command1.wcount   = 4;           // Word count 4
   bcmessage.mess_command1.tran_rec = 1;           // Transmit command
   bcmessage.errorid = 0;                          // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                        // 15 microsecond inter-message gap_time.
   // Write the message buffer data to the buffer
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                       // Index to the next message buffer                                             
   memset((char*)&bcmessage,0,sizeof(bcmessage));  // Clear out the message buffer data
   bcmessage.messno = messno;                      // Current buffer index
   bcmessage.messno_next = (BT_U16BIT)0;           // link back to the first message buffer
   bcmessage.control = BC_CONTROL_NOP;             // Noop buffer
   bcmessage.control |= BC_CONTROL_MFRAME_END;     // End of frame
   // Write the message buffer data to the buffer
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

} 

/****************************************************************************
*
*  Function:  bm_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message on a Bus Monitor  message. This function read the data 
*     in the BM message and prints message data to the screen.
****************************************************************************/
BT_INT _stdcall bm_intFunction(BT_UINT ch_id, struct api_int_fifo *sIntFIFO)
{
   BT_INT         tail, wcnt, status, i, messno;
   API_BM_MBUF    mbuf;
   char           outbuf[80];

   // Loop through all entries in the FIFO
   // Fetch entries from the FIFO: Get the tail pointer and extract the entry
   // it points to.   When (head == tail) then FIFO is empty.
   tail = sIntFIFO->tail_index;
   while ( tail != sIntFIFO->head_index )
   {
      //  Process a BM interrupt:
      //sIntFIFO->fifo[tail].buffer_off       // Byte address of buffer
      //sIntFIFO->fifo[tail].rtaddress        // RT address
      //sIntFIFO->fifo[tail].transrec         // Transmit/Receive
      //sIntFIFO->fifo[tail].subaddress       // Subaddress number
      //sIntFIFO->fifo[tail].wordcount        // Word count
      //sIntFIFO->fifo[tail].bufferID         // BM message number

      messno = sIntFIFO->fifo[tail].bufferID;

      // Read the message from the board.
      status = BusTools_BM_MessageRead(ch_id, messno, &mbuf);

      //print out the BM data.  NOTE: FOR EXAMPLE ONLY 
      // printing not recommended in user-callback functions.
		
      if(BusTools_BoardIsV6(ch_id))
         BusTools_TimeGetFmtString(API_TTD_IRIG_NS, &mbuf.time, outbuf);
      else
         BusTools_TimeGetFmtString(API_TTD_IRIG, &mbuf.time, outbuf);

      printf("BM>>>>> time tag = %s\n",outbuf);

      if(mbuf.int_status & BT1553_INT_RETRY)
         printf("Retry - %d\n",messno);

      if (mbuf.int_status & BT1553_INT_BROADCAST)
         printf("Broadcast - %d\n",messno);

      if(mbuf.int_status & BT1553_INT_RT_RT_FORMAT)
         printf("BM>>>>> RT<->RT (%d) \n",mbuf.messno);
      else if(mbuf.int_status & BT1553_INT_MODE_CODE)  
         printf("BM>>>>> Mode Code (%d) \n",mbuf.messno);
      else if (mbuf.command1.tran_rec)
         printf("BM>>>>> RT->BC (%d)  \n", mbuf.messno);
      else
         printf("BM>>>>> BC->RT (%d)  \n", mbuf.messno);
          
      printf("BM>>>>> RT-%d  \n",mbuf.command1.rtaddr);
      printf("BM>>>>> SA-%d  \n",mbuf.command1.subaddr);
      printf("BM>>>>> WC%d  \n",mbuf.command1.wcount);
      printf("BM>>>>> ST1 %4X \n",*(BT_U16BIT *)&mbuf.status1);
 
      if(mbuf.command1.wcount == 0)
         wcnt = 32;
      else
         wcnt = mbuf.command1.wcount;

      printf("BM>>>>> 0x%08lx\n",mbuf.int_status);
      for(i=0;i<wcnt; i++)
      {
         if(i%8==0 && i>0)
            printf("\n");
         printf("%04X ",mbuf.value[i]);
      }
      printf("\n");

      // Now update and store the tail pointer.
      tail++;                         // Next entry
      tail &= sIntFIFO->mask_index;   // Wrap the index
      sIntFIFO->tail_index = tail;    // Save the index
   }
   return( API_SUCCESS );
}

/****************************************************************************
*
*  Function:   
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a BM message.  The interrupt is set for the end-of
*     message.  
****************************************************************************/
void setupThe_BM_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;

   // Setup the FIFO structure for this board.
   memset(&sIntFIFO, 0, sizeof(API_INT_FIFO));   // clear out the API_INT_FIFO structure
   sIntFIFO.function       = bm_intFunction;     // Pointer to the user-callback function 
   sIntFIFO.iPriority      = THREAD_PRIORITY_TIME_CRITICAL; // System priority
   sIntFIFO.dwMilliseconds = INFINITE;           // Wait forever 
   sIntFIFO.FilterType     = EVENT_BM_MESSAGE;   // This will record BM messages

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register the user function.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);

}


