/*============================================================================*
 * FILE:        E X A M P L E _ B M _ P R O C E S S _ M S G _B L K . C
 *============================================================================*
 *
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:   EXAMPLE PROGRAM
 *
 *             This example sets up a Bus Monitor to process each Bus Monitor 
 *             message trasacted. This example also shows how to process the 
 *             messages using the function BusTools_BM_ReadLastMessageBlock.  
 *             This function parses the interrupt queue to determine the BM 
 *             messages that have transacted since the last call.  For a BM 
 *             message to record the BM initialization must define an interrupt 
 *             condition like BT1553_INT_END_OF_MESS.  This processing differs 
 *             from using BusTools_RegisterFunction in that your application must
 *             provide a timing loop to call BusTools_BM_ReadLastMessageBlock 
 *             periodically.  The timing must be set so the BM messages (and 
 *             other messages that may be recorded in the interrupt queue do
 *             not overwrite unprocessed entries.  This examples uses a 50 
 *             millisecond delay.  
 *
 *             This application use the Windows function kbhit to break out of 
 *             timing loop.  If using this example on non-Windows systems, you 
 *             will need to provide a kbhit function.
 *				
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
    08/21/13   Compatible with all O/S and F/W version
*/

#include "busapi.h"
#include <stdio.h>
#ifdef WIN32
#include <conio.h>
#endif //WIN32

BT_INT bm_readFunction(BT_UINT cardnum);


// Main program
int main(int argc, char **argv) 
{
   BT_INT   status, ch_id;
   BT_UINT  num_bm_buffers_actual, device, mode;
   BT_INT   exit_loop = 0;
   BT_UINT NUM_BM_BUFFERS;
   
   printf("Initializing API with BusTools_API_OpenChannel . . . ");

   if(argc == 2)
      device = atoi(argv[1]);
   else
      device = 0;

   // Open the device and get the channel id.
   mode = 3;//API_B_MODE | API_MANUAL_INT;  // 1553B protocol, use manual interrupts. 
   status = BusTools_API_OpenChannel( &ch_id, mode, device, CHANNEL_1);
   printf("BusTools_API_OpenChannel = %d\n",status);

   // Initialize the Bus Monitor.
   printf("BM Init . . . ");
   status = BusTools_BM_Init(ch_id, BM_ENABLE_BUS | BM_NO_HW_INT, BM_ENABLE_BUS);
   printf("BusTools_BM_Init = %d\n",status);

   status = BusTools_TimeTagMode(ch_id,API_TTD_RELM,API_TTI_ZERO,API_TTM_FREE, NULL,0,0,0);

   // Select External Bus.
   printf("Bus select . . . ");
   status = BusTools_SetInternalBus(ch_id, EXTERNAL_BUS);
   printf("BusTools_SetInternalBus = %d\n",status);

   if(BusTools_GetBoardType(ch_id)==R15USB)  // Set the number of BM Message stored in the BM Buffer
      NUM_BM_BUFFERS = 1000;                 // For R15-USB make sure to allocate a large number of messages
   else
      NUM_BM_BUFFERS = 100;                  // Otherwise a smaller number                

   printf("BM MSG Alloc . . . ");
   status = BusTools_BM_MessageAlloc(ch_id, NUM_BM_BUFFERS, &num_bm_buffers_actual, BT1553_INT_END_OF_MESS);
   printf("BusTools_SetInternalBus = %d\n",status);

   printf("Hit Enter to start Monitoring.\n");
   printf("  Enter 'Q/q' to stop Monitoring.\n");
   printf("  Enter 'D/d' to dump memory to file.\n");
   getchar();

   // Start monitoring.
   printf("BM Start . . . ");
   status = BusTools_BM_StartStop(ch_id, BM_START);
   printf("BusTools_BM_StartStop = %d\n",status);

   do
   {
      MSDELAY(10);
      status = bm_readFunction(ch_id);
      if(status != API_SUCCESS && status != API_BM_READ_NODATA)
         printf("bc_readFunction status = %d\n",status);

   }while(!kbhit());
   
   status = BusTools_BM_StartStop(ch_id,BM_STOP);
   printf("BusTools_BM_StartStop = %d\n",status);  
			
   // We're done.  Close API and board
   printf("\nClosing API . . . ");
   status = BusTools_API_Close(ch_id);

   printf("FINISHED.\n");
} // End of main

/****************************************************************************
*
*  Function:  bm_readFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message on a Bus Monitor  message. This function read the data 
*     in the BM message and prints message data to the screen.
****************************************************************************/
BT_INT  bm_readFunction(BT_UINT ch_id)
{
   BT_INT wcnt, status, i;
   BT_UINT mcount, mess_index;
   API_BM_MBUF mbuf[100];
   char outbuf[80];
   BT_INT RT_MASK = -1, SA_MASK = -1, TR = -1;

   // Loop through all entries in the FIFO
   // Fetch entries from the FIFO: Get the tail pointer and extract the entry
   // it points to.   When (head == tail) then FIFO is empty.
   status = BusTools_BM_ReadLastMessageBlock(ch_id, RT_MASK, SA_MASK, TR, &mcount, mbuf);
   if(status)
      return status;

    printf("%d ****************\n",mcount);//XXXXX
   for( mess_index=0; mess_index<mcount; mess_index++ )
   {
      //print out the BM data.  NOTE: FOR EXAMPLE ONLY 
      // printing not recommended in user-callback functions.
/*		
      if(BusTools_BoardIsV6(ch_id))
         BusTools_TimeGetFmtString(API_TTD_RELM_NS, &mbuf[mess_index].time, outbuf);
      else
         BusTools_TimeGetFmtString(API_TTD_RELM, &mbuf[mess_index].time, outbuf);

      printf("BM>>>>>time tag = %s\n",outbuf);


      if(mbuf[mess_index].int_status & BT1553_INT_RETRY)
         printf("Retry - %d\n",mbuf[mess_index].messno);

      if (mbuf[mess_index].int_status & BT1553_INT_BROADCAST)
         printf("Broadcast - %d\n",mbuf[mess_index].messno);

      if(mbuf[mess_index].int_status & BT1553_INT_RT_RT_FORMAT)
         printf("BM>>>>> RT<->RT (%d) \n",mbuf[mess_index].messno);
      else if(mbuf[mess_index].int_status & BT1553_INT_MODE_CODE)  
         printf("BM>>>>> Mode Code (%d) \n",mbuf[mess_index].messno);
      else if (mbuf[mess_index].command1.tran_rec)
         printf("BM>>>>> RT->BC (%d)  \n", mbuf[mess_index].messno);
      else
         printf("BM>>>>> BC->RT (%d)  \n", mbuf[mess_index].messno);
          
      printf("BM>>>>> RT-%d  \n",mbuf[mess_index].command1.rtaddr);
      printf("BM>>>>> SA-%d  \n",mbuf[mess_index].command1.subaddr);
      printf("BM>>>>> WC%d  \n",mbuf[mess_index].command1.wcount);
      printf("BM>>>>> ST1 %4X \n",*(BT_U16BIT *)&mbuf[mess_index].status1);
 
      if(mbuf[mess_index].command1.wcount == 0)
         wcnt = 32;
      else
         wcnt = mbuf[mess_index].command1.wcount;

      printf("BM>>>>> 0x%08lx\n",mbuf[mess_index].int_status);
      for(i=0;i<wcnt; i++)
      {
         if(i%8==0 && i>0)
            printf("\n");
         printf("%04X ",mbuf[mess_index].value[i]);
      }
      printf("\n");*/
   }
   return( API_SUCCESS );
}



