/*============================================================================*
 * FILE:                 E X A M P L E _ B M _ R E C O R D E R . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:    EXAMPLE PROGRAM
 *              This example program that sets up a Bus Monitor to record the
 *              message traffic to a file. The user is prompted for the the 
 *              number of messages to capture.  The program then captures those
 *              messages and writea them to BMD file (The format used by 
 *              BusTools/1553-Analyzer.  You can use BusTools/1553-Analyzer to
 *              view the data.
 *
 *				Since this is a monitor only example, you will 
 *				need something else generating bus traffic so there will be
 *				messages to capture.
 *				
 *===========================================================================*/

/* $Revision:  8.14 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
    08/21/13   Compatible with all O/S and F/W version
	03/27/2015 Update for BMDX file format
*/

#include "busapi.h"
#include <stdio.h>

typedef struct bmdx_header
{
   char       cTypeName[16]; //"BMDX"
   BT_U32BIT  uVersion; 
   BT_U32BIT  uFormatInfo;
   BT_U32BIT  uReserved;
}
BMDX_HEADER;

BMDX_HEADER bmdHeader;

void setupThe_BMREC_IntFIFO(int ch_id);
BT_INT  _stdcall bm_REC_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO);

API_INT_FIFO sIntFIFO;
BT_INT        msg_count = 0;
FILE          *BMDfile;

// Main program
int main(int argc, char **argv) 
{
   BT_INT   status, ch_id;
   BT_UINT  num_bm_buffers_actual, device, mode;
   BT_INT   num_msgs_to_capture;
   BT_INT   max_buffers_read = 0;
   BT_UINT NUM_BM_BUFFERS;
   char    c;
   
   printf("Initializing API with BusTools_API_OpenChannel . . . ");

   if(argc == 2)
      device = atoi(argv[1]);
   else
      device = 0;

   // Open the device and get the channel id.
   mode = API_B_MODE | API_SW_INTERRUPT;  // 1553B protocol, use SW interrupts.
   status = BusTools_API_OpenChannel( &ch_id, mode, device, CHANNEL_1);
   printf("BusTools_API_OpenChannel = %d\n",status);

   // Initialize the Bus Monitor.
   printf("BM Init . . . ");
   status = BusTools_BM_Init(ch_id, 1, 1);
   printf("BusTools_BM_Init = %d\n",status);

   // Select External Bus.
   printf("Bus select . . . ");
   status = BusTools_SetInternalBus(ch_id, EXTERNAL_BUS);
   printf("BusTools_SetInternalBus = %d\n",status);

   if(BusTools_GetBoardType(ch_id)==R15USB)  // Set the number of BM Message stored in the BM Buffer
      NUM_BM_BUFFERS = 1000;                 // For R15-USB make sure to allocate a large number of messages
   else
      NUM_BM_BUFFERS = 100;                  // Otherwise a smaller number                

   printf("BM MSG Alloc . . . ");
   status = BusTools_BM_MessageAlloc(ch_id, NUM_BM_BUFFERS, &num_bm_buffers_actual, BT1553_INT_END_OF_MESS);
   printf("BusTools_SetInternalBus = %d\n",status);

   // The API enables capture of messages for all RT/SA/TR/WC combinations.

   // Prompt user for number of messages to capture.
   printf("\nNumber of messages to capture? ");
   scanf("%d",&num_msgs_to_capture);

   setupThe_BMREC_IntFIFO(ch_id);  // This function uses BusTools_RegisterFunction to set up a user-callback 
                                   // that records BM data

   printf("Hit Enter to start Monitoring.\n");
   scanf("%c",&c);

   // Start monitoring.
   printf("BM Start . . . ");
   status = BusTools_BM_StartStop(ch_id, BM_START);
   printf("BusTools_BM_StartStop = %d\n",status);

   while (msg_count < num_msgs_to_capture)
   {
      printf("Message count = %d\r",msg_count);
   }
   
   status = BusTools_BM_StartStop(ch_id,BM_STOP);
   printf("BusTools_BM_StartStop = %d\n",status);  

   status = BusTools_RegisterFunction(ch_id, &sIntFIFO, UNREGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);		
			
   // We're done.  Close API and board
   printf("\nClosing API . . . ");
   status = BusTools_API_Close(ch_id);

   printf("FINISHED.\n");
} // End of main

/****************************************************************************
*
*  Function:  bm_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bm_REC_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BM_MBUF mbuf[1000];
   int       status,count;
   int       size;
   BT_INT tail;

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      status = BusTools_BM_MessageReadBlock(cardnum,  mbuf, 1000, 0, &count );
      if(status)
         printf("record error = %d\n",status);
      if(count > 0)
      { 
         size = fwrite((const void *)mbuf, sizeof(API_BM_MBUF),count,BMDfile);  
         msg_count+=count;  
      }

      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }
   return 0;
}


/****************************************************************************
*
*  Function:   
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a BC message.  The interrupt is set for only the end-of
*     message.  
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupThe_BMREC_IntFIFO(int cardnum)
{
   int status;
   int size;

   if((BMDfile = fopen("recorder_data.bmdx","wb"))==NULL)  // Open the .bmd file
   {
      printf("OPEN ERROR!!!\n");
   }

   memset(&bmdHeader, 0, sizeof(BMDX_HEADER));
   strcpy(&bmdHeader.cTypeName,"BMDX");
   if(BusTools_BoardIsV6(cardnum))
	   bmdHeader.uVersion=0;
   else
	   bmdHeader.uVersion=1;
   //write the BMDX header record.
   size = fwrite((const void *)&bmdHeader, sizeof(BMDX_HEADER),1,BMDfile); 

   // Setup the FIFO structure for this board.
   memset(&sIntFIFO, 0, sizeof(API_INT_FIFO));   // clear out the API_INT_FIFO structure
   sIntFIFO.function       = bm_REC_intFunction; // Pointer to the user-callback function 
   sIntFIFO.iPriority      = THREAD_PRIORITY_TIME_CRITICAL; // System priority
   sIntFIFO.dwMilliseconds = INFINITE;           // Wait forever 
   sIntFIFO.iNotification  = 0;               
   sIntFIFO.FilterType     = EVENT_RECORDER;     // This will record message for the BM Buffer

   // Call the register function to register the user function.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);

}
