/*===========================================================================*
 * FILE:                   example_bm_rt_process_msg_blk.c
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    Demonstration of BusTools/1553-API routines. This console 
 *              application shows how to configure the Remote Terminal,
 *              Bus Controller and Bus Monitor  and set up interrupt on BC, RT,
 *              BM messages.  It demonstrated to use the 
 *              BusTools_BM_ReadLastMessageBlock() and 
 *              BusTools_RT_ReadLastMessageBlock() to returns all the Bus Monitor 
 *              and remote terminal messages in the interrupt queue.
 *===========================================================================*/

/* $Revision:  8.18 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "busapi.h"
#ifdef WIN32
#include <conio.h>
#endif //WIN32

void BusController_Init(int,int);
void BusMonitor_Init(int);
void RemoteTerminal_Init(int);


BT_UINT rt_int_count[32];

static unsigned bm_tcnt=0,bc_tcnt=0,rt_tcnt=0;

BT_INT bm_readFunction(BT_UINT cardnum);
BT_INT rt_readFunction(BT_UINT cardnum);

#if defined (__WIN32__)
int main(int argc, char **argv)                     
#else
int tst_mrt()
#endif
{
   BT_INT status, cardnum;
   BT_UINT channel,device,mode;
   char ichar;
   BT_INT i=0;

   mode = API_B_MODE | API_SW_INTERRUPT; //API_B_MODE | API_SW_INTERRUPT;  //MIL-STD-1553B with Software (Polling) interrupts enabled
   channel = CHANNEL_1;  //default channel 1
   device = 2;

   status = BusTools_API_OpenChannel(&cardnum,mode,device,channel);
   printf("BusTools_API_OpenChannel status = %d\n",status);

   status = BusTools_SetInternalBus(cardnum,EXTERNAL_BUS);  // Set up bus External
   printf("BusTools_SetInternalBus status = %d\n",status);

   BusMonitor_Init(cardnum);   
   BusController_Init(cardnum,50);
   RemoteTerminal_Init(cardnum);  


   status = BusTools_BM_StartStop(cardnum,BM_START);
   printf("BusTools_BM_StartStop status = %d\n",status);

   status = BusTools_RT_StartStop(cardnum,RT_START);
   printf("BusTools_RT_StartStop status = %d\n",status);

   status = BusTools_BC_StartStop(cardnum,BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);
   
   do
   {
      MSDELAY(10);
      status = bm_readFunction(cardnum);
      if(status != API_SUCCESS && status != API_BM_READ_NODATA)
         printf("bc_readFunction status = %d\n",status);
      status = rt_readFunction(cardnum);
      if(status != API_SUCCESS && status != API_RT_READ_NODATA)
         printf("RT_readFunction status = %d\n",status);

   }while(!kbhit());
   
   
   do
   {
      ichar= getchar();
   }while(ichar != 'q' && ichar != 'Q'); 



   printf("main: closing API\n");

   status = BusTools_BC_StartStop(cardnum,BC_STOP); 
   printf("BusTools_BC_StartStop status = %d\n",status);

   MSDELAY(1000);

   status = BusTools_RT_StartStop(cardnum,RT_STOP);
   printf("BusTools_RT_StartStop status = %d\n",status);

   status = BusTools_BM_StartStop(cardnum,BM_STOP);
   printf("BusTools_BM_StartStop status = %d\n",status);

   

   status = BusTools_API_Close(cardnum);//
   printf("BusTools_API_Close status = %d -- %d\n",status, rt_tcnt);

}

void BusController_Init(int cardnum,int num_buffers)
{
   int status, messno,i;
   API_BC_MBUF bcmessage;

   //initialize the BC with interrupt on end-of-message 16 us no response 14 us late response, 1sec frame (1000000 usec).     
   status = BusTools_BC_Init(cardnum,0,BT1553_INT_END_OF_MESS,0,16,14,1000000,1);
   printf("BusTools_BC_Init status = %d\n",status);

   status = BusTools_BC_MessageAlloc(cardnum,50);//num_buffers);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);

   //Receive (BC->RT) Command)
   messno = 0;
   memset((char*)&bcmessage,0,sizeof(bcmessage));       
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on bus A
   bcmessage.control |= BC_CONTROL_RETRY;         // enable interrupt on message
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;    // Start of a minor frame
   bcmessage.mess_command1.rtaddr   = 7;          
   bcmessage.mess_command1.subaddr  = 7;
   bcmessage.mess_command1.wcount   = 0;
   bcmessage.mess_command1.tran_rec = RECEIVE;    // BC->RT
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 10;                       // 10 microsecond inter-message gap.

   for(i=0;i<32;i++)
       bcmessage.data[0][i] = 0xA000+i;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Transmit (RT->BC) Command
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message
   bcmessage.mess_command1.rtaddr   = 1;
   bcmessage.mess_command1.subaddr  = 23;
   bcmessage.mess_command1.wcount   = 32;
   bcmessage.mess_command1.tran_rec = TRANSMIT;   // RT->BC
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                       // 15 microsecond inter-message gap
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);


   //Receive (BC->RT)
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message
   bcmessage.mess_command1.rtaddr   = 1;
   bcmessage.mess_command1.subaddr  = 23;
   bcmessage.mess_command1.wcount   = 32;
   bcmessage.mess_command1.tran_rec = RECEIVE;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 20;                      // 20 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Receive (BC->RT) Command
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 4;
   bcmessage.mess_command1.wcount   = 4;
   bcmessage.mess_command1.tran_rec = RECEIVE;    // BC->RT
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 50;                       // 50 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Receive (BC->RT) Command
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;      // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message
   bcmessage.mess_command1.rtaddr   = 5;
   bcmessage.mess_command1.subaddr  = 5;
   bcmessage.mess_command1.wcount   = 8;
   bcmessage.mess_command1.tran_rec = 0;          // BC->RT
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // 100 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   // RT - RT Command
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_RTRTFORMAT;    // define RT-RT message
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 10;
   bcmessage.mess_command1.wcount   = 6;
   bcmessage.mess_command1.tran_rec = 0;          //RT-RT receive command

   bcmessage.mess_command2.rtaddr   = 6;
   bcmessage.mess_command2.subaddr  = 6;
   bcmessage.mess_command2.wcount   = 6;
   bcmessage.mess_command2.tran_rec = 1;          // RT-RT transmit command
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 25;                       // 25 microsecond inter-message gap.
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Mode Code
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message
   bcmessage.mess_command1.rtaddr   = 6;
   bcmessage.mess_command1.subaddr  = 0;          // Subaddress 0 is a mode code
   bcmessage.mess_command1.wcount   = 17;         // Mode Code 17 sync w/data  
   bcmessage.mess_command1.tran_rec = RECEIVE;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // 100 microsecond inter-message gap.
   bcmessage.data[0][0] = 0xABCD;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status); 

  
   //Noop message at end-of-frame
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)0;          // repeat starting at message 0
   bcmessage.control = BC_CONTROL_NOP;            // 
   bcmessage.control |= BC_CONTROL_MFRAME_END;    // End-of-Frame

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);
} 

void RemoteTerminal_Init(int cardnum)
{
   int status;
   int RT_MBUF_COUNT;
   API_RT_CBUF cbuf;       // RT Control Buffer Legal Word Count Mask (2 words)
   API_RT_MBUF_WRITE mbuf;
   API_RT_ABUF abuf;
   int         tr;
   int         subaddress;
   int i,j,ii,jj;
   int rt_cnt = 5;
   int rt_addr[] = {1,4,5,6,7};

   RT_MBUF_COUNT = 5;

   status = BusTools_RT_Init(cardnum,0);
   printf("BusTools_RT_Init status = %d\n",status);

   /**********************************************************************
   *  Output RT control buffers
   *   Enable or disable all word counts and mode codes as specified by
   *    the caller's arguements.
   **********************************************************************/

   cbuf.legal_wordcount = 0xffffffff;  // legailize all word counts
   for ( i = 0; i < rt_cnt; i++)
   { 
      for ( tr = 0; tr < 2; tr++ )
      {
         for ( subaddress = 0; subaddress < 32; subaddress++ )
         {
            status = BusTools_RT_CbufWrite(cardnum,rt_addr[i],subaddress,tr,RT_MBUF_COUNT,&cbuf);
         }
      }
   }

   /*******************************************************************
   *  Fill in message buffers for receive and transmit sub-units.
   *******************************************************************/
   mbuf.error_inj_id = 0;
   mbuf.enable = BT1553_INT_END_OF_MESS;   // Interrupt enables
   // For all subaddresses...
   for (ii = 0; ii<rt_cnt; ii++)
   {  
      for (i=0; i<32; i++)
      {
         // For each data buffer...
         for (j=0; j<RT_MBUF_COUNT; j++)
         {
            for(jj=0;jj<32;jj++)
               mbuf.mess_data[jj] = (BT_U16BIT)(0xAA00 + jj);
            status = BusTools_RT_MessageWrite(cardnum,rt_addr[ii],i,0,j,&mbuf);
            status = BusTools_RT_MessageWrite(cardnum,rt_addr[ii],i,1,j,&mbuf);
         }
      }
   }

   /************************************************
   *  Setup RT address control block
   ************************************************/

   abuf.enable_a          = 1;    // Disable Bus A...Note that the API bits are
   abuf.enable_b          = 1;    // Disable Bus B... reversed from the HW bits.
   abuf.inhibit_term_flag = 0;
   abuf.status            = 0x0000;
   abuf.command           = 0;
   abuf.bit_word          = 0;

   for(i=0;i<rt_cnt;i++)
   {
      status = BusTools_RT_AbufWrite(cardnum,rt_addr[i],&abuf);
      printf("BusTools_RT_AbufWrite status = %d\n",status);
   }
}
void BusMonitor_Init(int cardnum)
{
 int status;

   BT_UINT actual;

   status = BusTools_BM_Init(cardnum,1,1);
   printf("BusTools_BM_Init status = %d\n",status);
   status = BusTools_BM_MessageAlloc(cardnum, 40, &actual ,BT1553_INT_END_OF_MESS); // Interrupt enabled
   printf("BusTools_BM_MessageAlloc status = %d - actual =%d\n",status,actual);
}



/****************************************************************************
*
*  Function:  bm_readFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message on a Bus Monitor  message. This function read the data 
*     in the BM message and prints message data to the screen.
****************************************************************************/
BT_INT  bm_readFunction(BT_UINT ch_id)
{
   BT_INT status;
   BT_UINT mcount;
   API_BM_MBUF mbuf[100];
   BT_INT RT_MASK = -1, SA_MASK = -1, TR = -1;

   // Loop through all entries in the FIFO
   // Fetch entries from the FIFO: Get the tail pointer and extract the entry
   // it points to.   When (head == tail) then FIFO is empty.
   status = BusTools_BM_ReadLastMessageBlock(ch_id, RT_MASK, SA_MASK, TR, &mcount, mbuf);
   if(status)
      return status;

    printf("BM %d ****************\n",mcount);//XXXXX
    /*
   for( mess_index=0; mess_index<mcount; mess_index++ )
   {
      //print out the BM data.  NOTE: FOR EXAMPLE ONLY 
      // printing not recommended in user-callback functions.
	
      if(BusTools_BoardIsV6(ch_id))
         BusTools_TimeGetFmtString(API_TTD_RELM_NS, &mbuf[mess_index].time, outbuf);
      else
         BusTools_TimeGetFmtString(API_TTD_RELM, &mbuf[mess_index].time, outbuf);

      printf("BM>>>>>time tag = %s\n",outbuf);


      if(mbuf[mess_index].int_status & BT1553_INT_RETRY)
         printf("Retry - %d\n",mbuf[mess_index].messno);

      if (mbuf[mess_index].int_status & BT1553_INT_BROADCAST)
         printf("Broadcast - %d\n",mbuf[mess_index].messno);

      if(mbuf[mess_index].int_status & BT1553_INT_RT_RT_FORMAT)
         printf("BM>>>>> RT<->RT (%d) \n",mbuf[mess_index].messno);
      else if(mbuf[mess_index].int_status & BT1553_INT_MODE_CODE)  
         printf("BM>>>>> Mode Code (%d) \n",mbuf[mess_index].messno);
      else if (mbuf[mess_index].command1.tran_rec)
         printf("BM>>>>> RT->BC (%d)  \n", mbuf[mess_index].messno);
      else
         printf("BM>>>>> BC->RT (%d)  \n", mbuf[mess_index].messno);
          
      printf("BM>>>>> RT-%d  \n",mbuf[mess_index].command1.rtaddr);
      printf("BM>>>>> SA-%d  \n",mbuf[mess_index].command1.subaddr);
      printf("BM>>>>> WC%d  \n",mbuf[mess_index].command1.wcount);
      printf("BM>>>>> ST1 %4X \n",*(BT_U16BIT *)&mbuf[mess_index].status1);
 
      if(mbuf[mess_index].command1.wcount == 0)
         wcnt = 32;
      else
         wcnt = mbuf[mess_index].command1.wcount;

      printf("BM>>>>> 0x%08lx\n",mbuf[mess_index].int_status);
      for(i=0;i<wcnt; i++)
      {
         if(i%8==0 && i>0)
            printf("\n");
         printf("%04X ",mbuf[mess_index].value[i]);
      }
      printf("\n");
      
   }
   */
   return( API_SUCCESS );
}

/****************************************************************************
*
*  Function:  bm_readFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message on a Bus Monitor  message. This function read the data 
*     in the BM message and prints message data to the screen.
****************************************************************************/
BT_INT  rt_readFunction(BT_UINT ch_id)
{
   BT_INT status;
   BT_UINT mcount;
   API_RT_MBUF_READ mbuf[100];
   BT_INT RT_MASK = -1, SA_MASK = -1, TR = -1;

   // Loop through all entries in the FIFO
   // Fetch entries from the FIFO: Get the tail pointer and extract the entry
   // it points to.   When (head == tail) then FIFO is empty.
   status = BusTools_RT_ReadLastMessageBlock(ch_id, RT_MASK, SA_MASK, TR, &mcount, mbuf);
   if(status)
      return status;

    printf("RT %d ****************\n",mcount);//XXXXX
    /*
   for( mess_index=0; mess_index<mcount; mess_index++ )
   {
      //print out the RT data.  NOTE: FOR EXAMPLE ONLY 
      // printing not recommended in user-callback functions.
		
      if(BusTools_BoardIsV6(ch_id))
         BusTools_TimeGetFmtString(API_TTD_RELM_NS, &mbuf[mess_index].time, outbuf);
      else
         BusTools_TimeGetFmtString(API_TTD_RELM, &mbuf[mess_index].time, outbuf);

      printf("RT>>>>>time tag = %s\n",outbuf);


      if(mbuf[mess_index].int_status & BT1553_INT_RETRY)
         printf("Retry - %d\n",mbuf[mess_index].messno);

      if (mbuf[mess_index].int_status & BT1553_INT_BROADCAST)
         printf("Broadcast - %d\n",mbuf[mess_index].messno);

      if(mbuf[mess_index].int_status & BT1553_INT_RT_RT_FORMAT)
         printf("RT>>>>> RT<->RT (%d) \n",mbuf[mess_index].messno);
      else if(mbuf[mess_index].int_status & BT1553_INT_MODE_CODE)  
         printf("RT>>>>> Mode Code (%d) \n",mbuf[mess_index].messno);
      else if (mbuf[mess_index].command1.tran_rec)
         printf("RT>>>>> RT->BC (%d)  \n", mbuf[mess_index].messno);
      else
         printf("BM>>>>> BC->RT (%d)  \n", mbuf[mess_index].messno);
          
      printf("RT>>>>> RT-%d  \n",mbuf[mess_index].command1.rtaddr);
      printf("RT>>>>> SA-%d  \n",mbuf[mess_index].command1.subaddr);
      printf("RT>>>>> WC%d  \n",mbuf[mess_index].command1.wcount);
      printf("RT>>>>> ST1 %4X \n",*(BT_U16BIT *)&mbuf[mess_index].status1);
 
      if(mbuf[mess_index].command1.wcount == 0)
         wcnt = 32;
      else
         wcnt = mbuf[mess_index].command1.wcount;

      printf("RT>>>>> 0x%08lx\n",mbuf[mess_index].int_status);
      for(i=0;i<wcnt; i++)
      {
         if(i%8==0 && i>0)
            printf("\n");
         printf("%04X ",mbuf[mess_index].value[i]);
      }
      printf("\n");
      
   }
   */
   return( API_SUCCESS );
}

