/*===========================================================================*
 * FILE:                     E X A M P L E _ B M _ S H A R E. C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    This application demonstrates how to initialize a channel and 
 * the share the channel so other applications can use that channel.  This 
 * application initializes and shares the channel.  It then configures and runs 
 * a Bus Monitor.  This allows separate Bus Controller and Remote Terminal 
 * applications to join this channel.
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include "busapi.h"

void setupTheIntFIFO1(int);

API_INT_FIFO  sIntFIFO1;     // Thread FIFO structure

BT_U16BIT pwidth=100;

void main(int argc, char **argv)
{
   int status,cardnum;
   int pwFlag = 1;
   BT_UINT actual, device=0, mode = 0;
 

   mode = API_B_MODE | API_SW_INTERRUPT;
   if(argc == 2)
      device = atoi(argv[1]);

   printf("Starting Test.  Hit Enter to stop\n\n");
   printf("Calling Init\n");
   status = BusTools_API_OpenChannel(&cardnum,mode,device,CHANNEL_1);  //Initialize the channel
   printf("BusTools_API_OpenChannel status = %d\n",status);

   /* Share the channel so other application can join  */
   status = BusTools_API_ShareChannel(cardnum);
    printf("BusTools_API_ShareChannel status = %d\n",status);

   status = BusTools_SetInternalBus(cardnum,EXTERNAL_BUS);
   printf("BusTools_SetInternalBus status = %d\n",status);
          
   status = BusTools_BM_Init(cardnum,1,1);
   printf("BusTools_BM_Init status = %d\n",status);

   status = BusTools_BM_MessageAlloc(cardnum, 1024, &actual, BT1553_INT_END_OF_MESS);
   printf("BusTools_BM_MessageAlloc status = %d - actual =%d\n",status,actual);

   setupTheIntFIFO1(cardnum);

   status = BusTools_BM_StartStop(cardnum,1);
   printf("BusTools_BM_StartStop status = %d\n",status);

   while(getc(stdin) != '\n');

   printf("main: closing API\n");

   status = BusTools_BM_StartStop(cardnum,0);
   printf("BusTools_BM_StartStop status = %d\n",status);

   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, UNREGISTER_FUNCTION);

   BusTools_API_QuitChannel(cardnum, BM_QUIT | SHARE_QUIT);
   printf("BusTools_API_QuitChannel status = %d\n",status);

   BusTools_API_Close(cardnum);

}

/****************************************************************************
*
*  Function:  intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a BM Message.  
*
****************************************************************************/
BT_INT _stdcall bm_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{
   API_BM_MBUF mbuf;
   int       status;
   BT_INT tail;
   BT_UINT messno;

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)  // Process all message in from the interrupt queue
   {
      if(sIntFIFO->FilterType == EVENT_BM_MESSAGE)
	  {
         messno = sIntFIFO->fifo[tail].bufferID;
         status = BusTools_BM_MessageRead(cardnum,messno,&mbuf);// read the BM message data
         if (status)
            return status;
        
         printf("BM RT = %d\n",mbuf.command1.rtaddr);
         printf("BM SA = %d\n",mbuf.command1.subaddr);
        
	   }
       tail++;
       tail &= sIntFIFO->mask_index;
       sIntFIFO->tail_index = tail;
   }
   return 0;
}

/****************************************************************************
*
*  Function:  setupTheIntFIFO1
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a BC message.  The interrupt is set for only the end-of
*     message.  
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupTheIntFIFO1(int cardnum)
{
   int rt,sa,tr,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.function     = bm_intFunction;  // This is the name of your interrupt processing function
   sIntFIFO1.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.iNotification  = 0;
   sIntFIFO1.FilterType     = EVENT_BM_MESSAGE;
   
   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO1.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos
 
   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}

