/*============================================================================*
 * FILE:         E X A M P L E _ B M _ T R I G _ S T A R T _ S T O P . C
 *============================================================================*
 *
  *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    This example shows how to enable Bus Monitor start trigger 
 * and stop trigger using the function BusTools_BM_TriggerWrite. A start trigger 
 * specifies a condition or set of conditions that must occur before the Bus 
 * Monitor starts processing data. The stop trigger defines a condition of set 
 * of conditions that must occur to stop the Bus Monitor processing. When a start 
 * trigger is set it prevent Bus Monitor data from being recorded in the interrupt 
 * queue. A stop trigger disables the Bus Monitor message from going into the 
 * interrupt queue.  You can still record all the message traffic.  
 * 
 * In this example the Bus Monitor is configured to start only after a command is 
 * sent to RT1.  All traffic transacting prior is not processed.  Once started the 
 * Monitor stops processing messages when a command word with RT22 occurs when 
 * armed first by a command word to RT2.
  
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
  12/18/2007   Update for BM trigger out/in example
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include <time.h>
#include "busapi.h"

int BusController_Init(int);
int BusMonitor_Init(int);
int RemoteTerminal_Init(int);
void setupThe_BM_IntFIFO(int);

API_INT_FIFO  sIntFIFO1;     // FIFO structure for BM

void main(int argc, char **argv)                     
{
   BT_INT status,cardnum,device;
   BT_INT bc_rt_running = 0;
   BT_INT loop_exit = 0;
   BT_UINT pwFlag, rtaddr;
   char c;

   pwFlag = API_B_MODE | API_SW_INTERRUPT; // Use hardware interrupts 

   if(argc == 2)
      device = atoi(argv[1]);
   else
      device = 0;

   printf("Starting Test.  Hit Enter to stop\n\n");
   printf("Initializing Test\n");

   status = BusTools_API_OpenChannel(&cardnum,pwFlag,device,CHANNEL_1);
   printf("BusTools_API_Init status = %d\n",status);

   status = BusTools_SetInternalBus(cardnum,EXTERNAL_BUS);        // Set up external bus
   printf("BusTools_SetInternalBus status = %d\n",status);
 
   status = BusMonitor_Init(cardnum);
   status = BusController_Init(cardnum);
   status = RemoteTerminal_Init(cardnum);
   setupThe_BM_IntFIFO(cardnum);

   status = BusTools_BM_StartStop(cardnum,BM_START);
   printf("BusTools_BM_StartStop status = %d\n",status);

   status = BusTools_RT_StartStop(cardnum,RT_START);
   printf("BusTools_RT_StartStop status = %d\n",status);

   status = BusTools_BC_StartStop(cardnum,BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);

   printf("Enter q/Q to Quit\n      r/R for BM trigger start\n      s/S for BM trigger stop\n      d/D for memory dump\n");
   do
   {
      c = getc(stdin);
      switch(c)
      {
         case 'D':
         case 'd':
            BusTools_DumpMemory(cardnum,0xffffff,"bm_trig_dump.txt","dumping");
            break;
         case 'q':
         case 'Q':
            loop_exit=1;
            break;
         case 'r':
         case 'R':
            BusTools_BC_AperiodicRun(cardnum,10,0,0,0);
            break;
         case 's':
         case 'S':
            BusTools_BC_AperiodicRun(cardnum,20,0,0,0);
            break;
         default:
            continue;
      }
   }while(!loop_exit);

   printf("main: closing API\n");

   status = BusTools_BC_StartStop(cardnum,BC_STOP);
   printf("BusTools_BC_StartStop status = %d\n",status);

   status = BusTools_RT_StartStop(cardnum,RT_STOP);
   printf("BusTools_RT_StartStop status = %d\n",status);

   status = BusTools_BM_StartStop(cardnum,BM_STOP);
   printf("BusTools_BM_StartStop status = %d\n",status);

   status = BusTools_RegisterFunction(cardnum,&sIntFIFO1,UNREGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);

   status = BusTools_API_Close(cardnum);
   printf("BusTools_API_Close status = %d\n",status);
}

int BusController_Init(int cardnum)
{
   int status, messno;
   API_BC_MBUF bcmessage;

   // Initialize the Bus Comtroller for default timing interrupt on end-of-message
   status = BusTools_BC_Init(cardnum,0,0,0,16,14,1000000,1);
   printf("BusTools_BC_Init status = %d\n",status);

   // Allocate the need BC message buffers
   status = BusTools_BC_MessageAlloc(cardnum,50);
   printf("BusTools_BC_MessageAlloc status = %d\n",status); 

   messno = 0;
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control  = BC_CONTROL_MESSAGE;       // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on BUS A
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;
   bcmessage.mess_command1.rtaddr   = 8;
   bcmessage.mess_command1.subaddr  = 8;
   bcmessage.mess_command1.wcount   = 8;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 20;                      // Microsecond inter-message gap.
   bcmessage.data[0][0] = 0xA001;
   bcmessage.data[0][1] = 0xA001;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);
    
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control  = BC_CONTROL_MESSAGE;       // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on BUS A
   bcmessage.mess_command1.rtaddr   = 3;
   bcmessage.mess_command1.subaddr  = 3;
   bcmessage.mess_command1.wcount   = 3;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 20;                      // Microsecond inter-message gap.
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control  = BC_CONTROL_MESSAGE;       // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on BUS A
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 4;
   bcmessage.mess_command1.wcount   = 4;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 20;                      // Microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control  = BC_CONTROL_MESSAGE;       // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on BUS A
   bcmessage.mess_command1.rtaddr   = 5;
   bcmessage.mess_command1.subaddr  = 5;
   bcmessage.mess_command1.wcount   = 5;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;     // Default error injection buffer (no errors)
   bcmessage.gap_time = 20;    // 8 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control  = BC_CONTROL_MESSAGE;       // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on BUS A
   bcmessage.mess_command1.rtaddr   = 3;
   bcmessage.mess_command1.subaddr  = 8;
   bcmessage.mess_command1.wcount   = 8;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 20;                      // Microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(0);
   bcmessage.control  = BC_CONTROL_MESSAGE;       // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on BUS A
   bcmessage.control |= BC_CONTROL_MFRAME_END;
   bcmessage.mess_command1.rtaddr   = 16;
   bcmessage.mess_command1.subaddr  = 12;
   bcmessage.mess_command1.wcount   = 12;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 50;                       // Microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   /////////////////////////////////////////////////////////////////
   // Aperiodic message list used for the the start trigger event // 
   /////////////////////////////////////////////////////////////////
   messno=10;                                                                                           
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(0xffff);
   bcmessage.control  = BC_CONTROL_MESSAGE;       // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on BUS A
   bcmessage.mess_command1.rtaddr   = 1;
   bcmessage.mess_command1.subaddr  = 1;
   bcmessage.mess_command1.wcount   = 1;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 20;                       // Microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   ////////////////////////////////////////////////////////////////
   // Aperiodic message list used for the the stop trigger event // 
   ////////////////////////////////////////////////////////////////
   messno=20;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control  = BC_CONTROL_MESSAGE;       // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSB;          // Transmit on BUS A
   bcmessage.mess_command1.rtaddr   = 23;
   bcmessage.mess_command1.subaddr  = 2;
   bcmessage.mess_command1.wcount   = 4;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 20;                       // Microsecond inter-message gap.

   bcmessage.data[0][0] = 0xb000;
   bcmessage.data[0][1] = 0xb001;
   bcmessage.data[0][2] = 0xb002;
   bcmessage.data[0][3] = 0xb003;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(0xffff);
   bcmessage.control  = BC_CONTROL_MESSAGE;       // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.mess_command1.rtaddr   = 7;
   bcmessage.mess_command1.subaddr  = 8;
   bcmessage.mess_command1.wcount   = 8;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 20;                       // Microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   return status; 
} 

int RemoteTerminal_Init(int cardnum)
{
   int status;
   int RT_MBUF_COUNT, NUM_RT;
   API_RT_CBUF cbuf;       // RT Control Buffer Legal Word Count Mask (2 words)
   API_RT_MBUF_WRITE mbuf;
   API_RT_ABUF abuf;
   int         tr;
   int         subaddress;
   int i,j,ii;
   int rt_addr[] = {1,2,3,4,5,8,16,22};

   NUM_RT = 8;
   RT_MBUF_COUNT = 1;

   status = BusTools_RT_Init(cardnum,0);
   printf("BusTools_RT_Init status = %d\n",status);

   /**********************************************************************
   *  Output RT control buffers
   *   Enable or disable all word counts and mode codes as specified by
   *    the caller's arguements.
   **********************************************************************/

   cbuf.legal_wordcount = 0xffffffff;  // legailize all word counts
   for ( i = 0; i < NUM_RT; i++)
   {
      for ( tr = 0; tr < 2; tr++ )
      {
         for ( subaddress = 0; subaddress < 32; subaddress++ )
         {
            status = BusTools_RT_CbufWrite(cardnum,rt_addr[i],subaddress,tr,
			                            RT_MBUF_COUNT,&cbuf);
         }
      }
   }

   /*******************************************************************
   *  Fill in message buffers for receive and transmit sub-units.
   *******************************************************************/

   mbuf.enable = BT1553_INT_END_OF_MESS;   // Interrupt enables on Broadcast Message
   mbuf.error_inj_id = 0;

   // For all subaddresses...
   for (ii = 0; ii<NUM_RT; ii++)
   {  
      for (i=0; i<32; i++)
      {
         // Create data buffer data
         for (j=0; j<32; j++)
            mbuf.mess_data[j] = (BT_U16BIT)(0xF000 + i);
         // For each data buffer...
         for (j=0; j<RT_MBUF_COUNT; j++)
         {
            status = BusTools_RT_MessageWrite(cardnum,rt_addr[ii],i,0,j,&mbuf);
            status = BusTools_RT_MessageWrite(cardnum,rt_addr[ii],i,1,j,&mbuf);
         }
      }
   }

   /************************************************
   *  Setup RT address control block
   ************************************************/

   abuf.enable_a          = 1;    // Enable Bus A...Note that the API bits are
   abuf.enable_b          = 1;    // Enable Bus B... reversed from the HW bits.
   abuf.inhibit_term_flag = 0;
   abuf.status            = 0x0000;
   abuf.command           = 0;
   abuf.bit_word          = 0;

   for(i=0;i<NUM_RT;i++)
   {
      status = BusTools_RT_AbufWrite(cardnum,rt_addr[i],&abuf);
      printf("BusTools_RT_AbufWrite status = %d\n",status);
   }
   
   return status;
}

int BusMonitor_Init(int cardnum)
{
   int status;
   BT_UINT actual;
   API_BM_TBUF trig_buf;

   status = BusTools_BM_Init(cardnum,1,1);
   printf("BusTools_BM_Init status = %d\n",status);

   status = BusTools_BM_MessageAlloc(cardnum, 300, &actual, BT1553_INT_END_OF_MESS); // Interrupt on END_OF_MESS
   printf("BusTools_BM_MessageAlloc status = %d - actual =%d\n",status,actual);

   /* Set the trigger to output on a certain message status *
   ***********************************************************************
   *  BM Trigger Buffer definition
   *
   *     3 - Start Trigger Events A - B - C
   *     3 - Stop Trigger Events  E - F - G
   *
   *     control: dialog box entries for order of events:
   *                         START SIDE                  STOP SIDE
   *                    0 -> unconditionally             never
   *                    1 -> if A                        if E
   *                    2 -> if A AND B                  if E AND F
   *                    3 -> if A OR B                   if E OR F
   *                    4 -> if A with B (same message)  if E with F (same message) 
   *                    5 -> if B armed by A             if F armed by E
   *                    6 -> if A with B and C           if E with F and G
   *                    7 -> if A with B or C            if E with F or G
   *                    8 -> if C armed by A with B      if G armed by E with F
   *                    9 -> if A with B armed by C      if E with F armed by G
   *                   10 -> if A and B and C            if E and F and G
   *                   11 -> if A or B or C              if E or E or G
   *     type:    trigger type:
   *                    0 -> none
   *                    1 -> command
   *                    2 -> status
   *                    3 -> data
   *                    4 -> Not used
   *                    5 -> interrupt status lsb
   *                    6 -> interrupt status msb
   *
   *     mask:    16 bit mask (applied prior to compare)
   *     value:   16 bit value to compare
   *     word:    word number (data trigger - 3) or offset (user offset trigger - 5)
   *     count:   number of times event must occur before event_occured bit is set
   *
   **********************************************************************/
 
   memset(&trig_buf,0,sizeof(API_BM_TBUF));
   trig_buf.trig_ext = 0;         
   trig_buf.trig_ext_output = 0;        // External trigger on event
   trig_buf.control1 = 1;               // Start trigger on Event A below
   trig_buf.control2 = 1;               // Stop trigger on EVENT F when armed by E

   // START Event A:
   trig_buf.capture[0].type  = 1;       // Type = 1, command word (2=status, 3=data)
   trig_buf.capture[0].mask  = 0xf800;  // Mask to only look at RT address
   trig_buf.capture[0].value = 0x0800;  // Look for RT address 1
   trig_buf.capture[0].count = 1;       // Count of 1

   // STOP Event E:
   trig_buf.stop[0].type  = 6;			// Type = 1, command word
   trig_buf.stop[0].mask  = 0xffff;		// Mask to only look at RT address
   trig_buf.stop[0].value = 0x0001;	    // Look for RT address 2
   trig_buf.stop[0].count = 1;			// Count of 1
/*
   // STOP Event F:
   trig_buf.stop[1].type  = 6;			// Type = 1, command word
   trig_buf.stop[1].mask  = 0xffff;		// Mask to only look at RT address
   trig_buf.stop[1].value = 0x0401;	    // Look for RT address 22
   trig_buf.stop[1].count = 1;			// Count of 1
*/
   status = BusTools_BM_TriggerWrite(cardnum,&trig_buf);
   printf("BusTools_BM_TriggerWrite status = %d\n",status);
   return status;
}

/****************************************************************************
*
*  Function:  bm_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bm_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BM_MBUF mbuf;
   int       status;
   BT_INT tail;
   BT_UINT messno;
   char      outbuf[100];

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      messno = sIntFIFO->fifo[tail].bufferID;
      status = BusTools_BM_MessageRead(cardnum,messno,&mbuf);// read the BM message data
      if (status)
         return status;
	  if(sIntFIFO->fifo[tail].event_type & EVENT_BM_TRIG)
	  {
         BusTools_TimeGetFmtString(API_TTD_IRIG_NS,&mbuf.time, outbuf);
         printf("\nBM>>>>>  TRIGGER !!!!!\n");
         printf("BM>>>>> time tag = %s\n",outbuf);
         if (mbuf.command1.tran_rec)
            printf("BM>>>>> RT->BC (%d)  \n", messno);
         else
            printf("BM>>>>> BC->RT (%d)  \n", messno);
         printf("BM>>>>> RT-%d  \n",mbuf.command1.rtaddr);
         printf("BM>>>>> SA-%d  \n",mbuf.command1.subaddr);
         printf("BM>>>>> WC%d  \n",mbuf.command1.wcount);
         printf("BM>>>>> 0x%08lx\n",mbuf.int_status);
         printf("\n");
	  }
      else
      {
         BusTools_TimeGetFmtString(API_TTD_IRIG_NS,&mbuf.time, outbuf); printf("BM>>>>> time tag = %s\n",outbuf);
         if (mbuf.command1.tran_rec)
            printf("BM>>>>> RT->BC (%d)  \n", messno);
         else
            printf("BM>>>>> BC->RT (%d)  \n", messno);
         printf("BM>>>>> RT-%d  \n",mbuf.command1.rtaddr);
         printf("BM>>>>> SA-%d  \n",mbuf.command1.subaddr);
         printf("BM>>>>> WC%d  \n",mbuf.command1.wcount);
         printf("BM>>>>> 0x%08lx\n",mbuf.int_status);
      }
      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
  }
  return 0;
}

/****************************************************************************
*
*  Function:  setupThe_BM_IntFIFO 
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a BM message.  The interrupt is set for only the end-of
*     message.  
*
****************************************************************************/
void setupThe_BM_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.function     = bm_intFunction;
   sIntFIFO1.iPriority    = THREAD_PRIORITY_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.iNotification  = 0;
   sIntFIFO1.FilterType     = EVENT_BM_TRIG | EVENT_BM_MESSAGE;

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO1.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}

