/*===========================================================================*
 * FILE:             E X M A P L E _ D B C A . C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    This application shows how to setup Dynamic Bus Controller 
 * allocation for both the Remote Terminal and the Bus Controller.  This example 
 * uses two channels, one as the initial Bus Controller and the other as the 
 * initial Remote Terminal.  The RT channel also configures a Bus Controller, 
 * but it is not started. 
 * 
 * In this example the active BC is initialized to send a bus list using RTs 
 * 2, 4 and 6. There is also an aperiodic message to send a mode code 0 to RT1.  
 * If RT1 accepts the DBCA by setting the DBA bit in the 
 * Status word, the Bus Controller halts.  The inactive BC, configured by the RT 
 * channel, has a bus list to RTs 12, 14, and 16.  RT1 is setup to accept the DBCA 
 * Mode Code 0.  It automatically starts the Bus Controller.  User input from the 
 * console sends the aperiodic Mode Code 0 to RT1.  RT1 accepts the DBCA by 
 * setting the DBA bit in the status word of thje response to the mode code. 
 * Once that transacts, the initial BC uses a conditional branch to on the DBA 
 * to branch to ah HALT that stop the BC. The RT side BC takes over.
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include "busapi.h"

void BusController_Init(int);
void BusController_RT_Init(int cardnum);
void setupThe_RT_IntFIFO(int);
void setupThe_BC_IntFIFO(int cardnum);

int status;
API_BC_MBUF bcmessage;
unsigned messno,device;
unsigned icount=0,  last_count=0;
static unsigned total_bc_messages_processed=0;
unsigned errcount = 0;
unsigned ccount=0;
int pwFlag;
API_INT_FIFO sIntFIFO1, sIntFIFO2;


/* Set up RT to response to the BC Messages */

int RemoteTerminal_Init(int cardnum)
{
   int status;
   int RT_MBUF_COUNT;
   API_RT_CBUF cbuf;       // RT Control Buffer Legal Word Count Mask (2 words)
   API_RT_MBUF_WRITE mbuf;
   API_RT_ABUF abuf;
   int         tr;
   int         subaddress;
   int i,j,ii,jj;
   int rt_addr[] = {1,2,3,4,6,12,14,16};
   int rt_cnt = 8;

   RT_MBUF_COUNT = 2;

   status = BusTools_RT_Init(cardnum,0);
   printf("BusTools_RT_Init status = %d\n",status);

   /**********************************************************************
   *  Output RT control buffers
   *   Enable or disable all word counts and mode codes as specified by
   *    the caller's arguements.
   **********************************************************************/

   cbuf.legal_wordcount = 0xffffffff;  // Legailize all word counts
   for ( i = 0; i < rt_cnt; i++)
   {
      for ( tr = 0; tr < 1; tr++ )
      {
         for ( subaddress = 0; subaddress < 8; subaddress++ )
         {
            status = BusTools_RT_CbufWrite(cardnum,rt_addr[i],subaddress,tr,
			                            RT_MBUF_COUNT,&cbuf);
            if(status)
               return status;
         }
      }
   }

   /*******************************************************************
   *  Fill in message buffers for receive and transmit sub-units.
   *******************************************************************/
   mbuf.error_inj_id = 0;
   mbuf.enable = BT1553_INT_END_OF_MESS;   // Interrupt enables
   // For all subaddresses...
   for (ii = 0; ii<rt_cnt; ii++)
   {  
      for (i=0; i<8; i++)
      {
         // For each data buffer...
         for (j=0; j<RT_MBUF_COUNT; j++)
         {
            for(jj=0;jj<9;jj++)
               mbuf.mess_data[jj] = (BT_U16BIT)(0xA000 + j);
            status = BusTools_RT_MessageWrite(cardnum,rt_addr[ii],i,0,j,&mbuf);
            status = BusTools_RT_MessageWrite(cardnum,rt_addr[ii],i,1,j,&mbuf);
         }
      }
   }

   /************************************************
   *  Setup RT address control block
   ************************************************/

   abuf.enable_a          = 1;    // Enable Bus A...Note that the API bits are
   abuf.enable_b          = 1;    // Enable Bus B... reversed from the HW bits.
   abuf.inhibit_term_flag = 0;
   abuf.status            = 0x0000;
   abuf.command           = 0;
   abuf.bit_word          = 0;

   for(i=0;i<rt_cnt;i++)
   {
      if(rt_addr[i] == 1)                           // Enable DBCA for RT 1
         abuf.inhibit_term_flag = RT_ABUF_DBC_ENA; 
	  else
	     abuf.inhibit_term_flag = 0;                // Disable DBCA for other RTs

      status = BusTools_RT_AbufWrite(cardnum,rt_addr[i],&abuf);
   }

   return status;
}

void main(int argc, char **argv)
{
   BT_INT status;
   BT_INT cardnumBC,cardnumRT;
   char ichar;

   pwFlag = API_B_MODE | API_SW_INTERRUPT; 
   device = 0;
   
   /* Initializing Channel */
   status = BusTools_API_OpenChannel(&cardnumBC,pwFlag,device,CHANNEL_1);
   printf("BusTools_API_OpenChannel status = %d\n",status);

   status = BusTools_API_OpenChannel(&cardnumRT,pwFlag,device,CHANNEL_2);
   printf("BusTools_API_OpenChannel status = %d\n",status);

   status = BusTools_SetInternalBus(cardnumBC, EXTERNAL_BUS); /*Run internally for this example */
   printf("BusTools_SetInternalBus status = %d\n",status);

   status = BusTools_SetInternalBus(cardnumRT, EXTERNAL_BUS); /*Run internally for this example */
   printf("BusTools_SetInternalBus status = %d\n",status);

   //Set up the RT
   RemoteTerminal_Init(cardnumRT);   // Initialize the Remote Terminal
   BusController_RT_Init(cardnumRT); // Initialize the Bus Controller that start on Mode Code 0 DBCA
   setupThe_RT_IntFIFO(cardnumRT);   // Setup the interrupt callback for Mode Code 0 to RT 1 only

   //Set up the BC
   BusController_Init(cardnumBC);    // Initialize the Bus Controller
   setupThe_BC_IntFIFO(cardnumBC);   // Setup the interrupt callback for the HALT command only

   printf("Hit enter to start the test\n");
   getc(stdin);

   //Start the RT Running (CH2)
   status = BusTools_RT_StartStop(cardnumRT,RT_START);
   printf("BusTools_RT_StartStop status = %d\n",status);

   //Start the BC Running (CH1)
   status = BusTools_BC_StartStop(cardnumBC,BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);

   fflush(stdin);
   printf("\nEnter a or A to send Mode Code 0 (DBCA) to RT 1\n");
   do{
      ichar = getchar();
      if(ichar == 'a' || ichar == 'A')
         BusTools_BC_AperiodicRun(cardnumBC,30,0,0,0);  // Send Mode Code 0 to RT1
  
   }while (ichar != 'q');

   status = BusTools_BC_StartStop(cardnumBC,BC_STOP);
   printf("BusTools_BC_StartStop status = %d\n",status);

   status = BusTools_RT_StartStop(cardnumRT,RT_STOP);
   printf("BusTools_RT_StartStop status = %d\n",status);

   BusTools_API_Close(cardnumBC); 
   printf("BusTools_API_Close status = %d\n",status);
   BusTools_API_Close(cardnumRT);
   printf("BusTools_API_Close status = %d\n",status);
  
}

void BusController_Init(int cardnum)
{
   status = BusTools_BC_Init(cardnum,0,BT1553_INT_END_OF_MESS, 0,14,12,1000000,1);
   printf("BusTools_BC_Init status = %d\n",status);

   status = BusTools_BC_MessageAlloc(cardnum,50);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);

   messno = 0;	
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno+1);	
   bcmessage.control = BC_CONTROL_MESSAGE;      
   bcmessage.control |= BC_CONTROL_BUFFERA;     
   bcmessage.control |= BC_CONTROL_BUSA;
   bcmessage.control |= BC_CONTROL_MFRAME_BEG; 
   bcmessage.mess_command1.rtaddr   = 2;
   bcmessage.mess_command1.subaddr  = 2;
   bcmessage.mess_command1.wcount   = 2;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.gap_time = 10;
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno+1);
   bcmessage.control = BC_CONTROL_MESSAGE;      
   bcmessage.control |= BC_CONTROL_BUFFERA;
   bcmessage.control |= BC_CONTROL_BUSA;
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 4;
   bcmessage.mess_command1.wcount   = 4;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.gap_time = 10;
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno+1);
   bcmessage.control = BC_CONTROL_MESSAGE;      
   bcmessage.control |= BC_CONTROL_BUFFERA;
   bcmessage.control |= BC_CONTROL_BUSA;
   bcmessage.mess_command1.rtaddr   = 6;
   bcmessage.mess_command1.subaddr  = 6;
   bcmessage.mess_command1.wcount   = 6;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.gap_time = 10;
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(0);
   bcmessage.control  = BC_CONTROL_NOP;      ///NOOP
   bcmessage.control |= BC_CONTROL_MFRAME_END; 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);


   //Aperiodic messages Mode Code 0 (DBCA) to RT 1.
   messno = 30;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;      
   bcmessage.control |= BC_CONTROL_BUFFERA;
   bcmessage.control |= BC_CONTROL_BUSA;
   bcmessage.mess_command1.rtaddr   = 1;
   bcmessage.mess_command1.subaddr  = 0;
   bcmessage.mess_command1.wcount   = 0;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.gap_time = 50;
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));       // Clear all fields.
   bcmessage.messno				= messno;		        // Message number
   bcmessage.control            = BC_CONTROL_CONDITION;	// BC_CONTROL_CONDITION message.
   bcmessage.address            = API_BRANCH_MSTAT1;    // Status word 1
   bcmessage.data_mask          = 0x2;  	            // Look at DBA bit only
   bcmessage.data_value         = 0x2;                  // DBA bit
   bcmessage.messno_next        = messno + 1;           // If condition is FALSE, go here
   bcmessage.messno_branch      = messno + 2;           // If condition is TRUE, go here
   status = BusTools_BC_MessageWrite(cardnum, messno, &bcmessage);

   // If RT rejects the DBCA keep running
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)0xffff;
   bcmessage.control  = BC_CONTROL_NOP;      // NOOP
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);

   // If RT accepts the DBCA then halt the BC.
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)0xffff;
   bcmessage.control = BC_CONTROL_HALT;        // Halt the Bus Controller
   bcmessage.control |= BC_CONTROL_INTERRUPT;  // Interrupt on the Halt
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
}

void BusController_RT_Init(int cardnum)
{
   status = BusTools_BC_Init(cardnum,0,BT1553_INT_END_OF_MESS, 0,14,12,1000000,1);
   printf("BusTools_BC_Init status = %d\n",status);

   status = BusTools_BC_MessageAlloc(cardnum,10);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);

   messno = 0;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno+1);	
   bcmessage.control = BC_CONTROL_MESSAGE;      
   bcmessage.control |= BC_CONTROL_BUFFERA;     
   bcmessage.control |= BC_CONTROL_BUSA;
   bcmessage.control |= BC_CONTROL_MFRAME_BEG; 
   bcmessage.mess_command1.rtaddr   = 12;
   bcmessage.mess_command1.subaddr  = 12;
   bcmessage.mess_command1.wcount   = 12;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.gap_time = 10;
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno+1);
   bcmessage.control = BC_CONTROL_MESSAGE;      
   bcmessage.control |= BC_CONTROL_BUFFERA;
   bcmessage.control |= BC_CONTROL_BUSA;
   bcmessage.mess_command1.rtaddr   = 14;
   bcmessage.mess_command1.subaddr  = 14;
   bcmessage.mess_command1.wcount   = 14;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.gap_time = 10;
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno+1);
   bcmessage.control = BC_CONTROL_MESSAGE;      
   bcmessage.control |= BC_CONTROL_BUFFERA;
   bcmessage.control |= BC_CONTROL_BUSA;
   bcmessage.mess_command1.rtaddr   = 16;
   bcmessage.mess_command1.subaddr  = 16;
   bcmessage.mess_command1.wcount   = 16;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.gap_time = 10;
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(0);
   bcmessage.control  = BC_CONTROL_NOP;      ///NOOP
   bcmessage.control |= BC_CONTROL_MFRAME_END; 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
}

/****************************************************************************
*
*  Function:  rt_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT _stdcall rt_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{
   int status;
   API_RT_MBUF_READ rt_mbuf;
   BT_INT tail;

   BT_UINT messno,rt_addr,sub_addr,trans;
   static int sflg = 0;
   int wcount,i,mc;
   static int cnt = 1;

   mc = 0;
   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      messno   = sIntFIFO->fifo[tail].bufferID;
      rt_addr  = sIntFIFO->fifo[tail].rtaddress;        // RT address
      trans    = sIntFIFO->fifo[tail].transrec;         // Transmit/Receive
      sub_addr = sIntFIFO->fifo[tail].subaddress;       // Subaddress number

      status = BusTools_RT_MessageRead( cardnum, rt_addr, sub_addr, trans, messno, &rt_mbuf);
      if (status)
         return status;

      if(rt_mbuf.status & BT1553_INT_RT_RT_FORMAT)
         printf("RT<->RT (%d) \n",messno);
      else if(rt_mbuf.status & BT1553_INT_MODE_CODE)
      {
         printf("RT--MC  (%d) \n", messno);
         mc = 1;
      }
      else if (rt_mbuf.mess_command.tran_rec)
         printf("RT->BC (%d) \n", messno);
      else
         printf("BC->RT (%d)  \n", messno);
      printf("RT**RT-%d  \n",rt_mbuf.mess_command.rtaddr);
      printf("RT**SA-%d  \n",rt_mbuf.mess_command.subaddr);
      printf("RT**WC-%d  \n",rt_mbuf.mess_command.wcount);
      printf("RT**mstat-0x%04x\n",*((WORD*)(&rt_mbuf.mess_status)));
      printf("RT**int_stat-0x%08lx\n",rt_mbuf.status);

      if(mc)
      {
         if(wcount >=16)
            wcount = 1;
         else
            wcount = 0;
      }
      else
      {
         if (rt_mbuf.mess_command.wcount == 0)
            wcount = 32;           // Zero word count means 32 words.
         else
            wcount = rt_mbuf.mess_command.wcount ;               // See if this msg shorted than previous
      }

      for ( i = 0; i < wcount; i++ ) // Display all of the data words
      {
         // If this is an RT->BC message, and there is a Message Error
         //  or No Response, than xxxx the data (since the RT did not
         //    send any data words).
         
         printf("RT**%04x ",rt_mbuf.mess_data[i]);
      }
      printf("\n");
  
      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }
   return 0;
}

/****************************************************************************
*
*  Function:  bc_intCtlFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_intCtlFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
 
   static int sa_index;
   BT_INT tail;
   BT_UINT messno;

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      messno = sIntFIFO->fifo[tail].bufferID;
      printf(" BC CONTROL INTERRUPT on buffer ID - HALT %d\n",messno);  
              
      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }

   return 0;
}

/****************************************************************************
*
*  Function:  setupThe_RT_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a BC message.  The interrupt is set for only the end-of
*     message.  
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupThe_RT_IntFIFO(int cardnum)
{
   int status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.iNotification = 0;
   sIntFIFO1.function     = rt_intFunction;
   sIntFIFO1.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.bForceStartup  = 0;
   sIntFIFO1.FilterType     =  EVENT_RT_MESSAGE;

   sIntFIFO1.FilterMask[1][1][0] =  0x1;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}

void setupThe_BC_IntFIFO(int cardnum)
{
   int status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO2, 0, sizeof(API_INT_FIFO));
   sIntFIFO2.iNotification  = 0;
   sIntFIFO2.function       = bc_intCtlFunction;
   sIntFIFO2.iPriority      = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO2.dwMilliseconds = INFINITE;
   sIntFIFO2.bForceStartup  = 0;
   sIntFIFO2.FilterType     = EVENT_BC_CONTROL;


   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO2, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}

