/*===========================================================================*
 * FILE:             E X A M P L E _ E X T _ T R I G . C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 * FUNCTION:  This application shows how to setup interrupts on external 
 * trigger. In addition it shows how to setup discrete for input and output 
 * triggers. This test requires a wrap connector between discrete 7 and discrete 
 * 8 on the D50 connector. The test generates a output trigger and wraps output 
 * on discrete 7 to the input on discrete 8. The input trigger generates an 
 * interrupt.
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/


#include <stdio.h>
#include <busapi.h>

UINT timerId;
BT1553_TIME ttime;
API_INT_FIFO trig_fifo;
API_INT_FIFO tfifo;

BT_INT _stdcall intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{
   static CEI_UINT64 last_time=0;
   BT_INT tail;
  
   union bigtime
   {
      CEI_UINT64 btime;
	  BT1553_TIME ttime;
   }bt;

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {

      BusTools_TimeTagRead(cardnum, &bt.ttime);
	  if(last_time != 0) // don't print the first time.
         printf("Trigger interrupt time = %d\n",(bt.btime - last_time)/1000);
      last_time = bt.btime;	

      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }
   return 0;
}

BT_INT _stdcall timeFunction(BT_UINT cardnum, struct api_int_fifo *tIntFIFO)
{
   if(tIntFIFO->timeout)
   {
      BusTools_ExtTriggerOut(cardnum,1000); //generate a trigger to simulates and external trigger
   }
   return 0;
}


int main(int argc, char **argv)
{
	int status;
    BT_UINT flag = API_B_MODE | API_SW_INTERRUPT;
	BT_INT cardnum;
    BT_UINT device = 0, count =0;
    
	if(argc == 2)
		device = atoi(argv[1]);

    status = BusTools_API_OpenChannel(&cardnum,flag,device,CHANNEL_1);
    printf("BusTools_API_OpenChannel status = %d\n",status);

    status = BusTools_SetInternalBus(cardnum,INTERNAL_BUS);          // Set up internal bus
    printf("BusTools_SetInternalBus status = %d\n",status);

    status = BusTools_SetVoltage(cardnum,DIRECT_MAX_VOLTS,DIRECT);

    status = BusTools_BM_Init(cardnum,1,1);
    status = BusTools_BM_MessageAlloc(cardnum,10,&count,0);
    status = BusTools_BM_StartStop(cardnum,BM_START);

       //Set the discrete for both input and output triggers
	   //Require discrete 7 and discrete 8 to be wrapped on the 1553 cable
    if(BusTools_BoardIsV6(cardnum))
    {
       status = BusTools_SetV6TrigOut(cardnum, DISCRETE, 7);            //Enable Trigger out on Discrete 7
       status = BusTools_SetV6TrigIn(cardnum, DISCRETE, 8);             //Enable Trigger in on Discrete 8
       status = BusTools_DiscreteWrite(cardnum,8,0);                    //Clear Discrete 7
    }
    else
    {
       status = BusTools_DiscreteSetIO(cardnum,0x080,0x3ff);            // Set the DISC8 for out and DISC7 for in
       status = BusTools_DiscreteTriggerOut(cardnum,TRIGGER_OUT_DIS_8); // External trigger out on Discrete 7
       status = BusTools_DiscreteTriggerIn(cardnum,TRIGGER_IN_DIS_7);   // External trigger in  on Discrete 8   
    }
    status = BusTools_ExtTrigIntEnable(cardnum, EXT_TRIG_ENABLE);       // Enable interrupt on external trigger

	//check the discretes are working as expected
	{
       BT_U32BIT dval = 0x0;
	   status = BusTools_DiscreteWrite(cardnum,8,0);                       // Clear Discrete 8
	   status = BusTools_DiscreteWrite(cardnum,7,1);                       // Set Discrete 7
	   status = BusTools_DiscreteRead(cardnum,8,&dval);
	   printf("discrete 8 value = %d\n",dval);
	   status = BusTools_DiscreteWrite(cardnum,7,0);                       // Clear Discrete 7
  	   status = BusTools_DiscreteRead(cardnum,8,&dval);
	   printf("discrete 8 value = %d\n",dval);
	}

    // Setup the FIFO structure for trigger interrupt event.
    memset(&trig_fifo, 0, sizeof(trig_fifo));
    trig_fifo.function     = intFunction; // This is the interrupt function the API calls
    trig_fifo.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
    trig_fifo.dwMilliseconds = INFINITE;
    trig_fifo.iNotification  = 0;
    trig_fifo.FilterType     = EVENT_EXT_TRIG;  //Trigger Event
 
    // Call the register function to register and start the thread.
    status = BusTools_RegisterFunction(cardnum, &trig_fifo, REGISTER_FUNCTION);

	//Set up the FIFO for the timerout event (no BM event) 
    memset(&tfifo, 0, sizeof(tfifo));
    tfifo.function     = timeFunction; // This is the interrupt function the API calls
    tfifo.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
    tfifo.dwMilliseconds = 500; // time out in 500 milliseconds
    tfifo.iNotification  = 0;
    tfifo.FilterType     = EVENT_BM_MESSAGE;
 
    // Call the register function to register and start the thread.
    status = BusTools_RegisterFunction(cardnum, &tfifo, REGISTER_FUNCTION);
    
    getchar();

    BusTools_BM_StartStop(cardnum, BM_STOP);
   
    BusTools_ExtTrigIntEnable(cardnum, EXT_TRIG_DISABLE);
    status = BusTools_RegisterFunction(cardnum, &trig_fifo, UNREGISTER_FUNCTION); 
	status = BusTools_RegisterFunction(cardnum, &tfifo, UNREGISTER_FUNCTION); 
    status = BusTools_API_Close(cardnum);
}

