/*===========================================================================*
 * FILE:             E X A M P L E _ B C _ M U L T I _ B U F F E R . C
 *===========================================================================*
 *
  *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    Demonstration of BusTools/1553-API routines. This example shows
 *              how to erase, write and read flash for the specific flash blocks 
 *              for the  RAR15XMC_XT board.
 *              This examples requires Firmware version 6.5 or greater,and 
 *              BusTools/1553-API version 8.14 or greater. 
 *
 *===========================================================================*/

/* $Revision:  8.14 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  3/16/2015    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <conio.h>
#include "busapi.h"


#if defined (__WIN32__)
void main(int argc, char **argv)                     
#else
int tst_flash_apis()
#endif
{
   int status,cardnum;
   unsigned channel,device,mode;
   int i=0;
   BT_U8BIT pagedata[256], rdata[256];
   BT_INT sector = 0, pagenumber = 0, readoffset = 0;
   float lpu,wcs;
   int build;
   BT_UINT mrev, arev;

   mode = API_B_MODE | API_SW_INTERRUPT;  // MIL-STD-1553B with S/W interrupts enabled
   channel = CHANNEL_1;                   // Default channel 1

   if(argc == 2)
      device = atoi(argv[1]);
   else
      device = 0;                         // check your device number and slot 

   // Initialize the channel
   status = BusTools_API_OpenChannel(&cardnum,mode,device,channel);
   printf("BusTools_API_OpenChannel status = %d\n",status);

   status = BusTools_SetInternalBus(cardnum,EXTERNAL_BUS);  // Setup bus External
   printf("BusTools_SetInternalBus status = %d\n",status);

   // Set the time tag mode for display and initial setting
   BusTools_TimeTagMode(cardnum,API_TTD_IRIG_NS,API_TTI_IRIG,API_TTM_FREE,0,0,0,0);

   status = BusTools_GetRevision(cardnum,&mrev,&arev);
   printf("mrev = %x --- arev = %d\n",mrev,arev);
   status = BusTools_GetFWRevision(cardnum,&wcs,&lpu, &build);
   printf("The LPU revision = %3.2f -- The WCS revision = %3.2f, build = %d\n",lpu,wcs, build);
   
   memset((char*)rdata,0,sizeof(rdata)); 
   memset((char*)pagedata,0,sizeof(pagedata)); 
   
   for (i=0; i<256; i++)
   {
      pagedata[i] = i;
   }
   
   sector = 1;
   pagenumber = 0;
   readoffset = 0;

   // Enter Critical Section
   status = BusTools_FlashLogErase(cardnum, sector);
   // Exit Critical Section
   printf("BusTools_FlashErase status = %d\n",status);
      
   // Enter Critical Section
   BusTools_FlashLogRead(cardnum,sector, readoffset, 256, rdata);
   // Exit Critical Section
   printf("BusTools_FlashRead status = %d section = %d  offset = %d\n",status,sector,readoffset);
   
   /*
   for (i=0; i<256; i++)
   {
      printf("read data righ afeter erase i = %d value =%d\n",i,rdata[i]);
   
   }
   */

   // Enter Critical Section
   BusTools_FlashLogWrite(cardnum,sector, pagenumber, pagedata);
   // Exit Critical Section
   
   printf("BusTools_FlashWrite status = %d section = %d  page = %d\n",status,sector,pagenumber);
  
   // Enter Critical Section
   BusTools_FlashLogRead(cardnum,sector, readoffset, 256, rdata);
   // Exit Critical Section
   printf("BusTools_FlashRead status = %d section = %d  offset = %d\n",status,sector,readoffset);
   /*
   for (i=0; i<256; i++)
   {
      printf("read data i = %d value =%d\n",i,rdata[i]);
   
   }
   */
  
   printf("Hit Enter to stop and exit\n");
   getchar();                                              

   printf("main: closing API\n");

   
   status = BusTools_API_Close(cardnum);                  // Close the channel
   printf("BusTools_API_Close status = %d \n",status);
   printf("Hit Enter to stop and exit\n");
   getchar();                                              

   printf("main: closing API\n");

}

