/*============================================================================*
 * FILE:             E X A M P L E _ I R I G 1 . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:    EXAMPLE PROGRAM
 * This test program demonstrates the setup of the IRIG-B output and input.  
 * This program just configures IRIG for EXTERNAL or INTERNAL  source, sets the 
 * IRIG the current time/date and then loops 20 times reading and displaying the 
 * time every second.  This REQUIRES a board with the IRIG option.  The part 
 * number should include a 'W', which indicates IRIG.  For example:  
 * QPCX-1553-4MW.  Use BusTools_BoardHasIRIG to find out if you have an IRIG 
 * enabled board.
 *
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
	01/26/04   Initial version.  RSW
	01/27/04   Minor cleanup.  RSW
    09/20/04   Added option to initialize with BusTools_API_LoadChannel.  RSW
    05/04/2010 Update to work with new IRIG functions. RHC 
*/

#include "busapi.h"
#include <time.h>
#include <stdio.h>


// Main program
int main(int argc, char ** argv)
{
	int         status, i, ch_id;
	char        ichar, outbuf[100];
    BT1553_TIME rtime;
    BT_UINT dev_num, mode;

    if(argc == 2)
      dev_num = atoi(argv[1]);
    else
      dev_num = 0;  //Default to device 0

    // Open the device and get the channel id.
    mode = API_B_MODE | API_SW_INTERRUPT;  // 1553B protocol, use SW interrupts.
    status = BusTools_API_OpenChannel( &ch_id, mode, dev_num, CHANNEL_1);

	if (status == API_SUCCESS) 
    {
		printf("Success.\n\n");

        if(BusTools_BoardHasIRIG(ch_id) == 0)
           return API_FEATURE_SUPPORT;

		// Initialize and reset memory.  Minimum BM setup.
		status = BusTools_BM_Init(ch_id, 1, 1);
		if (status != API_SUCCESS) printf("Error %d on BusTools_BM_Init\n",status);

		printf("Do you have an external IRIG-B Genator connect to the 1553 interface board? ");
        ichar = getchar();
        printf("\n");
        if(ichar == 'y' || ichar == 'Y')
        {
           status = BusTools_IRIG_Config(ch_id, IRIG_EXTERNAL, IRIG_OUT_DISABLE); 
		   if (status != API_SUCCESS) 
              printf("Error %d on BusTools_IRIG_Config\n", status);

           status = BusTools_IRIG_Calibration(ch_id, 1);
           MSDELAY(2500); // wait 2.5 seconds
           status = BusTools_IRIG_Valid(ch_id);
           if(status)
           {
              printf("Invalid IRIG-B signal\n");     
              exit(0);
           }
        }
        else
        {
           time_t ttime;

           printf("Using localtime to set Internal IRIG-B timer\n");
           status = BusTools_IRIG_Config(ch_id, IRIG_INTERNAL, IRIG_OUT_ENABLE); 
		   if (status != API_SUCCESS)
              printf("Error %d on BusTools_IRIG_Config\n", status);

           ttime = time(NULL);
           
           status = BusTools_IRIG_SetTime(ch_id,ttime,1);
           MSDELAY(2500);

           if(status)
           {
              printf("Invalid IRIG-B signal\n");     
              exit(0);
           }
        }

		// Set time tag mode for IRIG.
		status = BusTools_TimeTagMode(ch_id, API_TTD_IRIG, API_TTI_IRIG, API_TTM_IRIG, 0, 0, 0, 0);
		if (status != API_SUCCESS) printf("Error %d on BusTools_TimeTagMode\n", status);

        for(i= 0; i < 20; i++)
        {
            status = BusTools_TimeTagRead(ch_id,&rtime);

            BusTools_TimeGetString(&rtime,outbuf);
            printf("Time - %s\n",outbuf);
            MSDELAY(1000);
        }
     
		// We're done.  Close API and board
		printf("\nClosing API . . . ");
		status = BusTools_API_Close(ch_id);
		if (status == API_SUCCESS)
			printf("Success.\n");
		else 
			printf("FAILURE, error = %d\n", status);
	} // End of if (initialization successful)
	else printf("FAILURE, error = %d\n", status);

    fflush(stdin);
	printf("FINISHED. Hit Enter to exit\n");
    getchar();
    return 0;
} // End of main
