/*============================================================================*
 * FILE:                 E X A M P L E _ R E V . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:    EXAMPLE PROGRAM
 *              This example program initializes a board and displays version 
 *              information and general board information.
 *
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
	09/20/04	Initial version.  RSW
*/

#include "busapi.h"
#include <stdio.h>


// Main program
int main(int argc, char **argv)
{
   BT_INT  status, ch_id, build, btype, temp;
   BT_INT chCnt;
   BT_UINT device, arev, frev, mode;
   BT_U32BIT serial_number;
   float wrev, lrev;
   DEVICE_INFO dInfo;

   // Use command line device or default device 0
   if(argc == 2)
      device = atoi(argv[1]);
   else
      device = 0;

   status = BusTools_GetDevInfo(device, &dInfo);
   printf("BusTools_GetDevInfo status = %s\n",BusTools_StatusGetString(status));
   if(status)
      exit(0);
   printf("%Vendor ID = %x Device ID = %x  Bus Type = %d\n",dInfo.VendorID,dInfo.DeviceID,dInfo.busType);


   // Open the device and get the channel id.
   mode = API_B_MODE | API_SW_INTERRUPT;  // 1553B protocol, use SW interrupts.
   status = BusTools_API_OpenChannel( &ch_id, mode, device, CHANNEL_1);
   if(status)
      printf("BusTools_API_OpenChannel status = %d\n",status);

   btype = BusTools_GetBoardType(ch_id);
   printf("Board Type         = %x - ",btype);    // Use hex to display the board type.  

   switch(btype)
   {
      case QPM1553:   /* This is the QPM-1553 native PMC board        */
         printf("QPM-1553\n");
         break;
      case QPCI1553:    /*This is the QPCI-1553 Native PCI board      */
         printf("QPCI-1553\n");
         break;
      case Q1041553P:   /* This is a PC\104 4-Ch PCI board            */
         printf("Q104-1553P\n");
         break;
      case RQVME2:      /* This is a RoHS QVME-1553                   */
         printf("RQVME2-1553/QVME-1553\n");
         break;
      case PCCD1553:    /* This is a Dual Channel PCCard-D1553        */
         printf("PCCard-D1553\n");
         break;
      case QCP1553:     /* This is the QCP-1553 board                 */
         printf("QCP-1553\n");
         break;
      case QPCX1553:    /* This is the QPCX-1553 Native PCI board     */
         printf("QPCX-1553\n");
         break;
      case R15EC:       /* This is the R15-EC express card board      */
         printf("R15-EC\n");
         break;
      case R15AMC:      /* This is the R15-AMC                        */
         printf("R15-AMC\n");
         break;
      case RPCIe1553:   /* This is the RPCIe-1553 PCI Express native  */
         printf("RPCIe-1553\n");
         break;
      case RXMC1553:    /* This is the RXMC-1553                      */
         printf("RXMC-1553\n");
         break; 
      case R15XMC2:     /* RXMC2-1553                                 */
         printf("RXMC2-1553\n");
         break;
      case LPCIE1553:   /* This is the low profile PCI express        */
         printf("LPCIe-1553\n");
         break;
      case MPCIE1553:   /* This is the low profile PCI express        */
         printf("MPCIe-1553\n");
         break;
      case RAR15XMCXT:  /* This is the RAR15-XMC XT IT or FIO         */
         printf("RAR15-XMC-XT/IT/FIO\n");
         status = BusTools_ReadBoardTemp(ch_id,INTERNAL, &temp);
         printf("INTERNAL TEMP = %d", temp);
         if(status == API_OVER_TEMP_ALARM)
            printf(" -- OVERTEMP!!!\n");
         else
            printf("\n");

         status = BusTools_ReadBoardTemp(ch_id,TFPGA, &temp);
         printf("TFPGA TEMP = %d", temp);
         if(status == API_OVER_TEMP_ALARM)
            printf(" -- OVERTEMP!!!\n");
         else
            printf("\n");


         status = BusTools_ReadBoardTemp(ch_id,TZBT, &temp);
         printf("TZBT TEMP = %d", temp);
         if(status == API_OVER_TEMP_ALARM)
            printf(" -- OVERTEMP!!!\n");
         else
            printf("\n");

         status = BusTools_ReadBoardTemp(ch_id,TCHANNEL, &temp);
         printf("TCHANNEL TEMP = %d", temp);
         if(status == API_OVER_TEMP_ALARM)
            printf(" -- OVERTEMP!!!\n");
         else
            printf("\n");

         status = BusTools_ReadBoardTemp(ch_id,TBOARD, &temp);
         printf("TBOARD TEMP = %d", temp);
         if(status == API_OVER_TEMP_ALARM)
            printf(" -- OVERTEMP!!!\n");
         else
            printf("\n\n");

         break;
      case R15USB:      /* This is the R15-USB                        */
         
         if(BusTools_BoardIsUSBMon(ch_id))
         {
             printf("R15-USB-MON\n");                 
         }
         else
         {
             printf("R15-USB\n");
         }
         break;
      default:
         printf("Unknown\n");
   } 
      
   // Use this function to get BusTools/1553-API revision
   status = BusTools_GetRevision(ch_id,&frev,&arev);
   if(status)
      printf("BusTools_GetRevision status = %d\n",status);
   printf("API Revision       = %d\n",arev);      // Use decimal to display the API version

   // Get firmware version information
   status = BusTools_GetFWRevision(ch_id, &wrev, &lrev, &build);
   if(status)
      printf("BusTools_GetFWRevision status = %d\n",status);

   printf("LPU Revision       = %f\n",lrev);
   printf("Microcode Revision = %f\n",wrev);
   if(BusTools_BoardIsV6(ch_id))
      printf("FPGA Build         = %x\n", build);  // Use hex to display the FPGA/LPU build number
   else
      printf("LPU Build          = %x\n");

   // report whether single-, dual-, or multi-function  
   status = BusTools_BoardIsMultiFunction(ch_id);
   switch(status)
   {
      case API_SINGLE_FUNCTION:
         printf("Board is single-function\n");
         break;
      case API_DUAL_FUNCTION:
         printf("Board is dual-function\n");
         break;
      case API_MULTI_FUNCTION:
         printf("Boards is multi-function\n");
         break;
      default:
         printf("BusTools_BoardIsMultiFunction errror status = %d\n",status);
   }

   {
      BT_U16BIT CSCreg, ACreg;
      status = BusTools_GetCSCRegs(ch_id,&CSCreg, &ACreg);
	  if(status == API_SUCCESS)
         printf("ACR = %x -- CSC = %x\n",ACreg,CSCreg);
   }
   // Report on the channel count
   chCnt = BusTools_GetChannelCount(ch_id);
   printf("Board has %d 1553 channels\n",chCnt);

   status = BusTools_BoardHasIRIG(ch_id);
   if(status == API_FEATURE_SUPPORT)
      printf("Board Has IRIG\n");
   else
   {
      if(status = 0)
         printf("Board does not have IRIG\n");
      else
         printf("BusTools_BoardHasIRIG error %d\n",status);
   }

   // Report Serial Number
   status = BusTools_GetSerialNumber(ch_id, &serial_number);
   if(status)
   {
      if(status = API_HARDWARE_NOSUPPORT)
         printf("Board type does not have serial number\n");
      else
         printf("BusTools_GetSerialNumber status = %d\n",status);
   }
   else
      printf("board serial number = %d\n",serial_number);

   {
      BT_U32BIT bytes;
      status = BusTools_MemoryAvailable(ch_id, &bytes);
      printf("Number of bytes available = %d\n",bytes);
   }

   

   // Close API and board
   printf("\nClosing API . . . ");
   status = BusTools_API_Close(ch_id);
   printf("BusTools_API_Close status = %d\n",status);

   fflush(stdin);
   printf("Hit Enter to exit\n");
   getchar();
}
