/*============================================================================*
 * FILE:                 E X A M P L E _ R T _EI _L A T E _ R S P . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:    EXAMPLE PROGRAM
 * This example program sets up RT1 with two subaddresses, SA1 RECEIVE and SA2 
 * TRANSMIT.  It also demonstrates how to inject errors into an RT message. In 
 * this case, we inject a LATE RESPONSE ERROR.  The user can specify the response 
 * time from 7-31us.  This can be done for SA1 RECEIVE or SA2 TRANSMIT.
 *
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
	03/14/02	Initial version.  RSW
    09/20/04    Added option to initialize with BusTools_API_LoadChannel.  RSW
*/

#include "busapi.h"
#include <stdio.h>

API_ENH_EIBUF error_buffer;

void setUpTheRT(BT_INT ch_id);

// Main program
int main(int argc, char ** argv) 
{
   BT_INT status, ch_id, i;
   char	c;
   BT_UINT mode, device;

   mode = API_B_MODE | API_SW_INTERRUPT;

   if(argc == 2)
      device = atoi(argv[1]);
   else
      device = 0;

   // Open the device and get the channel id.
   mode = API_B_MODE | API_SW_INTERRUPT;  // 1553B protocol, use SW interrupts.
   status = BusTools_API_OpenChannel( &ch_id, mode, device, CHANNEL_1);
   printf("BusTools_API_OpenChannel status = %d\n", status);
 
   // Select External Bus.
   status = BusTools_SetInternalBus(ch_id, EXTERNAL_BUS);
   printf("BusTools_SetInternalBus status = %d\n", status);

   setUpTheRT(ch_id);

	// Start the RT 
   status = BusTools_RT_StartStop(ch_id, RT_START);
   printf("BusTools_RT_StartStop status = %d\n", status);

   printf("\nInput Q to stop RT and exit, R to inj err on RCV SA1, T to inj err on TX SA2.\n");
   printf("C to clear errors.\n");

   do
   {
      scanf("%c",&c);
      fflush(stdin);
 
      switch (c) 
      {
         case 'R': // Inj error on RT1 SA1 RX
         case 'r': // Inj error on RT1 SA1 RX
            printf("Response Time in us (7-31)? "); scanf("%d",&i);
            error_buffer.buftype = EI_RT_REC;
            error_buffer.error[0].etype = EI_LATERESPONSE;
            error_buffer.error[0].edata = i*2;
            status = BusTools_EI_EbufWriteENH(ch_id, 1, &error_buffer);
            printf("BusTools_EI_EbufWriteENH status = %d\n", status);
            break;
         case 'T': // Inj error on RT1 SA2 TX
         case 't': // Inj error on RT1 SA1 RX
            printf("Response Time in us (7-31)? "); scanf("%d",&i);
            error_buffer.buftype = EI_RT_TRANS;
            error_buffer.error[0].etype = EI_LATERESPONSE;
            error_buffer.error[0].edata = i*2;
            status = BusTools_EI_EbufWriteENH(ch_id, 2, &error_buffer);
            printf("BusTools_EI_EbufWriteENH status = %d\n", status);
            break;
         case 'C':
         case 'c': // Inj error on RT1 SA1 RX
            // RT1 SA1 RX - Error buffer 1
            error_buffer.buftype = EI_RT_REC;
            for (i=0; i<EI_COUNT; i++)
            {
               error_buffer.error[i].etype = EI_NONE;
               error_buffer.error[i].edata = 0;
            }
            status = BusTools_EI_EbufWriteENH(ch_id, 1, &error_buffer);
            printf("BusTools_EI_EbufWriteENH status = %d\n", status);

            // RT1 SA2 TX - Error buffer 2
            error_buffer.buftype = EI_RT_TRANS;
            status = BusTools_EI_EbufWriteENH(ch_id, 2, &error_buffer);
            printf("BusTools_EI_EbufWriteENH status = %d\n", status);
            break;
      } // End of switch (c)
   } while (c != 'Q' && c != 'q'); // End of do-while

   status = BusTools_RT_StartStop(ch_id, RT_STOP);
   printf("BusTools_RT_StartStop status = %d\n", status);

   status = BusTools_API_Close(ch_id);
   printf("BusTools_API_Close status = %d\n", status);		
} // End of main

void setUpTheRT(BT_INT ch_id)
{
   BT_INT i,status;
   API_RT_ABUF rt_abuf;    // RT address buffer structure.
   API_RT_CBUF rt_cbuf;    // RT control buffer structures.
   API_RT_MBUF_WRITE msg_buffer_write;

   BT_UINT rtaddr=1;
   BT_UINT subaddr;
   BT_UINT NUM_BUFFERS = 1;
   BT_UINT rt_buffer = 0;

   // Set up an RT.
   status = BusTools_RT_Init(ch_id, 0);
   printf("BusTools_RT_Init status = %d\n", status);

   // Setup RT address buffer for our RT (RT1)
   rt_abuf.enable_a = 1;			// Respond on bus A
   rt_abuf.enable_b = 1;			// Respond on bus B
   rt_abuf.inhibit_term_flag = 0;	// Inhibit terminal flag in status word
   rt_abuf.status = 0x0000;		    // Set status word (done by F/W)
   rt_abuf.bit_word = 0x0000;		// Set BIT word (for mode code 19)
   status = BusTools_RT_AbufWrite(ch_id, rtaddr, &rt_abuf);
   printf("BusTools_RT_AbufWrite status = %d\n", status);

   // Setup control buffer - RT1, SA1, Receive, 1 buffer.
   subaddr = 1;
   rt_cbuf.legal_wordcount = 0xFFFFFFFF;  // any word count is legal.
   status = BusTools_RT_CbufWrite(ch_id, rtaddr, subaddr, RECEIVE, NUM_BUFFERS, &rt_cbuf);
   printf("BusTools_RT_CbufWrite status = %d\n", status);

   // Setup control buffer - RT1, SA2, Transmit, 1 buffer.
   subaddr = 2;
   rt_cbuf.legal_wordcount = 0xFFFFFFFF;  // any word count is legal.
   status = BusTools_RT_CbufWrite(ch_id, rtaddr, subaddr, TRANSMIT, NUM_BUFFERS, &rt_cbuf);
   printf("BusTools_RT_CbufWrite status = %d\n", status);

   // Define two error injection buffers, no errors initially.
   // RT1 SA1 RX - Error buffer 1
   error_buffer.buftype = EI_RT_REC;
   for (i=0; i<EI_COUNT; i++) 
   {
      error_buffer.error[i].etype = EI_NONE;
      error_buffer.error[i].edata = 0;
   }
   status =  BusTools_EI_EbufWriteENH(ch_id, 1, &error_buffer);
   printf("BusTools_EI_EbufWriteENH status = %d\n", status);

   // RT1 SA2 TX - Error buffer 2
   error_buffer.buftype = EI_RT_TRANS;
   status = BusTools_EI_EbufWriteENH(ch_id, 2, &error_buffer);
   printf("BusTools_EI_EbufWriteENH status = %d\n", status);

   // Write data and assign error inj buffers to message buffers.
   // RT1 SA1 RECEIVE.
   msg_buffer_write.enable = 0;		    // No interrupts enabled
   msg_buffer_write.error_inj_id = 1;	// Use error buffer 1
   for (i=0; i<32; i++)
      msg_buffer_write.mess_data[i] = 0xAB00 + i;

   subaddr = 1;
   status = BusTools_RT_MessageWrite(ch_id, rtaddr, subaddr, RECEIVE, rt_buffer, &msg_buffer_write);
   printf("BusTools_RT_MessageWrite status = %d\n", status);

   // RT1 SA2 TRANSMIT.
   subaddr = 2;
   msg_buffer_write.error_inj_id = 2;	// Use error buffer 2
   status = BusTools_RT_MessageWrite(ch_id, rtaddr, subaddr, TRANSMIT, rt_buffer, &msg_buffer_write);
   printf("BusTools_RT_MessageWrite status = %d\n", status);

}