/*===========================================================================*
 * FILE:      E X A M P L E _ R T _ E X T E N D E D _ S T A T U S . C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    This example shows how to configure a Remote Terminal and 
 * enable Extended Status updates.  Under normal 1553 operation the status 
 * returned by the RT is for all Sub-addresses and transmit and receive buffer.  
 * BusTools_RT_AbufWrite sets the status response for the RT and enable extended 
 * status for a RT.  Using Extended Status mode the RT can set the status word for 
 * each Sub-address, Transmit, Receive and buffer for the RT.  
 * 
 * In this example the RTs are set up with two buffers per RT/SA/TX/RX combination. 
 * For RT 3 Transmit and RT 4 receive the second buffer is set to respond with 
 * updated status.  RT 3 responds with Busy (BSY) and RT 4 responds with Message 
 * Error (ME).  This causes the status word for those two messages to toggle 
 * between the RT message status and the extended status programmed by 
 * BusTools_RT_MessageWriteStatusWord.  To use Extended Status set the Extended 
 * Status enable in the inhibit terminal flag parameter in call to 
 * BusTools_RT_AbufWrite.
 *
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "busapi.h"

void BusController_Init(int,int);
void RemoteTerminal_Init(int);

void setupThe_BC_IntFIFO(int cardnum);


BT_UINT rt_int_count[32];
API_INT_FIFO sIntFIFO1;      // Thread FIFO structure for BC.
static unsigned bm_tcnt=0,bc_tcnt=0,rt_tcnt=0;

int main(int argc, char **argv)                     
{
   int status,cardnum;
   unsigned channel,device,mode;
   int i=0;

   mode = API_B_MODE | API_SW_INTERRUPT;  // MIL-STD-1553B with Software (Polling) interrupts enabled
   channel = CHANNEL_1;  //default channel 1
   device = 0;           //default device 0

   status = BusTools_API_OpenChannel(&cardnum,mode,device,channel);
   printf("BusTools_API_OpenChannel status = %d\n",status);

   status = BusTools_SetInternalBus(cardnum,EXTERNAL_BUS);  // Set up bus 0 External or  1 Internal
   printf("BusTools_SetInternalBus status = %d\n",status);
 
   BusController_Init(cardnum,50);
   RemoteTerminal_Init(cardnum);  

   setupThe_BC_IntFIFO(cardnum);

   status = BusTools_RT_StartStop(cardnum,RT_START);
   printf("BusTools_RT_StartStop status = %d\n",status);

   status = BusTools_BC_StartStop(cardnum,BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);

   getchar();

   printf("main: closing API\n");

   status = BusTools_BC_StartStop(cardnum,BC_STOP); 
   printf("BusTools_BC_StartStop status = %d\n",status);

   status = BusTools_RT_StartStop(cardnum,RT_STOP);
   printf("BusTools_RT_StartStop status = %d\n",status);

   status = BusTools_API_Close(cardnum);//
   printf("BusTools_API_Close status = %d\n",status);

}

void BusController_Init(int cardnum,int num_buffers)
{
   int status, messno,i;
   API_BC_MBUF bcmessage;

   // Initialize the BC with interrupt on end-of-message 16 us no response 14 us late response, 1sec frame (1000000 usec).     
   status = BusTools_BC_Init(cardnum,0,BT1553_INT_END_OF_MESS,0,16,14,1000000,1);
   printf("BusTools_BC_Init status = %d\n",status);

   status = BusTools_BC_MessageAlloc(cardnum,50);//num_buffers);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);

   // Receive (BC->RT) Command)
   messno = 0;
   memset((char*)&bcmessage,0,sizeof(bcmessage));       
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // Enable interrupt on message
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;    // Start of a minor frame
   bcmessage.mess_command1.rtaddr   = 7;          
   bcmessage.mess_command1.subaddr  = 7;
   bcmessage.mess_command1.wcount   = 7;
   bcmessage.mess_command1.tran_rec = 0;          // BC->RT
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 10;                       // 10 microsecond inter-message gap.

   for(i=0;i<7;i++)
       bcmessage.data[0][i] = 0xA000+i;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Transmit (RT->BC) Command
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // Enable interrupt on message
   bcmessage.mess_command1.rtaddr   = 2;
   bcmessage.mess_command1.subaddr  = 2;
   bcmessage.mess_command1.wcount   = 2;
   bcmessage.mess_command1.tran_rec = 1;          // BC->RT
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 50;                       // 50 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Transmit (RT->BC) Command
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // Enable interrupt on message
   bcmessage.mess_command1.rtaddr   = 3;
   bcmessage.mess_command1.subaddr  = 3;
   bcmessage.mess_command1.wcount   = 3;
   bcmessage.mess_command1.tran_rec = 1;          // BC->RT
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 50;                       // 50 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Receive (BC->RT) Command
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // enable interrupt on message
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 4;
   bcmessage.mess_command1.wcount   = 4;
   bcmessage.mess_command1.tran_rec = 0;          // BC->RT
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 50;                       // 50 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);


   //Noop message at end-of-frame
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)0;          // Repeat starting at message 0
   bcmessage.control = BC_CONTROL_NOP;            // 
   bcmessage.control |= BC_CONTROL_MFRAME_END;    // End-of-Frame

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);
} 

void RemoteTerminal_Init(int cardnum)
{
   int status;
   int RT_MBUF_COUNT;
   API_RT_CBUF cbuf;       // RT Control Buffer Legal Word Count Mask (2 words)
   API_RT_MBUF_WRITE mbuf;
   API_RT_ABUF abuf;
   int         tr;
   int         subaddress;
   int i,j,ii,jj;
   int rt_cnt = 4;
   int rt_addr[] = {2,3,4,7};

   RT_MBUF_COUNT = 2;

   status = BusTools_RT_Init(cardnum,0);
   printf("BusTools_RT_Init status = %d\n",status);

   /**********************************************************************
   *  Output RT control buffers
   *   Enable or disable all word counts and mode codes as specified by
   *    the caller's arguements.
   **********************************************************************/

   cbuf.legal_wordcount = 0xffffffff;  // legailize all word counts
   for ( i = 0; i < rt_cnt; i++)
   { 
      for ( tr = 0; tr < 2; tr++ )
      {
         for ( subaddress = 0; subaddress < 32; subaddress++ )
         {
            status = BusTools_RT_CbufWrite(cardnum,rt_addr[i],subaddress,tr,RT_MBUF_COUNT,&cbuf);
         }
      }
   }

   /*******************************************************************
   *  Fill in message buffers for receive and transmit sub-units.
   *******************************************************************/
   mbuf.error_inj_id = 0;
   mbuf.enable = BT1553_INT_END_OF_MESS;   // Interrupt enables
   // For all subaddresses...
   for (ii = 0; ii<rt_cnt; ii++)
   {  
      for (i=0; i<32; i++)
      {
         // For each data buffer...
         for (j=0; j<RT_MBUF_COUNT; j++)
         {
            for(jj=0;jj<32;jj++)
               mbuf.mess_data[jj] = (BT_U16BIT)(0xAA00 + jj);
            status = BusTools_RT_MessageWrite(cardnum,rt_addr[ii],i,0,j,&mbuf);
            status = BusTools_RT_MessageWrite(cardnum,rt_addr[ii],i,1,j,&mbuf);
         }
      }
   }

   /************************************************
   *  Setup RT address control block
   ************************************************/

   abuf.enable_a          = 1;    // Disable Bus A...Note that the API bits are
   abuf.enable_b          = 1;    // Disable Bus B... reversed from the HW bits.
   abuf.inhibit_term_flag = RT_ABUF_EXT_STATUS; // Enable extended status mode
   abuf.status            = 0x0000;
   abuf.command           = 0;
   abuf.bit_word          = 0;

   for(i=0;i<rt_cnt;i++)
   {
      status = BusTools_RT_AbufWrite(cardnum,rt_addr[i],&abuf);
      printf("BusTools_RT_AbufWrite status = %d\n",status);
   }

   // Set the extended status word for the specified RT/SA/TX-RX/Buffer
   BusTools_RT_MessageWriteStatusWord(cardnum, 3,3,1,1, API_1553_STAT_BY, RT_EXT_STATUS);  // Set Busy
   BusTools_RT_MessageWriteStatusWord(cardnum, 4,4,0,1, API_1553_STAT_ME, RT_EXT_STATUS);  // Set Message Error
}


/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   int       bufnum;
   static int sa_index;
   int       i,j;
   int       status;
   int       wcount;
   BT_INT tail, rtrt,mc;
   BT_UINT messno;

   char outbuf[80];

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      messno = sIntFIFO->fifo[tail].bufferID;
      status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);         
      if (status)
         return status;

       BusTools_TimeGetFmtString(API_TTD_RELM_NS,&bcmessage.time_tag,outbuf);
       printf("Tag Time = %s\n",outbuf);          
 
       rtrt=0;
       mc=0;

       if(bcmessage.status & BT1553_INT_RETRY)
         printf("Retry - %d - %x\n",messno,bcmessage.status);
       else if (bcmessage.status & BT1553_INT_BROADCAST)
         printf("Broadcast - %d - %x\n",messno,bcmessage.status);
       else
         printf("%d - %x\n",messno,bcmessage.status);

       if(bcmessage.status & BT1553_INT_RT_RT_FORMAT)
       {
         printf("RT<->RT (%d) ",messno);
          rtrt=1;
       }
       else if(bcmessage.status & BT1553_INT_MODE_CODE)
       {
          printf("Mode Code --  (%d) ", messno);
          mc=1;
       }
       else if (bcmessage.mess_command1.tran_rec)
          printf("BC**RT->BC (%d)  ", messno);
       else
          printf("BC->RT (%d)  ", messno);
 
       printf("BC**RT-%d  \n",bcmessage.mess_command1.rtaddr);
       printf("BC**SA-%d  \n",bcmessage.mess_command1.subaddr);
       printf("BC**WC-%d  \n",bcmessage.mess_command1.wcount);
       if(rtrt)
       {   
          printf("BC**RT-%d  \n",bcmessage.mess_command2.rtaddr);
          printf("BC**SA-%d  \n",bcmessage.mess_command2.subaddr);
          printf("BC**WC-%d  \n",bcmessage.mess_command2.wcount);
       }
        
       printf("BC**status-1 0x%04x\n",*((WORD*)(&bcmessage.mess_status1)));
       if(rtrt)
          printf("BC**status-2 0x%04x\n",*((WORD*)(&bcmessage.mess_status2)));
          
       printf("BC**int_stat-0x%08lx\n",*((DWORD*)(&bcmessage.status)));
       if(bcmessage.status & BT1553_INT_NO_RESP)
          printf("XXXXXXXXXXXXXXXXXX NO RESPONSE %x!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n",bcmessage.mess_status1);

       wcount = bcmessage.mess_command1.wcount;
       if (wcount == 0)
          wcount = 32;           // Zero word count means 32 words.

       if(mc)
         if(wcount >=16)
            wcount = 1;

       j = wcount;               // See if this msg shorted than previous
    
       if (bcmessage.control & BC_CONTROL_BUFFERA)
          bufnum = 0;            // Display data from first BC buffer
       else
          bufnum = 1;            // Display data from second BC buffer

       for ( i = 0; i < j; i++ ) // Display all of the data words
       {
          // If this is an RT->BC message, and there is a Message Error
          //  or No Response, than xxxx the data (since the RT did not
          //    send any data words).
          if ( i >= wcount )
          {
             printf("    ");     // Data does not apply to this msg
          }
          else if ( bcmessage.mess_command1.tran_rec &&
            (bcmessage.mess_status1.me || (bcmessage.status&BT1553_INT_NO_RESP) ) )
          {
             printf("xxxx ");      // No data was transfered because of error
          }
          else
          {
             printf("BC**%04x ",bcmessage.data[bufnum][i]);
          }
       }
       printf("\n_____________________________________________________________\n");
   
       tail++;
       tail &= sIntFIFO->mask_index;
       sIntFIFO->tail_index = tail;
   }

   return 0;
}

/****************************************************************************
*
*  Function:  setupThe_BC_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO for a interrupt on a BC message.
*     The interrupt is set for the end-of message.  
*
****************************************************************************/
void setupThe_BC_IntFIFO(int cardnum)
{
   int rt,sa, tr, status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.iNotification = 0;
   sIntFIFO1.function     = bc_intFunction;
   sIntFIFO1.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.bForceStartup  = 0;
   sIntFIFO1.FilterType     =  EVENT_BC_MESSAGE;

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO1.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}

