/*============================================================================*
 * FILE:                 E X A M P L E _R T _ J O I N . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:    EXAMPLE PROGRAM
 * This example shows how an application joins an already initialized channel.  
 * RTs 1 - 8 are programmed and a callback function is setup to process RT 
 * message in the interrupt queue.    
 * 
 * This function requires that the channel joined already be initialize and shared 
 * by another application.  The user can run this example with example_bm_share and 
 * example_bc_join.   Joining a shared channel allows individual Remote Terminal, 
 * Bus Monitor and Bus Controller applications to run off a single channel.
 * 
 *===========================================================================*/


#include <stdio.h>
#include "busapi.h"

API_INT_FIFO sIntFIFO2;

/****************************************************************************
*
*  Function:  rt_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen. 
****************************************************************************/
BT_INT _stdcall rt_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{
   int status;
   API_RT_MBUF_READ rt_mbuf;
   BT_INT tail;

   BT_UINT messno,rt_addr,sub_addr,trans;
   BT1553_TIME ttime;
   static int sflg = 0;
   char      outbuf[100];
   int wcount,i,mc;
   static int cnt = 1;
   mc = 0;
   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
       printf("RT msg\n");
      if(sIntFIFO->fifo[tail].event_type & EVENT_RT_MESSAGE)
	  {
	      messno   = sIntFIFO->fifo[tail].bufferID;
          rt_addr  = sIntFIFO->fifo[tail].rtaddress;        // RT address
          trans    = sIntFIFO->fifo[tail].transrec;         // Transmit/Receive
          sub_addr = sIntFIFO->fifo[tail].subaddress;       // Subaddress number

	      status = BusTools_RT_MessageRead( cardnum, rt_addr, sub_addr, trans, messno, &rt_mbuf);
          if (status)
             return status;

          if(rt_mbuf.status & BT1553_INT_RT_RT_FORMAT)
             printf("RT<->RT (%d) \n",messno);
          else if(rt_mbuf.status & BT1553_INT_MODE_CODE)
          {
             printf("RT--MC  (%d)!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! \n", messno);
             mc = 1;
    
          }
          else if (rt_mbuf.mess_command.tran_rec)
             printf("RT->BC (%d) \n", messno);
          else
             printf("BC->RT (%d)  \n", messno);
          printf("RT**RT-%d  \n",rt_mbuf.mess_command.rtaddr);
          printf("RT**SA-%d  \n",rt_mbuf.mess_command.subaddr);
          printf("RT**WC-%d  \n",rt_mbuf.mess_command.wcount);
          printf("RT**mstat-0x%04x\n",*((WORD*)(&rt_mbuf.mess_status)));
          printf("RT**int_stat-0x%08lx\n",rt_mbuf.status);
          if (rt_mbuf.mess_command.wcount == 0)
             wcount = 32;           // Zero word count means 32 words.
          else
             wcount = rt_mbuf.mess_command.wcount ;               // See if this msg shorted than previous

          if(mc)
            if(wcount >=16)
               wcount = 1;

          for ( i = 0; i < wcount; i++ ) // Display all of the data words
          {
             // If this is an RT->BC message, and there is a Message Error
             //  or No Response, than xxxx the data (since the RT did not
             //    send any data words).
         
             printf("RT**%04x ",rt_mbuf.mess_data[i]);
		  }
          printf("\n");

       }   
       tail++;
       tail &= sIntFIFO->mask_index;
       sIntFIFO->tail_index = tail;
   }
   return 0;
}

/****************************************************************************
*
*  Function:  setupThe_RT_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a RT message.  The interrupt is set for only the end-of
*     message.  
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupThe_RT_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO2, 0, sizeof(sIntFIFO2));
   sIntFIFO2.function     = rt_intFunction;
   sIntFIFO2.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO2.dwMilliseconds = INFINITE;
   sIntFIFO2.iNotification  = 0;
   sIntFIFO2.FilterType     = EVENT_RT_MESSAGE;

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO2.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos
  
   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO2, 1);
   printf("BusTools_RegisterFunction status = %d\n",status);
}

int RemoteTerminal_Init(int cardnum)
{
   int status;
   int RT_MBUF_COUNT;
   API_RT_CBUF cbuf;       // RT Control Buffer Legal Word Count Mask (2 words)
   API_RT_MBUF_WRITE mbuf;
   API_RT_ABUF abuf;
   int         tr;
   int         subaddress;
   int i,j,ii,jj;
   int rt_addr[] = {1,2,3,4,5,6,7,8};
   int rt_cnt = 8;

   RT_MBUF_COUNT = 2;

   status = BusTools_RT_Init(cardnum,0);
   printf("BusTools_RT_Init status = %d\n",status);

   /**********************************************************************
   *  Output RT control buffers
   *   Enable or disable all word counts and mode codes as specified by
   *    the caller's arguements.
   **********************************************************************/

   cbuf.legal_wordcount = 0xffffffff;  // legailize all word counts
   for ( i = 0; i < rt_cnt; i++)
   {
      for ( tr = 0; tr < 1; tr++ )
      {
         for ( subaddress = 0; subaddress < 8; subaddress++ )
         {
            status = BusTools_RT_CbufWrite(cardnum,rt_addr[i],subaddress,tr,
			                            RT_MBUF_COUNT,&cbuf);
            if(status)
               return status;
         }
      }
   }


   /*******************************************************************
   *  Fill in message buffers for receive and transmit sub-units.
   *******************************************************************/
   mbuf.error_inj_id = 0;
   mbuf.enable = BT1553_INT_END_OF_MESS;   // Interrupt enables
   // For all subaddresses...
   for (ii = 0; ii<rt_cnt; ii++)
   {  
      for (i=0; i<8; i++)
      {
         for (j=0; j<RT_MBUF_COUNT; j++)
         {
            for(jj=0;jj<8;jj++)
               mbuf.mess_data[jj] = (BT_U16BIT)(0xA000 + j);
            status = BusTools_RT_MessageWrite(cardnum,rt_addr[ii],i,0,j,&mbuf);
            status = BusTools_RT_MessageWrite(cardnum,rt_addr[ii],i,1,j,&mbuf);
         }
      }
   }
//BusTools_DumpMemory(cardnum,DUMP_ALL,"RT_DUMP.txt","TST_RT RT Memory Dump");
   /************************************************
   *  Setup RT address control block
   ************************************************/

   abuf.enable_a          = 1;    // Enable Bus A...Note that the API bits are
   abuf.enable_b          = 1;    // Enable Bus B... reversed from the HW bits.
   abuf.inhibit_term_flag = 0;
   abuf.status            = 0x0000;
   abuf.command           = 0;
   abuf.bit_word          = 0;

   for(i=0;i<rt_cnt;i++)
   {
      status = BusTools_RT_AbufWrite(cardnum,rt_addr[i],&abuf);
     // printf("BusTools_RT_AbufWrite status = %d\n",status);
   }

   return status;
}

int main(int argc, char ** argv)                  
{
   int status,cardnum; 

   status = BusTools_API_JoinChannel(&cardnum,0,0);        // Joining already initialize channel
   printf("BusTools_API_JoinChannel status = %d\n",status);

   RemoteTerminal_Init(cardnum);                           // Initialize Remote Terminal
   setupThe_RT_IntFIFO(cardnum);                           // Set up callback function to process RT data

   BusTools_RT_StartStop(cardnum, RT_START);               // Start RT

   getchar();

   BusTools_RT_StartStop(cardnum, RT_STOP);                // Stop RT
   BusTools_API_QuitChannel(cardnum, RT_QUIT);             // Quit the channel

}