/*===========================================================================*
 * FILE:                     E X A M P L E _ R T _ M C _ S Y N C . C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    Demonstrates how to process Mode Code 17, Sync with Data.
 *
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/
#define IRIG_B


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "busapi.h"

int BusController_Init(int);
int RemoteTerminal_Init(int);
void setupThe_RT_IntFIFO(int);
void setupThe_BC_IntFIFO(int);
void setupThe_Trigger_IntFIFO(int cardnum);


API_INT_FIFO  sIntFIFO_RT;     // Thread FIFO structure for RT
API_INT_FIFO  sIntFIFO1;       // Thread FIFO structure for BC
API_INT_FIFO  trig_fifo;

void main(int argc, char **argv)                     
{
   int status,cardnum, BC_IS_RUNNING=0;
   unsigned channel,device,mode;
   char ichar;

   mode = API_B_MODE | API_SW_INTERRUPT;
   channel = CHANNEL_1;        // Default channel 1

   if(argc==2)
      device = atoi(argv[1]); // Command-line input
   else
      device = 0;             // Default device 0

   // Initialize the channel
   status = BusTools_API_OpenChannel(&cardnum,mode,device,channel);
   printf("BusTools_API_OpenChannel %d status = %d\n",cardnum,status);

   status = BusTools_SetInternalBus(cardnum,EXTERNAL_BUS);  // Set up bus EXTERNAL Bus
   printf("BusTools_SetInternalBus status = %d\n",status);

   // Select to time base depending on firmware version
   if(BusTools_BoardIsV6(cardnum)) // V6 Firmware use nanosecond time tags
      status = BusTools_TimeTagMode(cardnum, API_TTD_DATE_NS, API_TTI_IRIG64, API_TTM_FREE, 0, 0, 0, 0);
   else                            // Older firmware uses microsecond time tags
      status = BusTools_TimeTagMode(cardnum, API_TTD_DATE, API_TTI_IRIG, API_TTM_FREE, 0, 0, 0, 0);
   printf("BusTools_TimeTagMode status = %d\n",status);

   { 
      // Show BusTools/1553-API version and firmware versions
      float lpu,wcs;
      int build;
      BT_UINT mrev, arev;
      status = BusTools_GetRevision(cardnum,&mrev,&arev);
      printf("mrev = %x --- arev = %d\n",mrev,arev);
      status = BusTools_GetFWRevision(cardnum,&wcs,&lpu, &build);
      printf("The LPU revision = %3.2f -- The WCS revision = %3.2f, build = %d\n",lpu,wcs, build);
   }

   BusTools_SetOptions(cardnum,0,0,1,0); // "trigger output on Sync Mode Code" 

   if(BusTools_BoardIsV6(cardnum))
   {

      status = BusTools_SetV6TrigIn(cardnum, DISCRETE, 7);             // Enable Trigger in on Discrete 7
      printf("BusTools_DiscreteSetIO status = %d\n",status);
   
      /* Use this to have the board generate it's own 1PPS signal */
      status = BusTools_SetV6TrigOut(cardnum, DISCRETE, 8);            // Enable Trigger out on Discrete 8
      printf("BusTools_DiscreteSetIO status = %d\n",status);   

	  status = BusTools_DiscreteWrite(cardnum,7,0);                    // Clear Discrete 7
      printf("BusTools_DiscreteWrite status = %d\n",status); 
   }
   else
   {
      // Setup input trigger to be discrete 7, output trigger on discrete 8.
      status = BusTools_DiscreteSetIO(cardnum, 0x155, 0x3ff);
      printf("BusTools_DiscreteSetIO status\n", status);

      status = BusTools_DiscreteTriggerOut(cardnum, TRIGGER_OUT_DIS_8);
      printf("BusTools_DiscreteTriggerOut status\n", status);
   
      status = BusTools_DiscreteTriggerIn(cardnum, TRIGGER_IN_DIS_7);
      printf("DiscreteTriggerIn Status\n", status);
   }

   status = BusTools_ExtTrigIntEnable(cardnum, EXT_TRIG_ENABLE); // Enable interrupt on input trigger
   printf("BusTools_ExtTrigIntEnable status = %d\n",status); 

   // Set up the Remote Terminal to process Mode Code 17
   status = RemoteTerminal_Init(cardnum);

   printf("Do you want to simulate the Bus Controller in the test (Y/N)? ");
   ichar = getchar();
   if(ichar == 'y' || ichar == 'Y')
   {
      BC_IS_RUNNING = 1;
      status = BusController_Init(cardnum);
   }

   //Set up Remote Terminal on MC-17 interrupts using BusTools_RegisterFunction   
   setupThe_RT_IntFIFO(cardnum); 
   setupThe_Trigger_IntFIFO(cardnum);

   // Start the Remote Terminal
   status = BusTools_RT_StartStop(cardnum,RT_START);
   printf("BusTools_RT_StartStop status = %d\n",status);

   if(BC_IS_RUNNING)
   {
      // Set up BC interrupts using BusTools_RegisterFunction
      setupThe_BC_IntFIFO(cardnum);

      // Start the Bus Controller
      status = BusTools_BC_StartStop(cardnum,BC_START);
      printf("BusTools_BC_StartStop status = %d\n",status);
   }
  
   do
   {
      ichar=getc(stdin);
      switch(ichar) // Memory dump
      {
         case 'd':
         printf("Dumping memory\n");
         BusTools_DumpMemory(cardnum,DUMP_ALL,"example_rt_mc17","Memory Dump");
         break;
         case 's':
         BusTools_BC_AperiodicRun(cardnum,20,0,0,0);
         break;
      }
   }while(ichar != 'q');

   // Stop the Bus Controller and Remote Terminal
   status = BusTools_BC_StartStop(cardnum,BC_STOP); 
   printf("BusTools_BC_StartStop status = %d\n",status);

   status = BusTools_RT_StartStop(cardnum,RT_STOP);
   printf("BusTools_RT_StartStop status = %d\n",status);

   // Unregister the callback functions
   status = BusTools_RegisterFunction(cardnum,&sIntFIFO_RT,UNREGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);

   if(BC_IS_RUNNING)
   {
      status = BusTools_RegisterFunction(cardnum,&sIntFIFO1,UNREGISTER_FUNCTION);
      printf("BusTools_RegisterFunction status = %d\n",status);
   }

   // Close the channel
   status = BusTools_API_Close(cardnum);//
   printf("BusTools_API_Close status = %d\n",status);
}

int RemoteTerminal_Init(int cardnum)
{
   BT_INT status;
   BT_INT RT_MBUF_COUNT,RT_MBUF_INDEX;
   API_RT_CBUF cbuf;      
   API_RT_MBUF_WRITE mbuf;
   API_RT_ABUF abuf;
   BT_INT rt = 4;  
   BT_INT tx=1,rx=0;
   BT_INT sa = 4;
   BT_INT i;

   RT_MBUF_COUNT = 1;
   RT_MBUF_INDEX = 0;

   status = BusTools_RT_Init(cardnum,0);
   printf("BusTools_RT_Init status = %d\n",status);

   /**********************************************************************
   *  Output RT control buffers.  Enable all word counts
   **********************************************************************/
   cbuf.legal_wordcount = 0xffffffff;  // legailize all word counts
   status = BusTools_RT_CbufWrite(cardnum,rt,sa,rx,RT_MBUF_COUNT,&cbuf);
   printf("BusTools_RT_CbufWrite RX status = %d\n",status);

   status = BusTools_RT_CbufWrite(cardnum,rt,sa,tx,RT_MBUF_COUNT,&cbuf);
   printf("BusTools_RT_CbufWrite TX status = %d\n",status);

   /*******************************************************************
   *  Fill in message buffers for receive and transmit sub-units.
   *******************************************************************/
   mbuf.error_inj_id = 0;
   mbuf.enable = BT1553_INT_END_OF_MESS;   // Interrupt enables

   status = BusTools_RT_MessageWrite(cardnum,rt,sa,rx,RT_MBUF_INDEX,&mbuf);
   printf("BusTools_RT_MessageWrite RX status = %d\n",status);


   for(i=0;i<32;i++)
      mbuf.mess_data[i]= 0x4000 + i;
   status = BusTools_RT_MessageWrite(cardnum,rt,sa,tx,RT_MBUF_INDEX,&mbuf);
   printf("BusTools_RT_MessageWrite TX status = %d\n",status);

   /************************************************
   *  Setup RT address control block
   ************************************************/

   abuf.enable_a          = 1;    // Enable Bus A...Note that the API bits are
   abuf.enable_b          = 1;    // Enable Bus B... reversed from the HW bits.
   abuf.inhibit_term_flag = 0;
   abuf.status            = 0x0000;
   abuf.command           = 0;
   abuf.bit_word          = 0;

   status = BusTools_RT_AbufWrite(cardnum,rt,&abuf);
   printf("BusTools_RT_AbufWrite status = %d\n",status);

   return status;
}

int BusController_Init(int cardnum)
{
   int status, messno,i;
   API_BC_MBUF bcmessage;

   status = BusTools_BC_Init(cardnum,0,BT1553_INT_END_OF_MESS,0,16,14,1000000,1);
   printf("BusTools_BC_Init status = %d\n",status);

   // Allocate Bus Controller Message Buffers
   status = BusTools_BC_MessageAlloc(cardnum,30);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);

   messno = 0;
   memset((char*)&bcmessage,0,sizeof(bcmessage));       
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_MFRAME_BEG; 
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 4;
   bcmessage.mess_command1.wcount   = 4;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                      // 15 microsecond inter-message gap.

   for(i=0;i<4;i++)
       bcmessage.data[0][i] = 0xA000+i;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // Interrupt on this message
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 4;
   bcmessage.mess_command1.wcount   = 4;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid =  0;                       // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                      // 15 microsecond inter-message gap.q
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_MFRAME_END;
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 0;
   bcmessage.mess_command1.wcount   = 17;  
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                      // 15 microsecond inter-message gap.
   bcmessage.data[0][0] = 0xABCD;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)0;
   bcmessage.control = BC_CONTROL_NOP;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);


   // Aperiodic synce mode code
   messno=20;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(0xffff);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A;
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // Interrupt on this message
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 0;
   bcmessage.mess_command1.wcount   = 1;  
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                      // 15 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   return status; 
} 


/****************************************************************************
*
*  Function:  rt_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function is invoked when Mode Code 17 
*     is sent to RT4.  When that happens, the callback function increment the
*     in the data in transmit buffer for RT4 SA4.
*
****************************************************************************/
BT_INT _stdcall rt_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{
   int status;
   API_RT_MBUF_READ rt_rmbuf;
   API_RT_MBUF_WRITE rt_wmbuf;
   BT_INT tail,indx;
   BT_UINT messno,rt_addr,sub_addr,trans;


   rt_wmbuf.error_inj_id = 0;
   rt_wmbuf.enable = BT1553_INT_END_OF_MESS;   // Interrupt enables

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      messno   = sIntFIFO->fifo[tail].bufferID;
      rt_addr  = sIntFIFO->fifo[tail].rtaddress;        // RT address
      trans    = sIntFIFO->fifo[tail].transrec;         // Transmit/Receive
      sub_addr = sIntFIFO->fifo[tail].subaddress;       // Subaddress number
      
      status = BusTools_RT_MessageRead( cardnum, rt_addr, sub_addr, trans, messno, &rt_rmbuf);// Read the sync data
      if (status)
         return status;

      if(rt_rmbuf.mess_data[0] = 0xABCD)                          // if sync data equals pattern
      {
        BusTools_RT_MessageRead(cardnum, 4,4,1,0,&rt_rmbuf);
        for(indx=0;indx<4;indx++)
           rt_wmbuf.mess_data[indx] = rt_rmbuf.mess_data[indx]+1; //update data in the RT message buffer
        status = BusTools_RT_MessageWrite(cardnum, 4,4,1,0,&rt_wmbuf);
      }
         
      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }
   return 0;
}

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   static int sa_index;
   int       i,j;
   int       status;
   int       wcount;
   BT_INT tail;
   BT_UINT messno;
   static int nFlag = 1;

   char outbuf[80];

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      messno = sIntFIFO->fifo[tail].bufferID;
      status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);    
      if (status)
         return status;

      printf("%d - %x\n",messno,bcmessage.status);

      BusTools_TimeGetString(&bcmessage.time_tag, outbuf);
      printf("tag time = %s\n",outbuf);
       
      if (bcmessage.mess_command1.tran_rec)
         printf("BC**RT->BC (%d)  ", messno);
      else
         printf("BC->RT (%d)  ", messno);
 
      printf("BC**RT-%d  \n",bcmessage.mess_command1.rtaddr);
      printf("BC**SA-%d  \n",bcmessage.mess_command1.subaddr);
      printf("BC**WC-%d  \n",bcmessage.mess_command1.wcount);
      printf("BC**status-1 0x%04x\n",*((WORD*)(&bcmessage.mess_status1)));
      printf("BC**int_stat-0x%08lx\n",*((DWORD*)(&bcmessage.status)));
      wcount = bcmessage.mess_command1.wcount;
      if (wcount == 0)
         wcount = 32;           // Zero word count means 32 words.

      j = wcount;               // See if this msg shorted than previous
    
      for ( i = 0; i < j; i++ ) // Display all of the data words
      {
         if ( i >= wcount )
         {
            printf("    ");     // Data does not apply to this msg
         }
         printf("BC**%04x ",bcmessage.data[0][i]);
      }
      printf("\n");
               
      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }

   return 0;
}

/****************************************************************************
*
*  Function:  setupThe_BC_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a BC message.  The interrupt is set for only the end-of
*     message.  
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupThe_BC_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.iNotification = 0;
   sIntFIFO1.function     = bc_intFunction;
   sIntFIFO1.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.bForceStartup  = 0;
   sIntFIFO1.FilterType     =  EVENT_BC_MESSAGE;

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO1.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}


/****************************************************************************
*
*  Function:  setupThe_RT_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a RT message.  The interrupt is set for only the end-of
*     message.  
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupThe_RT_IntFIFO(int cardnum)
{
   int rt=4,tr=0,sa=0,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO_RT, 0, sizeof(sIntFIFO_RT));
   sIntFIFO_RT.function     = rt_intFunction;
   sIntFIFO_RT.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO_RT.dwMilliseconds = INFINITE;
   sIntFIFO_RT.iNotification  = 0;
   sIntFIFO_RT.FilterType     = EVENT_RT_MESSAGE;

   sIntFIFO_RT.FilterMask[rt][tr][sa] =  0x20000;  // Enable callback on MC-17
  
   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO_RT, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}

BT_INT _stdcall intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{
   static BT_U32BIT last_micro=0;
   BT_INT tail;

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      printf("TRIGGER ON SYNC!!!!!!!!!!!!\n");	
      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }
   return 0;
}

/****************************************************************************
*
*  Function:  setupThe_1PPS_Deadman_timer
*
*  Description:
*     This routine setup the interrupt FIFO to detect when the 1PPS pulse is
*     not delivered on time
****************************************************************************/
void setupThe_Trigger_IntFIFO(int cardnum)
{
   BT_INT status;

   // Setup the FIFO structure for trigger interrupt event.
   memset(&trig_fifo, 0, sizeof(trig_fifo));
   trig_fifo.function       = intFunction; // This is the interrupt function the API calls
   trig_fifo.iPriority      = THREAD_PRIORITY_ABOVE_NORMAL;
   trig_fifo.dwMilliseconds = INFINITE;    //
   trig_fifo.iNotification  = 0;
   trig_fifo.FilterType     = EVENT_EXT_TRIG; 
   status = BusTools_RegisterFunction(cardnum, &trig_fifo, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);  
}
