/*===========================================================================*
 * FILE:           E X A M P L E _R T _ M O D E _ C O D E . C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    This example shows multi-buffering of mode code data.  Uses 
 * Mode Code 17, synchronize with data and Mode code 19, Transmit Bit Word.  
 * This example shows how to setup data for Mode codes if that are in the same 
 * bus list.
 *
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/
#define IRIG_B


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "busapi.h"

int BusController_Init(int,int);
int BusMonitor_Init(int);
int RemoteTerminal_Init(int);
void setupThe_BC_RT_IntFIFO(int);

API_INT_FIFO  sIntFIFO1;     // Thread FIFO structure for BC

void main(int argc, char **argv)                     
{
   int status,cardnum;
   int help=0;
   unsigned channel,device,mode;
   char ichar;

   mode = API_B_MODE | API_SW_INTERRUPT;
   channel = CHANNEL_1;  //default channel 1
   device = 0;           //default device 0


   status = BusTools_API_OpenChannel(&cardnum,mode,device,channel);
   printf("BusTools_API_OpenChannel %d status = %d\n",cardnum,status);

   status = BusTools_SetInternalBus(cardnum,EXTERNAL_BUS);  // Set up bus 0 External or  1 Internal
   printf("BusTools_SetInternalBus status = %d\n",status);

   if(BusTools_BoardIsV6(cardnum))
      BusTools_TimeTagMode(cardnum, API_TTD_RELM_NS, API_TTI_ZERO, API_TTM_FREE, NULL,0,0,0);
   else
      BusTools_TimeTagMode(cardnum, API_TTD_RELM, API_TTI_ZERO, API_TTM_FREE, NULL,0,0,0);

   status = RemoteTerminal_Init(cardnum);  
   status = BusController_Init(cardnum,50);
   
   setupThe_BC_RT_IntFIFO(cardnum); //Set up BC and RT interrupts in the same callback using BusTools_RegisterFunction

   status = BusTools_RT_StartStop(cardnum,RT_START);
   printf("BusTools_RT_StartStop status = %d\n",status);

   status = BusTools_BC_StartStop(cardnum,BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);
  
   while((ichar=getc(stdin))!='q')
   {
      if(ichar == 'd') // memory dump
      {
         static int count=0;

         char dump_file_name[20];
         sprintf(dump_file_name,"mem_dump%d.txt",count++);
         printf("Dumping memory to %s\n",dump_file_name);
         BusTools_DumpMemory(cardnum,DUMP_ALL,dump_file_name,"TST_RT Memory Dump");
      }
   }

   printf("main: closing API\n");

   status = BusTools_BC_StartStop(cardnum,BC_STOP); 
   printf("BusTools_BC_StartStop status = %d\n",status);

   status = BusTools_RT_StartStop(cardnum,RT_STOP);
   printf("BusTools_RT_StartStop status = %d\n",status);

   status = BusTools_API_Close(cardnum);//
   printf("BusTools_API_Close status = %d\n",status);

}

int RemoteTerminal_Init(int cardnum)
{
   int status;
   int RT_MBUF_COUNT;
   API_RT_CBUF cbuf;       // RT Control Buffer Legal Word Count Mask (2 words)
   API_RT_MBUF_WRITE mbuf;
   API_RT_ABUF abuf;
   int         tr;
   int         subaddress;
   int i,j,ii;
   int rt_addr[] = {2,3};
   int rt_cnt = 2;

   RT_MBUF_COUNT = 10;

   status = BusTools_RT_Init(cardnum,0);
   printf("BusTools_RT_Init status = %d\n",status);

   /**********************************************************************
   *  Output RT control buffers
   *   Enable or disable all word counts and mode codes as specified by
   *    the caller's arguements.
   **********************************************************************/

   cbuf.legal_wordcount = 0xffffffff;  // legailize all word counts
   for ( i = 0; i < rt_cnt; i++)
   { 
      for ( tr = 0; tr < 2; tr++ )
      {
         for ( subaddress = 0; subaddress < 32; subaddress++ )
         {
            status = BusTools_RT_CbufWrite(cardnum,rt_addr[i],subaddress,tr,RT_MBUF_COUNT,&cbuf);
            if(status)
               return status;
         }
      }
   }

   /*******************************************************************
   *  Fill in message buffers for receive and transmit sub-units.
   *******************************************************************/
   mbuf.error_inj_id = 0;
   mbuf.enable = BT1553_INT_END_OF_MESS;   // Interrupt enables
   // For all subaddresses...
   for (ii = 0; ii<rt_cnt; ii++)
   {  
      for (i=0; i<8; i++)
      {
         // For each data buffer...
         for (j=0; j<RT_MBUF_COUNT; j++)
         {
            mbuf.mess_data[0]=0xaa00+j;
            status = BusTools_RT_MessageWrite(cardnum,rt_addr[ii],i,0,j,&mbuf);
            status = BusTools_RT_MessageWrite(cardnum,rt_addr[ii],i,1,j,&mbuf);
         }
      }
   }

   /************************************************
   *  Setup RT address control block
   ************************************************/

   abuf.enable_a          = 1;    // Enable Bus A...Note that the API bits are
   abuf.enable_b          = 1;    // Enable Bus B... reversed from the HW bits.
   abuf.inhibit_term_flag = 0;
   abuf.status            = 0x0000;
   abuf.command           = 0;
   abuf.bit_word          = 0x1234;

   status = BusTools_RT_AbufWrite(cardnum,2,&abuf);
   printf("BusTools_RT_AbufWrite status = %d\n",status);

   abuf.enable_a          = 1;    // Enable Bus A...Note that the API bits are
   abuf.enable_b          = 1;    // Enable Bus B... reversed from the HW bits.
   abuf.inhibit_term_flag = RT_ABUF_MBUF_BWD;
   abuf.status            = 0;
   abuf.command           = 0;
   abuf.bit_word          = 0x1234;

   status = BusTools_RT_AbufWrite(cardnum,3,&abuf);
   printf("BusTools_RT_AbufWrite status = %d\n",status);

   status = BusTools_RT_AbufRead(cardnum,3,&abuf);

   return status;

}

int BusController_Init(int cardnum,int num_buffers)
{
   int status, messno;
   API_BC_MBUF bcmessage;


   status = BusTools_BC_Init(cardnum,0,BT1553_INT_END_OF_MESS,0,16,14,1000000,1);
   printf("BusTools_BC_Init status = %d\n",status);

   status = BusTools_BC_MessageAlloc(cardnum,10);//num_buffers);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);

   messno = 0;
   memset((char*)&bcmessage,0,sizeof(bcmessage));       
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;   // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_MFRAME_BEG; 
   bcmessage.mess_command1.rtaddr   = 2;
   bcmessage.mess_command1.subaddr  = 0;
   bcmessage.mess_command1.wcount   = 17;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;     // Default error injection buffer (no errors)
   bcmessage.gap_time = 10;    // 15 microsecond inter-message gap.

   bcmessage.data[0][0] = 0x2000;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 3;
   bcmessage.mess_command1.subaddr  = 0;
   bcmessage.mess_command1.wcount   = 17;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;     // Default error injection buffer (no errors)
   bcmessage.gap_time = 10;    // 15 microsecond inter-message gap

   bcmessage.data[0][0] = 0x3000;
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 3;
   bcmessage.mess_command1.subaddr  = 0;
   bcmessage.mess_command1.wcount   = 19;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;       // Default error injection buffer (no errors)
   bcmessage.gap_time = 10;    // 15 microsecond inter-message gap
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;      // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 2;
   bcmessage.mess_command1.subaddr  = 0;
   bcmessage.mess_command1.wcount   = 19;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;     // Default error injection buffer (no errors)
   bcmessage.gap_time = 10;    // 15 microsecond inter-message gap
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)0;
   bcmessage.control = BC_CONTROL_NOP;
   bcmessage.control |= BC_CONTROL_MFRAME_END;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   return status; 
} 

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   API_RT_MBUF_READ rt_mbuf;
   int       bufnum;
   static BT_INT sa_index;
   BT_INT i, j, status, wcount, tail, mc;
   BT_UINT messno, rt_addr,trans,sub_addr;
   static int nFlag = 1;
   char outbuf[80];

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      if(sIntFIFO->fifo[tail].event_type == EVENT_RT_MESSAGE) // Handle RT interrupt here
      {     
         rt_addr  = sIntFIFO->fifo[tail].rtaddress;        // RT address
         trans    = sIntFIFO->fifo[tail].transrec;         // Transmit/Receive
         sub_addr = sIntFIFO->fifo[tail].subaddress;       // Subaddress number
         messno    = sIntFIFO->fifo[tail].bufferID;

         status = BusTools_RT_MessageRead( cardnum, rt_addr, sub_addr, trans, messno, &rt_mbuf);
         if (status)
            return status;

         BusTools_TimeGetString(&rt_mbuf.time, outbuf);
         printf("tag time = %s\n",outbuf);
 
         if(rt_mbuf.status & BT1553_INT_MODE_CODE)
         {
            printf("RT--MC  (%d) \n", messno);
            mc = 1;
         }
         else if (rt_mbuf.mess_command.tran_rec)
            printf("RT->BC (%d) \n", messno);
         else
            printf("BC->RT (%d)  \n", messno);

         printf("RT**RT-%d  \n",rt_mbuf.mess_command.rtaddr);
         printf("RT**SA-%d  \n",rt_mbuf.mess_command.subaddr);
         printf("RT**WC-%d  \n",rt_mbuf.mess_command.wcount);
         printf("RT**mstat-0x%04x\n",*((WORD*)(&rt_mbuf.mess_status)));
         printf("RT**int_stat-0x%08lx\n",rt_mbuf.status);
         if (rt_mbuf.mess_command.wcount == 0)
            wcount = 32;           // Zero word count means 32 words.
         else
            wcount = rt_mbuf.mess_command.wcount ;               // See if this msg shorted than previous

         if(mc)
            if(wcount >=16)
               wcount = 1;

         for ( i = 0; i < wcount; i++ ) // Display all of the data words
            printf("RT**%04x ",rt_mbuf.mess_data[i]);
         printf("\n___________________________________________________________\n"); 
      }
      if(sIntFIFO->fifo[tail].event_type == EVENT_BC_MESSAGE)  // Handle BC interrupt here
      {
         messno = sIntFIFO->fifo[tail].bufferID;
         status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);
         if (status)
            return status;
         mc=0;       

         if(bcmessage.status & BT1553_INT_MODE_CODE)
         {
            printf("BC**Mode Code --  (%d) \n", messno);
            mc=1;
         }
         else if (bcmessage.mess_command1.tran_rec)
            printf("BC**RT->BC (%d)  ", messno);
         else
            printf("BC**BC->RT (%d)  ", messno);
 
         printf("BC**RT-%d  \n",bcmessage.mess_command1.rtaddr);
         printf("BC**SA-%d  \n",bcmessage.mess_command1.subaddr);
         if(mc)
            printf("BC**MC-%d  \n",bcmessage.mess_command1.wcount);
         else
            printf("BC**WC-%d  \n",bcmessage.mess_command1.wcount);
         printf("BC**status-1 0x%04x\n",*((WORD*)(&bcmessage.mess_status1)));
         printf("BC**int_stat-0x%08lx\n",*((DWORD*)(&bcmessage.status)));

         wcount = bcmessage.mess_command1.wcount;
         if (wcount == 0)
            wcount = 32;           // Zero word count means 32 words.

         if(mc)
           if(wcount >=16)
              wcount = 1;
   
         j = wcount;               // See if this msg shorted than previous
    
         if (bcmessage.control & BC_CONTROL_BUFFERA)
            bufnum = 0;            // Display data from first BC buffer
         else
            bufnum = 1;            // Display data from second BC buffer

         for ( i = 0; i < j; i++ ) // Display all of the data words
         {
            // If this is an RT->BC message, and there is a Message Error
            //  or No Response, than xxxx the data (since the RT did not
            //    send any data words).
            if ( i >= wcount )
            {
               printf("    ");     // Data does not apply to this msg
            }
            else if ( bcmessage.mess_command1.tran_rec &&
              (bcmessage.mess_status1.me || (bcmessage.status&BT1553_INT_NO_RESP) ) )
            {
               printf("xxxx ");      // No data was transfered because of error
            }
            else
            {
               printf("BC**%04x ",bcmessage.data[bufnum][i]);
            }
         }
         printf("\n___________________________________________________________\n"); 
      }      
      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }

   return 0;
}

/****************************************************************************
*
*  Function:  setupThe_BC_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a BC message.  The interrupt is set for only the end-of
*     message.  
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupThe_BC_RT_IntFIFO(int cardnum)
{
   int status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.iNotification = 0;
   sIntFIFO1.function     = intFunction;
   sIntFIFO1.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.bForceStartup  = 0;
   sIntFIFO1.FilterType     =  EVENT_BC_MESSAGE | EVENT_RT_MESSAGE;

   sIntFIFO1.FilterMask[3][1][0] =  0xffffffff;  // Enable all Word Counts on RT3/SA0/TX combos
   sIntFIFO1.FilterMask[2][1][0] =  0xffffffff;  // Enable all Word Counts on RT2/SA0/TX combos
   sIntFIFO1.FilterMask[3][0][0] =  0xffffffff;  // Enable all Word Counts on RT3/SA0/RX combos
   sIntFIFO1.FilterMask[2][0][0] =  0xffffffff;  // Enable all Word Counts on RT2/SA0/RX combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}
