/*===========================================================================*
 * FILE:                     E X A M P L E _ R T _ M O N I T O R . C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    This examples shows how to setup the Remote Terminal in monitor
 *              only mode.  In this mode the RT captures all messages to the
 *              specified RT but will not respond to the RT messages.   
 *
 *              With the exception of calling BusTools_RT_MonitorInit the Remote
 *              terminal is configured identically to a real Remote Terminal.
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/
#define IRIG_B


#include <stdio.h>
#include "busapi.h"

int RemoteTerminalMonitor_Init(BT_INT cardnum, BT_INT rt_addr);
void setupThe_RT_IntFIFO(BT_INT cardnum, BT_INT rt_addr);

BT_UINT rt_int_count[32];
API_INT_FIFO  rtIntFIFO;     // Thread FIFO structure for all RTs


#if defined (__WIN32__)
void main(int argc, char **argv)                     
#else
int tst_rt_mrt_bm()
#endif
{
   BT_INT status,cardnum, rt_addr=0xff;
   BT_UINT channel,device,mode;

   printf("Enter the RT Address to monitor: ");
   scanf("%d",&rt_addr);
   printf("\n");

   mode = API_B_MODE | API_SW_INTERRUPT;    //MIL-STD-1553B with S/W interrupts enabled
   channel = CHANNEL_1;  // Default channel 1
   device = 0;           // Default device 0

   status = BusTools_API_OpenChannel(&cardnum,mode,device,channel);
   printf("BusTools_API_OpenChannel status = %d\n",status);

   status = BusTools_SetInternalBus(cardnum, EXTERNAL_BUS);  // Set up bus external bus
   printf("BusTools_SetInternalBus status = %d\n",status);

   // Use the correct time base to display time tags
   if(BusTools_BoardIsV6(cardnum))  // Nanosecond for V6 firmware
      BusTools_TimeTagMode(cardnum, API_TTD_RELM_NS, API_TTI_ZERO, API_TTM_FREE, NULL,0,0,0);
   else                             // Microsecond for earlier firmware
      BusTools_TimeTagMode(cardnum, API_TTD_RELM, API_TTI_ZERO, API_TTM_FREE, NULL,0,0,0);

   // Initialize the Remote Terminal in monitor only mode
   status = RemoteTerminalMonitor_Init(cardnum, rt_addr);  

   setupThe_RT_IntFIFO(cardnum, rt_addr);

   status = BusTools_RT_StartStop(cardnum,RT_START);
   printf("BusTools_RT_StartStop status = %d\n",status);

   fflush(stdin);
   getchar();

   printf("main: closing API\n");

   status = BusTools_RT_StartStop(cardnum,RT_STOP);
   printf("BusTools_RT_StartStop status = %d\n",status);

   status = BusTools_API_Close(cardnum);//
   printf("BusTools_API_Close status = %d\n",status);

}

int RemoteTerminalMonitor_Init(BT_INT cardnum, BT_INT rt_addr )
{
   BT_INT status, i, j;
   int RT_MBUF_COUNT;
   API_RT_CBUF cbuf;       // RT Control Buffer Legal Word Count Mask
   API_RT_MBUF_WRITE mbuf;
   API_RT_ABUF abuf;
   int         tr;
   int         subaddress;

   RT_MBUF_COUNT = 1;

   status = BusTools_RT_Init(cardnum,0);
   printf("BusTools_RT_Init status = %d\n",status);

   /**********************************************************************
   *  Output RT control buffers
   *   Enable or disable all word counts and mode codes as specified by
   *    the caller's arguements.
   **********************************************************************/

   cbuf.legal_wordcount = 0xffffffff;  // legailize all word counts
 
   for ( tr = 0; tr < 2; tr++ )
   {
      for ( subaddress = 0; subaddress < 32; subaddress++ )
      {
         status = BusTools_RT_CbufWrite(cardnum,rt_addr,subaddress,tr,RT_MBUF_COUNT,&cbuf);
         if(status)
            return status;
      }
   }

   /*******************************************************************
   *  Fill in message buffers for receive and transmit sub-units.
   *******************************************************************/
   mbuf.error_inj_id = 0;
   mbuf.enable = BT1553_INT_END_OF_MESS;   // Interrupt enables
   // For all subaddresses...

   for (i=0; i<32; i++)
   {
      // For each data buffer...
      for (j=0; j<RT_MBUF_COUNT; j++)
      {
            status = BusTools_RT_MessageWrite(cardnum,rt_addr,i,0,j,&mbuf);
            status = BusTools_RT_MessageWrite(cardnum,rt_addr,i,1,j,&mbuf);
      }
   }

   /************************************************
   *  Setup RT address control block
   ************************************************/

   abuf.enable_a          = 1;    // Eisable Bus A...Note that the API bits are
   abuf.enable_b          = 1;    // Eisable Bus B... reversed from the HW bits.
   abuf.inhibit_term_flag = 0;    // RT Monitor on this RT
   abuf.status            = 0x0000;
   abuf.command           = 0;
   abuf.bit_word          = 0;

   status = BusTools_RT_AbufWrite(cardnum,rt_addr,&abuf);
   printf("BusTools_RT_AbufWrite status = %d\n",status);

   // Enable Monitor Mode - Will not responds
   status = BusTools_RT_MonitorEnable(cardnum, rt_addr, RT_MONITOR_ENABLE); 

   return status;

}

/****************************************************************************
*
*  Function:  rt_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT _stdcall rt_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{
   int status;
   API_RT_MBUF_READ rt_mbuf;
   BT_INT tail;

   BT_UINT messno,rt_addr,sub_addr,trans;
   static int sflg = 0;
   char      outbuf[100];
   int wcount,i,mc,rtrtint=0;
   static int cnt = 1;


   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      mc = 0;
      printf("RT msg\n");
      if(sIntFIFO->fifo[tail].event_type & EVENT_RT_MESSAGE)
	  {
          printf("\n_______________________________________________________________________________\n");
	      messno   = sIntFIFO->fifo[tail].bufferID;
          rt_addr  = sIntFIFO->fifo[tail].rtaddress;        // RT address
          trans    = sIntFIFO->fifo[tail].transrec;         // Transmit/Receive
          sub_addr = sIntFIFO->fifo[tail].subaddress;       // Subaddress number

	      status = BusTools_RT_MessageRead( cardnum, rt_addr, sub_addr, trans, messno, &rt_mbuf);
          if (status)
             return status;

          BusTools_TimeGetString(&rt_mbuf.time, outbuf);
          printf("tag time = %s\n",outbuf);

          if(rt_mbuf.status & BT1553_INT_RT_RT_FORMAT)
          {
             printf("RT<->RT ********************************************(%d) \n",messno);
          }
          else if(rt_mbuf.status & BT1553_INT_MODE_CODE)
          {
             printf("RT--MC  (%d)!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! \n", messno);
             mc = 1;
    
          }
          else if (rt_mbuf.mess_command.tran_rec)
             printf("RT->BC (%d) \n", messno);
          else
             printf("BC->RT (%d)  \n", messno);
          printf("RT**RT-%d  \n",rt_mbuf.mess_command.rtaddr);
          printf("RT**SA-%d  \n",rt_mbuf.mess_command.subaddr);
          printf("RT**WC-%d  \n",rt_mbuf.mess_command.wcount);
          printf("RT**mstat-0x%04x\n",*((WORD*)(&rt_mbuf.mess_status)));
          printf("RT**int_stat-0x%08lx\n",rt_mbuf.status);
          if (rt_mbuf.mess_command.wcount == 0)
             wcount = 32;           // Zero word count means 32 words.
          else
             wcount = rt_mbuf.mess_command.wcount;               // See if this msg shorted than previous

          if(mc)
            if(wcount >=16)
               wcount = 1;

          for ( i = 0; i < wcount; i++ ) // Display all of the data words
          {
             // If this is an RT->BC message, and there is a Message Error
             //  or No Response, than xxxx the data (since the RT did not
             //    send any data words).
         
             printf("RT**%04x ",rt_mbuf.mess_data[i]);
             if(i%8 == 0 && i>0)
                printf("\n");
		  }
          printf("\n");
       }   
       tail++;
       tail &= sIntFIFO->mask_index;
       sIntFIFO->tail_index = tail;
   }
   return 0;
}

/****************************************************************************
*
*  Function:  setupThe_RT_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a RT message.  The interrupt is set for only the end-of
*     message.  
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupThe_RT_IntFIFO(BT_INT cardnum, BT_INT rt_addr)
{
   int tr,sa,status;
   // Setup the FIFO structure for this board.
   memset(&rtIntFIFO, 0, sizeof(API_INT_FIFO));
   rtIntFIFO.function     = rt_intFunction;
   rtIntFIFO.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   rtIntFIFO.dwMilliseconds = INFINITE;
   rtIntFIFO.iNotification  = 0;
   rtIntFIFO.FilterType     = EVENT_RT_MESSAGE;
 
   for (tr = 0; tr < 2; tr++ )
      for (sa = 0; sa < 32; sa++ )
         rtIntFIFO.FilterMask[rt_addr][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos
  
   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &rtIntFIFO, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}

