/*===========================================================================*
 * FILE:           E X A M P L E _ R T  _ W R A P . C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    Demonstration of BusTools/1553-API routines. This console 
 *              application shows how to manually wrap RT receive and transmit
 *              messages buffers. 
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "busapi.h"

int BusController_Init(int,int);
int RemoteTerminal_Init(int);
void setupThe_BC_IntFIFO(int cardnum);

API_INT_FIFO sIntFIFO1;      // Thread FIFO structure for BC.

#if defined (__WIN32__)
void main(int argc, char **argv)                     
#else
int rt_wrap()
#endif
{
   int status,cardnum;
   unsigned channel,device,mode;
   int i=0;

   mode = API_B_MODE | API_SW_INTERRUPT;  //MIL-STD-1553B with software interrupt enabled
   channel = CHANNEL_1;  //default channel 1
   device  = 0;           //default device 0

   status = BusTools_API_OpenChannel(&cardnum,mode,device,channel);
   printf("BusTools_API_OpenChannel status = %d\n",status);

   status = BusTools_SetInternalBus(cardnum,EXTERNAL_BUS);  // Set up bus 0 External or  1 Internal
   printf("BusTools_SetInternalBus status = %d\n",status);

   status = BusController_Init(cardnum,20);
   status = RemoteTerminal_Init(cardnum);  

   setupThe_BC_IntFIFO(cardnum);

   status = BusTools_RT_StartStop(cardnum,RT_START);
   printf("BusTools_RT_StartStop status = %d\n",status);

   status = BusTools_BC_StartStop(cardnum,BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);

   getchar();

   printf("main: closing API\n");

   status = BusTools_BC_StartStop(cardnum,BC_STOP); 
   printf("BusTools_BC_StartStop status = %d\n",status);

   status = BusTools_RT_StartStop(cardnum,RT_STOP);
   printf("BusTools_RT_StartStop status = %d\n",status);

   status = BusTools_API_Close(cardnum);//
   printf("BusTools_API_Close status = %d\n",status);

}

int BusController_Init(int cardnum,int num_buffers)
{
   int status, messno;
   API_BC_MBUF bcmessage;


   //initialize the BC with interrupt on end-of-message 16 us no response 14 us late response, 1sec frame (1000000 usec).     
   status = BusTools_BC_Init(cardnum,0,BT1553_INT_END_OF_MESS,0,16,14,1000000,1);
   printf("BusTools_BC_Init status = %d\n",status);

   status = BusTools_BC_MessageAlloc(cardnum,50);//num_buffers);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);

   messno = 0;
   memset((char*)&bcmessage,0,sizeof(bcmessage));       
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // Enable interrupt on message
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;    // Start of a minor frame
   bcmessage.mess_command1.rtaddr   = 22;          
   bcmessage.mess_command1.subaddr  = 11;
   bcmessage.mess_command1.wcount   = 2;
   bcmessage.mess_command1.tran_rec = 0;          // BC->RT
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 10;                       // 10 microsecond inter-message gap.
   bcmessage.data[0][0] = 0xabcd;
   bcmessage.data[0][1] = 0xefef;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);


   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_INTERRUPT;     // Enable interrupt on message
   bcmessage.mess_command1.rtaddr   = 22;
   bcmessage.mess_command1.subaddr  = 11;
   bcmessage.mess_command1.wcount   = 2;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 20;                      // 20 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)0;          // Repeat starting at message 0
   bcmessage.control = BC_CONTROL_NOP;            // 
   bcmessage.control |= BC_CONTROL_MFRAME_END;    // End-of-Frame

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);


   return status; 
}

void wrap_RT_buffer(BT_INT cardnum, BT_INT RT, BT_INT SA)
{
   BT_INT status;
   BT_U32BIT  startptr, endptr, temp_ptr, rx_ptr_offset, tx_ptr_offset;
   BT_U16BIT  rx_mbuf_ptr, tx_mbuf_ptr;
   BT_UINT temp;

   status = BusTools_GetAddr(cardnum, GETADDR_RTFILTER, &startptr, &endptr);
   /***** Get current MBUF pointer offset for RT Receive SA *****
   Offset into Filter Table for RT Receive SA
			              RT          TR        SA                */
   temp_ptr = startptr + (RT*64)*2 + (RECEIVE*32)*2 + SA*2;  

   // Read the filter table entry - this is the pointer to the Control Buffer.
   temp = 0;
   status = BusTools_MemoryRead(cardnum, temp_ptr, 2, &temp); 

   // Add 2 words, convert to a BYTE value.  Now we have the byte offset
   // to the CURRENT MSG BUFFER POINTER for RT Receive SA.  Store this.
   rx_ptr_offset = ( (unsigned long) temp + 2 ) * 2;

   // Now we can read the 19-bit MBUF pointer value from the control buffer.
   status = BusTools_MemoryRead(cardnum, rx_ptr_offset, 2, &rx_mbuf_ptr);

   /***** Get current MBUF pointer offset for RT Transmit SA *****/
   // Offset into Flt Tbl for RT1 Transmit SA2
   //                     RT          TR        SA
   temp_ptr = startptr + (RT*64)*2 + (TRANSMIT*32)*2 + SA*2;

   // Read the filter table entry - this is the pointer to the Control Buffer.
   temp = 0;
   status = BusTools_MemoryRead(cardnum, temp_ptr, 2, &temp);

   // Add 2 words, convert to a BYTE value.  Now we have the byte offset
   // to the CURRENT MSG BUFFER POINTER for RT Transmit SA.  Store this.
   tx_ptr_offset = ( (unsigned long) temp + 2 ) * 2;

   // Now we can read the 19-bit MBUF pointer value from the control buffer.
   status = BusTools_MemoryRead(cardnum, tx_ptr_offset, 2, &tx_mbuf_ptr);

   /***** Wrap the buffers by setting the TX pointer to point to the RCV buffer *****/
   // Change the message buffer pointer in the control buffer for RT SA TX to
   // use the same message buffer as RT SA RCV.
   status = BusTools_MemoryWrite(cardnum, tx_ptr_offset, 2, &rx_mbuf_ptr);

   // Now we are setup with RT SA WRAPPED.
} 


void wrap_RT_bufferV6(BT_INT cardnum, BT_INT RT, BT_INT SA)
{
   BT_INT status;
   BT_U32BIT  startptr, endptr, temp_ptr, rx_ptr_offset, tx_ptr_offset;
   BT_U32BIT  rx_mbuf_ptr, tx_mbuf_ptr;
   BT_UINT temp;

   status = BusTools_GetAddr(cardnum, GETADDR_RTFILTER, &startptr, &endptr);
   /***** Get current MBUF pointer offset for RT Receive SA *****
   Offset into Filter Table for RT Receive SA
			                          RT           TR           SA                */
   temp_ptr = startptr + ((BT_U32BIT)(RT<<8) + (RECEIVE<<7) + (SA<<2));  

   // Read the filter table entry - this is the pointer to the Control Buffer.
   temp = 0;
   status = BusTools_MemoryRead2(cardnum,RAM32, temp_ptr, 4, &temp); 

   // Add 2 words, convert to a BYTE value.  Now we have the byte offset
   // to the CURRENT MSG BUFFER POINTER for RT Receive SA.  Store this.
   rx_ptr_offset = BusTools_RelAddr(cardnum,temp);

   // Now we can read the 32-bit MBUF pointer value from the control buffer.
   status = BusTools_MemoryRead2(cardnum, RAM32, rx_ptr_offset, 4, &rx_mbuf_ptr);

   /***** Get current MBUF pointer offset for RT Transmit SA *****/
   // Offset into Flt Tbl for RT1 Transmit SA2
   //                                 RT          TR            SA
   temp_ptr = startptr + ((BT_U32BIT)(RT<<8) + (TRANSMIT<<7) + (SA<<2));

   // Read the filter table entry - this is the pointer to the Control Buffer.
   temp = 0;
   status = BusTools_MemoryRead2(cardnum, RAM32, temp_ptr, 4, &temp);

   // Add 2 words, convert to a BYTE value.  Now we have the byte offset
   // to the CURRENT MSG BUFFER POINTER for RT Transmit SA.  Store this.
   tx_ptr_offset = BusTools_RelAddr(cardnum,temp);

   // Now we can read the 32-bit MBUF pointer value from the control buffer.
   status = BusTools_MemoryRead2(cardnum, RAM32, tx_ptr_offset, 4, &tx_mbuf_ptr);

   /***** Wrap the buffers by setting the TX pointer to point to the RCV buffer *****/
   // Change the message buffer pointer in the control buffer for RT SA TX to
   // use the same message buffer as RT SA RCV.
   status = BusTools_MemoryWrite2(cardnum, RAM32, tx_ptr_offset, 4, &rx_mbuf_ptr);
   // Now we are setup with RT SA WRAPPED.
} 


int RemoteTerminal_Init(int cardnum)
{
   int status;
   API_RT_CBUF cbuf;       // RT Control Buffer Legal Word Count Mask (2 words)
   API_RT_MBUF_WRITE mbuf;
   API_RT_ABUF abuf;

   status = BusTools_RT_Init(cardnum,0);
   printf("BusTools_RT_Init status = %d\n",status);

   /**********************************************************************
   *  Output RT control buffers
   *   Enable or disable all word counts and mode codes as specified by
   *    the caller's arguements.
   **********************************************************************/

   cbuf.legal_wordcount = 0xffffffff;  // legailize all word counts

   status = BusTools_RT_CbufWrite(cardnum,22,11,0,1,&cbuf);
   status = BusTools_RT_CbufWrite(cardnum,22,11,1,1,&cbuf);

   /*******************************************************************
   *  Fill in message buffers for receive and transmit sub-units.
   *******************************************************************/
   memset(&mbuf,0,sizeof(API_RT_MBUF_WRITE));
   mbuf.error_inj_id = 0;
   mbuf.enable = BT1553_INT_END_OF_MESS;   // Interrupt enables
   // For all subaddresses...
 
   status = BusTools_RT_MessageWrite(cardnum,22,11,0,0,&mbuf);
   status = BusTools_RT_MessageWrite(cardnum,22,11,1,0,&mbuf);

   /*******************************************************************
   *  Wrap the buffers.
   *******************************************************************/

   if(BusTools_BoardIsV6(cardnum))
      wrap_RT_bufferV6(cardnum,22,11);
   else
      wrap_RT_buffer(cardnum, 22, 11);

   /************************************************
   *  Setup RT address control block
   ************************************************/

   abuf.enable_a          = 1;    // Disable Bus A...Note that the API bits are
   abuf.enable_b          = 1;    // Disable Bus B... reversed from the HW bits.
   abuf.inhibit_term_flag = 0;
   abuf.status            = 0x0000;
   abuf.command           = 0;
   abuf.bit_word          = 0;
   status = BusTools_RT_AbufWrite(cardnum,22,&abuf);
   printf("BusTools_RT_AbufWrite status = %d\n",status);

   return status;

}

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   static int sa_index;
   int       status,i;
   BT_INT tail, j,wcount;
   BT_UINT messno;
   static int nFlag = 1;

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      messno = sIntFIFO->fifo[tail].bufferID;
      status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);
      if (status)
         return status; 

      if (bcmessage.mess_command1.tran_rec)
         printf("BC**RT->BC (%d)  ", messno);
      else
         printf("BC->RT (%d)  ", messno);
 
      printf("BC**RT-%d  \n",bcmessage.mess_command1.rtaddr);
      printf("BC**SA-%d  \n",bcmessage.mess_command1.subaddr);
      printf("BC**WC-%d  \n",bcmessage.mess_command1.wcount);
      printf("BC**status-1 0x%04x\n",*((WORD*)(&bcmessage.mess_status1)));
      printf("BC**int_stat-0x%08lx\n",*((DWORD*)(&bcmessage.status)));

      wcount = bcmessage.mess_command1.wcount;
      if (wcount == 0)
         wcount = 32;           // Zero word count means 32 words.

      j = wcount;               // See if this msg shorted than previous
   

      for ( i = 0; i < j; i++ ) // Display all of the data words
      {
         printf("BC**%04x ",bcmessage.data[0][i]);
      }
      printf("\n");  
            
      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }

   return 0;
}


/****************************************************************************
*
*  Function:  setupThe_BC_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO for a interrupt on a BC message.
*     The interrupt is set for the end-of message.  
*
****************************************************************************/
void setupThe_BC_IntFIFO(int cardnum)
{
   int status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.iNotification = 0;
   sIntFIFO1.function     = bc_intFunction;
   sIntFIFO1.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.bForceStartup  = 0;
   sIntFIFO1.FilterType     =  EVENT_BC_MESSAGE;

   sIntFIFO1.FilterMask[22][1][11] =  0xffffffff;  // Enable all Word Counts on RT 22 SA 11 TX

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}

