/*===========================================================================*
 * FILE:                T I M E T A G _ R E A D _ T E S T . C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2017 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    This example set the time tag to either zero or the present time
 *              and data.  Then it reads the time tag register every 500 
 *              milliseconds (.5 seconds) and prints the raw nanoseconds or
 *              microseconds depending the firmware version.  It also 
 *              converts the time to a string and prints the time string.
 *===========================================================================*/

/* $Revision:  8.22 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
 11/26/2013    Update for V8.x API and F/W 6.x
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/
//#define IRIG_B


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "busapi.h"

void setupThe_TT_IntFIFO(int);
API_INT_FIFO  sIntFIFO1;     // Thread FIFO structure for BC
static unsigned tcnt=0;

int g_1553_DeviceNum = -1;
int g_wCondorBoardInstance = 1;

int main(int argc, char **argv)
{
   BT_INT status,cardnum,time_set,i;
   BT_UINT channel,device,mode;
   DeviceList dlist;
   CEI_CHAR ichar;

   mode=1;    //default S/W interrupts
   channel=0; //default channel 1
   device=0;  //default device 0

   if(argc == 2)
      device = atoi(argv[1]);  // User entered device

   // List all 1553 devices installed
   status = BusTools_ListDevices(&dlist);
   for(i=0; i<dlist.num_devices; i++)
      printf("Board Type %s ID %x is device %d\n",dlist.name[i],dlist.device_name[i],dlist.device_num[i]);     
     
   status = BusTools_API_OpenChannel(&cardnum,mode,device,channel);
   printf("BusTools_API_OpenChannel, status = %d\n",status);
   if(status)
      exit(0);

   status = BusTools_SetInternalBus(cardnum,INTERNAL_BUS);      // Set up Internal Bus
   printf("BusTools_SetInternalBus status = %d\n",status);

   { 
      BT_UINT fw_rev,api_rev;
      status = BusTools_GetRevision(cardnum,&fw_rev,&api_rev);  // Get the API and f/w revisions
      printf("The F/W revision = %x -- The API revision = %d\n",fw_rev,api_rev);
   }

   printf("Enter 0 for to start time tag from zero (0)\nEnter 1 to set time tag to current time/data\nEnter 2 to set time tag to external 1pps clock RXMC2 board\n");
   printf("Enter Q/q/ to exit\n");
   scanf("%d",&time_set);

   if(BusTools_BoardIsV6(cardnum))
   {   
      switch (time_set)
	  {
		  case 0:
			status = BusTools_TimeTagMode( cardnum, API_TTD_RELM_NS, API_TTI_ZERO, API_TTM_FREE, NULL, 0, 0, 0 );
			break;
		  case 1:
			status = BusTools_TimeTagMode( cardnum, API_TTD_DATE_NS, API_TTI_IRIG, API_TTM_FREE, NULL, 0, 0, 0 );
			break;
		  case 2: // only the RXMC2-1553 supports external clock 
			status = BusTools_TimeTagMode( cardnum, API_TTD_RELM_NS, API_TTI_ZERO, API_TTM_XCLK, NULL, 1000, 0, 0 );
			break;
		  default:
			break;
	  }
  }
   else
   {   
      switch (time_set)
	  {
		  case 0:
			status = BusTools_TimeTagMode( cardnum, API_TTD_RELM, API_TTI_ZERO, API_TTM_FREE, NULL, 0, 0, 0 );
			break;
		  case 1:
			status = BusTools_TimeTagMode( cardnum, API_TTD_DATE, API_TTI_IRIG, API_TTM_FREE, NULL, 0, 0, 0 );
 			break;
		  case 2: // only the RXMC2-1553 supports external clock 
			status = BusTools_TimeTagMode( cardnum, API_TTD_RELM, API_TTI_ZERO, API_TTM_XCLK, NULL, 1000, 0, 0 );
			break;
		  default:
			break;
	  }
   }
   printf("BusTools_TimeTagMode status = %d\n", status);
  
   if(status == 0) { 
     setupThe_TT_IntFIFO(cardnum); //Set up BC interrupts using BusTools_RegisterFunction

     fflush(stdin);
     do {
       ichar = getchar();
     } while(ichar != 'q' && ichar != 'Q');

     status = BusTools_RegisterFunction(cardnum,&sIntFIFO1,0);
     printf("BusTools_RegisterFunction status = %d\n",status);  
   }

   printf("main: closing API\n");

   status = BusTools_API_Close(cardnum);
   printf("BusTools_API_Close status = %d\n",status);
}



/****************************************************************************
*
*  Function:  tt_intFunction
*
*  Description:
*     This routine is the user callback function invoke periodically
*
****************************************************************************/
BT_INT  _stdcall tt_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{
   BT1553_TIME rtime;
   static BT_U32BIT mlast=0;
   char tstrng[80];
  
   tcnt++;
   BusTools_TimeTagRead(cardnum,&rtime);
   if(BusTools_BoardIsV6(cardnum))
      printf("%d - Time Tag = %x   %x (nS) dif = %d (mS)\n",tcnt,rtime.topuseconds,rtime.microseconds,(rtime.microseconds-mlast)/1000000); 
   else
      printf("%d - Time Tag = %x   %x (uS) dif = %d (mS)\n",tcnt,rtime.topuseconds,rtime.microseconds,(rtime.microseconds-mlast)/1000);
   mlast = rtime.microseconds;
   BusTools_TimeGetString(&rtime,tstrng);
   printf("%s\n\n",tstrng);
   
   return 0;
}

/****************************************************************************
*
*  Function:  setupThe_BC_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a BC message.  The interrupt is set for only the end-of
*     message.  
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupThe_TT_IntFIFO(int cardnum)
{
   int status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.function     = tt_intFunction;
   sIntFIFO1.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = 500;
   sIntFIFO1.cardnum        = cardnum;

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, 1);
   printf("BusTools_RegisterFunction status = %d\n",status);
}
