/*============================================================================*
 * FILE:                    B T U S E R 0 . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:   BusTools API User DLL Interface Example DLL.
 *
 *             This module is an example which demonstrates how to setup a
 *             User Interface DLL.
 *
 * DESCRIPTION:  This user-supplied User Interface DLL is loaded during API
 *               initialization.  The functions in this module are called by
 *               the BusTools/1553-API whenever the associated API function
 *               is called.
 *
 *               Each function is responsible for insuring that an infinite
 *               calling loop is not generated if it calls an API function.
 *
 * Each function should return
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *
 * User API ENTRY POINTS:
 *      UsrAPI_Init         - Called from BusTools_API_Init and BusTools_API_InitExtended
 *      UsrAPI_Close        - Called from BusTools_API_Close
 *      UsrBC_MessageAlloc  - Called from BusTools_BC_MessageAlloc
 *      UsrBC_MessageRead   - Called from BusTools_BC_MessageRead
 *      UsrBC_MessageUpdate - Called from BusTools_BC_MessageUpdate
 *      UsrBC_MessageWrite  - Called from BusTools_BC_MessageWrite
 *      UsrBC_StartStop     - Called from BusTools_BC_StartStop
 *      UsrBM_MessageAlloc  - Called from BusTools_BM_MessageAlloc
 *      UsrBM_MessageRead   - Called from BusTools_BM_MessageRead
 *      UsrBM_StartStop     - Called from BusTools_BM_StartStop
 *      UsrRT_CbufWrite     - Called from BusTools_RT_CbufWrite
 *      UsrRT_MessageRead   - Called from BusTools_RT_MessageRead
 *      UsrRT_StartStop     - Called from BusTools_RT_StartStop
 *
 *===========================================================================*/

/* $Revision:  4.30 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  06/22/2000   Initial version.V4.06.ajh
 */

/*---------------------------------------------------------------------------*
 *                    INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include "busapi.h"


/*===========================================================================*
 * User API ENTRY POINT:          U s r A P I _ I n i t
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever either
 *              BusTools_API_Init or BusTools_API_InitExtended is called.
 *
 * DESCRIPTION: This is a template for a user-defined function.  If it is not
 *              being used it should be removed from the DLL to avoid the
 *              overhead of the API calling the function.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrAPI_Init(
   BT_UINT   cardnum,        // (i) card number (0 - based)
   BT_U32BIT *base_address,  // (i) base address of board/carrier memory area
   BT_U32BIT *ioaddr,        // (i) board/carrier io address
   BT_UINT   *flag,          // (io) 0->emulation driver, 1->hardware driver, 2->Enable HW interrupts
   BT_UINT   *platform,      // (i) execution platform: PLATFORM_PC, PLATFORM_VMIC
   BT_UINT   *boardType,     // (i) PCI1553 or ISA1553
   BT_UINT   *carrier,       // (i) NATIVE, IP1553_ISA, IP1553_PCI, IP1553_VME,
                             //     IP1553_CP, IP1553_VXI.
   BT_UINT   *slot,          // (i) SLOT_A, SLOT_B, SLOT_C, SLOT_D, etc.
   BT_UINT   *mapping)       // (i) carrier memory map: CARRIER_MAP_DEFAULT
{
   static int NotFirstTime = 0;
   if ( NotFirstTime )
      return API_CONTINUE;

   NotFirstTime = 1;
   MessageBox(NULL, "User Function Entered", "UsrAPI_Init", MB_OK | MB_SYSTEMMODAL);

   return API_CONTINUE;      // Have the API function continue normally
}
#endif

/*===========================================================================*
 * User API ENTRY POINT:          U s r A P I _ C l o s e
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever
 *              BusTools_API_Close is called.
 *
 * DESCRIPTION: This is a template for a user-defined function.  If it is not
 *              being used it should be removed from the DLL to avoid the
 *              overhead of the API calling the function.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrAPI_Close(BT_UINT cardnum)
{
   static int NotFirstTime = 0;
   if ( NotFirstTime )
      return API_CONTINUE;

   NotFirstTime = 1;
   MessageBox(NULL, "User Function Entered", "UsrAPI_Close", MB_OK | MB_SYSTEMMODAL);

   return API_CONTINUE;      // Have the API function continue normally
}
#endif

/*===========================================================================*
 * User API ENTRY POINT:        U s r B C _ M e s s a g e A l l o c
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever
 *              BusTools_BC_MessageAlloc is called.
 *
 * DESCRIPTION: This is a template for a user-defined function.  If it is not
 *              being used it should be removed from the DLL to avoid the
 *              overhead of the API calling the function.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrBC_MessageAlloc(
   BT_UINT cardnum,         // (i) card number (0 - based)
   BT_UINT *count)          // (i) number of BC messages to allocate
{
   static int NotFirstTime = 0;
   if ( NotFirstTime )
      return API_CONTINUE;

   NotFirstTime = 1;
   MessageBox(NULL, "User Function Entered", "UsrBC_MessageAlloc", MB_OK | MB_SYSTEMMODAL);

   return API_CONTINUE;      // Have the API function continue normally
}
#endif

/*===========================================================================*
 * User API ENTRY POINT:        U s r B C _ M e s s a g e R e a d
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever
 *              BusTools_BC_MessageRead is called.
 *
 * DESCRIPTION: This is a template for a user-defined function.  If it is not
 *              being used it should be removed from the DLL to avoid the
 *              overhead of the API calling the function.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrBC_MessageRead(
   BT_UINT cardnum,         // (i) card number (0 - based)
   BT_UINT *messno,         // (i) index of BC message to read
   API_BC_MBUF *api_message) // (o) user's buffer to write message into
{
   static int NotFirstTime = 0;
   if ( NotFirstTime )
      return API_CONTINUE;

   NotFirstTime = 1;
   MessageBox(NULL, "User Function Entered", "UsrBC_MessageRead", MB_OK | MB_SYSTEMMODAL);

   return API_CONTINUE;      // Have the API function continue normally
}
#endif

/*===========================================================================*
 * User API ENTRY POINT:        U s r B C _ M e s s a g e U p d a t e
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever
 *              BusTools_BC_MessageUpdate is called.
 *
 * DESCRIPTION: This is a template for a user-defined function.  If it is not
 *              being used it should be removed from the DLL to avoid the
 *              overhead of the API calling the function.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrBC_MessageUpdate(
   BT_UINT cardnum,         // (i) card number (0 - based)
   BT_UINT *mblock_id,      // (i) index of BC message to update
   BT_U16BIT *buffer)       // (i) pointer to data to copy to BC message
{
   static int NotFirstTime = 0;
   if ( NotFirstTime )
      return API_CONTINUE;

   NotFirstTime = 1;
   MessageBox(NULL, "User Function Entered", "UsrBC_MessageUpdate", MB_OK | MB_SYSTEMMODAL);

   return API_CONTINUE;      // Have the API function continue normally
}
#endif

/*===========================================================================*
 * User API ENTRY POINT:        U s r B C _ M e s s a g e W r i t e
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever
 *              BusTools_BC_MessageWrite is called.
 *
 * DESCRIPTION: This is a template for a user-defined function.  If it is not
 *              being used it should be removed from the DLL to avoid the
 *              overhead of the API calling the function.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrBC_MessageWrite(
   BT_UINT cardnum,         // (i) card number (0 - based)
   BT_UINT *messno,         // (i) index of BC message to write to
   API_BC_MBUF *api_message) // (i) pointer to user-specified BC message
{
   static int NotFirstTime = 0;
   if ( NotFirstTime )
      return API_CONTINUE;

   NotFirstTime = 1;
   MessageBox(NULL, "User Function Entered", "UsrBC_MessageWrite", MB_OK | MB_SYSTEMMODAL);

   return API_CONTINUE;      // Have the API function continue normally
}
#endif

/*===========================================================================*
 * User API ENTRY POINT:        U s r B C _ S t a r t S t o p
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever
 *              BusTools_BC_StartStop is called.
 *
 * DESCRIPTION: This is a template for a user-defined function.  If it is not
 *              being used it should be removed from the DLL to avoid the
 *              overhead of the API calling the function.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrBC_StartStop(
   BT_UINT cardnum,         // (i) card number (0 - based)
   BT_UINT *flag)           // (i) 1 -> start BC (at message 0), 0 -> stop BC
{
   static int NotFirstTime = 0;
   if ( NotFirstTime )
      return API_CONTINUE;

   NotFirstTime = 1;
   MessageBox(NULL, "User Function Entered", "UsrBC_StartStop", MB_OK | MB_SYSTEMMODAL);

   return API_CONTINUE;      // Have the API function continue normally
}
#endif

/*===========================================================================*
 * User API ENTRY POINT:        U s r B M _ M e s s a g e A l l o c
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever
 *              BusTools_BM_MessageAlloc is called.
 *
 * DESCRIPTION: This is a template for a user-defined function.  If it is not
 *              being used it should be removed from the DLL to avoid the
 *              overhead of the API calling the function.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrBM_MessageAlloc(
   BT_UINT cardnum,         // (i) card number (0 - based)
   BT_UINT *mbuf_count,
   BT_UINT *mbuf_actual,
   BT_U32BIT *enable)
{
   static int NotFirstTime = 0;
   if ( NotFirstTime )
      return API_CONTINUE;

   NotFirstTime = 1;
   MessageBox(NULL, "User Function Entered", "UsrBM_MessageAlloc", MB_OK | MB_SYSTEMMODAL);

   return API_CONTINUE;      // Have the API function continue normally
}
#endif

/*===========================================================================*
 * User API ENTRY POINT:        U s r B M _ M e s s a g e R e a d
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever
 *              BusTools_BM_MessageRead is called.
 *
 * DESCRIPTION: This is a template for a user-defined function.  If it is not
 *              being used it should be removed from the DLL to avoid the
 *              overhead of the API calling the function.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrBM_MessageRead(
   BT_UINT cardnum,         // (i) card number (0 - based)
   BT_UINT *mbuf_id,
   API_BM_MBUF *mbuf)
{
   static int NotFirstTime = 0;
   if ( NotFirstTime )
      return API_CONTINUE;

   NotFirstTime = 1;
   MessageBox(NULL, "User Function Entered", "UsrBM_MessageRead", MB_OK | MB_SYSTEMMODAL);

   return API_CONTINUE;      // Have the API function continue normally
}
#endif

/*===========================================================================*
 * User API ENTRY POINT:        U s r B M _ S t a r t S t o p
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever
 *              BusTools_BM_StartStop is called.
 *
 * DESCRIPTION: This is a template for a user-defined function.  If it is not
 *              being used it should be removed from the DLL to avoid the
 *              overhead of the API calling the function.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrBM_StartStop(
   BT_UINT cardnum,         // (i) card number (0 - based)
   BT_UINT *flag)           // (i) 1 -> Start BM, 0 -> Stop BM
{
   static int NotFirstTime = 0;
   if ( NotFirstTime )
      return API_CONTINUE;

   NotFirstTime = 1;
   MessageBox(NULL, "User Function Entered", "UsrBM_StartStop", MB_OK | MB_SYSTEMMODAL);

   return API_CONTINUE;      // Have the API function continue normally
}
#endif

/*===========================================================================*
 * User API ENTRY POINT:        U s r R T _ C b u f W r i t e
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever
 *              BusTools_RT_CbufWrite is called.
 *
 * DESCRIPTION: This is a template for a user-defined function.  If it is not
 *              being used it should be removed from the DLL to avoid the
 *              overhead of the API calling the function.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrRT_CbufWrite(
   BT_UINT cardnum,         // (i) card number (0 - based)
   BT_UINT *rtaddr,         // (i) RT address (0 - based)
   BT_UINT *subaddr,        // (i) RT subaddress (0 - based)
   BT_UINT *tr,             // (i) Transmit/Receive flag (1->rt transmit)
   BT_INT  *mbuf_count,     // if negative, one pass through buffers only
   API_RT_CBUF *apicbuf)    // (i) pointer to API RT control buf
{
   static int NotFirstTime = 0;
   if ( NotFirstTime )
      return API_CONTINUE;

   NotFirstTime = 1;
   MessageBox(NULL, "User Function Entered", "UsrRT_CbufWrite", MB_OK | MB_SYSTEMMODAL);

   return API_CONTINUE;      // Have the API function continue normally
}
#endif

/*===========================================================================*
 * User API ENTRY POINT:        U s r R T _ M e s s a g e R e a d
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever
 *              BusTools_RT_MessageRead is called.
 *
 * DESCRIPTION: This is a template for a user-defined function.  If it is not
 *              being used it should be removed from the DLL to avoid the
 *              overhead of the API calling the function.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrRT_MessageRead(
   BT_UINT cardnum,         // (i) card number (0 - based)
   BT_UINT *rtaddr,         // (i) RT address (0 - based)
   BT_UINT *subaddr,        // (i) RT subaddress (0 - based)
   BT_UINT *tr,             // (i) Transmit/Receive flag (1->rt transmit)
   BT_UINT *mbuf_id,        // (i) RT MBUF number
   API_RT_MBUF_READ *mbuf)
{
   static int NotFirstTime = 0;
   if ( NotFirstTime )
      return API_CONTINUE;

   NotFirstTime = 1;
   MessageBox(NULL, "User Function Entered", "UsrRT_MessageRead", MB_OK | MB_SYSTEMMODAL);

   return API_CONTINUE;      // Have the API function continue normally
}
#endif

/*===========================================================================*
 * User API ENTRY POINT:        U s r R T _ S t a r t S t o p
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever
 *              BusTools_RT_StartStop is called.
 *
 * DESCRIPTION: This is a template for a user-defined function.  If it is not
 *              being used it should be removed from the DLL to avoid the
 *              overhead of the API calling the function.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrRT_StartStop(
   BT_UINT cardnum,         // (i) card number (0 - based)
   BT_UINT *flag)           // (i) 1 -> start RT simulation, 0 -> stop simulation
{
   static int NotFirstTime = 0;
   if ( NotFirstTime )
      return API_CONTINUE;

   NotFirstTime = 1;
   MessageBox(NULL, "User Function Entered", "UsrRT_StartStop", MB_OK | MB_SYSTEMMODAL);

   return API_CONTINUE;      // Have the API function continue normally
}
#endif
