/*============================================================================*
 * FILE:                    B T U S E R 1 . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:   BusTools API User DLL Interface Example DLL.
 *
 *             This module is an example which demonstrates how to setup a
 *             User Interface DLL.
 *
 * DESCRIPTION:  This template DLL demonstrates how a User Interface DLL can be
 *               used to modify the data that a BC message transmits to an RT.
 *               It intercepts all BusTools_BC_MessageWrite() function calls,
 *               and clears the Interrupt Enable on all messages except the
 *               last message in the minor frame.
 *               It sets up a BusTools_RegisterFunction that monitors the
 *               Bus Controller for the last message in the minor frame.
 *               When that message occurs the thread runs and modifies the data
 *               in the BC message the user defined DLL.
 *               In this case, the BusTools_RegisterFunction, when notified
 *               of a BC message, will increment one of the data words in the
 *               message through a call to BusTools_BC_MessageUpdate()..
 *
 * Each function should return
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User DLL function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *
 * User API ENTRY POINTS:
 *      UsrAPI_Init         - Called from BusTools_API_Init and BusTools_API_InitExtended
 *      UsrAPI_Close        - Called from BusTools_API_Close
 *      UsrBC_MessageAlloc  - Called from BusTools_BC_MessageAlloc
 *      UsrBC_MessageRead   - Called from BusTools_BC_MessageRead
 *      UsrBC_MessageUpdate - Called from BusTools_BC_MessageUpdate
 *      UsrBC_MessageWrite  - Called from BusTools_BC_MessageWrite
 *      UsrBC_StartStop     - Called from BusTools_BC_StartStop
 *      UsrBM_MessageAlloc  - Called from BusTools_BM_MessageAlloc
 *      UsrBM_MessageRead   - Called from BusTools_BM_MessageRead
 *      UsrBM_StartStop     - Called from BusTools_BM_StartStop
 *      UsrRT_CbufWrite     - Called from BusTools_RT_CbufWrite
 *      UsrRT_MessageRead   - Called from BusTools_RT_MessageRead
 *      UsrRT_StartStop     - Called from BusTools_RT_StartStop
 *
 *===========================================================================*/

/* $Revision:  4.30 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  06/22/2000   Initial version.V4.06.ajh
 */

/*---------------------------------------------------------------------------*
 *                    INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include "busapi.h"


/*===========================================================================*
 * User API ENTRY POINT:          U s r A P I _ I n i t
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever either
 *              BusTools_API_Init or BusTools_API_InitExtended is called.
 *
 * DESCRIPTION: This is a template for a user-defined function.  If it is not
 *              being used it should be removed from the DLL to avoid the
 *              overhead of the API calling the function.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrAPI_Init(
   BT_UINT   cardnum,        // (i) card number (0 - based)
   BT_U32BIT *base_address,  // (i) base address of board/carrier memory area
   BT_U32BIT *ioaddr,        // (i) board/carrier io address
   BT_UINT   *flag,          // (io) 0->emulation driver, 1->hardware driver, 2->Enable HW interrupts
   BT_UINT   *platform,      // (i) execution platform: PLATFORM_PC, PLATFORM_VMIC
   BT_UINT   *boardType,     // (i) IP1553, IP1553SF, IP1553MF, PCI1553 or ISA1553
   BT_UINT   *carrier,       // (i) NATIVE, IP1553_ISA, IP1553_PCI, IP1553_VME,
                             //     IP1553_CP, IP1553_VXI.
   BT_UINT   *slot,          // (i) SLOT_A, SLOT_B, SLOT_C, SLOT_D, etc.
   BT_UINT   *mapping)       // (i) carrier memory map: CARRIER_MAP_DEFAULT
{
   return API_NEVER_CALL_AGAIN;  // Continue and never call me again
}
#endif

/*===========================================================================*
 * User API ENTRY POINT:          U s r A P I _ C l o s e
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever
 *              BusTools_API_Close is called.
 *
 * DESCRIPTION: This is a template for a user-defined function.  If it is not
 *              being used it should be removed from the DLL to avoid the
 *              overhead of the API calling the function.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrAPI_Close(BT_UINT cardnum)
{
   return API_NEVER_CALL_AGAIN;  // Continue and never call me again
}
#endif

/*===========================================================================*
 * User API ENTRY POINT:        U s r B C _ M e s s a g e A l l o c
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever
 *              BusTools_BC_MessageAlloc is called.
 *
 * DESCRIPTION: This is a template for a user-defined function.  If it is not
 *              being used it should be removed from the DLL to avoid the
 *              overhead of the API calling the function.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrBC_MessageAlloc(
   BT_UINT cardnum,         // (i) card number (0 - based)
   BT_UINT *count)          // (i) number of BC messages to allocate
{
   return API_NEVER_CALL_AGAIN;  // Continue and never call me again
}
#endif

/*===========================================================================*
 * User API ENTRY POINT:        U s r B C _ M e s s a g e R e a d
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever
 *              BusTools_BC_MessageRead is called.
 *
 * DESCRIPTION: This is a template for a user-defined function.  If it is not
 *              being used it should be removed from the DLL to avoid the
 *              overhead of the API calling the function.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrBC_MessageRead(
   BT_UINT cardnum,         // (i) card number (0 - based)
   BT_UINT *messno,         // (i) index of BC message to read
   API_BC_MBUF *api_message) // (o) user's buffer to write message into
{
   return API_NEVER_CALL_AGAIN;  // Continue and never call me again
}
#endif

/*===========================================================================*
 * User API ENTRY POINT:        U s r B C _ M e s s a g e U p d a t e
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever
 *              BusTools_BC_MessageUpdate is called.
 *
 * DESCRIPTION: This is a template for a user-defined function.  If it is not
 *              being used it should be removed from the DLL to avoid the
 *              overhead of the API calling the function.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrBC_MessageUpdate(
   BT_UINT cardnum,         // (i) card number (0 - based)
   BT_UINT *mblock_id,      // (i) index of BC message to update
   BT_U16BIT *buffer)       // (i) pointer to data to copy to BC message
{
   return API_NEVER_CALL_AGAIN;  // Continue and never call me again
}
#endif

/*===========================================================================*
 * User API ENTRY POINT:        U s r B C _ M e s s a g e W r i t e
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever
 *              BusTools_BC_MessageWrite is called.
 *
 * DESCRIPTION: This function intercepts all BusTools_BC_MessageWrite()
 *              function calls, and clears the Interrupt Enable on all
 *              BC messages except the last message in the minor frame.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrBC_MessageWrite(
   BT_UINT cardnum,          // (i) card number (0 - based)
   BT_UINT *messno,          // (i) index of BC message to write to
   API_BC_MBUF *api_message) // (i) pointer to user-specified BC message
{
   // Determine if this message is the one that triggers the data update.
   // If it is, make sure the interrupt enable is set, otherwise clear the
   //  interrupt enable.
   // We trigger off of the first message in the minor frame.  Since this is
   //  the message that we update, triggering on it gives us the maximum time
   //  to update the message before it is transmitted again.
   api_message->control &= ~BC_CONTROL_INTERRUPT;    // Turn off the interrupt

   // Not a message?
   if ( (api_message->control & BC_CONTROL_TYPEMASK) != BC_CONTROL_MESSAGE )
      return API_CONTINUE;      // Have the API function continue normally

   // Not the beginning of the frame?
   if ( (api_message->control & BC_CONTROL_MFRAME_BEG) == 0 )
      return API_CONTINUE;      // Have the API function continue normally

   api_message->control |= BC_CONTROL_INTERRUPT;    // Turn on the interrupt

   return API_CONTINUE;      // Have the API function continue normally
}
#endif

/*===========================================================================*
 * User ENTRY POINT:        demo_bc_watch_function
 *===========================================================================*
 *
 * FUNCTION:    This register function is called whenever a BC message interrupt
 *              is detected.
 *
 * DESCRIPTION: This function is called by the API whenever a BC message with the
 *              interrupt bit set is detected.
 *              This function increments the data in the message data buffer, and
 *              shows how to register a BC interrupt thread and process the
 *              resulting thread FIFO entries and interrupt calls.
 *
 * It will return:
 *   API_SUCCESS - Thread continues execution
 *===========================================================================*/

BT_INT _stdcall demo_bc_watch_function(
   BT_UINT cardnum,
   struct api_int_fifo *sIntFIFO)
{
   /***********************************************************************
   *  Local variables
   ***********************************************************************/
   BT_U16BIT   data[33];       // Data buffer we update
   BT_INT      tail;           // FIFO Tail index
   BT_UINT     messno;         // Message number to be updated
   BT_INT      status;

   /***********************************************************************
   *  Loop on all entries in the FIFO.  Get the tail pointer and extract
   *   the FIFO entry it points to.   When head == tail FIFO is empty
   ***********************************************************************/
   tail = sIntFIFO->tail_index;
   while ( tail != sIntFIFO->head_index )
   {
      // Extract the buffer ID from the FIFO
      messno = sIntFIFO->fifo[tail].bufferID;
      //  and read the message data from the board.
      status = BusTools_BC_MessageReadData(cardnum, messno, data);
      if ( status )
         return status;

      // Update the data buffer per the magic (pat.pending) algorithm:
      data[0] = (BT_U16BIT)(data[0] + data[1]);
      data[2] += (short)1;

      // Now write the data back to the message buffer:
      status = BusTools_BC_MessageUpdate(cardnum, messno, data);
      if ( status )
         return status;

      // Now update and store the tail pointer.
      tail++;                         // Next entry
      tail &= sIntFIFO->mask_index;   // Wrap the index
      sIntFIFO->tail_index = tail;    // Save the index
   }
   return API_SUCCESS;
}

/*===========================================================================*
 * User API ENTRY POINT:        U s r B C _ S t a r t S t o p
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever
 *              BusTools_BC_StartStop is called.
 *
 * DESCRIPTION: This function is called by the API whenever the BC is started or
 *              stopped.  This function sets up a BusTools_RegisterFunction
 *              that monitors the BC for the first message in the minor frame.
 *              When that message occurs the thread runs and modifies the data
 *              in the BC message.
 *              In this case, the BusTools_RegisterFunction, when notified
 *              of a BC message, will increment one of the data words in the
 *              message through a call to BusTools_BC_MessageUpdate()..
 *              If the BC is being stopped then the register function is distroyed.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
static API_INT_FIFO  sIntFIFO1;     // Thread FIFO structure

#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrBC_StartStop(
   BT_UINT cardnum,         // (i) card number (0 - based)
   BT_UINT *flag)           // (i) 1 -> start BC (at message 0), 0 -> stop BC
{
   int    rt, tr, sa;
   int    status;

   /***********************************************************************
   *  If the BC is starting, register a thread for the board.
   *  If the BC is shutting down, unregister the thread.
   **********************************************************************/
   if ( *flag )
   {
      // Setup the FIFO structure for this board.
      memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
      sIntFIFO1.function     = demo_bc_watch_function;
      sIntFIFO1.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
      sIntFIFO1.dwMilliseconds = INFINITE;
      sIntFIFO1.iNotification  = 0;       // Dont care about startup or shutdown
      sIntFIFO1.FilterType     = EVENT_BC_MESSAGE;
      for ( rt=0; rt < 32; rt++ )
         for (tr = 0; tr < 2; tr++ )
            for (sa = 0; sa < 32; sa++ )
               sIntFIFO1.FilterMask[rt][tr][sa] = 0xFFFFFFFF;  // Enable all messages

      // Call the register function to register and start the BC thread.
      status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, 1);
      if ( status )
         return status;
   }
   else
   {
      // Call the register function to unregister and stop the BC thread.
      status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, 0);
      if ( status )
         return status;
   }
   return API_CONTINUE;      // Have the API function continue normally
}
#endif

/*===========================================================================*
 * User API ENTRY POINT:        U s r B M _ M e s s a g e A l l o c
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever
 *              BusTools_BM_MessageAlloc is called.
 *
 * DESCRIPTION: This is a template for a user-defined function.  If it is not
 *              being used it should be removed from the DLL to avoid the
 *              overhead of the API calling the function.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrBM_MessageAlloc(
   BT_UINT cardnum,         // (i) card number (0 - based)
   BT_UINT *mbuf_count,
   BT_UINT *mbuf_actual,
   BT_U32BIT *enable)
{
   return API_NEVER_CALL_AGAIN;  // Continue and never call me again
}
#endif

/*===========================================================================*
 * User API ENTRY POINT:        U s r B M _ M e s s a g e R e a d
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever
 *              BusTools_BM_MessageRead is called.
 *
 * DESCRIPTION: This is a template for a user-defined function.  If it is not
 *              being used it should be removed from the DLL to avoid the
 *              overhead of the API calling the function.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrBM_MessageRead(
   BT_UINT cardnum,         // (i) card number (0 - based)
   BT_UINT *mbuf_id,
   API_BM_MBUF *mbuf)
{
   return API_NEVER_CALL_AGAIN;  // Continue and never call me again
}
#endif

/*===========================================================================*
 * User API ENTRY POINT:        U s r B M _ S t a r t S t o p
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever
 *              BusTools_BM_StartStop is called.
 *
 * DESCRIPTION: This is a template for a user-defined function.  If it is not
 *              being used it should be removed from the DLL to avoid the
 *              overhead of the API calling the function.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrBM_StartStop(
   BT_UINT cardnum,         // (i) card number (0 - based)
   BT_UINT *flag)           // (i) 1 -> Start BM, 0 -> Stop BM
{
   return API_NEVER_CALL_AGAIN;  // Continue and never call me again
}
#endif

/*===========================================================================*
 * User API ENTRY POINT:        U s r R T _ C b u f W r i t e
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever
 *              BusTools_RT_CbufWrite is called.
 *
 * DESCRIPTION: This is a template for a user-defined function.  If it is not
 *              being used it should be removed from the DLL to avoid the
 *              overhead of the API calling the function.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrRT_CbufWrite(
   BT_UINT cardnum,         // (i) card number (0 - based)
   BT_UINT *rtaddr,         // (i) RT address (0 - based)
   BT_UINT *subaddr,        // (i) RT subaddress (0 - based)
   BT_UINT *tr,             // (i) Transmit/Receive flag (1->rt transmit)
   BT_INT  *mbuf_count,     // if negative, one pass through buffers only
   API_RT_CBUF *apicbuf)    // (i) pointer to API RT control buf
{
   return API_NEVER_CALL_AGAIN;  // Continue and never call me again
}
#endif

/*===========================================================================*
 * User API ENTRY POINT:        U s r R T _ M e s s a g e R e a d
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever
 *              BusTools_RT_MessageRead is called.
 *
 * DESCRIPTION: This is a template for a user-defined function.  If it is not
 *              being used it should be removed from the DLL to avoid the
 *              overhead of the API calling the function.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrRT_MessageRead(
   BT_UINT cardnum,         // (i) card number (0 - based)
   BT_UINT *rtaddr,         // (i) RT address (0 - based)
   BT_UINT *subaddr,        // (i) RT subaddress (0 - based)
   BT_UINT *tr,             // (i) Transmit/Receive flag (1->rt transmit)
   BT_UINT *mbuf_id,        // (i) RT MBUF number
   API_RT_MBUF_READ *mbuf)
{
   return API_NEVER_CALL_AGAIN;  // Continue and never call me again
}
#endif

/*===========================================================================*
 * User API ENTRY POINT:        U s r R T _ S t a r t S t o p
 *===========================================================================*
 *
 * FUNCTION:    This User Interface function is called whenever
 *              BusTools_RT_StartStop is called.
 *
 * DESCRIPTION: This is a template for a user-defined function.  If it is not
 *              being used it should be removed from the DLL to avoid the
 *              overhead of the API calling the function.
 *
 * It will return:
 *   API_CONTINUE       - API function should continue execution normally
 *   API_RETURN_SUCCESS - API function should return immediately with API_SUCCESS
 *   API_NEVER_CALL_AGAIN - User function is never to be called again
 *
 *   or some other BusTools error code that the API function should
 *              pass back to the application.
 *===========================================================================*/
#if 1       /* Change the 1 to a 0 if this function is not being used */
#if defined(__BORLANDC__)
#pragma argsused
#endif
NOMANGLE BT_INT CCONV UsrRT_StartStop(
   BT_UINT cardnum,         // (i) card number (0 - based)
   BT_UINT *flag)           // (i) 1 -> start RT simulation, 0 -> stop simulation
{
   return API_NEVER_CALL_AGAIN;  // Continue and never call me again
}
#endif
