/*===========================================================================*
 * FILE:                     V X W _ D E M O . C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    Demonstration of BusTools/1553-API routines. These routines
 *              show how to initialize the Bus Conroller, Bus Monitor and
 *              Remote Terminal.  The also show how to access data using the
 *              interrupt queue to get the messages transacted on the bus.
 *
 * Notes:       To compile for the VME-1553 use the BusTools_API_InitExtended
 *              call for that board.  To compile for the PCI-1553, cPCI-1553, 
 *              or PMC-1553 use the BusTools_API_InitExtended for those cards.
 *              When compiling define _UNIX_ and VXWORKS and do not use the 
 *              -ansi directive.  
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  06/11/2001    initial release
  01/27/2003    Update form improved examples
  06/20/2003    Add interrupt example
  01/25/2011	Remove legacy interrupt support only use BusTools_RegisterFunction
  03/08/2012    Add support for F/W 6.0 64-bit nanosecond time tags
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/
#include <stdio.h>
#include <time.h>
#ifndef WIN32
#include <ioLib.h>
#endif

#include "busapi.h"

#ifdef irig_time_setup
void runThe_IRIG_test(int);
#endif


int BusController_Init(int);
int BusMonitor_Init(int);
int RemoteTerminal_Init(int);
BT_INT bm_readFunction(BT_UINT cardnum, BT_UINT mcount, API_BM_MBUF * mbuf);

API_INT_FIFO  sIntFIFO1;     // Thread FIFO structure for BC
API_INT_FIFO  sIntFIFO2;     // Thread FIFO structure for RT
API_INT_FIFO  sIntFIFO3;     // Thread FIFO structure for BM

void setupThe_BC_IntFIFO(int);
void setupThe_RT_IntFIFO(int);
void setupThe_BM_IntFIFO(int cardnum);

int total_bc_msgs;
int total_bm_msgs;
int total_rt_msgs;

BT_INT use_hw_int;
BT_INT use_regfunc;

#ifndef WIN32
BT_INT iod;
#endif

#ifdef WIN32
int main(int argc, char **argv)
#else
int demo(BT_UINT device, int channel)
#endif
{
   int status;
   BT_UINT pwFlag, cardnum=0;
   char ichar; 
#ifdef WIN32
   BT_UINT device=0, channel=0;
#endif

#ifndef WIN32
   iod = ioTaskStdGet(0,1);
#endif

#ifdef WIN32
   pwFlag = API_B_MODE  | API_SW_INTERRUPT; //  Use polling
#else
   pwFlag = API_B_MODE  | API_HW_ONLY_INT; //  Use polling
#endif
   
#if defined(VXW_PCI_PPC) || defined(WIN32) 
   printf("Initializing channel %d on device %d\n",channel,device);
   status = BusTools_API_OpenChannel(&cardnum,pwFlag,device,channel);
#elif VXW_VME_PPC
   if(device>MAX_BTA)
   {
	  BT_U32BIT addr32,addr16;
	  addr32 = channel;
	  addr16 = device;
	  printf("Initializing VME A32 = %x A16 = %x\n",addr32,addr16);
      status = BusTools_API_InitExtended(cardnum,addr32,addr16,&pwFlag,PLATFORM_PC,QVME1553,NATIVE_32,CHANNEL_1,CARRIER_MAP_A32S);
   }
#else
#error PCI or VME not specified
#endif
      
   printf("Initialization status = %d\n",status);
   if(status != 0)   
   {  
	  printf("Init fail with error %d\n",status);
      BusTools_API_Close(cardnum);
      return 0;
   }

   BusTools_SetInternalBus(cardnum, EXTERNAL_BUS);
   if(BusTools_BoardIsV6(cardnum))
      BusTools_TimeTagMode(cardnum,API_TTD_RELM_NS,API_TTI_ZERO,API_TTM_FREE,NULL,0,0,0);
   else
      BusTools_TimeTagMode(cardnum,API_TTD_RELM,API_TTI_ZERO,API_TTM_FREE,NULL,0,0,0);
;
   { 
      float lpu,wcs;
      int build;
      BT_UINT mrev, arev;
      status = BusTools_GetRevision(cardnum,&mrev,&arev);
      printf("mrev = %x --- arev = %d\n",mrev,arev);
      status = BusTools_GetFWRevision(cardnum,&wcs,&lpu, &build);
      printf("The LPU revision = %3.2f -- The WCS revision = %3.2f, build = %d\n",lpu,wcs, build);
   }

   printf("\nVxW_Demo: \n");
   printf("Start the BM (1), BC (2) and RT (3).  To display data type:\n");
   printf("c - for Bus Controller\n");
   printf("r - for Remote Terminal\n");
   printf("m - for Bus Monitor\n");
   printf("Select options from Menu below... \n");

   printf("\nHit q to Exit\n");
   printf("Type 1 to start the Bus Monitor\n");
   printf("Type 2 to start the Bus Controller\n");
   printf("Type 3 to start the Remote Terminal\n");
   printf("Type 4 to stop the Bus Monitor\n");
   printf("Type 5 to stop the Bus Controller\n");
   printf("Type 6 to stop the Remote Terminal\n");
   printf("Type C for Bus Controller output - 'x' to quit\n");
   printf("Type M for Bus Monitor output - 'x' to quit\n");
   printf("Type L for Bus Monitor read Last message block\n");
   printf("Type R for Remote Terminal output - 'x' to quit\n");
   printf("Type E for External Bus\n");
   printf("Type I for Internal Bus\n");
   printf("Type D for Direct Couple\n");
   printf("Type T for Transformer Couple\n");
   printf("Type B for Internal BIT test\n");
   printf("type H to read and display the heartbeat register\n");
   printf("Type ? for lists of commands\n\n");
                             
   while((ichar = getc(stdin)) != 'q')
   {
      printf("\n --> ");
      switch(ichar)
      {
         case '1':
            status = BusMonitor_Init(cardnum);
            break;
         case '2':
            status = BusController_Init(cardnum);
            break;
         case '3':
            status = RemoteTerminal_Init(cardnum);
            break;
         case '4':
        	status = BusTools_BM_StartStop(cardnum,0);
            printf("BusTools_BM_StartStop status = %d\n",status);
            break;
         case '5':
            status = BusTools_BC_StartStop(cardnum,0);
            printf("BusTools_BC_StartStop status = %d\n",status);
            break;
         case '6':
            status = BusTools_RT_StartStop(cardnum,0);
            printf("BusTools_RT_StartStop status = %d\n",status);
            break;
         case 'C':
         case 'c':
 	        setupThe_BC_IntFIFO(cardnum);
			while((ichar=getchar())!='x'){};
		    status = BusTools_RegisterFunction(cardnum,&sIntFIFO1,UNREGISTER_FUNCTION);
            break;
         case 'M':
         case 'm':
 	        setupThe_BM_IntFIFO(cardnum);
	        while((ichar=getchar())!='x'){};
		    status = BusTools_RegisterFunction(cardnum,&sIntFIFO3,UNREGISTER_FUNCTION);
            break;
         case 'L':
         case 'l':
		    while(1)
		    {
		       BT_UINT mcount;
		       API_BM_MBUF bm_mbuf[200];
	           MSDELAY(50);
	           BusTools_BM_ReadLastMessageBlock(cardnum,-1,-1,-1,&mcount,bm_mbuf);
	           if(mcount>0)
	        	   bm_readFunction(cardnum,mcount,bm_mbuf);
		    }		
            break;
         case 'N':
         case 'n':            
            status = BusTools_BC_MessageNoop(cardnum,2,0,0);
            printf("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!BusTools_BC_MessageNoop status = %d\n",status);
            break;
         case 'R':
         case 'r':
		    setupThe_RT_IntFIFO(cardnum);
		    while((ichar=getchar())!='x'){};
		    status = BusTools_RegisterFunction(cardnum,&sIntFIFO2,UNREGISTER_FUNCTION);
		    break;
         case 'E':
         case 'e':
            status = BusTools_SetInternalBus(cardnum,0);  //Use the external Bus
            printf("BusTools_SetInternal status = %d\n",status);
         case 'I':
         case 'i':
            status = BusTools_SetInternalBus(cardnum,1); //Use the internal Bus
            printf("BusTools_SetInternalBus status = %d\n",status);
            break;
         case 'd':
         case 'D':
            status = BusTools_SetVoltage(cardnum, 500, 0 ); //Select Direct coupling
            printf("BusTools_SetVoltage status = %d\n",status);
            break;
            break;
         case 't':
         case 'T':
               
            break;
         case 'b':
         case 'B':
            status = BusTools_BIT_InternalBit(cardnum, 100);  //Run internal BIT
            printf("BusTools_BIT_InternalBit status = %d\n",status);
            break;    
         case 'H':
         case 'h':
         {
            BT_U32BIT beat;
            status = BusTools_MemoryRead2(cardnum,HWREG,0x14,1,&beat);
            printf("The beat is now %d\n",beat);
            break;
         }
         case '?':
               printf("\nHit q to Exit\n");
               printf("Type 1 to start the Bus Monitor\n");
               printf("Type 2 to start the Bus Controller\n");
               printf("Type 3 to start the Remote Terminal\n");
               printf("Type 4 to stop the Bus Monitor\n");
               printf("Type 5 to stop the Bus Controller\n");
               printf("Type 6 to stop the Remote Terminal\n");
               printf("Type C for Bus Controller output - 'x' to quit\n");
               printf("Type F for BusToolsRegisterFunction BC Display\n");
               printf("Type M for Bus Monitor output - 'x' to quit\n");
               printf("Type R for Remote Terminal output - 'x' to quit\n");
               printf("Type E for External Bus\n");
               printf("Type I for Internal Bus\n");
               printf("Type D for Direct Couple\n");
               printf("Type T for Transformer Couple\n");
               printf("type H to read and display the heartbeat register\n");
               printf("Type B for Internal BIT test\n");
               printf("Type ? for lists of commands\n\n"); 
      }     
   }
   printf("main: closing API\n");
   
   status = BusTools_BC_StartStop(cardnum,BC_STOP);
   status = BusTools_BM_StartStop(cardnum,BM_STOP);
   status = BusTools_RT_StartStop(cardnum,RT_STOP);
   
   status = BusTools_API_Close(cardnum);
   printf("BusTools_API_Close status = %d\n",status);

   return 0;
}

int BusController_Init(int cardnum)
{
   int status, messno, i;
   API_BC_MBUF bcmessage;
   BT_U16BIT retry[9];
   BT1553_TIME stime, etime;

   status = BusTools_BC_Init(cardnum,0,BT1553_INT_END_OF_MESS,BC_RETRY_NRSP,16,14,1000000,1);//BT1553_INT_END_OF_MESS
   printf("BusTools_BC_Init status = %d\n",status);

   status = BusTools_BC_MessageAlloc(cardnum,50); //Allocate 50 BC message buffers
   printf("BusTools_BC_MessageAlloc status = %d\n",status); 

   memset(retry,0,sizeof(BT_U16BIT)*9);

   retry[0] = RETRY_ALTERNATE_BUS;
   retry[1] = RETRY_ALTERNATE_BUS;
   retry[2] = RETRY_SAME_BUS;
   retry[3] = RETRY_ALTERNATE_BUS;
   retry[4] = 0;
   retry[5] = 0;
   retry[6] = 0;
   retry[7] = 0;
   retry[8] = 0;
   status = BusTools_BC_RetryInit(cardnum,retry);
   printf("BusTools_BC_RetryInit status = %d\n",status);

   //Message 0 recevie (BC->RT)
   messno = 0;
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;          // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;         // use buffer A
   bcmessage.control |= BC_CONTROL_BUSB;           // Transmit on Bus B
   bcmessage.control |= BC_CONTROL_INTERRUPT;
//   bcmessage.control |= BC_CONTROL_RETRY;
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;
   bcmessage.mess_command1.rtaddr   = 2;
   bcmessage.mess_command1.subaddr  = 2;
   bcmessage.mess_command1.wcount   = 32;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;     // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;    // 8 microsecond inter-message gap.
   for(i=0;i<32;i++)
   {
      bcmessage.data[0][i] = 0x2000 + i;
   }

   BusTools_TimeTagRead(cardnum,&stime);
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   BusTools_TimeTagRead(cardnum,&etime);
   printf("BusTools_BC_MessageWrite status = %d--%d\n",status,(etime.microseconds-stime.microseconds)/1000);


   BusTools_TimeTagRead(cardnum,&stime);
   status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);
   BusTools_TimeTagRead(cardnum,&etime);
   printf("BusTools_BC_MessageRead status = %d--%d\n",status,(etime.microseconds-stime.microseconds)/1000);


   //Message 1 transmit (RT->BC)    
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;          // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;         // use buffer A
   bcmessage.control |= BC_CONTROL_BUSB;        // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_RETRY;
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 3;
   bcmessage.mess_command1.subaddr  = 3;
   bcmessage.mess_command1.wcount   = 3;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;     // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;    // 8 microsecond inter-message gap.
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   // Message 2 recevie (BC->RT)
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;          // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;         // use buffer A
   bcmessage.control |= BC_CONTROL_BUSB;            // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_RETRY;
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 4;
   bcmessage.mess_command1.wcount   = 4;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;     // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;    // 8 microsecond inter-message gap.
   for(i=0;i<4;i++)
   {
      bcmessage.data[0][i] = 0x4000 + i;
   }
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);
   

   // Message 3 transmit (RT->BC) 
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSB;   // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_RETRY;
   bcmessage.mess_command1.rtaddr   = 7;
   bcmessage.mess_command1.subaddr  = 5;
   bcmessage.mess_command1.wcount   = 5;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;     // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;    // 8 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(0);
   bcmessage.control = BC_CONTROL_NOP;
   bcmessage.control |= BC_CONTROL_MFRAME_END;
   
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   printf("calling BusTools_BC_StartStop\n");
   status = BusTools_BC_StartStop(cardnum,BC_START);  //Start the BC Running
   printf("BusTools_BC_StartStop status = %d\n",status);

   return status;
}

int BusMonitor_Init(int cardnum)
{
   int status;
   BT_UINT actual;
   
   status = BusTools_BM_Init(cardnum,1,1);   
   printf("BusTools_BM_Init status = %d\n",status);
   
   status = BusTools_BM_MessageAlloc(cardnum,250, &actual,BT1553_INT_END_OF_MESS);  //BT1553_INT_END_OF_MESS
   printf("BusTools_BM_MessageAlloc status = %d - actual = %d\n",status,actual);

   status = BusTools_BM_StartStop(cardnum,BM_START);
   printf("BusTools_BM_StartStop status = %d\n",status);
   return status;
}

int RemoteTerminal_Init(int cardnum)
{
   int RT_MBUF_COUNT = 4; 
   int status;
   API_RT_CBUF cbuf;       // RT Control Buffer Legal Word Count Mask (2 words)
   API_RT_MBUF_WRITE mbuf;
   API_RT_ABUF abuf;

   int tr;
   int subaddress;
   int i,j,ii,k;
   int rt_addr[] = {2,3,4,5};

   status = BusTools_RT_Init(cardnum,0);
   printf("BusTools_RT_Init status = %d\n",status);

   /**********************************************************************
   *  Output RT control buffers
   *   Enable or disable all word counts and mode codes as specified by
   *    the caller's arguements.
   **********************************************************************/

   cbuf.legal_wordcount = 0xffffffff;  // legailize all word counts
   
   for ( i = 0; i < 4; i++)
   {
      for ( tr = 0; tr < 2; tr++ )
      {
         for ( subaddress = 0; subaddress < 32; subaddress++ )
         {
            status = BusTools_RT_CbufWrite(cardnum,rt_addr[i],subaddress,tr,
			                            RT_MBUF_COUNT,&cbuf);
         }
      }
   }

   /*******************************************************************
   *  Fill in message buffers for receive and transmit sub-units.
   *******************************************************************/
   mbuf.enable = BT1553_INT_END_OF_MESS;   // Interrupt enables on end of message
   mbuf.error_inj_id = 0;

   // For all RT...
   for (ii = 0; ii<4; ii++)
   {
      for (i=0; i<32; i++)
      {
         // For each data buffer..
         for (k=0; k<RT_MBUF_COUNT; k++)
         {
            // Create data buffer data
            for (j=0; j<32; j++)
            {
               mbuf.mess_data[j] =(BT_U16BIT)((rt_addr[ii]<<12) | (k+1)<<8 | i);
               //mbuf.mess_data[j] = 0xf000 + j;
            }
            status = BusTools_RT_MessageWrite(cardnum,rt_addr[ii],i,0,k,&mbuf);
            status = BusTools_RT_MessageWrite(cardnum,rt_addr[ii],i,1,k,&mbuf);
         }
      }
   }

   /************************************************
   *  Setup RT address control block
   ************************************************/
   abuf.enable_a          = 1;    // Enable Bus A...Note that the API bits are
   abuf.enable_b          = 1;    // Enable Bus B... reversed from the HW bits.
   abuf.inhibit_term_flag = 0;
   abuf.status            = 0x0000;
   abuf.command           = 0;
   abuf.bit_word          = 0;

   for(i=0;i<4;i++)
   {
      status = BusTools_RT_AbufWrite(cardnum,rt_addr[i],&abuf);
      printf("BusTools_RT_AbufWrite status = %d\n",status);
   }

   status = BusTools_RT_StartStop(cardnum,RT_START);
   printf("BusTools_RT_StartStop status = %d\n",status);

   return 0;
}


/****************************************************************************
*
*  Function:  rt_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT _stdcall rt_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{
   int status;
   API_RT_MBUF_READ rt_mbuf;
   BT_INT tail;
   BT_UINT messno;
   BT_UINT rt_addr,sub_addr,trans;
   int wcount;

#ifndef WIN32
   ioTaskStdSet(0,1,iod);
#endif

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      messno   = sIntFIFO->fifo[tail].bufferID;
      rt_addr  = sIntFIFO->fifo[tail].rtaddress;        // RT address
      trans    = sIntFIFO->fifo[tail].transrec;         // Transmit/Receive
      sub_addr = sIntFIFO->fifo[tail].subaddress;       // Subaddress number
          
      status = BusTools_RT_MessageRead(cardnum, rt_addr, sub_addr, trans, messno, &rt_mbuf);
      wcount = rt_mbuf.mess_command.wcount;

      printf("  %d--RT-%d  \n",cardnum,rt_mbuf.mess_command.rtaddr);
      printf("  %d--SA-%d  \n",cardnum,rt_mbuf.mess_command.subaddr);
      printf("  %d--WC-%d  \n",cardnum,rt_mbuf.mess_command.wcount);
      printf("  %d--mstat-0x%04x\n",cardnum,*((WORD*)(&rt_mbuf.mess_status)));
      printf("  %d--int_stat-0x%08x\n",cardnum,rt_mbuf.status);

 /*    if(trans)  //update the transmit data
     {
         mbuf_w.enable = BT1553_INT_END_OF_MESS;   // Interrupt enables on end of message
         mbuf_w.error_inj_id = 0;
         for ( i = 0; i < wcount; i++ )            // Update the data words          
         {
            mbuf_w.mess_data[i] = rt_mbuf.mess_data[i]+2;
         }
         BusTools_RT_MessageWrite(cardnum, rt_addr, sub_addr, trans, messno, &mbuf_w);
     }*/

     tail++;
     tail &= sIntFIFO->mask_index;
     sIntFIFO->tail_index = tail;
   }
   return 0;
}

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     an BC message interrupt.  This is interrupt function used with
*     BusTools_RegisterFunction
*
****************************************************************************/
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   int       bufnum;
   int       i,j;
   int       status;
   int       wcount;
   BT_INT tail, rtrt,mc;
   BT_UINT messno;
   BT_U16BIT bc_data[32];
   char outbuf[100];
   
#ifndef WIN32
   ioTaskStdSet(0,1,iod);
#endif

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {   
       messno = sIntFIFO->fifo[tail].bufferID;            
       status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);
       if (status)
          return status;
       
       rtrt=0;
       mc=0;
       
       BusTools_TimeGetString(&bcmessage.time_tag,outbuf);
       printf("BC Message Time - %s\n",outbuf);

       if(bcmessage.status & BT1553_INT_RETRY)
       {
          printf("********** RETRY ON MESSAGE (%d) \n", messno);
       }

       if(bcmessage.status & BT1553_INT_RT_RT_FORMAT)
       {
          printf("RT<->RT (%d) \n",messno);
          rtrt=1;
       }
       else if(bcmessage.status & BT1553_INT_MODE_CODE)
       {
          printf("Mode Code --  (%d) \n", messno);
          mc=1;
       }
          
       else if (bcmessage.mess_command1.tran_rec)
          printf("BC**RT->BC (%d-%d)  \n", messno,bcmessage.messno_next);
       else
          printf("BC->RT (%d-%d)  \n", messno,bcmessage.messno_next);
 
       printf("BC**RT-%d  \n",bcmessage.mess_command1.rtaddr);
       printf("BC**SA-%d  \n",bcmessage.mess_command1.subaddr);
       printf("BC**WC-%d  \n",bcmessage.mess_command1.wcount);
       if(rtrt)
       {   
          printf("BC**RT-%d  \n",bcmessage.mess_command2.rtaddr);
          printf("BC**SA-%d  \n",bcmessage.mess_command2.subaddr);
          printf("BC**WC-%d  \n",bcmessage.mess_command2.wcount);
       }
        
       printf("BC**status-1 0x%04x\n",*((WORD*)(&bcmessage.mess_status1)));
       if(rtrt)
          printf("BC**status-2 0x%04x\n",*((WORD*)(&bcmessage.mess_status2)));
          
          printf("BC**int_stat-0x%08lx",*((DWORD*)(&bcmessage.status)));

       wcount = bcmessage.mess_command1.wcount;
       if (wcount == 0)
          wcount = 32;           // Zero word count means 32 words.

       if(mc)
         if(wcount >=16)
            wcount = 1;

       j = wcount;               // See if this msg shorted than previous
    
       if (bcmessage.control & BC_CONTROL_BUFFERA)
          bufnum = 0;            // Display data from first BC buffer
       else
          bufnum = 1;            // Display data from second BC buffer

       for ( i = 0; i < j; i++ ) // Display all of the data words
       {
          if(i%8==0)
             printf("\n");
          // If this is an RT->BC message, and there is a Message Error
          //  or No Response, than xxxx the data (since the RT did not
          //    send any data words).
          if ( i >= wcount )
          {
             printf("              ");     // Data does not apply to this msg
          }
          else if ( bcmessage.mess_command1.tran_rec &&
            (bcmessage.mess_status1.me || (bcmessage.status&BT1553_INT_NO_RESP) ) )
          {
             printf("xxxx ");      // No data was transfered because of error
          }
          else
          {
             printf("BC**%04x ",bcmessage.data[bufnum][i]);
          }
          if(bcmessage.mess_command1.rtaddr==2 && bcmessage.mess_command1.subaddr==2)
          {
             bc_data[i] = bcmessage.data[bufnum][i]+2;
          }
          if(bcmessage.mess_command1.rtaddr==2 && bcmessage.mess_command1.subaddr==2)
          {
             status = BusTools_BC_MessageUpdateBuffer(cardnum,messno,0,bc_data);
             if(status)
            	 printf("XXXXXXXXXXXXXXXXXXX %d\n",status);
          }
       }
       printf("\n\n");
         
      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }
   return 0;
}

/****************************************************************************
*
*  Function:  bm_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bm_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BM_MBUF mbuf;
   int       status;
   BT_INT tail,rtrt;
   BT_UINT messno;
   char      outbuf[100];

#ifndef WIN32
   ioTaskStdSet(0,1,iod);
#endif

   rtrt = 0;
   tail = sIntFIFO->tail_index;

   while(tail != sIntFIFO->head_index)
   {
      messno = sIntFIFO->fifo[tail].bufferID;
      status = BusTools_BM_MessageRead(cardnum,messno,&mbuf);// read the BM message data
      if(mbuf.int_status & BT1553_INT_RT_RT_FORMAT)
      {
          printf("RT<->RT (%d) \n",mbuf.messno);
          rtrt=1;
      }

	  BusTools_TimeGetString(&mbuf.time, outbuf);
      printf("BM>>>>> time tag = %s\n",outbuf);

      if (mbuf.command1.tran_rec)
         printf("BM>>>>> RT->BC (%d)  \n", mbuf.messno);
      else
         printf("BM>>>>> BC->RT (%d)  \n", mbuf.messno);
		
      printf("BM>>>>> RT-%d  \n",mbuf.command1.rtaddr);
      printf("BM>>>>> SA-%d  \n",mbuf.command1.subaddr);
      printf("BM>>>>> WC%d  \n",mbuf.command1.wcount);
      printf("BM>>>>> 0x%08x\n",mbuf.int_status);
                     
      tail++;
	  tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }
   printf("\n");
   return 0;
}

void setupThe_BC_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;

   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.function     = bc_intFunction;
   sIntFIFO1.iPriority    = THREAD_PRIORITY_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.iNotification  = 0;
   sIntFIFO1.bForceStartup  = 0;
   sIntFIFO1.FilterType     = EVENT_BC_MESSAGE;  

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO1.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction BC status = %d\n",status);
}

void setupThe_BM_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;

   // Setup the FIFO structure for this board.
   //memset(&sIntFIFO3, 0, sizeof(API_INT_FIFO));
   sIntFIFO3.function     = bm_intFunction;
   sIntFIFO3.iPriority    = THREAD_PRIORITY_NORMAL;
   sIntFIFO3.dwMilliseconds = INFINITE;
   sIntFIFO3.iNotification  = 0;
   sIntFIFO3.bForceStartup  = 0;
   sIntFIFO3.FilterType     = EVENT_BM_MESSAGE;  

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO3.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO3, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction BC status = %d\n",status);
}

/****************************************************************************
*
*  Function:  setupThe_RT_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a RT message.  The interrupt is set for only the end-of
*     message.  
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupThe_RT_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO2, 0, sizeof(sIntFIFO2));
   sIntFIFO2.function     = rt_intFunction;
   sIntFIFO2.iPriority    = THREAD_PRIORITY_NORMAL;
   sIntFIFO2.dwMilliseconds = INFINITE;
   sIntFIFO2.iNotification  = 0;
   sIntFIFO2.FilterType     = EVENT_RT_MESSAGE;

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO2.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos
  
   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO2, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction RT status = %d\n",status);
}

/****************************************************************************
*
*  Function:  bm_readFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT bm_readFunction(BT_UINT cardnum, BT_UINT mcount, API_BM_MBUF * mbuf) // struct api_int_fifo *sIntFIFO)
{   
   BT_UINT i;
   BT_U16BIT cword;

#ifndef WIN32
   ioTaskStdSet(0,1,iod);
#endif

   for(i=0; i<mcount; i++)
   {
      if (mbuf[i].command1.tran_rec)
         printf("BM>>>>> RT->BC (%d)  \n", mbuf[i].messno);
      else
         printf("BM>>>>> BC->RT (%d)  \n", mbuf[i].messno);
		
      printf("BM>>>>> RT-%d  \n",mbuf[i].command1.rtaddr);
      printf("BM>>>>> SA-%d  \n",mbuf[i].command1.subaddr);
      printf("BM>>>>> WC%d  \n",mbuf[i].command1.wcount);
      printf("BM>>>>> 0x%08x\n",mbuf[i].int_status);
      if(mbuf[i].int_status & BT1553_INT_BUSB)
         printf("BUS B\n");
      else
         printf("BUS A\n");
      
      if(mbuf[i].command1.rtaddr == 2)
      {
         BusTools_BC_ControlWordRead(cardnum,0,&cword);
    	 if(cword & BC_CONTROL_CHANNELB) // toggle the bus
            cword = BC_CONTROL_MESSAGE | BC_CONTROL_BUFFERA | BC_CONTROL_BUSA | BC_CONTROL_INTERRUPT | BC_CONTROL_RETRY;
         else
            cword = BC_CONTROL_MESSAGE | BC_CONTROL_BUFFERA | BC_CONTROL_BUSB | BC_CONTROL_INTERRUPT | BC_CONTROL_RETRY;
    	 BusTools_BC_ControlWordUpdate(cardnum,0, cword, 0);
      }
   }
   printf("\n\n");
   return 0;
}

