/*===========================================================================*
 * FILE:                    f o u r _ c h - i n t . C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    Demonstration of BusTools/1553-API routines. These routines
 *              show how to initialize the Bus Conroller, Bus Monitor and
 *              Remote Terminal.  The also show how to access data using the
 *              interrupt queue to get the messages transacted on the bus.
 *
 * Notes:       To compile for the VME-1553 use the BusTools_API_InitExtended
 *              call for that board.  To compile for the PCI-1553, cPCI-1553, 
 *              or PMC-1553 use the BusTools_API_InitExtended for those cards.
 *              When compiling define _UNIX_ and VXWORKS and do not use the 
 *              -ansi directive.  
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
  1/27/2003    Update form improved examples
  6/20/2003    Add interrupt example
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include "busapi.h"

BT_INT _stdcall rt1_intFunction(BT_UINT, struct api_int_fifo *);
BT_INT _stdcall rt2_intFunction(BT_UINT, struct api_int_fifo *);
BT_INT _stdcall rt3_intFunction(BT_UINT, struct api_int_fifo *);
BT_INT _stdcall bc_intFunction(BT_UINT, struct api_int_fifo *);

API_INT_FIFO  sIntFIFO_BC;      // Thread FIFO structure for BC
API_INT_FIFO  sIntFIFO_RT1;     // Thread FIFO structure for RT
API_INT_FIFO  sIntFIFO_RT2;     // Thread FIFO structure for RT
API_INT_FIFO  sIntFIFO_RT3;     // Thread FIFO structure for RT

void setupThe_BC_IntFIFO(int);
void setupThe_RT1_IntFIFO(int);
void setupThe_RT2_IntFIFO(int);
void setupThe_RT3_IntFIFO(int);

int BusController_Init(int);
int RemoteTerminal_Init(int,int);
void user_int_process(int card);

void bc_interrupt(BT_UINT cardnum);
void rt1_interrupt(BT_UINT cardnum);
void rt2_interrupt(BT_UINT cardnum);
void rt2_interrupt(BT_UINT cardnum);

int total_bc_msgs;
int total_rt1_msgs;
int total_rt2_msgs;
int total_rt3_msgs;

void int4()
{
   int  status;
   UINT pwFlag;
   char ichar;

   BT_UINT cardnum,cardnumRT1,cardnumRT2,cardnumRT3;

   pwFlag = API_B_MODE | API_HW_ONLY_INT;

   status = BusTools_API_OpenChannel(&cardnum,pwFlag,0,CHANNEL_1);
   printf("BusTools_API_OpenChannel for cardnumBC(%d) = %d\n",cardnum,status);

   status = BusTools_API_OpenChannel(&cardnumRT1,pwFlag,0,CHANNEL_2);
   printf("BusTools_API_OpenChannel for cardnumRT1(%d) = %d\n",cardnumRT1,status);

   status = BusTools_API_OpenChannel(&cardnumRT2,pwFlag,0,CHANNEL_3);
   printf("BusTools_API_OpenChannel for cardnumRT2(%d) = %d\n",cardnumRT2,status);

   status = BusTools_API_OpenChannel(&cardnumRT3,pwFlag,0,CHANNEL_4);
   printf("BusTools_API_OpenChannel for cardnumRT3(%d) = %d\n",cardnumRT3,status);

   {
      unsigned lpurev,apirev;
      status = BusTools_GetRevision(cardnum,&lpurev,&apirev);
      printf("LPU Revison = %4.0x -- API Revision = %d\n",lpurev,apirev);
   }
   
   RemoteTerminal_Init(cardnumRT1,1);
   RemoteTerminal_Init(cardnumRT2,2);
   RemoteTerminal_Init(cardnumRT3,3);
   BusController_Init(cardnum);
 
   setupThe_BC_IntFIFO(cardnum); //Set up BC interrupts using BusTools_RegisterFunction   
   setupThe_RT1_IntFIFO(cardnumRT1);
   setupThe_RT2_IntFIFO(cardnumRT2);
   setupThe_RT3_IntFIFO(cardnumRT3);

   printf("Hit the entr key to start interrupts running\n");
   printf("Then type Q or q to stop\n");
   getchar();

   status = BusTools_RT_StartStop(cardnumRT1,RT_START);
   printf("BusTools_RT_StartStop 1 status = %d\n",status);

   status = BusTools_RT_StartStop(cardnumRT2,RT_START);
   printf("BusTools_RT_StartStop 2 status = %d\n",status);

   status = BusTools_RT_StartStop(cardnumRT3,RT_START);
   printf("BusTools_RT_StartStop 3 status = %d\n",status);

   status = BusTools_BC_StartStop(cardnum,BC_START);  //Start the BC Running
   printf("BusTools_BC_StartStop status = %d\n",status);

   do{
      ichar = getchar();
      switch(ichar)
      {
      case 1:
         BusTools_RT_StartStop(cardnumRT1, RT_STOP);
         BusTools_API_Close(cardnumRT1);
         break;
      case 2:
         BusTools_RT_StartStop(cardnumRT2, RT_STOP);
         BusTools_API_Close(cardnumRT2);
         break;
      case 3:
         BusTools_RT_StartStop(cardnumRT3, RT_STOP);
         BusTools_API_Close(cardnumRT3);
         break;
      }

   }while (ichar != 'Q' && ichar != 'q');

   printf("main: closing API\n");

   status = BusTools_RegisterFunction(cardnum, &sIntFIFO_BC, UNREGISTER_FUNCTION);
   printf("BusTools_RegisterFunction Close status = %d\n",status);

   status = BusTools_RegisterFunction(cardnumRT1, &sIntFIFO_RT1, UNREGISTER_FUNCTION);
   printf("BusTools_RegisterFunction Close status = %d\n",status);

   status = BusTools_RegisterFunction(cardnumRT2, &sIntFIFO_RT2, UNREGISTER_FUNCTION);
   printf("BusTools_RegisterFunction Close status = %d\n",status);

   status = BusTools_RegisterFunction(cardnumRT3, &sIntFIFO_RT3, UNREGISTER_FUNCTION);
   printf("BusTools_RegisterFunction Close status = %d\n",status);

   status = BusTools_API_Close(cardnum);
   printf("BusTools_API_Close status = %d\n",status);

   status = BusTools_API_Close(cardnumRT1);
   printf("BusTools_API_Close status = %d\n",status);

   status = BusTools_API_Close(cardnumRT2);
   printf("BusTools_API_Close status = %d\n",status);

   status = BusTools_API_Close(cardnumRT3);
   printf("BusTools_API_Close status = %d\n",status);

   printf("Hit the entr key to stop the test\n");
   getchar();
}

int BusController_Init(int cardnum)
{
   int status, messno;
   API_BC_MBUF bcmessage;

   status = BusTools_BC_Init(cardnum,0,BT1553_INT_END_OF_MESS,0,16,14,1000000,1);//BT1553_INT_END_OF_MESS
   printf("BusTools_BC_Init status = %d\n",status);

   status = BusTools_BC_MessageAlloc(cardnum,10); //Allocate 10 BC message buffers
   printf("BusTools_BC_MessageAlloc status = %d\n",status);

   status = BusTools_SetInternalBus(cardnum,EXTERNAL_BUS);
   printf("BusTools_SetInternalBus status = %d\n",status);

   //Message 0 recevie (BC->RT)
   messno = 0;
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;          // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;         // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;            // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;
   bcmessage.mess_command1.rtaddr   = 1;
   bcmessage.mess_command1.subaddr  = 1;
   bcmessage.mess_command1.wcount   = 1;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                           // Default error injection buffer (no errors)
   bcmessage.gap_time = 10;                         // 10 microsecond inter-message gap.
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status);

   //Message 1 transmit (RT->BC)    
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;          // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;         // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;            // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 2;
   bcmessage.mess_command1.subaddr  = 2;
   bcmessage.mess_command1.wcount   = 2;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                           // Default error injection buffer (no errors)
   bcmessage.gap_time = 10;                         // 10 microsecond inter-message gap. 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status);

   //Message 1 transmit (RT->BC)    
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;          // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;         // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;            // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 2;
   bcmessage.mess_command1.subaddr  = 2;
   bcmessage.mess_command1.wcount   = 2;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                           // Default error injection buffer (no errors)
   bcmessage.gap_time = 10;                         // 10 microsecond inter-message gap. 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status);

   // Message 4 receive (BC->RT)
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;          // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;         // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;            // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 3;
   bcmessage.mess_command1.subaddr  = 3;
   bcmessage.mess_command1.wcount   = 3;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                          // Default error injection buffer (no errors)
   bcmessage.gap_time = 10;                        // 10 microsecond inter-message gap. 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite %d status = %d\n",messno,status);

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = 0;
   bcmessage.control = BC_CONTROL_NOP; 
   bcmessage.control |= BC_CONTROL_MFRAME_END; 

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   return 0;
}

int RemoteTerminal_Init(int cardnum, int rt_addr)
{
   int RT_MBUF_COUNT = 1; 
   int status;
   API_RT_CBUF cbuf;       // RT Control Buffer Legal Word Count Mask (2 words)
   API_RT_MBUF_WRITE mbuf;
   API_RT_ABUF abuf;
   int         tr;
   int         subaddress;
   int i,j,k;

   printf("Initializing RT %d\n",rt_addr);
   status = BusTools_RT_Init(cardnum,0);
   printf("BusTools_RT_Init status = %d\n",status);

   /************************************************
   *  Setup RT address control block
   ************************************************/
   abuf.enable_a          = 1;    // Enable Bus A...Note that the API bits are
   abuf.enable_b          = 1;    // Enable Bus B... reversed from the HW bits.
   abuf.inhibit_term_flag = 0;
   abuf.status            = 0x0000;
   abuf.command           = 0;
   abuf.bit_word          = 0;

   status = BusTools_RT_AbufWrite(cardnum,rt_addr,&abuf);
   printf("BusTools_RT_AbufWrite status = %d\n",status);


   /**********************************************************************
   *  Output RT control buffers
   *   Enable or disable all word counts and mode codes as specified by
   *    the caller's arguements.
   **********************************************************************/

   cbuf.legal_wordcount = 0xffffffff;  // legailize all word counts
   for ( tr = 0; tr < 2; tr++ )
   {
      for ( subaddress = 0; subaddress < 32; subaddress++ )
      {
         status = BusTools_RT_CbufWrite(cardnum,rt_addr,subaddress,0,RT_MBUF_COUNT,&cbuf);
         status = BusTools_RT_CbufWrite(cardnum,rt_addr,subaddress,1,RT_MBUF_COUNT,&cbuf);
      }
   }
   printf("BusTools_RT_CbufWrite status = %d\n",status);

   /*******************************************************************
   *  Fill in message buffers for receive and transmit sub-units.
   *******************************************************************/
   mbuf.enable = BT1553_INT_END_OF_MESS;   // Interrupt enables on end of message
   mbuf.error_inj_id = 0;

   // For all subaddresses...
   for (i=0; i<32; i++)
   {
      // For each data buffer..
      for (k=0; k<RT_MBUF_COUNT; k++)
      {
         // Create data buffer data
         for (j=0; j<32; j++)
         {
            mbuf.mess_data[j] = (rt_addr<<12) + j;
         }
         status = BusTools_RT_MessageWrite(cardnum,rt_addr,i,0,k,&mbuf);
         status = BusTools_RT_MessageWrite(cardnum,rt_addr,i,1,k,&mbuf);
      }
   }

   status = BusTools_SetInternalBus(cardnum,EXTERNAL_BUS);
   printf("BusTools_SetInternalBus status = %d\n",status);

   return 0;  
}

/*******************************************************************/
BT_INT _stdcall rt1_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{
   BT_INT tail, wcount,i;
   BT_UINT messno,rt_addr,sub_addr,trans;
   int status;
   API_RT_MBUF_READ  rt_rbuf;
   API_RT_MBUF_WRITE rt_wbuf;

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      messno   = sIntFIFO->fifo[tail].bufferID;
      rt_addr  = sIntFIFO->fifo[tail].rtaddress;        // RT address
      trans    = sIntFIFO->fifo[tail].transrec;         // Transmit/Receive
      sub_addr = sIntFIFO->fifo[tail].subaddress;       // Subaddress number

      status = BusTools_RT_MessageRead( cardnum, rt_addr, sub_addr, trans, messno, &rt_rbuf);
      wcount = rt_rbuf.mess_command.wcount; 
  
      for(i=0;i<wcount;i++)
         rt_wbuf.mess_data[i]  = rt_rbuf.mess_data[i]+=1;

      rt_wbuf.error_inj_id = 0;
      rt_wbuf.enable = BT1553_INT_END_OF_MESS;   // Interrupt enables
      status = BusTools_RT_MessageWrite(cardnum,rt_addr,sub_addr,trans,messno,&rt_wbuf);

      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }
   return 0;
}

/*******************************************************************/
BT_INT _stdcall rt2_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{
   BT_INT tail, wcount,i;
   BT_UINT messno,rt_addr,sub_addr,trans;
   int status;
   API_RT_MBUF_READ  rt_rbuf;
   API_RT_MBUF_WRITE rt_wbuf;

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      messno   = sIntFIFO->fifo[tail].bufferID;
      rt_addr  = sIntFIFO->fifo[tail].rtaddress;        // RT address
      trans    = sIntFIFO->fifo[tail].transrec;         // Transmit/Receive
      sub_addr = sIntFIFO->fifo[tail].subaddress;       // Subaddress number

      status = BusTools_RT_MessageRead( cardnum, rt_addr, sub_addr, trans, messno, &rt_rbuf);

      wcount = rt_rbuf.mess_command.wcount;   
      for(i=0;i<wcount;i++)
         rt_wbuf.mess_data[i]  = rt_rbuf.mess_data[i]+=2;

      rt_wbuf.error_inj_id = 0;
      rt_wbuf.enable = BT1553_INT_END_OF_MESS;   // Interrupt enables
      status = BusTools_RT_MessageWrite(cardnum,rt_addr,sub_addr,trans,messno,&rt_wbuf);

      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }
   return 0;

}

/*******************************************************************/
BT_INT _stdcall rt3_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{
   BT_INT tail, wcount,i;
   BT_UINT messno,rt_addr,sub_addr,trans;
   int status;
   API_RT_MBUF_READ  rt_rbuf;
   API_RT_MBUF_WRITE rt_wbuf;

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      messno   = sIntFIFO->fifo[tail].bufferID;
      rt_addr  = sIntFIFO->fifo[tail].rtaddress;        // RT address
      trans    = sIntFIFO->fifo[tail].transrec;         // Transmit/Receive
      sub_addr = sIntFIFO->fifo[tail].subaddress;       // Subaddress number

      status = BusTools_RT_MessageRead( cardnum, rt_addr, sub_addr, trans, messno, &rt_rbuf);
      wcount = rt_rbuf.mess_command.wcount;   

      for(i=0;i<wcount;i++)
         rt_wbuf.mess_data[i]  = rt_rbuf.mess_data[i]+=3;

      rt_wbuf.error_inj_id = 0;
      rt_wbuf.enable = BT1553_INT_END_OF_MESS;   // Interrupt enables
      status = BusTools_RT_MessageWrite(cardnum,rt_addr,sub_addr,trans,messno,&rt_wbuf);

      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }
   return 0;
}

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   int       bufnum;
   static int sa_index;
   int       i,j;
   int       status;
   int       wcount;
   BT_INT tail, rtrt,mc;
   BT_UINT messno;
   BT1553_TIME rtime;

   char outbuf[80];

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      messno = sIntFIFO->fifo[tail].bufferID;
      status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);         
      if (status)
         return status;

       BusTools_TimeGetFmtString(API_TTD_RELM,&bcmessage.time_tag,outbuf);
       printf("Tag Time = %s\n",outbuf);          
 
       rtrt=0;
       mc=0;

       if(bcmessage.status & BT1553_INT_RETRY)
         printf("Retry - %d - %x\n",messno,bcmessage.status);
       else if (bcmessage.status & BT1553_INT_BROADCAST)
         printf("Broadcast - %d - %x\n",messno,bcmessage.status);
       else
         printf("%d - %x\n",messno,bcmessage.status);

       if(bcmessage.status & BT1553_INT_RT_RT_FORMAT)
       {
         printf("RT<->RT (%d) ",messno);
          rtrt=1;
       }
       else if(bcmessage.status & BT1553_INT_MODE_CODE)
       {
          printf("Mode Code --  (%d) ", messno);
          mc=1;
       }
       else if (bcmessage.mess_command1.tran_rec)
          printf("BC**RT->BC (%d)  ", messno);
       else
          printf("BC->RT (%d)  ", messno);
 
       printf("BC**RT-%d  \n",bcmessage.mess_command1.rtaddr);
       printf("BC**SA-%d  \n",bcmessage.mess_command1.subaddr);
       printf("BC**WC-%d  \n",bcmessage.mess_command1.wcount);
       if(rtrt)
       {   
          printf("BC**RT-%d  \n",bcmessage.mess_command2.rtaddr);
          printf("BC**SA-%d  \n",bcmessage.mess_command2.subaddr);
          printf("BC**WC-%d  \n",bcmessage.mess_command2.wcount);
       }
        
       printf("BC**status-1 0x%04x\n",*((WORD*)(&bcmessage.mess_status1)));
       if(rtrt)
          printf("BC**status-2 0x%04x\n",*((WORD*)(&bcmessage.mess_status2)));
          
       printf("BC**int_stat-0x%08lx\n",*((DWORD*)(&bcmessage.status)));
       if(bcmessage.status & BT1553_INT_NO_RESP)
          printf("XXXXXXXXXXXXXXXXXX NO RESPONSE %x!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n",bcmessage.mess_status1);

       wcount = bcmessage.mess_command1.wcount;
       if (wcount == 0)
          wcount = 32;           // Zero word count means 32 words.

       if(mc)
         if(wcount >=16)
            wcount = 1;

       j = wcount;               // See if this msg shorted than previous
    
       if (bcmessage.control & BC_CONTROL_BUFFERA)
          bufnum = 0;            // Display data from first BC buffer
       else
          bufnum = 1;            // Display data from second BC buffer

       for ( i = 0; i < j; i++ ) // Display all of the data words
       {
          // If this is an RT->BC message, and there is a Message Error
          //  or No Response, than xxxx the data (since the RT did not
          //    send any data words).
          if ( i >= wcount )
          {
             printf("    ");     // Data does not apply to this msg
          }
          else if ( bcmessage.mess_command1.tran_rec &&
            (bcmessage.mess_status1.me || (bcmessage.status&BT1553_INT_NO_RESP) ) )
          {
             printf("xxxx ");      // No data was transfered because of error
          }
          else
          {
             printf("BC**%04x ",bcmessage.data[bufnum][i]);
          }
       }
       printf("\n_____________________________________________________________\n");
       if(bcmessage.mess_command1.tran_rec==0)
       {
          for(i=0;i<wcount;i++)
          {
             bcmessage.data[bufnum][i]+=1;
          }
          BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
       }
   
       tail++;
       tail &= sIntFIFO->mask_index;
       sIntFIFO->tail_index = tail;
   }

   return 0;
}

void setupThe_BC_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;

   // Setup the FIFO structure for this board.
   memset(&sIntFIFO_BC, 0, sizeof(sIntFIFO_BC));
   sIntFIFO_BC.function       = bc_intFunction;
   sIntFIFO_BC.iPriority      = THREAD_PRIORITY_CRITICAL;
   sIntFIFO_BC.dwMilliseconds = INFINITE;
   sIntFIFO_BC.FilterType     = EVENT_BC_MESSAGE;

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO_BC.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO_BC, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction BC status = %d\n",status);
}


void setupThe_RT1_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO_RT1, 0, sizeof(sIntFIFO_RT1));
   sIntFIFO_RT1.function     = rt1_intFunction;
   sIntFIFO_RT1.iPriority    = THREAD_PRIORITY_CRITICAL;
   sIntFIFO_RT1.dwMilliseconds = INFINITE;
   sIntFIFO_RT1.iNotification  = 0;
   sIntFIFO_RT1.FilterType     = EVENT_RT_MESSAGE;

   sIntFIFO_RT1.FilterMask[1][1][1] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos
  
   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO_RT1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction RT1 status = %d\n",status);
}

void setupThe_RT2_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO_RT2, 0, sizeof(sIntFIFO_RT2));
   sIntFIFO_RT2.function     = rt2_intFunction;
   sIntFIFO_RT2.iPriority    = THREAD_PRIORITY_CRITICAL;
   sIntFIFO_RT2.dwMilliseconds = INFINITE;
   sIntFIFO_RT2.iNotification  = 0;
   sIntFIFO_RT2.FilterType     = EVENT_RT_MESSAGE;

   sIntFIFO_RT2.FilterMask[2][1][2] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos
  
   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO_RT2, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction RT2 status = %d\n",status);
}
void setupThe_RT3_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO_RT3, 0, sizeof(sIntFIFO_RT3));
   sIntFIFO_RT3.function     = rt3_intFunction;
   sIntFIFO_RT3.iPriority    = THREAD_PRIORITY_NORMAL;
   sIntFIFO_RT3.dwMilliseconds = INFINITE;
   sIntFIFO_RT3.iNotification  = 0;
   sIntFIFO_RT3.FilterType     = EVENT_RT_MESSAGE;

   sIntFIFO_RT3.FilterMask[3][1][3] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos
  
   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO_RT3, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction RT3 status = %d\n",status);
}
