/*============================================================================*
 * FILE:                    B T - W E P . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *
 *===========================================================================*
 *
 * FUNCTION:   BusTools/1553-API Library:
 *             Windows-specific DLL functions.  This module contains the
 *             functions which are unique to the Microsoft Windows programming
 *             environment, specifically the functions which are required by
 *             Windows DLL's.  These functions are not required by the API,
 *             only by the operating system.
 *
 *             This is a good place to put other system-required functions that
 *             are otherwise not used by the BusTools API.
 *
 * USER ENTRY POINTS: 
 *     none
 *
 * EXTERNAL NON-USER SYSTEM ENTRY POINTS:
 *     LocalPeekMsg       - Does a PeekMesssge() loop to allow PostMessage()'s
 *                          to be processed.
 *     LibMain            - Is called by LibEntry.
 *     WEP(int)           - Performs cleanup when the DLL is unloaded
 *     DllEntryPoint      - is the 32-bit version of LibMain
 *
 * INTERNAL ROUTINES:
 *     none
 *
 *===========================================================================*/

/* $Revision:  4.30 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  01/04/1999   Added the LocalPeekMsg() function.V2.95.ajh
  01/18/2000   Omitted saving handle in 32-bit.V4.00.ajh
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include "busapi.h"
#include "apiint.h"
#include "globals.h"

/*===========================================================================*
 * NON-USER ENTRY POINT:          W E P, DLL ENTRY POINT
 *===========================================================================*
 *
 * FUNCTION:    Microsoft Windows DLL Exit Procedure.
 *
 * DESCRIPTION: Microsoft Windows DLL exit function.  This is a
 *              system level function and is not accessible by the DLL, and
 *              is called to allow the DLL to cleanup before being unloaded.
 *
 *      It will return:
 *              1
 *===========================================================================*/

/*===========================================================================*
 * ENTRY POINT:      D L L E N T R Y P O I N T
 *===========================================================================*
 *
 * FUNCTION:    DLL Entry Point Procedure
 *
 * DESCRIPTION: Needed for 32 bit DLL under WIN32.
 *
 * It will return:  BOOL
 *
 *===========================================================================*/

 BOOL WINAPI DllEntryPoint ( HANDLE    hDLL,
                            DWORD     dwReason,
                            LPVOID    lpReserved)
{
   static int nAttached = 0;     // Number of processes/threads attached to us.

   switch (dwReason)
   {
      case DLL_PROCESS_ATTACH:
          nAttached++;
          break;
      case DLL_PROCESS_DETACH:
          nAttached--;
          if ( nAttached <= 0 )
             BusTools_API_Close( 99 );   // Shutdown board access and threads.
          break;
      case DLL_THREAD_ATTACH:
          //nAttached++;
          break;
      case DLL_THREAD_DETACH:
          //nAttached--;
          break;
      default:
          break;
    }
    return TRUE;
}


