/*===========================================================================*
 * FILE:                   B T V X I M A P . C
 *===========================================================================*
 *
*      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:    This stand-alone DLL is a component of the BusTools/15553-API.
 *              The entry point BusTools_API_InitExternal() is called by the
 *              API whenever a board with a platform of "PLATFORM_USER" is to
 *              be opened or closed.
 *              This function is expected to setup whatever mapping is required
 *              by the specified board, and return the pointers to the API.
 *              When the API is closing the board it will also call this routine
 *              to give it a chance to free any allocated resources.
 *
 * Products Supported by this "PLATFORM_USER" BTVXIMAP.DLL:

  
          REQUIRED PARAMETERS

  
      Product: VXI-1553-IPD or VME-1553-IPD6
      ------------------------------------
          Product-name    Carrier-name    Target Device(s)
          ------------    ------------    ---------------- 
          VXI-1553-IPD    VIPC616_618     IP-D1553
          VME-1553-IPD    VIPC616_618     IP-D1553

          REQUIRED PARAMETERS

          VIPC616_618 Carrier, IP-D1553 card type, Slot_A - Slot_D,
          Carrier A32 Memory Address and Carrier A16 I/O Address

     Product: VME-1553-IP3
      --------------------
          Product-name    Carrier-name    Target Device(s)  
          ------------    ------------    ---------------- 
          VME-1553-IPD     VIPC310         IP-D1553-S or -M 

          REQUIRED PARAMETERS

          VIPC310 Carrier, IP-D1553 card type, Slot_A, Slot_B
          Carrier A24 Memory Address and Carrier A16 I/O Address

      Product: VME-1553-IPD or VXI-1553-IPD
      -------------------------------------
          Product-name    Carrier-name    Target Device(s)
          ------------    ------------    ----------------
          VXI-1553-IPD    AVME9660        IP-D1553
          VME-1553-IPD    AVME9660        IP-D1553 
 

        Product: VME-1553 or VXI-1553
      -------------------------------------
          Product-name       Carrier-name 
          ------------       ------------      
		  VME-1553         NATIVE_24/NATIVE_32   
          QVME-155         NATIVE_24/NATIVE_32        
		  VXI-1553         NATIVE_24/NATIVE_32   


 * API ENTRY POINTS: (only used by btdrv.c):
 *    BusTools_API_InitExternal Setup adapter page access data pointers for
 *                              specified BusTools board.
 * Local Routines:
 *    vxiInit()  - Open the NI VXI library
 *    vxiClose() - Close the NI VXI library
 *===========================================================================*/

/* $Revision: 5.43 Release $
   Date       Revision
  ----------  -----------------------------------------------------------------
  07/05/1999  Initial version.V3.20.ajh
  02/03/2000  BDW - Added compiler directive VME_AIC on the function invocation
                    MapVXIAddress() to compile for a VME/VXI-AIC target versus
                    the VIPC-616/618 target.
  02/22/2000  Added builds for VME/VXI-AIC-xx02 (BTVXIAIC.DLL) (VME_AIC) and the
              VXI-1553-IP-xx (VIPC-616/618 or Acromag AVME-9660 carrier) targets
              (BTVXICAR.DLL) (VXI_AIC) to API release build.  Calculated the
              address of the ID PROM for systems that support it rather than
              assuming that the PROM version was 3.V4.01.ajh
  03/12/2000  Changed to support VME/VXI-AIC-xx02/3 et.al. within one DLL
              (BTVXIMAP.DLL) by using the calling args to properly map the
              target environment and the carrier, and the carrier slot.V4.01.ajh
  08/14/2000  Changed the assumed PROM version for the IP-1553 on the MVME162
              carrier from V3 to V5.V4.09.ajh
  03/29/2001  Changed the assumed PROM version for the IP-1553 on the MVME162
              carrier from V5 to V6.V4.33.ajh
  01/30/2002  Added comments for the VME/VXI-1553 native board.
  07/09/2002  Added IP-D1553 support.
  05/03/2004  Change from VXI library to VISA library. 
  11/22/2004  Add function Condor_IP_ID to test for the IP module type
  07/29/2005  Add support for vbtGetCEIDevInfo
 */

/*---------------------------------------------------------------------------*
 *          INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/

#include <NiVisa.h>
#include <stdio.h>
#include "busapi.h"
#include "lowlevel.h"
#include "btdrv.h"

/*---------------------------------------------------------------------------*
 *          INTERNAL FUNCTION PROTOTYPES
 *---------------------------------------------------------------------------*/

static int vxiInit (int device, BT_UINT cardType, BT_UINT CurrentCarrier);
static int vxiClose(int device);

void map_ipd1553(char *lpbase, void *window_2_address,BT_UINT cardType, BT_UINT CurrentCarrier,BT_UINT CurrentSlot,void *PageAddr[4]);


/*---------------------------------------------------------------------------*
 *          INTERNAL MAPPING DATABASE
 *---------------------------------------------------------------------------*/

//*****************************************************************************
// Each physical device (e.g., each carrier) is mapped one time only by this
//  function, beginning at its base memory/IO addresses.  When it is mapped,
//  it's entire memory and I/O space is mapped.
// The returned pointers are adjusted by adding the offsets needed to access
//  the specified IP slot, so the called can use the bt_PageAddr[] pointers
//  directly.
// When the caller wishes to access another IP on a carrier that has already
//  been mapped, these functions simply increment the use count and return
//  adjusted values of the previously mapped regions.
// When closing a mapping region, the use count is decremented and only if it
//  is zero is the mapping window closed.
//*****************************************************************************

// Define the max number of physical devices (carriers) we support
//#define MAX_DEVICES   12

// Per physical device (e.g., carrier):
//  Memory Mapping parameters map the entire carrier [all slots]
typedef struct _MAP_DATA_STRUCT {
    int      DeviceMapOpen;    // Physical device Memory Map open count.
    unsigned base_addr_1;      // Physical nemory space base address
    unsigned base_addr_2;      // Physical I/O space base address
    unsigned mem_size_1;       // Physical memory space region size
    unsigned mem_size_2;       // Physical I/O space region size
    void    *mapped_addr_1;    // Mapped memory pointer
    void    *mapped_addr_2;    // Mapped I/O pointer
    int      CurrentCarrier;   // Type of carrier being mapped
    int      CurrentCardType;  // API card type.
} MAP_DATA_STRUCT, *PMAP_DATA_STRUCT;

static MAP_DATA_STRUCT pData[MAX_DEVICES];    // Fixed allocated to make life easier.

static int  NumDevices = 0;     // Number of physical devices currently mapped.
static int  NumVXIOpenLib = 0;  // Number of instances currently open.
ViStatus    visaStatus; 
static ViSession   defaultRM;
static int haveRM = FALSE;
static int memOpen = FALSE;
ViSession instrument[2][16] = {{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},}; 
ViAddr mappedAddress[16];
/*===========================================================================*
 * NON-USER GLOBAL ENTRY POINT:    D L L E N T R Y P O I N T
 *===========================================================================*
 *
 * FUNCTION:    DLL Entry Point Procedure
 *
 * DESCRIPTION: Needed for 32 bit DLL under WIN32.
 *              Microsoft Windows DLL entry/exit function.  This is a
 *              system level function and is not accessible by the DLL, and
 *              is called to allow the DLL to initialize when loaded, and
 *              cleanup before being unloaded.
 *
 * It will return:  BOOL
 *
 *===========================================================================*/
BOOL WINAPI DllEntryPoint(HANDLE    hDLL,
                          DWORD     dwReason,
                          LPVOID    lpReserved)
{
   static int nAttached = 0;     // Number of processes/threads attached to us.

   switch (dwReason)
   {
   case DLL_PROCESS_ATTACH:
      //hUInst = hDLL;                 // Save our process handle.
      // if (!InitializeProcess (hDLL))
      //    return FALSE;
      // MessageBox(NULL, "DLL_PROCESS_ATTACH", "DllEntryPoint",
      //            MB_OK | MB_SYSTEMMODAL);
      nAttached++;
      break;
   case DLL_PROCESS_DETACH:
      nAttached--;
      // MessageBox(NULL, "DLL_PROCESS_DETACH", "DllEntryPoint",
      //            MB_OK | MB_SYSTEMMODAL);
      //if ( nAttached <= 0 )
          // Shutdown board access and threads.
      break;
   case DLL_THREAD_ATTACH:
      //nAttached++;
      break;
   case DLL_THREAD_DETACH:
      //nAttached--;
      break;
   default:
      break;
   }
   return TRUE;
}


/*===========================================================================*
 * INTERNAL ENTRY POINT:             v x i C l o s e
 *===========================================================================*
 *
 * FUNCTION:    Unmap the VXI windows and close the VXI library.
 *
 * DESCRIPTION: Return ARS_SUCCESS/ARS_FAILURE based on results of call to
 *              UnMapVXIAddress.  If succesful, the mapped PCI address spaces
 *              specified for the window_1 and window_2 
 *              will be returned to the OS.  Once this is
 *              complete the NI VXI Library is closed via invocation of
 *              CloseVXIlibrary();
 *
 *===========================================================================*/
static int vxiClose(int device)
{
   visaStatus = viUnmapAddress (instrument[0][device]);
   visaStatus = viUnmapAddress (instrument[1][device]);
   if (visaStatus == VI_SUCCESS) 
   {
   
      visaStatus = viClose (instrument[0][device]);
      visaStatus = viClose (instrument[1][device]);
      instrument[0][device] = 0;
      instrument[1][device] = 0;

      if (visaStatus == VI_SUCCESS) 
      {
         visaStatus = viClose (defaultRM);
         haveRM = FALSE;
      }
   }
   else
      return BTD_VIUNMAP_ERR;

   NumDevices--;
   return BTD_OK;
}

/*===========================================================================*
 * INTERNAL ENTRY POINT:             v x i I n i t
 *===========================================================================*
 *
 * FUNCTION:    Map two address spaces through the PCI-MXI-2 interface.
 *              This function provides the capability to map two VMEbus
 *              address spaces in the PCI-MXI-2 VXI realm.
 *
 *              base_addr_1    VXI - VMEbus base address for address space 1
 *              base_addr_2    VXI - VMEbus base address for address space 2
 *              mem_size_1     VXI - VMEbus address space 1 size in bytes
 *              mem_size_2     VXI - VMEbus address space 2 size in bytes
 *              CurrentCarrier Carrier type
 *              *mapped_addr_1 PCI base address allocated to address space 1
 *              *mapped_addr_2 PCI base address allocated to address space 2
 *
 * DESCRIPTION: Return ARS_SUCCESS/ARS_FAILURE based on results of call to
 *              MapVXIAddress.  If succesful, return the mapped addresses via
 *              the variables "mapped_addr_*". 
 *
 * IMPORTANT NOTE:  The invocation of MapVXIAddress() for the IP-1553 I/O Space
 *                  is different between the VME-AIC and an unintelligent
 *                  carrier such as the VIPC-616 or VIPC-618.  See the
 *                  commented parameters in the second invocation of
 *                  MapVXIAddress below.
 *
 *===========================================================================*/
static int vxiInit(int device, BT_UINT cardType,BT_UINT CurrentCarrier)   // (i) Instance of the device to open and map
{ 
   unsigned long jaddr;

   visaStatus = VI_SUCCESS;
   if(haveRM==FALSE)
   {   
      visaStatus = viOpenDefaultRM (&defaultRM);
      haveRM=TRUE;
   }
         
   if (visaStatus == VI_SUCCESS) 
   { 
      if(instrument[0][device] == 0)
      {
         visaStatus = viOpen (defaultRM, "VXI0::MEMACC", VI_NULL, 
                             VI_NULL, &instrument[0][device]);
         if (visaStatus != VI_SUCCESS)
            return BTD_VIOPEN_FAIL;   
      }
      if(instrument[1][device] == 0)
      {
         visaStatus = viOpen (defaultRM, "VXI0::MEMACC", VI_NULL, 
                           VI_NULL, &instrument[1][device]);
            if (visaStatus != VI_SUCCESS)
         return BTD_VIOPEN_FAIL;
      }
    
      if(CurrentCarrier == AVME9660 || CurrentCarrier == NATIVE_24)
         visaStatus = viMapAddress (instrument[0][device], VI_A24_SPACE,
                                    (unsigned long)pData[device].base_addr_1, 
                                    pData[device].mem_size_1, VI_FALSE, VI_NULL, 
                                    &mappedAddress[0]);
      else
         visaStatus = viMapAddress (instrument[0][device], VI_A32_SPACE,
                                    (unsigned long)pData[device].base_addr_1, 
                                    pData[device].mem_size_1, VI_FALSE, VI_NULL, 
                                    &mappedAddress[0]);

      if ( visaStatus != VI_SUCCESS )
         return BTD_VIMAPADDRESS_FAIL;   // Unable to map device

      pData[device].mapped_addr_1 = (void *)mappedAddress[0];

      jaddr = (unsigned long)pData[device].base_addr_2 & 0xfc00;
      visaStatus = viMapAddress (instrument[1][device], VI_A16_SPACE, jaddr, 0x400, 
                                 VI_FALSE, VI_NULL, &mappedAddress[0]);

      (unsigned long)mappedAddress[0]+=(unsigned long)pData[device].base_addr_2&0x3ff;

      if ( visaStatus != VI_SUCCESS )
        return BTD_VIMAPADDRESS_FAIL;   // Unable to map device

      pData[device].mapped_addr_2 = (void *)mappedAddress[0];
      return BTD_OK;
  }
  return BTD_VIOPENDEFAULTRM;
}

/*===========================================================================*
 * GLOBAL ENTRY POINT:   B u s T o o l s _ A P I _ I n i t E x t e r n a l
 *===========================================================================*
 *
 * FUNCTION:    Perform user-specified memory mapping functions.
 *              This function is an example of the memory mapping functions
 *              that a user needs to provide, when mapping a BusTools board
 *              into host address space.
 *              This function must be provided if the API does not support the
 *              mapping or access method supported by the users OS and hardware
 *              environment.
 *
 * DESCRIPTION: This function is called by the API whenever a BusTools_API_Initx
 *              function call is made with a platform = "PLATFORM_USER".
 *              wOpen is set to 1->open device, 0->close device.
 *              dwAddress, wIOaddress, cardType, carrier and slot are simply
 *              passed through by the API.
 *              This functions must setup the bt_PageAddr[] access pointers and
 *              return a value of BTD_OK, as listed below for each board type:
 *
 *           IP-D1553:
 *              bt_PageAddr[0] maps the board dual-port RAM page (2K bytes)
 *              bt_PageAddr[1] points to the I/O register
 *              bt_PageAddr[2] PROM version or points to the ID PROM
 *              bt_PageAddr[3] points to Interrupt Vector reg (not used)
 *
 *   Parameters:
 *       BT_UINT   cardnum        card number (0 - based)
 *       BT_UINT   wOpen          1->open device, 0->close device
 *       BT_U32BIT dwAddress      base address of memory area
 *       BT_U32BIT wIOaddress     board/carrier io address
 *       BT_UINT   cardType       PCI1553, PMC1553 or ISA1553
 *       BT_UINT   carrier        NATIVE.
 *       BT_UINT   slot           SLOT_A, SLOT_B, SLOT_C, SLOT_D, etc.
 *       void * PageAddr[4]       array of board addresses, as described above
 *
 *
 *   NOTES FOR THE IP-D1553 INSTALLED ON A VXI/VME-AIC WITH PCI-MXI-2 INTERFACE
 *
 *   The values for dwAddress and wIOaddress
 *
 *   1) dwAddress is set to the INI file value for the parameter MEMAddr.
 *      MEMAddr should be set to the IP-1553 Memory Space Address defined in 
 *      the MVME-162 Environment "Bug Mode" Parameter "IP A Base Address"
 *
 *   2) wIOaddress is set to the INI file value for the parameter IoAddr.
 *      IoAddr should be set to the IP-1553 I/O Space Address defined by the
 *      MVME-162 Environment "Bug Mode" Parameters for the slave translation
 *      register set 2.  The current settings for these parameters are
 *      defined as follows:
 *
 *      Slave Enable #2 [Y/N] = Y?
 *      Slave Starting Address #2 = 00F50000?
 *      Slave Ending Address #2   = 00F60000?
 *      Slave Address Translation Address #2 = FF000000?
 *      Slave Address Translation Select #2  = FF000000?
 *      Slave Control #2 = 0BFF?
 *
 *      This translates VMEbus address space $F50000 - $F60000 to the
 *      entire MVME-162 IPIC Address Space, including I/O and ID regions.
 *      One thing to note is that word addressing is not supported, and
 *      only even words are accessable with this addressing scheme.
 *      IP A I/O Space is located at $F58000 and ID Space is located at 
 *      $F58080.  IP C I/O Space is located at $F58200 and ID Space is 
 *      located at $F58280.
 *
 *   BusTools_API_InitExternal() will return:
 *              BTD_OK if successful, non-zero upon error.
 *
 *   Note that the base addresses passed into this function ARE valid for
 *   both the OPEN and the CLOSE!  And the remaining arguements ARE ALSO
 *   valid!
 *
 *===========================================================================*/
// This routine is called by BusTools_API_Initxxx() to map the board addresses.
// This code supports one single board, and that board maps exactly two regions.
NOMANGLE BT_INT CCONV BusTools_API_InitExternal(
   BT_UINT   cardnum,        // (i) card number (0 - based)
   BT_UINT   wOpen,          // (i) 1->open device, 0->close device
   BT_U32BIT dwAddress,      // (i) base address of board/carrier memory area
   BT_U32BIT wIOaddress,     // (i) base address of board/carrier io area
   BT_UINT   cardType,       // (i) PCI1553, PMC1553 or ISA1553
   BT_UINT   CurrentCarrier, // (i) MVME162, VIPC616_618, AVME9660.
   BT_UINT   CurrentSlot,    // (i) SLOT_A, SLOT_B, SLOT_C, SLOT_D, etc.
   void     *PageAddr[4])    // (o) array of board addresses.
{
   /*********************************************************************
   *  Local variables
   *********************************************************************/
   void   *window_1_address;  // Mapped memory area pointer
   void   *window_2_address;  // Mapped I/O area pointer
   int     status;            // Returned function status
   int     device;            // Physical device being accessed
   char    szMsg[200];        // Error message buffer

   /*********************************************************************
   *  Test and verify that the specified board/carrier/slot etc. is
   *   understood by this function.  If not return BTD_NON_SUPPORT or
   *   another error code.
   *********************************************************************/
   if ( cardnum >= MAX_BTA )
      return API_BUSTOOLS_BADCARDNUM;

   if ( cardType != IPD1553  &&
	    cardType != VME1553  &&
        cardType != QVME1553 &&
		cardType != VXI1553 )
      return BTD_NON_SUPPORT;


   if ( CurrentCarrier == VIPC616_618 )      {}
   else if ( CurrentCarrier == AVME9660 )    {}
   else if ( CurrentCarrier == NATIVE_24 )   {}
   else if ( CurrentCarrier == NATIVE_32 )   {}
   else if ( CurrentCarrier == NI_A32_MAP_16 )  {}
   else if ( CurrentCarrier == NI_A32_MAP_32 )  {}
   else
      return BTD_NON_SUPPORT;

   if(cardType == VME1553 || cardType == VXI1553 || cardType == QVME1553)
   {
	   if(CurrentSlot < CHANNEL_1 || CurrentSlot > CHANNEL_4)
		   return BTD_NON_SUPPORT;
   }

   /*********************************************************************
   * Locate the physical device entry in the MAP_DATA_STRUCT (pData)
   *  structure.  Locate it based on the device base addresses, which
   *  are supplied by the caller for both open and close!
   *********************************************************************/
   for ( device = 0; device < NumDevices; device++ )
   {
      if ( pData[device].DeviceMapOpen == 0 )
         continue;                                   // Entry not in use.
      if (  pData[device].CurrentCarrier != (BT_INT)CurrentCarrier )
         continue;                                   // Carrier type no match
      if (  pData[device].base_addr_1 != dwAddress )
         continue;                                   // Memory address no match
      if (  pData[device].base_addr_2 == wIOaddress )
         break;                                      // Memory and IO address match
   }

   // See if there is no device structure entry allocated for this board.
   if ( device < NumDevices )
   {
      // "device" index has been allocated, let's use it.
   }
   else
   {  // "device" index has not been allocated, grab the next device slot.
      if ( wOpen )
      {
         device = NumDevices;    // Allocate the device at the end of the list
         NumDevices++;
         pData[device].DeviceMapOpen   = 0;
         pData[device].base_addr_1     = dwAddress;
         pData[device].base_addr_2     = wIOaddress;
         pData[device].CurrentCardType = cardType;
         pData[device].CurrentCarrier  = CurrentCarrier;
         if ( CurrentCarrier == VIPC616_618 )
         {  // We map the entire 32 Meg region, but we only use the first
            //  16 Meg + 2 Kb memory area.  And in the first 16 Meg we only use
            //   the first 2 Kb of that region...
            pData[device].mem_size_1 = 0x2000000; // Map 32 Meg bytes in A32 space.
            pData[device].mem_size_2 = 0x400;     // Map 1024 bytes in A16 space.
         }
         else if ( CurrentCarrier == AVME9660 )
         {  // We map 2 Meg, but we only use the first 1 Meg + 2 Kb.  And in the
            //  first 1 Meg we only use the first 2 Kb...
            pData[device].mem_size_1 = 0x400000; // Map 4 Meg bytes in A24 space for IP-D1553.
            pData[device].mem_size_2 = 0x400;    // Map 1024 bytes in A16 space.
         }
         else if ( CurrentCarrier == VIPC310 )
         {  // We map 64K bytes, but we only use the first 2K bytes...
            pData[device].mem_size_1 = 0x10000;  // Map 64K bytes in A24 space.
            pData[device].mem_size_2 = 0x200;    // Map 512 bytes in A16 space.
         }
		 else if ( CurrentCarrier == NATIVE_24 || CurrentCarrier == NATIVE_32)
         {  // We map the entire 32 Meg region, but we only use the first
            //  16 Meg + 2 Kb memory area.  And in the first 16 Meg we only use
            //   the first 2 Kb of that region...
            pData[device].mem_size_1 = 0x800000; // Map 8 Meg bytes in A24/A32 space.
            pData[device].mem_size_2 = 0x40;     // Map 64 bytes in A16 space.
         }
		 else if ( CurrentCarrier == NI_A32_MAP_16)
         {  // We map the entire 32 Meg region, but we only use the first
            //  16 Meg + 2 Kb memory area.  And in the first 16 Meg we only use
            //   the first 2 Kb of that region...
            pData[device].mem_size_1 = 0x1000000; // Map 8 Meg bytes in A24/A32 space.
            pData[device].mem_size_2 = 0x40;     // Map 64 bytes in A16 space.
         }
		 else if ( CurrentCarrier == NI_A32_MAP_32)
         {  // We map the entire 32 Meg region, but we only use the first
            //  16 Meg + 2 Kb memory area.  And in the first 16 Meg we only use
            //   the first 2 Kb of that region...
            pData[device].mem_size_1 = 0x2000000; // Map 8 Meg bytes in A24/A32 space.
            pData[device].mem_size_2 = 0x40;     // Map 64 bytes in A16 space.
         }
      }
      else
         return BTD_ERR_NOTSETUP;  // Can't close if not setup and open
   }

   /*********************************************************************
   *  Given the HW base addresses, map the addresses into host address
   *   space.  Perform any other initialization specific to the board,
   *   OS or carrier that needs to be performed.
   *********************************************************************/
   if ( wOpen )
   {
      /******************************************************************
      * If this device is not open, map it.
      ******************************************************************/
      NumVXIOpenLib++;
      if ( pData[device].DeviceMapOpen == 0 )
      {
         status = vxiInit(device,cardType,CurrentCarrier);
         if ( status != BTD_OK )
         {
            vxiClose(device);
            NumVXIOpenLib--;
            if ( NumVXIOpenLib <= 0 )
            return status;
         }
      }

      // Count this instance of the open device.
      pData[device].DeviceMapOpen++;

      // Fetch the mapped carrier base addresses
      window_1_address = pData[device].mapped_addr_1;
      window_2_address = pData[device].mapped_addr_2;

      /******************************************************************
      * Setup the IP-1553 access addresses, based on the carrier type,
      *  the slot ID and the carrier base addresses.
      ******************************************************************/
      if(cardType == VME1553 || cardType == QVME1553 || cardType == VXI1553)
	          PageAddr[3] = NULL;                   // Interrupt Vector not used.
 
	  PageAddr[0] = (char *)window_1_address;  // mem pointer.
      PageAddr[1] = (char *)window_2_address;  // VXI/VME Registers

      return BTD_OK;
   }
   /*********************************************************************
   *  Close access to the specified device (carrier slot).
   *********************************************************************/
   pData[device].DeviceMapOpen--;         // Count of open slots
   if ( pData[device].DeviceMapOpen == 0 )
      status = vxiClose(device);
   else
      status = BTD_OK;

   /*********************************************************************
   *  Close the VXI library if this is the last device instance.
   *********************************************************************/
   NumVXIOpenLib--;
   if ( NumVXIOpenLib <= 0 )
   {
      if ( status < 0 )
      {
         sprintf(szMsg, "Error %X closing the VXI Library\n", status);
         MessageBox(NULL, szMsg, "BusTools_API_InitExternal()", MB_OK | MB_SYSTEMMODAL);
         return BTD_ERR_NOACCESS;   // Unable to map device
      }
   }
   return status;
}

NOMANGLE BT_INT CCONV Condor_IP_ID(BT_UINT addr16, BT_INT slot)
{
   ViStatus    visaStatus;
   ViSession   instrument;
   ViSession   defaultRM;
   ViAddr vi_addr;
   BT_INT retval;

   BT_U32BIT IoAddr,offset;
   unsigned char ip_board_type;

   offset = (unsigned)(slot*0x100);

   /* Window A16 mapping using National Instruments VISA library */
   {

      visaStatus = VI_SUCCESS;   
      visaStatus = viOpenDefaultRM (&defaultRM);
         
      if (visaStatus == VI_SUCCESS) 
      { 
         visaStatus = viOpen (defaultRM, "VXI0::MEMACC", VI_NULL, 
                             VI_NULL, &instrument);
         if (visaStatus != VI_SUCCESS)
            exit(0);   
      }

      visaStatus = viMapAddress (instrument, VI_A16_SPACE, addr16, 0x400, 
                                   VI_FALSE, VI_NULL, &vi_addr);
      if ( visaStatus != VI_SUCCESS )
      {
         printf("Unable to map VISA A16 Address\n");
         exit(0); 
      }
      IoAddr = (unsigned)vi_addr + offset;
   } 
   /*
    * Mapping successful Now check for board type at IDPROM offset 0xb 
    */

   /*  Value is 05 for IP-1553-M, 08 for IP-1553-S, 0C for IP-D1553-1M, 
       0D for IP-D1553-1S, 0E for IP-D1553-2M and 0F for IP-D1553-2S.  */ 
        
   ip_board_type = ((unsigned char *)(IoAddr))[0x80+0xb];
   switch(ip_board_type)
   {
      case 5:
      case 8:
         retval = 0;
         break;
      case 0xc:
      case 0xd:
      case 0xe:
      case 0xf:
          retval = 1;
          break; 
      default:
          retval = -1;
   }
   visaStatus = viUnmapAddress (instrument);
   if (visaStatus == VI_SUCCESS) 
   {  
      visaStatus = viClose (instrument);
      if (visaStatus == VI_SUCCESS) 
         visaStatus = viClose (defaultRM);
   }
   return retval;
}

NOMANGLE BT_INT CCONV vbtGetCEIVMEDevInfo(BT_UINT addr16, DEVICE_INFO *pInfo)
{
   ViStatus    visaStatus;
   ViSession   instrument;
   ViSession   defaultRM;
   ViAddr vi_addr;

   BT_U16BIT *IoAddr;
   BT_U32BIT offset;
   BT_INT slot,i;
   unsigned char ipd_id;
   BT_U16BIT vme_id [] = {0x150, //   1   single-function	no
                          0x151, //   2   single-function	no
                          0x153, //   4   single-function	no
                          0x15C, //   1   multi-function	no
                          0x15D, //   2   multi-function	no
                          0x15F, //   4   multi-function	no
                          0x350, //   1   single-function	yes
                          0x351, //   2   single-function	yes
                          0x353, //   4   single-function	yes
                          0x35C, //   1   multi-function	yes
                          0x35D, //   2   multi-function	yes
                          0x35F};//   4   multi-function	yes
   BT_INT vme_nchan[] = {1,2,4,1,2,4,1,2,4,1,2,4};
   BT_INT vme_mode[]  = {0,0,0,1,1,1,0,0,0,1,1,1};
   BT_INT vme_irig[]  = {0,0,0,0,0,0,1,1,1,1,1,1};                  

   /* Window A16 mapping using National Instruments VISA library */

   visaStatus = VI_SUCCESS;   
   visaStatus = viOpenDefaultRM (&defaultRM);
         
   if (visaStatus == VI_SUCCESS) 
   { 
      visaStatus = viOpen (defaultRM, "VXI0::MEMACC", VI_NULL, 
                           VI_NULL, &instrument);
      if (visaStatus != VI_SUCCESS)
         return BTD_VIOPEN_FAIL;   
   }

   visaStatus = viMapAddress (instrument, VI_A16_SPACE, addr16, 0x400, 
                              VI_FALSE, VI_NULL, &vi_addr);
   if ( visaStatus != VI_SUCCESS )
      return BTD_VIMAPADDRESS_FAIL;

   slot = addr16 & 0x7;                  // this is for IPD 
   offset = (unsigned)(slot*0x100);      //

   IoAddr = (BT_U16BIT *)(vi_addr) + offset;

   if((IoAddr[0] & 0x0fff) == 0xe94) // this is a VME-1553 or QVME-1553
   {
      for(i=0;i<12;i++)
      {
         if(vme_id[i] == (IoAddr[1]&0x0fff))
            break;
      }
      if(i==12)
         return BTD_ERR_NODETECT;

      pInfo->busType = 3;  //VMEBus
      pInfo->DeviceID = 0; //no deviceID
      pInfo->VendorID = 0xE94; //Manufactur ID
      pInfo->nchan = vme_nchan[i];
      pInfo->mode = vme_mode[i];
      pInfo->irig = vme_irig[i];
      pInfo->memSections = 2; //A16 and A24/A32
   }
   else
   {
      if((IoAddr[4] & 0x00ff) == 0x79) // Test for IP-D1553
      {
          ipd_id = (unsigned char)(IoAddr[5] & 0x00ff);
          pInfo->busType = 4;  //IP Bus
          pInfo->DeviceID = 0; //no deviceID
          pInfo->VendorID = 0x79; //Manufactur ID
          switch (ipd_id)
          {
             case '0xc':
                pInfo->nchan = 1;
                pInfo->mode  = 1;
                break;
             case '0xd':
                pInfo->nchan = 1;
                pInfo->mode  = 0;
                break;
             case '0xe':
                pInfo->nchan = 2;
                pInfo->mode  = 1;
                break;
             case '0xf':
                pInfo->nchan = 2;
                pInfo->mode  = 0;
                break;
         }
         pInfo->irig = 0;
         pInfo->memSections = 2; //A16 and A24/A32
      }    
      return BTD_ERR_NODETECT;                      
   }

   visaStatus = viUnmapAddress (instrument);
   if (visaStatus == VI_SUCCESS) 
   {  
      visaStatus = viClose (instrument);
      if (visaStatus == VI_SUCCESS) 
         visaStatus = viClose (defaultRM);
   }
   return API_SUCCESS;
}
