 /*============================================================================*
 * FILE:                       U S B . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2017 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:   This file contains all function related USB configuration and 
 *             input/outputs.
 *
 * USER ENTRY POINTS: 
 *
 * EXTERNAL NON-USER ENTRY POINTS:
 *
 * INTERNAL ROUTINES:
 *===========================================================================*/

/* $Revision:  8.22 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  04/22/2014   modified usb_open and usb_error. bch
  11/05/2014   Fix the error display.
  11/16/2017   Changes to address warnings when using VS9 Wp64 compiler option.
*/


#include <stdio.h>
#include "busapi.h"
#include "apiint.h" 
#include "btdrv.h"
#include "globals.h"
#include "cei_install.h"

#ifdef INCLUDE_USB_SUPPORT
#define MAX_RWR_SIZE 8192
#define MAX_RWR 64


CEI_INT32 LL_InitReadRequestBlock
(
    CEI_DEV_HANDLE dev_handle,   // (i) handle of device instance
    CEI_UINT32 offset_bytes,     // (i) offset (in bytes) to begin reading/
                                 //     writing at within the device memory 
                                 //     region specified by rgn_usr_addr
    CEI_UINT32 request_bytes,    // (i) number of bytes to read or write
    pCEI_VOID data_ptr,          // (i) pointer to caller-allocated buffer of 
                                 //     length request_bytes which contains data
                                 //     to write (if direction is CEI_RWR_WRITE)
                                 //     or which receives data that was read (if 
                                 //     direction is CEI_RWR_READ)

    pCEI_READ_WRITE_REQ rwr_ptr, // (o) pointer to caller-allocated buffer of 
                                 //     max_rwrs read/write request structures
                                 //     that, upon successful return, is 
                                 //     initialized and ready to pass to 
                                 //     LL_ExecuteReadWriteRequest
    pCEI_UINT32 num_rwrs         // (o) receives the number of read/write
                                 //     request structures written into the 
                                 //     rwr_ptr buffer (upon successful
                                 //     return)
);

CEI_INT32 LL_InitWriteRequestBlock
(
    CEI_DEV_HANDLE dev_handle,   // (i) handle of device instance
    CEI_UINT32 offset_bytes,     // (i) offset (in bytes) to begin reading/
                                 //     writing at within the device memory 
                                 //     region specified by rgn_usr_addr
    CEI_UINT32 request_bytes,    // (i) number of bytes to read or write
    pCEI_VOID data_ptr,          // (i) pointer to caller-allocated buffer of 
                                 //     length request_bytes which contains data
                                 //     to write (if direction is CEI_RWR_WRITE)
                                 //     or which receives data that was read (if 
                                 //     direction is CEI_RWR_READ)

    pCEI_READ_WRITE_REQ rwr_ptr, // (o) pointer to caller-allocated buffer of 
                                 //     max_rwrs read/write request structures
                                 //     that, upon successful return, is 
                                 //     initialized and ready to pass to 
                                 //     LL_ExecuteReadWriteRequest
    pCEI_UINT32 num_rwrs         // (o) receives the number of read/write
                                 //     request structures written into the 
                                 //     rwr_ptr buffer (upon successful
                                 //     return)
);

// array of structs to receive device info
CEI_UINT32 numDevicesFound = 0;
CEI_INT bm_allocated[16];
CEI_INT user_bm_buffer_size;
static CEI_UINT close_count[MAX_BTA] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

void displayError(const CHAR *sErr,...);

// cei_install wrapper functions
extern CEI_INSTALL_MODULE *CEI_DLL_GetDynamicLink(CEI_INT32 optionFlags, CEI_INSTALL_ERR err);

CEI_INSTALL_MODULE *get_cei_dll(CEI_INSTALL_ERR cei_err) {
  static CEI_INSTALL_MODULE *cei_dll = NULL;

  if(cei_dll == NULL)
    cei_dll = CEI_DLL_GetDynamicLink(0, cei_err);

  return cei_dll;
}

// Wraps call to LL_OpenDeviceByLogicalId.  Refer to LL_OpenDeviceByLogicalId documentation in cei_install.h for details.
CEI_INT32 DLL_EXPORTED LL_OpenDeviceByLogicalId(CEI_UINT32 id,pCEI_DEV_HANDLE dev_handle_ptr,CEI_INSTALL_ERR err) {
  CEI_INSTALL_MODULE *cei_dll = get_cei_dll(err);
  if(cei_dll == NULL)
    return FALSE;
  else            
    return cei_dll->LL_OpenDeviceByLogicalId(id,dev_handle_ptr,err);
}

// Wraps call to LL_ExecuteReadWriteRequest.  Refer to LL_ExecuteReadWriteRequest documentation in cei_install.h for details.
CEI_INT32 DLL_EXPORTED LL_ExecuteReadWriteRequest(CEI_DEV_HANDLE dev_handle,CEI_UINT32 rwr_ver,pCEI_READ_WRITE_REQ rwr_ptr,CEI_UINT32 rwr_count,CEI_INSTALL_ERR err) {
  CEI_INSTALL_MODULE *cei_dll = get_cei_dll(err);
  if(cei_dll == NULL)
    return FALSE;
  else            
    return cei_dll->LL_ExecuteReadWriteRequest(dev_handle,rwr_ver,rwr_ptr,rwr_count,err);
}

// Wraps call to LL_InitReadWriteRequestStruct.  Refer to LL_InitReadWriteRequestStruct documentation in cei_install.h for details.
CEI_INT32 DLL_EXPORTED LL_InitReadWriteRequestStruct(CEI_DEV_HANDLE dev_handle,CEI_UINT32 direction,pCEI_VOID rgn_usr_addr,CEI_UINT32 offset_bytes,CEI_UINT32 request_bytes,pCEI_VOID data_ptr,CEI_UINT32 rwr_ver,pCEI_READ_WRITE_REQ rwr_ptr,CEI_INSTALL_ERR err) {
  CEI_INSTALL_MODULE *cei_dll = get_cei_dll(err);
  if(cei_dll == NULL)
    return FALSE;
  else            
    return cei_dll->LL_InitReadWriteRequestStruct(dev_handle,direction,rgn_usr_addr,offset_bytes,request_bytes,data_ptr,rwr_ver,rwr_ptr,err);
}

// Wraps call to LL_CloseDevice.  Refer to LL_CloseDevice documentation in cei_install.h for details.
CEI_INT32 DLL_EXPORTED LL_CloseDevice(CEI_DEV_HANDLE dev_handle,CEI_INSTALL_ERR err) {
  CEI_INSTALL_MODULE *cei_dll = get_cei_dll(err);
  if(cei_dll == NULL)
    return FALSE;
  else            
    return cei_dll->LL_CloseDevice(dev_handle,err);
}


BT_INT usb_open(BT_UINT cardnum)
{
   CEI_INSTALL_ERR cei_err="";
   if (!LL_OpenDeviceByLogicalId(api_device[cardnum],&gpDevice[cardnum],cei_err))
   {
     displayError("usb_open: LL_OpenDeviceByLogicalId failed (device: %d, err: %s)", api_device[cardnum], cei_err);
     return BTD_LL_OPEN_ERR | API_INSTALL_ERROR;
   }
   return API_SUCCESS;
}

usb_error(BT_INT cardnum,CEI_INSTALL_ERR cei_err, char * fname)
{
   close_count[cardnum]++;
   if(close_count[cardnum]==1)
     BusTools_API_Close(cardnum);

//   sprintf(szMsg, "%s\nError in %s. May be due to USB disconnect. Application Closing",fname, cei_err);
//   MessageBox(NULL, szMsg, "Fatal USB Interface Error", MB_OK | MB_SYSTEMMODAL);
   displayError("%s failed. May be due to USB disconnect. API closed. (device: %d, err: %s)", fname, api_device[cardnum], cei_err);
   MessageBox(NULL, install_error_string, "Fatal USB Interface Error", MB_OK | MB_SYSTEMMODAL);

   exit(1);
}

void usb_close(BT_INT cardnum)
{
   CEI_INSTALL_ERR cei_err;
   CEI_UINT32 retval=0;   

   retval = LL_CloseDevice(gpDevice[cardnum],cei_err);

   if(usb_host_hwreg[cardnum] != NULL)
   {
      CEI_FREE(usb_host_hwreg[cardnum]);
      usb_host_hwreg[cardnum] = NULL;
   }
   if(usb_int_queue[cardnum] != NULL)
   {
      CEI_FREE(usb_int_queue[cardnum]);
      usb_int_queue[cardnum] = NULL;
   }
   if(ptrMbuf[cardnum] != NULL)
   {
      CEI_FREE(ptrMbuf[cardnum]);
      ptrMbuf[cardnum] = NULL;
   }
}

/*===========================================================================*
 * ENTRY POINT:         u s b _ g e t _ b m _ d a t a
 *===========================================================================*
 *
 * FUNCTION:    Read data from selected range of adapter memory into a caller
 *              supplied buffer.
 *
 * DESCRIPTION:
 *
 *      It will return:
 *              nothing.
 *===========================================================================*/

BT_INT usb_get_bm_data(BT_INT cardnum)
{
   CEI_READ_WRITE_REQ usbData[64];
   BT_U32BIT bm_head_pointer;
   BT_INT bcount=0, rcount;
   CEI_UINT32 retval=0;
   CEI_INSTALL_ERR cei_err;

   bm_head_pointer = REL_ADDR(cardnum,usb_host_hwreg[cardnum][0xc2]);  // This was read earlier in the processing HWREG_BM_HEAD_PTR

   if(bm_head_pointer < btmem_bm_mbuf[cardnum] || bm_head_pointer > btmem_bm_mbuf_next[cardnum]) 
      return 0x1234; //API_BM_BAD_HEAD_PTR;

   if(bm_rec_prev[cardnum] != bm_head_pointer)
   {
      if(bm_rec_prev[cardnum] < bm_head_pointer) // Only need a single read from tail to head (no wrap)      
      {
         // Read in the Bus Monitor Buffers tail to head
         retval =  LL_InitReadRequestBlock(gpDevice[cardnum],
                                           (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][RAM_PAGE] + bm_rec_prev[cardnum])),
                                           (CEI_UINT32)(bm_head_pointer - bm_rec_prev[cardnum]),
                                           (pCEI_UCHAR)ptrMbuf[cardnum] + (bm_rec_prev[cardnum] - btmem_bm_mbuf[cardnum]),
                                           usbData,
                                           &rcount);

      }
      else  // the buffer wrapped - need to reaf from tail to end - then from start to head
      {
         // Read in the Bus Monitor Buffers tail to end
         retval =  LL_InitReadRequestBlock(gpDevice[cardnum],
                                           (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][RAM_PAGE] + bm_rec_prev[cardnum])),
                                           (CEI_UINT32)(btmem_bm_mbuf_next[cardnum] - bm_rec_prev[cardnum]),
                                           (pCEI_UCHAR)ptrMbuf[cardnum] + (bm_rec_prev[cardnum] - btmem_bm_mbuf[cardnum]),
                                           usbData,
                                           &bcount);                 

         rcount = bcount;

         // Read in the Bus Monitor Buffers start to head
         retval = LL_InitReadRequestBlock(gpDevice[cardnum],
                                          (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][RAM_PAGE] + btmem_bm_mbuf[cardnum])),
                                          (CEI_UINT32)(bm_head_pointer - btmem_bm_mbuf[cardnum] ),
                                          (pCEI_UCHAR)ptrMbuf[cardnum],
                                          &usbData[bcount],
                                          &bcount);

         rcount+=bcount;
      }

      if(rcount < 63)
      {
         retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                      CEI_RWR_WRITE,
                      NULL,
                      (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][HWREG_PAGE] + (HWREG_BM_TAIL_PTR*4))),
                      4,
                      (pCEI_UCHAR)&usb_host_hwreg[cardnum][0xc2],
                      CEI_READ_WRITE_REQ_VER,
                      &usbData[rcount],
                      cei_err);
         if(retval==0)
            usb_error(cardnum,cei_err,"usb_get_bm_data");

         rcount++;    
      }
      else
         printf("Tail Pointer not updated\n");

      retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, usbData, rcount,cei_err);
      if(retval==0)
         usb_error(cardnum,cei_err,"usb_get_bm_data");
      
   }
   return API_SUCCESS;
}

/*===========================================================================*
 * ENTRY POINT:         u s b _ g e t _ i n t _ d a t a
 *===========================================================================*
 *
 * FUNCTION:    Read data from selected range of adapter memory into a caller
 *              supplied buffer.
 *
 * DESCRIPTION:
 *
 *      It will return:
 *              nothing.
 *===========================================================================*/

BT_INT usb_get_int_data(BT_INT cardnum)
{
   CEI_UINT32 retval;
   CEI_READ_WRITE_REQ usbData[2];
   CEI_INSTALL_ERR cei_err;

   // Read in the Hardware Registers
   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                     CEI_RWR_READ,
                     NULL,
                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][HWREG_PAGE])),
                     (CEI_UINT32)0x400,
                     (pCEI_UCHAR)usb_host_hwreg[cardnum],
                     CEI_READ_WRITE_REQ_VER,
                     &usbData[0],
                     cei_err);
   if(!retval)
      return 0xc001;

   // Read in the interrupt queue
   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                     CEI_RWR_READ,
                     NULL,
                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][RAM_PAGE] + BTMEM_IQ_V6)),
                     (CEI_UINT32)(sizeof(IQ_MBLOCK_V6) * 512),
                     (pCEI_UCHAR)usb_int_queue[cardnum],
                     CEI_READ_WRITE_REQ_VER,
                     &usbData[1],
                     cei_err);

   if(!retval)
      return 0xc002;

   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, usbData, 2,cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usb_get_int_data");

   return 0;
}

/*===========================================================================*
 *  ENTRY POINT:         u s b R e a d R A M
 *===========================================================================*
 *
 * FUNCTION:    Read data from selected range of adapter memory into a caller
 *              supplied buffer.
 *
 * DESCRIPTION:
 *
 *      It will return:
 *              nothing.
 *===========================================================================*/

void usbReadRAM(
   BT_UINT   cardnum,      // (i) card number
   BT_U16BIT *lpbuffer,    // (o) host buffer (destination)
   BT_U32BIT byteOffset,   // (i) byte offset within adapter memory (source)
   BT_UINT   wordsToRead)   // (i) number of bytes to copy
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   CEI_UINT32 retval;
   CEI_READ_WRITE_REQ usbData;
   CEI_INSTALL_ERR cei_err;

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                     CEI_RWR_READ,
                     NULL,
                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][RAM_PAGE]+byteOffset)),
                     wordsToRead*2,
                     (pCEI_UCHAR)lpbuffer,
                     CEI_READ_WRITE_REQ_VER,
                     &usbData,
                     cei_err);

   if(retval==0)
      usb_error(cardnum,cei_err,"usbReadRAM");

      

   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, &usbData, 1,cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbReadRAM");

}

/*===========================================================================*
 *  ENTRY POINT:         u s b R e a d R A M
 *===========================================================================*
 *
 * FUNCTION:    Read data from selected range of adapter memory into a caller
 *              supplied buffer.
 *
 * DESCRIPTION:
 *
 *      It will return:
 *              nothing.
 *===========================================================================*/

void usbReadBMRAM(
   BT_UINT   cardnum,      // (i) card number
   BT_U16BIT *lpbuffer,    // (o) host buffer (destination)
   BT_U32BIT byteOffset,   // (i) byte offset within adapter memory (source)
   BT_UINT   wordsToRead)   // (i) number of bytes to copy
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   BT_UINT    i;                  // Word counter

   byteOffset = byteOffset - btmem_bm_mbuf[cardnum];

   for ( i = 0; i < wordsToRead; i++ )
   {
      lpbuffer[i] = ((BT_U16BIT *)(bt_PageAddr[cardnum][BM_SIM_PAGE]+byteOffset))[i];
      flipw(&lpbuffer[i]); // for PowerPC
   }
}

/*===========================================================================*
 *  ENTRY POINT:         u s b R e a d R e l R A M
 *===========================================================================*
 *
 * FUNCTION:    Read data from selected range of adapter memory into a caller
 *              supplied buffer.
 *
 * DESCRIPTION:
 *
 *      It will return:
 *              nothing.
 *===========================================================================*/

void usbReadRelRAM(
   BT_UINT   cardnum,      // (i) card number
   BT_U16BIT *lpbuffer,    // (o) host buffer (destination)
   BT_U32BIT byteOffset,   // (i) byte offset within adapter memory (source)
   BT_UINT   wordsToRead)   // (i) number of bytes to copy
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   CEI_UINT32 retval;
   CEI_READ_WRITE_REQ usbData;
   CEI_INSTALL_ERR cei_err;

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                     CEI_RWR_READ,
                     NULL,
                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][BASE_PAGE]+byteOffset)),
                     wordsToRead*2,
                     (pCEI_UCHAR)lpbuffer,
                     CEI_READ_WRITE_REQ_VER,
                     &usbData,
                     cei_err);

   if(retval==0)
      usb_error(cardnum,cei_err,"usbReadRelRAM");

   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, &usbData, 1,cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbReadRelRAM");
}

/*===========================================================================*
 * EXTERNAL ENTRY POINT:      u s b W r i t e R A M
 *===========================================================================*
 *
 * FUNCTION:    Write data from caller supplied buffer into adapter memory.
 *
 * DESCRIPTION:
 *
 *      It will return:
 *              nothing
 *===========================================================================*/
void usbWriteRAM(
   BT_UINT   cardnum,      // (i) card number
   BT_U16BIT *lpbuffer,    // (i) host buffer (source)
   BT_U32BIT byteOffset,   // (o) byte offset within adapter (destination)
   BT_UINT   wordsToWrite) // (i) number of bytes to copy
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   BT_INT retval;
   CEI_READ_WRITE_REQ usbData[64];
   CEI_INSTALL_ERR cei_err;
   BT_INT rcount;

   retval =  LL_InitWriteRequestBlock(gpDevice[cardnum],
                                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][RAM_PAGE] + byteOffset)),
                                     wordsToWrite*2,
                                     (pCEI_UCHAR)lpbuffer,
                                     usbData,
                                     &rcount);

   if(retval)
      usb_error(cardnum,cei_err,"usbWriteRAM");

   if(rcount==0)
      return;

   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, usbData, rcount,cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbWriteRAM");
}

/*===========================================================================*
 * EXTERNAL ENTRY POINT:      u s b W r i t e R e l R A M
 *===========================================================================*
 *
 * FUNCTION:    Write data from caller supplied buffer into adapter memory.
 *
 * DESCRIPTION:
 *
 *      It will return:
 *              nothing
 *===========================================================================*/
void usbWriteRelRAM(
   BT_UINT   cardnum,      // (i) card number
   BT_U16BIT *lpbuffer,    // (i) host buffer (source)
   BT_U32BIT byteOffset,   // (o) byte offset within adapter (destination)
   BT_UINT   wordsToWrite) // (i) number of bytes to copy
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   BT_INT retval;
   CEI_READ_WRITE_REQ usbData;
   CEI_INSTALL_ERR cei_err;

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                     CEI_RWR_WRITE,
                     NULL,
                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][BASE_PAGE]+byteOffset)),
                     wordsToWrite*2,
                     (pCEI_UCHAR)lpbuffer,
                     CEI_READ_WRITE_REQ_VER,
                     &usbData,
                     cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbWriteRelRAM");

   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, &usbData, 1,cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbWriteRelRAM");
}

/*===========================================================================*
 *  ENTRY POINT:         u s b R e a d R A M 3 2
 *===========================================================================*
 *
 * FUNCTION:    Read data from selected range of adapter memory into a caller
 *              supplied buffer.
 *
 * DESCRIPTION:
 *
 *      It will return:
 *              nothing.
 *===========================================================================*/
void usbReadRAM32(
   BT_UINT   cardnum,      // (i) card number
   BT_U32BIT *lpbuffer,    // (o) host buffer (destination)
   BT_U32BIT byteOffset,   // (i) byte offset within adapter memory (source)
   BT_UINT   wordsToRead)   // (i) number of bytes to copy
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   CEI_UINT32 retval;
   CEI_READ_WRITE_REQ usbData[64];
   CEI_INSTALL_ERR cei_err;
   BT_INT rcount;

   retval =  LL_InitReadRequestBlock(gpDevice[cardnum],
                                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][RAM_PAGE]+byteOffset)),
                                     wordsToRead*4,
                                     (pCEI_UCHAR)lpbuffer,
                                     usbData,
                                     &rcount);
   if(retval)
      usb_error(cardnum,"","usbReadRAM32");

   if(rcount == 0)
      return;

   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, usbData, rcount,cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbReadRAM32");
}


/*===========================================================================*
 *  ENTRY POINT:         u s b R e a d B M R A M 3 2
 *===========================================================================*
 *
 * FUNCTION:    Read data from selected range of Host side BM memory into a caller
 *              supplied buffer.
 *
 * DESCRIPTION:
 *
 *      It will return:
 *              nothing.
 *===========================================================================*/
void usbReadBMRAM32(
   BT_UINT   cardnum,      // (i) card number
   BT_U32BIT *lpbuffer,    // (o) host buffer (destination)
   BT_U32BIT byteOffset,   // (i) byte offset within adapter memory (source)
   BT_UINT   wordsToRead)   // (i) number of bytes to copy
{
   BT_UINT i;
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   byteOffset = byteOffset - btmem_bm_mbuf[cardnum];

#ifdef WORD_SWAP
   for ( i = 0; i < wordsToRead; i++ )
   {
      lpbuffer[i] = ((BT_U32BIT *)(bt_PageAddr[cardnum][BM_SIM_PAGE]+byteOffset))[i];
   }
   for( i = 0; i < wordsToRead; i++ )
   {
      flipw(&lpbuffer[i]);
   }
#else //WORD_SWAP
   for ( i = 0; i < wordsToRead; i++ )
   {
      lpbuffer[i] = flips(((BT_U32BIT *)(bt_PageAddr[cardnum][BM_SIM_PAGE]+byteOffset))[i]);
   }
#endif //WORD_SWAP
}

/*===========================================================================*
 * EXTERNAL ENTRY POINT:         v b t R e a d I n t Q u e u e
 *===========================================================================*
 *
 * FUNCTION:    Read data from selected range of interrupt queue into a caller
 *              supplied buffer.  This function takes the byte destination address
 *              and reads a int queue record filing the host
 *              buffer
 * DESCRIPTION:
 *
 *      It will return:
 *              nothing.
 *===========================================================================*/

void usbReadIntQueue(
   BT_UINT   cardnum,       // (i) card number
   BT_U32BIT *lpbuffer,     // (o) host buffer (destination)
   BT_U32BIT byteOffset)    // (i) BYTE offset within adapter memory (source)
{
   BT_UINT index;

   index=(byteOffset-BTMEM_IQ_V6)/sizeof(IQ_MBLOCK_V6);
   ((IQ_MBLOCK_V6 *)lpbuffer)->msg_ptr = usb_int_queue[cardnum][index].msg_ptr;
   ((IQ_MBLOCK_V6 *)lpbuffer)->mode    = usb_int_queue[cardnum][index].mode;
}

/*===========================================================================*
 *  ENTRY POINT:         u s b R e a d R e l R A M 3 2
 *===========================================================================*
 *
 * FUNCTION:    Read data from selected range of adapter memory into a caller
 *              supplied buffer.
 *
 * DESCRIPTION:
 *
 *      It will return:
 *              nothing.
 *===========================================================================*/
void usbReadRelRAM32(
   BT_UINT   cardnum,      // (i) card number
   BT_U32BIT *lpbuffer,    // (o) host buffer (destination)
   BT_U32BIT byteOffset,   // (i) byte offset within adapter memory (source)
   BT_UINT   wordsToRead)   // (i) number of bytes to copy
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   CEI_UINT32 retval;
   CEI_READ_WRITE_REQ usbData;
   CEI_INSTALL_ERR cei_err;

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                     CEI_RWR_READ,
                     NULL,
                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][BASE_PAGE] + byteOffset)),
                     wordsToRead*4,
                     (pCEI_UCHAR)lpbuffer,
                     CEI_READ_WRITE_REQ_VER,
                     &usbData,
                     cei_err);

   if(retval==0)
      usb_error(cardnum,cei_err,"usbReadRelRAM32");

   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, &usbData, 1,cei_err);

   if(retval==0)
      usb_error(cardnum,cei_err,"usbReadRelRAM32");
}

/*===========================================================================*
 *  ENTRY POINT:         u s b R e a d S h a r e M e m o r y
 *===========================================================================*
 *
 * FUNCTION:    Read data from selected range of shared memory into a caller
 *              supplied buffer.
 *
 * DESCRIPTION:
 *
 *      It will return:
 *              nothing.
 *===========================================================================*/
void usbReadSharedMemory(
   BT_UINT   cardnum,      // (i) card number
   BT_U32BIT *lpbuffer,    // (o) host buffer (destination)
   BT_U32BIT byteOffset,   // (i) byte offset within adapter memory (source)
   BT_UINT   wordsToRead)   // (i) number of bytes to copy
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   CEI_UINT32 retval;
   CEI_READ_WRITE_REQ usbData;
   CEI_INSTALL_ERR cei_err;

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                     CEI_RWR_READ,
                     NULL,
                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][SHR_MEM_PAGE] + byteOffset)),
                     wordsToRead*4,
                     (pCEI_UCHAR)lpbuffer,
                     CEI_READ_WRITE_REQ_VER,
                     &usbData,
                     cei_err);

   if(retval==0)
      usb_error(cardnum,cei_err,"usbReadSharedMemory");

   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, &usbData, 1,cei_err);

   if(retval==0)
      usb_error(cardnum,cei_err,"usbReadSharedMemory");
}

/*===========================================================================*
 * EXTERNAL ENTRY POINT:      u s b W r i t e R A M 3 2
 *===========================================================================*
 *
 * FUNCTION:    Write data from caller supplied buffer into adapter memory.
 *
 * DESCRIPTION:
 *
 *      It will return:
 *              nothing
 *===========================================================================*/

void usbWriteRAM32(
   BT_UINT   cardnum,      // (i) card number
   BT_U32BIT *lpbuffer,    // (i) host buffer (source)
   BT_U32BIT byteOffset,   // (o) byte offset within adapter (destination)
   BT_UINT   wordsToWrite) // (i) number of bytes to copy
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   BT_INT retval;
   CEI_READ_WRITE_REQ usbData[64];
   CEI_INSTALL_ERR cei_err;
   BT_INT rcount;

   retval =  LL_InitWriteRequestBlock(gpDevice[cardnum],
                                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][RAM_PAGE] + byteOffset)),
                                     wordsToWrite*4,
                                     (pCEI_UCHAR)lpbuffer,
                                     usbData,
                                     &rcount);

   if(retval)
      usb_error(cardnum,cei_err,"usbWriteRAM32");

   if(rcount==0)
      return;

   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, usbData,rcount,cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbWriteRAM32");
}

 /*===========================================================================*
 * EXTERNAL ENTRY POINT:     u s b W r i t e B M R A M 3 2
 *===========================================================================*
 *
 * FUNCTION:    Write data from caller supplied buffer into adapter memory.
 *
 * DESCRIPTION:
 *
 *      It will return:
 *              nothing
 *===========================================================================*/

void usbWriteBMRAM32(
   BT_UINT   cardnum,      // (i) card number
   BT_U32BIT *lpbuffer,     // (i) host buffer (source)
   BT_U32BIT byteOffset,   // (o) byte offset within adapter (destination)
   BT_UINT   wordsToWrite) // (i) number of bytes to copy
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   BT_UINT    i;

   byteOffset = byteOffset - btmem_bm_mbuf[cardnum];
#ifdef WORD_SWAP
   for( i = 0; i < wordsToWrite; i++ )
   {
      flipw(lpbuffer[i]);
   }
   for ( i = 0; i < wordsToWrite; i++ )
   {
      ((BT_U32BIT *)(bt_PageAddr[cardnum][BM_SIM_PAGE]+byteOffset))[i] = lpbuffer[i];
   }
#else //WORD_SWAP
   for ( i = 0; i < wordsToWrite; i++ )
   {
      ((BT_U32BIT *)(bt_PageAddr[cardnum][BM_SIM_PAGE]+byteOffset))[i] = lpbuffer[i];
   }
#endif //WORD_SWAP

}

/*===========================================================================*
 * EXTERNAL ENTRY POINT:      u s b W r i t e R e l R A M 3 2
 *===========================================================================*
 *
 * FUNCTION:    Write data from caller supplied buffer into adapter memory.
 *
 * DESCRIPTION:
 *
 *      It will return:
 *              nothing
 *===========================================================================*/

void usbWriteRelRAM32(
   BT_UINT   cardnum,      // (i) card number
   BT_U32BIT *lpbuffer,    // (i) host buffer (source)
   BT_U32BIT byteOffset,   // (o) byte offset within adapter (destination)
   BT_UINT   wordsToWrite) // (i) number of bytes to copy
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   BT_INT retval;
   CEI_READ_WRITE_REQ usbData;
   CEI_INSTALL_ERR cei_err;

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                     CEI_RWR_WRITE,
                     NULL,
                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][BASE_PAGE] + byteOffset)),
                     wordsToWrite*4,
                     (pCEI_UCHAR)lpbuffer,
                     CEI_READ_WRITE_REQ_VER,
                     &usbData,
                     cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbWriteRelRAM32");

   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, &usbData, 1,cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbWriteRelRAM32");
}

/*===========================================================================*
 * EXTERNAL ENTRY POINT:      u s b W r i t e S h a r e M e m o r y
 *===========================================================================*
 *
 * FUNCTION:    Write data from caller supplied buffer into shared memory.
 *
 * DESCRIPTION:
 *
 *      It will return:
 *              nothing
 *===========================================================================*/

void usbWriteSharedMemory(
   BT_UINT   cardnum,      // (i) card number
   BT_U32BIT *lpbuffer,    // (i) host buffer (source)
   BT_U32BIT byteOffset,   // (o) byte offset within adapter (destination)
   BT_UINT   wordsToWrite) // (i) number of bytes to copy
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   BT_INT retval;
   CEI_READ_WRITE_REQ usbData;
   CEI_INSTALL_ERR cei_err;

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                     CEI_RWR_WRITE,
                     NULL,
                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][SHR_MEM_PAGE] + byteOffset)),
                     wordsToWrite*4,
                     (pCEI_UCHAR)lpbuffer,
                     CEI_READ_WRITE_REQ_VER,
                     &usbData,
                     cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbWriteSharedMemory");

   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, &usbData, 1,cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbWriteSharedMemory");
}



/*===========================================================================*
 * EXTERNAL ENTRY POINT:      u s b W r i t e H I F
 *===========================================================================*
 *
 * FUNCTION:    Write data from caller supplied buffer into adapter memory.
 *
 * DESCRIPTION:
 *
 *      It will return:
 *              nothing
 *===========================================================================*/

void usbWriteHIF(
   BT_UINT   cardnum,      // (i) card number
   BT_U32BIT *lpbuffer,    // (i) host buffer (source)
   BT_U32BIT byteOffset,   // (o) byte offset within adapter (destination)
   BT_UINT   wordsToWrite) // (i) number of bytes to copy
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   BT_INT retval;
   CEI_READ_WRITE_REQ usbData;
   CEI_INSTALL_ERR cei_err;

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                     CEI_RWR_WRITE,
                     NULL,
                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][CSC_REG_PAGE] +byteOffset)),
                     wordsToWrite*4,
                     (pCEI_UCHAR)lpbuffer,
                     CEI_READ_WRITE_REQ_VER,
                     &usbData,
                     cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbWriteHIF");

   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, &usbData, 1,cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbWriteHIF");
}


/*===========================================================================*
 * EXTERNAL ENTRY POINT:         u s b R e a d H I F
 *===========================================================================*
 *
 * FUNCTION:    Read data from selected range of adapter memory into a caller
 *              supplied buffer.
 *
 * DESCRIPTION:
 *
 *      It will return:
 *              nothing.
 *===========================================================================*/

void usbReadHIF(
   BT_UINT   cardnum,       // (i) card number
   BT_U32BIT *lpbuffer,      // (o) host buffer (destination)
   BT_U32BIT byteOffset,    // (i) byte offset within adapter memory (source)
   BT_UINT   wordsToRead)   // (i) number of bytes to copy
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   BT_INT retval;
   CEI_READ_WRITE_REQ usbData;
   CEI_INSTALL_ERR cei_err;

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                     CEI_RWR_READ,
                     NULL,
                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][CSC_REG_PAGE] + byteOffset)),
                     wordsToRead*4,
                     (pCEI_UCHAR)lpbuffer,
                     CEI_READ_WRITE_REQ_VER,
                     &usbData,
                     cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbReadHIF");

   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, &usbData, 1,cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbReadHIF");
}

/*===========================================================================*
 * EXTERNAL ENTRY POINT:       u s b S e t R e g i s t e r
 *===========================================================================*
 *
 * FUNCTION:    Sets the value of a specified HW or RAM register.
 *
 * DESCRIPTION: The current frame register is first saved, then set to page
 *              zero.  The specified register is then written and the frame reg
 *              is restored to the saved value.  The old value of the specified
 *              register is returned to the caller.  The register offset is
 *              assumed to be a WORD offset, NOT a BYTE offset...
 *
 *      It will return:
 *              Previous value of the specified register.
 *===========================================================================*/

void usbSetRegister(
   BT_UINT   cardnum,       // (i) card number (0 based)
   BT_U32BIT regnum,        // (i) register number, WORD offset
   BT_U32BIT regval)        // (i) new value
{
   BT_INT retval;
   CEI_READ_WRITE_REQ usbData;
   CEI_INSTALL_ERR cei_err;

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                     CEI_RWR_WRITE,
                     NULL,
                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][HWREG_PAGE] + regnum*4)),
                     4,
                     (pCEI_UCHAR)&regval,
                     CEI_READ_WRITE_REQ_VER,
                     &usbData,
                     cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbSetRegister");

   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, &usbData, 1,cei_err); 
   if(retval==0)
      usb_error(cardnum,cei_err,"usbSetRegister");
   
   usb_host_hwreg[cardnum][regnum] = regval;
}

/*===========================================================================*
 * EXTERNAL ENTRY POINT:      u s b G e t R e g i s t e r
 *===========================================================================*
 *
 * FUNCTION:    Read the value of a specified HW or RAM register.
 *
 * DESCRIPTION: The current frame register is first saved, then set to page
 *              zero.  The specified register is then read and the frame reg
 *              is restored to the saved value.  The value of the specified
 *              register is returned to the caller.  The register offset is
 *              assumed to be a WORD offset, NOT a BYTE offset...
 *
 *      It will return:
 *              Value of the specified register.
 *===========================================================================*/

BT_U32BIT usbGetRegister(
   BT_UINT   cardnum,       // (i) card number (0 based)
   BT_U32BIT regnum)        // (i) register number, WORD offset
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   BT_UINT retval;
   BT_U32BIT rdata;
   CEI_READ_WRITE_REQ usbData;
   CEI_INSTALL_ERR cei_err;

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                     CEI_RWR_READ,
                     NULL,
                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][HWREG_PAGE] + regnum*4)),
                     4,
                     (pCEI_UCHAR)&rdata,
                     CEI_READ_WRITE_REQ_VER,
                     &usbData,
                     cei_err);

   if(retval==0)
      usb_error(cardnum,cei_err,"usbGetRegister");
   
   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, &usbData, 1,cei_err); 
   if(retval==0)
      usb_error(cardnum,cei_err,"usbGetRegister");

   usb_host_hwreg[cardnum][regnum] = rdata;
   return usb_host_hwreg[cardnum][regnum];
}

CEI_ULONG usbGetIqHeadPtr(BT_UINT cardnum)
{
   return usb_host_hwreg[cardnum][HWREG_IQ_HEAD_PTR];
}

CEI_ULONG usbGetBMHeadPtr(BT_UINT cardnum)
{
   return usb_host_hwreg[cardnum][HWREG_BM_HEAD_PTR];
}

/*===========================================================================*
 * EXTERNAL ENTRY POINT:      u s b G e t C S C R e g i s t e r
 *===========================================================================*
 *
 * FUNCTION:    Read the value of a specified HW or RAM register.
 *
 * DESCRIPTION: Read the CSC and ACR registers
 *
 *      It will return:
 *              Value of the specified register.
 *===========================================================================*/

BT_U32BIT usbGetCSCRegister(
   BT_UINT   cardnum,       // (i) card number (0 based)
   BT_U32BIT regnum)        // (i) register number, WORD offset
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   BT_U32BIT rdata=0;
   BT_INT retval;
   CEI_READ_WRITE_REQ usbData;
   CEI_INSTALL_ERR cei_err;

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                     CEI_RWR_READ,
                     NULL,
                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][CSC_REG_PAGE] + regnum * 4)),
                     4,
                     (CEI_UCHAR *)&rdata,
                     CEI_READ_WRITE_REQ_VER,
                     &usbData,
                     cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbGetCSCRegister");


   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, &usbData, 1,cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbGetCSCRegister");

   return rdata;
}

/*===========================================================================*
 * EXTERNAL ENTRY POINT:      u s b S e t C S C R e g i s t e r
 *===========================================================================*
 *
 * FUNCTION:    Read the value of a specified HW or RAM register.
 *
 * DESCRIPTION: Read the CSC and ACR registers
 *
 *      It will return:
 *              Value of the specified register.
 *===========================================================================*/

void usbSetCSCRegister(
   BT_UINT   cardnum,       // (i) card number (0 based)
   BT_U32BIT regnum,
   BT_U32BIT regval)        // (i) register number, WORD offset
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   BT_INT retval;
   CEI_READ_WRITE_REQ usbData;
   CEI_INSTALL_ERR cei_err;

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                     CEI_RWR_WRITE,
                     NULL,
                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][CSC_REG_PAGE] + regnum*4)),
                     4,
                     (pCEI_UCHAR)&regval,
                     CEI_READ_WRITE_REQ_VER,
                     &usbData,
                     cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbSetCSCRegister");

   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, &usbData, 1,cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbSetCSCRegister");
}



/*===========================================================================*
 * EXTERNAL ENTRY POINT:      u s b G e t T T R e g i s t e r
 *===========================================================================*
 *
 * FUNCTION:    Read the value of a specified HW or RAM register.
 *
 * DESCRIPTION: Read the CSC and ACR registers
 *
 *      It will return:
 *              Value of the specified register.
 *===========================================================================*/

BT_U32BIT usbGetTTRegister(
   BT_UINT   cardnum,       // (i) card number (0 based)
   BT_U32BIT regnum)        // (i) register number, WORD offset
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   BT_U32BIT rdata;
   BT_INT retval;
   CEI_READ_WRITE_REQ usbData;
   CEI_INSTALL_ERR cei_err;

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                     CEI_RWR_READ,
                     NULL,
                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][TTREG_PAGE] + regnum*4)),
                     4,
                     (CEI_UCHAR *)&rdata,
                     CEI_READ_WRITE_REQ_VER,
                     &usbData,
                     cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbGetTTRegister");

   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, &usbData, 1,cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbGetTTRegister");

   return rdata;
}

/*===========================================================================*
 * EXTERNAL ENTRY POINT:      u s b S e t T T R e g i s t e r
 *===========================================================================*
 *
 * FUNCTION:    Read the value of a specified HW or RAM register.
 *
 * DESCRIPTION: Read the CSC and ACR registers
 *
 *      It will return:
 *              Value of the specified register.
 *===========================================================================*/

void usbSetTTRegister(
   BT_UINT   cardnum,       // (i) card number (0 based)
   BT_U32BIT regnum,
   BT_U32BIT regval)        // (i) register number, WORD offset
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   BT_INT retval;
   CEI_READ_WRITE_REQ usbData;
   CEI_INSTALL_ERR cei_err;

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                     CEI_RWR_WRITE,
                     NULL,
                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][TTREG_PAGE] + regnum*4)),
                     4,
                     (pCEI_UCHAR)&regval,
                     CEI_READ_WRITE_REQ_VER,
                     &usbData,
                     cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbSetTTRegister");


   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, &usbData, 1,cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbSetTTRegister");
}


/*===========================================================================*
 * EXTERNAL ENTRY POINT:      u s b G e t T T R e g i s t e r
 *===========================================================================*
 *
 * FUNCTION:    Read the value of a specified HW or RAM register.
 *
 * DESCRIPTION: Read the CSC and ACR registers
 *
 *      It will return:
 *              Value of the specified register.
 *===========================================================================*/

BT_U32BIT usbGetTrigRegister(
   BT_UINT   cardnum,       // (i) card number (0 based)
   BT_U32BIT regnum)        // (i) register number, WORD offset
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   BT_U32BIT rdata;
   BT_INT retval;
   CEI_READ_WRITE_REQ usbData;
   CEI_INSTALL_ERR cei_err;

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                     CEI_RWR_READ,
                     NULL,
                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][TRIG_PAGE] + regnum*4)),
                     4,
                     (CEI_UCHAR *)&rdata,
                     CEI_READ_WRITE_REQ_VER,
                     &usbData,
                     cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbGetTrigRegister");             


   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, &usbData, 1,cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbGetTrigRegister");

   return rdata;
}

/*===========================================================================*
 * EXTERNAL ENTRY POINT:      u s b S e t T r i g R e g i s t e r
 *===========================================================================*
 *
 * FUNCTION:    Read the value of a specified HW or RAM register.
 *
 * DESCRIPTION: Read the CSC and ACR registers
 *
 *      It will return:
 *              Value of the specified register.
 *===========================================================================*/

void usbSetTrigRegister(
   BT_UINT   cardnum,       // (i) card number (0 based)
   BT_U32BIT regnum,
   BT_U32BIT regval)        // (i) register number, WORD offset
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   BT_INT retval;
   CEI_READ_WRITE_REQ usbData;
   CEI_INSTALL_ERR cei_err;

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                     CEI_RWR_WRITE,
                     NULL,
                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][TRIG_PAGE] + regnum*4)),
                     4,
                     (pCEI_UCHAR)&regval,
                     CEI_READ_WRITE_REQ_VER,
                     &usbData,
                     cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbSetTrigRegister");

   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, &usbData, 1,cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbSetTrigRegister");
}



/*===========================================================================*
 * EXTERNAL ENTRY POINT:       u s b S e t D i s c r e t e
 *===========================================================================*
 *
 * FUNCTION:    Sets the discrete register.
 *
 * DESCRIPTION: Set the specified discrete register to the specify value.
 *
 *      It will return:
 *              Previous value of the specified register.
 *===========================================================================*/

void usbSetDiscrete(
   BT_UINT   cardnum,       // (i) card number (0 based)
   BT_U32BIT regnum,        // (i) register number, WORD offset
   BT_U32BIT regval)        // (i) new value
{
   CEI_READ_WRITE_REQ usbData;
   CEI_INSTALL_ERR cei_err;
   BT_INT retval;

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],   
                     CEI_RWR_WRITE,
                     NULL,
                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][CSC_REG_PAGE] + regnum*4)),
                     4,
                     (CEI_UCHAR *)&regval,
                     CEI_READ_WRITE_REQ_VER,
                     &usbData,
                     cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbSetDiscrete");

   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, &usbData, 1,cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbSetDiscrete");
}

/*===========================================================================*
 * EXTERNAL ENTRY POINT:      u s b G e t D i s c r e t e
 *===========================================================================*
 *
 * FUNCTION:    Read discrete reigsters.
 *
 * DESCRIPTION: The routine read the specified discrete registers.
 *
 *      It will return:
 *              Value of the specified register.
 *===========================================================================*/

BT_U32BIT usbGetDiscrete(
   BT_UINT   cardnum,       // (i) card number (0 based)
   BT_U32BIT regnum)        // (i) register number, WORD offset
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   BT_U32BIT rdata;
   CEI_READ_WRITE_REQ usbData;
   BT_INT retval;
   CEI_INSTALL_ERR cei_err;

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                     CEI_RWR_READ,
                     NULL,
                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][CSC_REG_PAGE] + regnum*4)),
                     4,
                     (CEI_UCHAR *)&rdata,
                     CEI_READ_WRITE_REQ_VER,
                     &usbData,
                     cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbGetDiscrete");

   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, &usbData, 1,cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbGetDiscrete");

   return rdata;
}

/*===========================================================================*
 * EXTERNAL ENTRY POINT:      u s b R e a d T i m e T a g
 *===========================================================================*
 *
 * FUNCTION:    Read the value of the 45-bit time tag counter register.
 *
 * DESCRIPTION: The value of the time tag register is read and returned.
 *              Care is taken to insure that the components do not wrap
 *              while reading the 3 16-bit words.  Byte offsets 0x2A-0x2F.
 *
 *      It will return:
 *              Value of the specified register.
 *===========================================================================*/

void usbReadTimeTag(
   BT_UINT   cardnum,          // (i) card number (0 based)
   BT_U32BIT * timetag)    // (o) resulting 48-bit time value from HW
{
   BT_INT retval;
   BT_U32BIT wdata = 0;
   BT_U32BIT rdata[2];
   CEI_INSTALL_ERR cei_err;
   CEI_READ_WRITE_REQ usbData[4];

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                     CEI_RWR_WRITE,
                     NULL,
                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][TTREG_PAGE] + TTREG_LATCH*4)),
                     4,
                     (CEI_UCHAR *)&wdata,
                     CEI_READ_WRITE_REQ_VER,
                     &usbData[0],
                     cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbReadTimeTag");

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                     CEI_RWR_READ,
                     NULL,
                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][TTREG_PAGE] + TTREG_READ_LOW*4)),
                     4,
                     (CEI_UCHAR *)&rdata[0],
                     CEI_READ_WRITE_REQ_VER,
                     &usbData[1],
                     cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbReadTimeTag");

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                     CEI_RWR_READ,
                     NULL,
                     (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][TTREG_PAGE] + TTREG_READ_HIGH*4)),
                     4,
                     (CEI_UCHAR *)&rdata[1],
                     CEI_READ_WRITE_REQ_VER,
                     &usbData[2],
                     cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbReadTimeTag");

   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, usbData, 3,cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbReadTimeTag");


   timetag[0]=rdata[0];
   timetag[1]=rdata[1];
}

/*===========================================================================*
 * EXTERNAL ENTRY POINT:      u s b W r i t e T i m e T a g
 *===========================================================================*
 *
 * FUNCTION:    Write the value of the 45-bit time tag load register.
 *
 * DESCRIPTION: The value of the time tag load register is set to the specified
 *              value.
 *
 *      It will return:
 *              nothing.
 *===========================================================================*/

void usbWriteTimeTag(
   BT_UINT   cardnum,             // (i) card number
   BT1553_TIME * timetag)         // (i) 64-bit time value to load into register
{ 
   BT_INT retval;
   CEI_INSTALL_ERR cei_err;
   CEI_READ_WRITE_REQ usbData[2];

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                  CEI_RWR_WRITE,
                  NULL,
                  (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][TTREG_PAGE] + TTREG_LOAD_LOW*4)),
                  4,
                  (CEI_UCHAR *)&timetag->microseconds,
                  CEI_READ_WRITE_REQ_VER,
                  &usbData[0],
                  cei_err);

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                  CEI_RWR_WRITE,
                  NULL,
                  (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][TTREG_PAGE] + TTREG_LOAD_HIGH*4)),
                  4,
                  (CEI_UCHAR *)&timetag->topuseconds,
                  CEI_READ_WRITE_REQ_VER,
                  &usbData[1],
                  cei_err);

   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, usbData, 2,cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbWriteTimeTag");
}

/*===========================================================================*
 * EXTERNAL ENTRY POINT:      u s b W r i t e T i m e T a g I n c r
 *===========================================================================*
 *
 * FUNCTION:    Write the value of the 45-bit time tag load register.
 *
 * DESCRIPTION: The value of the time tag load register is set to the specified
 *              value.
 *
 *      It will return:
 *              nothing.
 *===========================================================================*/

void usbWriteTimeTagIncr(
   BT_UINT   cardnum,             // (i) card number
   BT_U32BIT incr)                // (i) increment value
{
   //TTREG_INCREMENT
   BT_INT retval;
   CEI_INSTALL_ERR cei_err;
   CEI_READ_WRITE_REQ usbData;

   retval = LL_InitReadWriteRequestStruct(gpDevice[cardnum],
                  CEI_RWR_WRITE,
                  NULL,
                  (CEI_UINT32)((CEI_UINT64)(bt_PageAddr[cardnum][TTREG_PAGE] + TTREG_INCREMENT*4)),
                  4,
                  (CEI_UCHAR *)&incr,
                  CEI_READ_WRITE_REQ_VER,
                  &usbData,
                  cei_err);

   retval = LL_ExecuteReadWriteRequest(gpDevice[cardnum],CEI_READ_WRITE_REQ_VER, &usbData, 1,cei_err);
   if(retval==0)
      usb_error(cardnum,cei_err,"usbWriteTimeTagIncr");
}


/*===========================================================================*
 * ENTRY POINT: LL_InitReadWriteRequestBlock
 *===========================================================================*
 *
 * FUNCTION:    Perform a block read/write request. 
 *
 * RETURN VAL:  TRUE    operation completed successfully 
 *              FALSE   operation failed - error description stored in 'err'
 *
 * DESCRIPTION: Prepare as many read/write request structures as are necessary
 *              to represent a read/write of the given number of bytes from/to
 *              the given device byte offset to/from the given host buffer.
 *
 *              This function initializes a set of read/write request structs
 *              that can be executed with LL_ExecuteReadWriteRequest.
 *           
 *              See LL_ExecuteReadWriteRequest documentation for a discussion
 *              of read/write request support. 
 *
 * EXAMPLE:     fixme - tbd
 *
 *===========================================================================*/
CEI_INT32 LL_InitReadRequestBlock
(
    CEI_DEV_HANDLE dev_handle,   // (i) handle of device instance
    CEI_UINT32 offset_bytes,     // (i) offset (in bytes) to begin reading/
                                 //     writing at within the device memory 
                                 //     region specified by rgn_usr_addr
    CEI_UINT32 request_bytes,    // (i) number of bytes to read or write
    pCEI_VOID data_ptr,          // (i) pointer to caller-allocated buffer of 
                                 //     length request_bytes which contains data
                                 //     to write (if direction is CEI_RWR_WRITE)
                                 //     or which receives data that was read (if 
                                 //     direction is CEI_RWR_READ)

    pCEI_READ_WRITE_REQ rwr_ptr, // (o) pointer to caller-allocated buffer of 
                                 //     max_rwrs read/write request structures
                                 //     that, upon successful return, is 
                                 //     initialized and ready to pass to 
                                 //     LL_ExecuteReadWriteRequest
    pCEI_UINT32 num_rwrs         // (o) receives the number of read/write
                                 //     request structures written into the 
                                 //     rwr_ptr buffer (upon successful
                                 //     return)
)
{

   CEI_UINT32 rwr_start_offset;      // starting offset for particular RWR
   CEI_UINT32 rwr_len_bytes;         // length of RWR request (in bytes)
   CEI_INSTALL_ERR cei_err;          // temporary error string buffer

   // make sure we have enough RWR structs to perform the read/write
   if ((MAX_RWR_SIZE * MAX_RWR) < request_bytes)
      return 2345; 

   // convert block read/write into read/write request structures
   for (*num_rwrs=0;;(*num_rwrs)++)
   {
      // determine start offset for this RWR
      rwr_start_offset = *num_rwrs * MAX_RWR_SIZE;
      if (rwr_start_offset >= request_bytes)
         break;

      // determine byte count for this RWR
      rwr_len_bytes = min(MAX_RWR_SIZE,request_bytes - rwr_start_offset);

      // initialize the RWR structure
      if (!LL_InitReadWriteRequestStruct(dev_handle,CEI_RWR_READ,NULL,offset_bytes+rwr_start_offset,rwr_len_bytes,(pCEI_CHAR)data_ptr+rwr_start_offset,CEI_READ_WRITE_REQ_VER,&rwr_ptr[*num_rwrs],cei_err))
         return 3456;
   }

   // operation was successful
   return API_SUCCESS;
}

CEI_INT32 LL_InitWriteRequestBlock
(
    CEI_DEV_HANDLE dev_handle,   // (i) handle of device instance
    CEI_UINT32 offset_bytes,     // (i) offset (in bytes) to begin reading/
                                 //     writing at within the device memory 
                                 //     region specified by rgn_usr_addr
    CEI_UINT32 request_bytes,    // (i) number of bytes to read or write
    pCEI_VOID data_ptr,          // (i) pointer to caller-allocated buffer of 
                                 //     length request_bytes which contains data
                                 //     to write (if direction is CEI_RWR_WRITE)
                                 //     or which receives data that was read (if 
                                 //     direction is CEI_RWR_READ)

    pCEI_READ_WRITE_REQ rwr_ptr, // (o) pointer to caller-allocated buffer of 
                                 //     max_rwrs read/write request structures
                                 //     that, upon successful return, is 
                                 //     initialized and ready to pass to 
                                 //     LL_ExecuteReadWriteRequest
    pCEI_UINT32 num_rwrs         // (o) receives the number of read/write
                                 //     request structures written into the 
                                 //     rwr_ptr buffer (upon successful
                                 //     return)
)
{

   CEI_UINT32 rwr_start_offset;      // starting offset for particular RWR
   CEI_UINT32 rwr_len_bytes;         // length of RWR request (in bytes)
   CEI_INSTALL_ERR cei_err;          // temporary error string buffer

   // make sure we have enough RWR structs to perform the read/write
   if ((MAX_RWR_SIZE * MAX_RWR) < request_bytes)
      return 2345; 

   // convert block read/write into read/write request structures
   for (*num_rwrs=0;;(*num_rwrs)++)
   {
      // determine start offset for this RWR
      rwr_start_offset = *num_rwrs * MAX_RWR_SIZE;
      if (rwr_start_offset >= request_bytes)
         break;

      // determine byte count for this RWR
      rwr_len_bytes = min(MAX_RWR_SIZE,request_bytes - rwr_start_offset);

      // initialize the RWR structure
      if (!LL_InitReadWriteRequestStruct(dev_handle,CEI_RWR_WRITE,NULL,offset_bytes+rwr_start_offset,rwr_len_bytes,(pCEI_CHAR)data_ptr+rwr_start_offset,CEI_READ_WRITE_REQ_VER,&rwr_ptr[*num_rwrs],cei_err))
         return 3456;
   }

   // operation was successful
   return API_SUCCESS;
}

#endif


